  MODULE Background

  USE pointmod, only : point_type

  implicit none

  integer, parameter :: POLY        = 1
  integer, parameter :: CHEBY       = 2
  integer, parameter :: FOUR_SERIES = 3

  integer, parameter :: NMAXCOEFB = 36 

  type back_condition_type
    integer :: btype = CHEBY     ! background type
    logical :: auto  = .false.   ! automatic selection of number of coefficient
    integer :: ncoef = 6         ! number of coefficients
  end type back_condition_type
!!!FIXME add coefb,ncoefb,thzerob,pointb,npointb in a different struct background_type

!corr  type background_type
!corr    private
!corr    type(back_condition_type)                   :: cond    ! settings for create backgrund
!corr    real, dimension(:), allocatable             :: yb      ! background
!corr    type(point_type), dimension(:), allocatable :: points  ! background points
!corr    real                                        :: thzerob ! zero of polynomial
!corr    real, dimension(:), allocatable             :: coef    ! number of coefficients
!corr
!corr  contains
!corr    procedure :: set_cond
!corr
!corr    procedure :: get_cond
!corr    procedure :: get_points
!corr    procedure :: get_coef
!corr    procedure :: zeroback
!corr
!corr    procedure :: make => make_background
!corr  end type background_type
!!!!FIXME - 1) fare una update(x) che aggiorna il back se cambiano i punti e/o i coefficienti
!!!!FIXME - e conterra' la getbackground e la autobackground. forse va messo xmin e xmax nella classe background
!!!!FIXME - 2) la set_cond deve fare la param_back se coef è allocato e auto è false
!!!!FIXME      quindi la sequenza set_cond, update aggiorna il back se cambiano le condizioni
!!!!FIXME   3) la set_point deve contenere la param_back; la sequenza set_points, update aggiorna 
!!!!FIXME      se sono cambiati i punti

  integer, private, parameter :: dbl = kind(1.0d0)
!corr  private genera_background

  CONTAINS

!corr   subroutine set_cond(back,cond)
!corr!
!corr!  Set conditions for background
!corr!
!corr   class(background_type), intent(inout) :: back
!corr   type(back_condition_type), intent(in) :: cond
!corr   back%cond = cond
!corr   end subroutine set_cond
!corr
!corr!----------------------------------------------------------------------------------------
!corr
!corr   subroutine get_cond(back,cond)
!corr!
!corr!  Get conditions for background
!corr!
!corr   class(background_type), intent(in)     :: back
!corr   type(back_condition_type), intent(out) :: cond
!corr   cond = back%cond
!corr   end subroutine get_cond
!corr
!corr!----------------------------------------------------------------------------------------
!corr  
!corr   subroutine get_points(back,points)
!corr!
!corr!  Get background points
!corr!
!corr   USE pointmod
!corr   class(background_type), intent(in)                         :: back
!corr   type(point_type), dimension(:), allocatable, intent(inout) :: points
!corr   call copy_points(points,back%points)
!corr   end subroutine get_points
!corr
!----------------------------------------------------------------------------------------
!corr
!corr   subroutine get_coef(back,coef)  !!!!FIXME - add allocation on coef
!corr   class(background_type), intent(in) :: back
!corr   real, dimension(:), intent(out)    :: coef
!corr   coef = back%coef
!corr   end subroutine get_coef
!corr
!----------------------------------------------------------------------------------------
!corr
!corr   integer function zeroback(back)
!corr   class(background_type), intent(in) :: back
!corr   zeroback = back%thzerob
!corr   end function zeroback
!corr
!----------------------------------------------------------------------------------------
!corr
!corr   subroutine make_background(back,x,y,yb)
!corr   USE arrayutil
!corr   class(background_type), intent(inout)        :: back
!corr   real, dimension(:), intent(in)               :: x,y
!corr   real, dimension(:), allocatable, intent(out) :: yb
!corr   integer                                      :: np
!corr   call resize_array(back%coef,NMAXCOEFB)
!corr   call new_array(yb,size(y))
!corr   call genera_background(x,y,yb,back%coef,back%thzerob,back%points,np,back%cond)
!corr   end subroutine make_background
!corr
!----------------------------------------------------------------------------------------

   subroutine genera_background(xcount,ycount,bcount,coefb,thzerob,pointb,npointb,backt)
   real, dimension(:), intent(in)    :: xcount    ! x
   real, dimension(:), intent(in)    :: ycount    ! y
   real, dimension(:), intent(out)   :: bcount    ! background
   real, dimension(:), intent(out)   :: coefb
   real, intent(out)                 :: thzerob
   type(point_type), dimension(:), allocatable, intent(out) :: pointb
   integer, intent(out)              :: npointb
   type(back_condition_type), intent(inout) :: backt
!
!  Seleziona punti di background sull'osservato
   call piazza_punti(xcount,ycount,npointb,pointb)
!
!  Definisci origine (thminbkg) del polinomio
   thzerob = (xcount(1) + xcount(size(xcount))) / 2.0
!
!  Cerca in automatico il miglior numero di coefficienti
   call autobackground(xcount,bcount,backt%btype,pointb(:npointb),thzerob,coefb,backt%ncoef)
!
   end subroutine genera_background

!----------------------------------------------------------------------------------------

   subroutine autobackground(xcount,bcount,typeback,backp,thzerob,coefb,ncoef)
!
   implicit none
   real, dimension(:), intent(in)   :: xcount
   real, dimension(:), intent(out)  :: bcount
   integer, intent(in)              :: typeback
   type(point_type), dimension(:), intent(in) :: backp
   real, intent(in)                 :: thzerob
   real, dimension(:), intent(out)  :: coefb
   integer, intent(out)             :: ncoef
   integer                          :: npointb
   integer                          :: nbk,ncoefmax
   real                             :: rpbmin,rpback
   real, allocatable                :: bcalc(:)
   real, allocatable                :: coefbkg0(:)
   real                             :: thmin0,thmax0
!
   npointb = size(backp,1)                         ! migliorare introducendo controllo
   select case (typeback)                          ! su background troppo alti rispetto all'osservato
       case (POLY,FOUR_SERIES)                     ! magari rimuovendo il punto piu' vicino
           ncoefmax = 8
       case (CHEBY)
           ncoefmax = 20  !!!10
   end select
!
   thmin0 = xcount(1)              
   thmax0 = xcount(size(xcount))  
   rpbmin = huge(1.0)
   !ncoefmax = min(npointb-2,ncoefmax)   ! il num. coeff. non può essere piu' grande dei punti
   ncoefmax = min(nint(npointb*0.70),ncoefmax)   ! il num. coeff. non può essere piu' grande dei punti
   allocate(bcalc(npointb),coefbkg0(ncoefmax))
!
   do nbk=1,ncoefmax
!
!     Calcola coefficienti
      call param_back(typeback,backp(:npointb),nbk,coefbkg0,thmin0,thmax0,thzerob)
!
!     Calcola background nei punti selezionati
      !call getbackground(typeback,bcalc(:npointb),backp(:npointb)%x,coefbkg0(:nbk),thmin0,thmax0,thzerob)
      call getbackground(typeback,bcalc(:npointb),backp(:npointb)%x,coefbkg0(:nbk),thzerob)
!
!     Calcola errp
      rpback = ( SUM(abs(backp(:npointb)%y-bcalc(:))) / SUM(backp(:npointb)%y) ) * 100.
!
!     Seleziona errep minimo
      if (rpback < rpbmin) then
          rpbmin = rpback
          ncoef = nbk
          coefb(:nbk) = coefbkg0(:nbk)
      endif
   enddo
!
!  Calcola il background
   !call getbackground(typeback,bcount,xcount,coefb(:ncoef),thmin0,thmax0,thzerob)
   call getbackground(typeback,bcount,xcount,coefb(:ncoef),thzerob)
!
   end subroutine autobackground

  !---------------------------------------------------------------------------------------

   subroutine piazza_punti(xval,yval,npoints,points)
!
!  Cerca i punti i background
!
   USE pointmod
        USE General, only:alambda
        USE Counts
   real, dimension(:), intent(in)    :: xval,yval ! dove piazzare i punti di background
   integer, intent(out)              :: npoints   ! num. di punti
   type(point_type), dimension(:), allocatable, intent(inout) :: points    ! punti di background
   real                              :: size_window
   integer                           :: minpoint
   integer                           :: nval
   type(point_type), allocatable, dimension(:) :: dati
   integer :: nplim
   real, parameter :: SWIN_CU = 2.0   ! default window size for Cu wavelength
   real :: tstep
   integer :: sizep
      !real, dimension(size(yvali)) :: yval
!
!   yval(:) = yvali(:)  ! prova-elimina poi yvali
     !call bruckner_back(yvali,yval,100,20)
!
   nval = size(xval)
   !minpoint = min(40,nval)
   minpoint = min(10,nval)
!
!  Definisci l'ampiezza di una finestra (in gradi) nella quale piazzare il punto
!
   !size_window = 2.0
   !size_window = deltatval(SWIN_CU,0.085,alambda)
   !  write(0,*)'SIZE=',size_window
   size_window = delta_from_lambda(SWIN_CU,1.54056,alambda,4.0)
     !write(0,*)'SIZE=',size_window
   !size_window = deltatval(2.0,0.040,alambda)
      !write(0,*)'size=',size_window
      !write(0,*)'size=',deltadval(4.0,0.15,1.54056)
      !write(0,*)'size=',deltatval(4.0,deltadval(4.0,0.15,1.54056),alambda)
!
!!!!!!!!!!!!test
!
!  Calculate amplitude of window in terms of number of points
   tstep = sum(xval(2:nval) - xval(1:nval-1))/(nval-1)
   sizep = nint(size_window/tstep)
   !write(0,*)'npoints=',npoints
!!!!!!!!!!!!test
   !xpmin = minval(xval)
   !xpmax = maxval(xval)
   !npoints = max(nint((xpmax - xpmin) / size_window),minpoint)
   !npoints = min (npoints,nval)  ! npoints non puo' essere piu grande di nval
             !write(0,*)'lim=',xpmin,xpmax,npoints,minpoint,nval
!
!  Piazza i punti
   allocate(dati(size(xval)))
   dati%x = xval
   dati%y = yval
   call getminpoints_old(xval,yval,npoints,points,sizep)
   !call filtra_punti(points,4,0.1); npoints = size(points)
   !call getminpoints(dati,npoints,points)
   !write(0,*)'punti prima=',npoints
   !do i=1,min(npoints,10)
   !   write(0,*)i,points(i)
   !enddo
!
!  Primo e ultimo punto del pattern
   if (points(1)%x /= dati(1)%x) then           ! aggiungi solo se non esiste gia'
       call add_point(points,dati(1),1)
       npoints = npoints + 1                  ! incrementa npointback
   endif
   if (points(npoints)%x /= dati(nval)%x) then  ! aggiungi solo se non esiste gia'
       call add_point(points,dati(nval))
       npoints = npoints + 1                  ! incrementa npointback
   endif
!
!  Aggiungi punti agli estremi per stabilizzare il polinomio
   nplim = 3
   if (npoints >= nplim) then
       call insert_points(points,1,nplim,dati,0.5)
       npoints = size(points)
   endif
   !write(0,*)'punti dopo=',npoints
   !do i=1,min(npoints,10)
   !   write(0,*)i,points(i)
   !enddo
   nplim = 3
   if (npoints >= nplim*3) then
       call insert_points(points,npoints-nplim,npoints,dati,0.5)
       npoints = size(points)
   endif
!
   npoints = size(points)
!
   end subroutine piazza_punti

 !------------------------------------------------------------------------------------

   subroutine insert_points(points,ipos,fpos,dati,freq)
!
!  Inserisci punti fra ipos e fpos
!
   USE pointmod
   USE nr
   USE math_util
   USE arrayutil
   type(point_type), dimension(:), allocatable, intent(inout) :: points
   integer, intent(in)                                        :: ipos,fpos
   type(point_type), dimension(:), intent(in)                 :: dati
   real, intent(in)                                           :: freq
   integer, parameter                                         :: NPMAX = 5
   type(point_type), dimension(:), allocatable                :: pdat
   type(point_type), dimension(:), allocatable                :: padd
   integer, dimension(:), allocatable                :: iorder
   integer                                                    :: np
   integer                                                    :: nadd,npadd
   integer                                                    :: i,j
   integer                                                    :: loc
   real                                                       :: diff
   real                                                       :: rangey
!corr   real, dimension(2) :: retta
!
   np = size(points)
   allocate(padd(np))
   nadd = 0
   rangey = abs(points(fpos)%y - points(ipos)%y)
!!!equazione della retta per fpos e ipos come y=mx+c
   !mretta = (points(fpos)%y - points(ipos)%y) / (points(fpos)%x - points(ipos)%x)
   !cretta = -mretta
!corr   retta = straight_line(points(fpos)%x,points(fpos)%y,points(ipos)%x,points(ipos)%y)
     !write(0,*)'retta=',points(ipos)
     !write(0,*)'retta=',points(ipos)%x*retta(1) + retta(2)
!!!
!corr       write(0,*)'rangey=',rangey
       !write(0,*)'ipos=',ipos,' fpos=',fpos
   do i=ipos,fpos-1
!
!     valuta il numero di punti da aggiungere
      !npadd = min((points(i+nadd+1)%x - points(i+nadd)%x) / freq,real(NPMAX))
      npadd = nint(min((points(i+1)%x - points(i)%x) / freq,real(NPMAX)))
      if (npadd == 0) npadd = 1
      call new_points(pdat,npadd)
!
!     genera npadd punti
      !call create_point(points(i+nadd),points(i+1+nadd),pdat(:npadd))
      call create_point(points(i),points(i+1),pdat(:npadd))
!
!     Controllo per i punti al di sopra dell'osservato (dati)
      do j=1,npadd
         loc = clocate(dati%x,pdat(j)%x)
         diff = pdat(j)%y - dati(loc)%y
         if (diff > 0) then           ! punto in alto
             pdat(j)%y = dati(loc)%y 
               !write(0,*)'alto ',j,pdat(j)
         else                         ! punto troppo in basso
             if ((dati(loc)%y <= max(points(ipos)%y,points(fpos)%y)) .or.  &   !point is below the range ipos-fpos
              (abs(diff) < 0.4*rangey)) then
                 pdat(j)%y = dati(loc)%y ! la soluzione migliore e' un controllo
             else
                 cycle
             endif
       !old      if (abs(diff) < 0.4*rangey) pdat(j)%y = dati(loc)%y ! la soluzione migliore e' un controllo
               !write(0,*)'diff',diff,'punto ',j,pdat(j)          ! sull'osservato
     !write(0,*)'retta=',pdat(j)%x*retta(1) + retta(2)
         endif
         nadd = nadd + 1
         padd(nadd) = pdat(j)
      enddo
   enddo
   if (nadd > 0) then
      call add_points(points,padd(:nadd),ipos)
      np = size(points)
      allocate(iorder(np))
      call indexx(points%x,iorder)
      points(:) = points(iorder)
   endif
!
   end subroutine insert_points

 !------------------------------------------------------------------------------------

   subroutine create_point(point1,point2,pointn)
!
!  Crea n punti tra point1 e point2
!
   type(point_type), intent(in)                :: point1,point2
   type(point_type), dimension(:), intent(out) :: pointn
   real                                        :: stepx,stepy
   integer                                     :: i
   integer                                     :: np
!
   np = size(pointn)     ! numero di punti da creare
   stepx = (point2%x - point1%x) / (np+1)
   stepy = (point2%y - point1%y) / (np+1)
   do i=1,np
      pointn(i)%x = point1%x + i*stepx
      pointn(i)%y = point1%y + i*stepy
   enddo
!
   end subroutine create_point

 !------------------------------------------------------------------------------------

   subroutine getminpoints_old(xvet,yvet,np,punti,wind)
!
!  Cerca i minimi di un vettore
!
   USE pointmod
   integer, intent(out)            :: np
   real, dimension(:), intent(in)    :: xvet, yvet
   type(point_type), dimension(:), allocatable, intent(inout) :: punti
   integer, intent(in)               :: wind
   integer                           :: i   !!!,wind
   integer, dimension(1)             :: loc
   integer                           :: minp,maxp,posm,posmold
   integer                           :: npoint
   integer                           :: nsizep, nplus
!
   npoint = size(xvet)
   !corrwind = size(xvet) / np   ! ampiezza della finestra
!
!  Rialloca su npp senza salvare il suo contenuto
   nplus = max(2,npoint/10)
   nsizep = nplus
   call new_points(punti,nsizep)
!
   posmold = 1
   maxp = 0
   i=0
   do
      i=i+1
      if (nsizep < i) then
          nsizep = nsizep + nplus
          call resize_points(punti,nsizep)
      endif          
      minp = 1 + maxp
      maxp = min(minp + wind - 1,npoint)
      punti(i)%y = MINVAL( yvet(minp : maxp) )
      loc = MINLOC( yvet(minp : maxp) )
      posm = loc(1) + minp - 1
      !write(25,*)'punto n.',i,minp,maxp,posm
      !if ((posm == minp .or. posm == maxp) .and. (minp /= 1)) then
      if ((posm == minp .or. posm == maxp)) then
          !write(25,*)'       punto rigettato'
          minp = max(posm - wind/2,posmold+1)
          maxp = min(minp + wind - 1,npoint)
          punti(i)%y = MINVAL( yvet(minp : maxp) )
          loc = MINLOC( yvet(minp : maxp) )
          posm = loc(1) + minp - 1
          !write(25,*)'punto n.',i,minp,maxp,posm
          !if (posm == minp)write(25,*)'punto forse non valido'
      endif
      posmold = posm
      punti(i)%x = xvet(posm)
      if (maxp == npoint) exit
   enddo
   np = i
!
   call resize_points(punti,np)
!
   end subroutine getminpoints_old

 !------------------------------------------------------------------------------------

   subroutine getminpoints(dati,np,punti)
!
!  Cerca i minimi nel vettore dati
!   
   USE smoothing
   USE stat
   USE peak_util
!
   type(point_type), dimension(:), intent(in) :: dati
   integer, intent(out)                  :: np
   type(point_type), allocatable, intent(out) :: punti(:)
   integer                               :: npoint
!corr   integer                               :: i
!
   npoint = size(dati)
!
!  Cerca i minimi (ovvero i massimi di -point) dallo studio della derivata prima
!corr   call findmaxd1((/(point_type(dati(i)%x,-dati(i)%y),i=1,npoint)/),20,punti,np)
   call findmaxd1(dati(:)%x,-dati(:)%y,20,punti,np)
   punti%y = -punti%y
!   
!  Filtra i punti
   call filtra_punti(punti,4,0.1)
   np = size(punti)
!
   end subroutine getminpoints

  !-----------------------------------------------------------------------------

   subroutine findmaxd1_old(dati,npoints,pk,np)
   USE ssmoothing
   USE pointmod
!
   implicit none
   type(point_type), dimension(:), intent(in)     :: dati
   integer, intent(in)                :: npoints
   type(point_type), allocatable, intent(inout) :: pk(:)
   integer, intent(out)               :: np
   real, dimension(size(dati))       :: ders
   integer                            :: ndat
   integer                            :: i
   integer                            :: ini,fin
   integer, dimension(1)              :: locm
   integer, parameter                 :: nplus = 100
   integer                            :: nsizep
   integer                            :: pos,posnew
   integer                            :: nprange
!
!  nsizep è un ragionevale valore iniziale di allocazione per xp e yp
   nsizep = nplus
!
!  Rialloca su npp senza salvare il suo contenuto
   call new_points(pk,nsizep)
!
!  calcola smoothing della derivata prima
   ders = savgol_smooth(dati%y,npoints,npoints,1,2)  ! pol=5 prima
!
   ndat = size(dati)     ! num. di dati
!
!  selezione i punti in cui la derivata da positiva diventa negativa
   np = 0
   nprange = nint(npoints*0.5)
   pos = -1     ! inizializza pos 
   do i=2,ndat
      if (ders(i) < 0 .and. ders(i-1) > 0) then          
!
!         seleziona il + grande nell'intorno definito da npoints         
          ini = max(1,i-nprange)
          fin = min(ndat,i+nprange)
          locm = maxloc(dati(ini:fin)%y)
          posnew = locm(1)+ini-1
          if (posnew /= pos) then  ! potrebbe selezionare un punto già preso prima
              pos = posnew
              np=np+1
              if (nsizep < np) then
                  nsizep = nsizep + nplus
                  call resize_points(pk,nsizep)
              endif          
              pk(np) = dati(pos)              
          endif
      endif
   enddo
!
   call resize_points(pk,np)
!
   end subroutine findmaxd1_old

 !------------------------------------------------------------------------------------

   subroutine filtra_punti(punti,nad,sog)
   USE stat
   USE pointmod
!
!  Filtra punti rimuovendo quelli che hanno un valore superiore alla media dei punti adiacenti
!
   type(point_type), allocatable, intent(inout) :: punti(:) ! i punti da filtrare
   integer, intent(in)                     :: nad      ! num. di punti adiacenti
   real, intent(in)                        :: sog      ! soglia di rimozione punti
   integer                                 :: i
   !type(point_type), dimension(size(punti)) :: puntir ! i punti da filtrare
   real, dimension(size(punti)) :: yvalr ! i punti da filtrare
   real                                    :: yave,yavel,yaver
   integer :: npunti
!corr   real :: ymax
!   
!-------- sostituire puntir con real yvalr
   npunti = size(punti) 
!
!  normalizza le intensita'
   yvalr(:) = punti(:)%y - minval(punti(:)%y)
   yvalr(:) = yvalr(:)/maxval(yvalr)
   i = nad
   do 
      i = i + 1
      if (i > npunti - nad) exit
      yavel = sum(yvalr(i-nad:i-1))/nad                    ! media a sinistra
      yaver = sum(yvalr(i+1:i+nad))/nad                    ! media a destra
      if (yvalr(i) > yavel .and. yvalr(i) > yaver) then  ! Il punto è piu' in alto rispetto alle medie 
          yave = (yavel + yaver)/2
              write(0,*)'punty',punti(i)%x,yvalr(i),i,yavel,yaver
              write(0,*)'punty',punti(i)%x,yvalr(i),i,(yvalr(i) - yave),sog
!
!         Rimuovi il punto i se la differenza relativa rispetto alla media e > di sog          
          if (yvalr(i) - yave > sog) then 
              write(0,*)'eliminato punto ',yvalr(i),punti(i)%x,i
              yvalr(i:npunti-1) = yvalr(i+1:npunti)
              punti(i:npunti-1) = punti(i+1:npunti)
              npunti = npunti - 1
              i = i - 1
          endif
      else
              write(0,*)'puntn',punti(i)%x,yvalr(i),i,yavel,yaver
      endif
   enddo
!
!  Ricompatta i punti a npunti
   call resize_points(punti,npunti)
!
   end subroutine

  !-----------------------------------------------------------------------------

   subroutine param_back(itype,bobs,ncoef,coef,tmin,tmax,th0)
!
!  Calcola coefficienti di background della curva passante per i punti bobs
!
   USE nr; USE nrtype
   USE GENLSQ
   USE trig_constants
!
   implicit none
   integer, intent(in)                      :: itype
   type(point_type), dimension(:), intent(in)         :: bobs
   integer, intent(in)                      :: ncoef
   real, dimension(ncoef), intent(out) :: coef
   real, intent(in)                         :: tmin,tmax
   real, intent(in), optional               :: th0
   integer                          :: i,j,kk
   real, dimension(36)              :: tt
   real                             :: xche
   real(dbl), dimension(size(bobs),ncoef) :: am
   real(dbl), dimension(size(bobs))       :: bm
   real(dbl), dimension(ncoef)              :: x
   integer                                  :: np
   integer :: ier
!
   np = size(bobs)
   bm = bobs(:np)%y
!
   select case (itype)

    case (POLY)
         am(:,1) = 1.0
         do i=2,ncoef
            am(:,i) = (bobs(:)%x/th0 - 1.0)**(i-1)
         enddo

    case (CHEBY)
         am(:,1) = 0.5
         tt(1) = 1.0
         do i=1,np
            xche = 2.0*(bobs(i)%x - tmin)/(tmax-tmin) - 1.0
            tt(2) = xche
            do j=2,ncoef-1
               tt(j+1) = 2.*xche*tt(j) - tt(j-1)
            enddo
            do kk=2,ncoef
                am(i,kk) = tt(kk)
            enddo
         enddo

    case (FOUR_SERIES)
         do i=1,ncoef
            am(:,i) = cos((i-1)*bobs(:)%x*dtor)
         enddo

   end select

   call solve_linlsq(am,bm,x,ier)
   coef(:ncoef) = x
     !write(0,*)'coef=',x
!
   end subroutine param_back

  !-------------------------------------------------------------------------------------
#if 0
   subroutine getbackground(itype,backg,xback,coef,tmin,tmax,th0)
!
!  Calcola il background nei punti xback noti i coefficienti
!
   USE nr
   USE trig_constants
!
   implicit none
   integer, intent(in)             :: itype      ! tipo di background
   real, dimension(:), intent(out) :: backg      ! background
   real, dimension(:), intent(in)  :: xback      ! x del punto di background
   real, dimension(:), intent(in)  :: coef       ! coefficienti di background
   integer                         :: np         ! num. di punti in cui calcolare il background
   integer                         :: ncoef      ! num. di coefficienti
   real, intent(in)                :: tmin,tmax
   real, intent(in), optional      :: th0
   integer :: i,j,k
   real    :: thy
   real    :: aa,bb,xche
!
   np = size(backg)
   ncoef = size(coef)
!
   select case (itype)

      case (POLY)
       do j=1,np
          thY=xback(j)/th0 - 1.0
          backg(j)= coef(1)
          do k=2,ncoef
             backg(j)= backg(j)+coef(k)*thY**(k-1)
          enddo
       enddo

     case (CHEBY)
       aa = -1.0
       bb = 1.0
       do i=1,np
          xche = 2.0*(xback(i) - tmin)/(tmax-tmin) - 1.0
          backg(i) = chebev(aa,bb,coef(:ncoef),xche)
       enddo

     case (FOUR_SERIES)
       backg = 0.0
       do i=1,np
         do j=1,ncoef
            backg(i) = backg(i) + coef(j)*cos((j-1)*xback(i)*dtor)
         enddo
       enddo

   end select
!
   end subroutine getbackground
#endif
  !-------------------------------------------------------------------------------------

   subroutine getbackground(itype,backg,xback,coef,th0)
!
!  Calcola il background nei punti xback noti i coefficienti
!
   USE nr
   USE trig_constants
!
   implicit none
   integer, intent(in)             :: itype      ! tipo di background
   real, dimension(:), intent(out) :: backg      ! background
   real, dimension(:), intent(in)  :: xback      ! x del punto di background
   real, dimension(:), intent(in)  :: coef       ! coefficienti di background
   integer                         :: np         ! num. di punti in cui calcolare il background
   integer                         :: ncoef      ! num. di coefficienti
   real                :: tmin,tmax
   real, intent(in), optional      :: th0
   integer :: i,j,k
   real    :: thy
   real    :: aa,bb,xche
!
   np = size(backg)
   ncoef = size(coef)
!
   select case (itype)

      case (POLY)
       do j=1,np
          thY=xback(j)/th0 - 1.0
          backg(j)= coef(1)
          do k=2,ncoef
             backg(j)= backg(j)+coef(k)*thY**(k-1)
          enddo
       enddo

     case (CHEBY)
       tmin = xback(1)
       tmax = xback(np)
       aa = -1.0
       bb = 1.0
       do i=1,np
          xche = 2.0*(xback(i) - tmin)/(tmax-tmin) - 1.0
          backg(i) = chebev(aa,bb,coef(:ncoef),xche)
       enddo

     case (FOUR_SERIES)
       backg = 0.0
       do i=1,np
         do j=1,ncoef
            backg(i) = backg(i) + coef(j)*cos((j-1)*xback(i)*dtor)
         enddo
       enddo

   end select
!
   end subroutine getbackground

  !-------------------------------------------------------------------------------------

   subroutine bruckner_back(ycount,bcount,ncicli,sizew)
!
   real, dimension(:), intent(in)  :: ycount    ! y
   real, dimension(:), intent(out) :: bcount    ! background
   integer, intent(in)             :: ncicli
   integer, intent(in)             :: sizew
   real                            :: media_conteggi
   real                            :: soglia
!
   bcount(:) = ycount(:)             ! inizializza background
!
!  Poni il background uguale a soglia per tutti i punti > di soglia
   media_conteggi = sum(bcount(:)) / size(bcount)
   soglia = media_conteggi !+ 2*(media_conteggi - minval(bcount))
       soglia = (maxval(bcount) + minval(bcount)) / 2.0
   where ( bcount(:) > soglia) bcount(:) = soglia
!
   call smoothback(bcount,ncicli,sizew)
!
   end subroutine bruckner_back

  !-------------------------------------------------------------------------------------

   subroutine smoothback(bcount,ncicli,sizew)
   !USE variables
   !USE interfaces
   USE ssmoothing
   !USE utiltec
!
   real, dimension(:), intent(inout)   :: bcount    ! background
   integer, intent(in) :: ncicli
   integer, intent(in) :: sizew
!corr   integer             :: ncmin,ncmax
!corr   integer             :: nctot
   real ,dimension(size(bcount)) :: back_smooth
   integer             :: i
!corr   character(len=80)   :: string
!
   !ncmin = clocate(counts%x,tmin)
   !ncmax = clocate(counts%x,tmax)
   !nctot = ncmax - ncmin + 1
   !allocate (back_smooth(nctot))
   do i=1,ncicli
      back_smooth(:) = averagesmooth(bcount,sizew)                     ! esegue smoothing
      where( bcount(:) > back_smooth(:) ) bcount(:) = back_smooth(:)   ! prende la minore delle 2 intensita'
      !call plot()
      !write(string,'(i4)')i
      !string = 'Estimation of background. Iteration n. '//trim(adjustl(string))
      !call printmess(2,string)
   enddo
   !call printmess(2,'Estimation of background. Filter')
!
   end subroutine smoothback

 END MODULE Background

!-------------------------------------------------------------------------------------------------
