#include <mrconfig.h>

extern GtkWidget *finestre[6];

typedef struct {
     int    nlines;           // number of lines used for searching
     int    syst[6];          // crystal system tested
     float  abcmax;           // min, max value on cell dimensions
     float  betamin, betamax; // min, max beta angle for monoclinic cell
     float  volmin, volmax;   // min, max volume in ang**3
     float  mol_weight;       // molecular weight
     float  dens;             // measured density
     float  delden;           // absolute error on measured density
     float  eps;              // peak position error
     int    zero_s;           // estimate zero point
     int    zero_ref;         // zero-point refinement
     float  zero;             // experimental zero-point
     int    n_imp;            // max impurity lines
     int    search_type;      // search type: option (dicvol), ngrid (McMaille)
     float  fom;              // Minimum figure-of-merit
     float  width;            // Windth of the peak (~2*FWHM) (McMaille only)
                } index_settings;

void SavePeaks(int filt);
void redrawpattern(int action);

extern "C" void open_indexation_win_(index_settings ind0[], index_settings ind1[]);
extern "C" void close_indexation_win_();
extern "C" void finish_index_(index_settings ind[], int sel[]);
extern "C" void open_cell_list_();

enum {page_NTREOR, page_DICVOL, page_MCMAILLE};

static _StyleType styleF = {true, 1, NULL, GTK_SHADOW_NONE, 6, 6, 15, 6};

static const char *radios[] = {"N-TREOR09", "DICVOL06", "McMaille"};
int nradios = (int) G_N_ELEMENTS(radios);

static const char *opt_bb[] = {
     "Monte Carlo (NGrid=0)",
     "Grid search (NGrid=1)",
     "Monte Carlo + Grid search (NGrid=2)",
     "Black box mode - Monte Carlo on all symmetries (Ngrid=3)",
     "Black box mode - Monte Carlo on all symmetries but without triclinic search (Ngrid=-3)",
     "Black box mode - Monte Carlo on all symmetries + grid search (Ngrid=4)"};
static const int search_t[] = {0, 1, 2, 3, -3, 4};

static int Nopt_bb = (int) G_N_ELEMENTS(search_t);

static const char *crysts[] = {"Cubic", "Tetragonal", "Hexagonal",
                               "Orthorhombic", "Monoclinic", "Triclinic"};
namespace C_Indexing {
                       index_settings indx_set0[3], indx_set[3];
                       int select_pro[3];
                     }

void abc_and_vol(GtkWidget *table, int irow)
{
  const char *max_str = "Max";
  const char *min_str = "Min";

  std::string labspec;
  labspec.assign("Volume (");
  labspec.append(special_car(Aring));
  labspec.append("<sup>3</sup>)");
  GtkWidget *label = LABEL_NEW(labspec.c_str(), NULL);
  set_alignment(label, 0, 0.5);
  table_attach(table, label, 0, 1, irow, irow+1, TRUE);
  label = LABEL_NEW(min_str, NULL);
  set_alignment(label, 0, 0.5);
  table_attach(table, label, 1, 2, irow, irow+1, TRUE);
  label = LABEL_NEW(max_str, NULL);
  set_alignment(label, 0, 0.5);
  table_attach(table, label, 3, 4, irow, irow+1, TRUE);
  labspec.assign("a,b,c (");
  labspec.append(special_car(Aring));
  labspec.append(")");
  label = LABEL_NEW(labspec.c_str(), NULL);
  set_alignment(label, 0, 0.5);
  table_attach(table, label, 0, 1, irow+1, irow+2, TRUE);
  label = LABEL_NEW(min_str, NULL);
  set_alignment(label, 0, 0.5);
  table_attach(table, label, 1, 2, irow+1, irow+2, TRUE);
  label = LABEL_NEW(max_str, NULL);
  set_alignment(label, 0, 0.5);
  table_attach(table, label, 3, 4, irow+1, irow+2, TRUE);
  GtkWidget *entry = gtk_entry_new();
  set_width_chars(GTK_ENTRY(entry), 10);
  table_attach(table, entry, 2, 3, irow, irow+1);
  g_object_set_data(G_OBJECT(table), "volmin", entry);
  entry = gtk_entry_new();
  set_width_chars(GTK_ENTRY(entry), 10);
  table_attach(table, entry, 4, 5, irow, irow+1);
  g_object_set_data(G_OBJECT(table), "volmax", entry);
  entry = gtk_entry_new();
  set_width_chars(GTK_ENTRY(entry), 10);
  gtk_entry_set_text(GTK_ENTRY(entry), fToStr2(0, 3).c_str());
  table_attach(table, entry, 2, 3, irow+1, irow+2);
  g_object_set_data(G_OBJECT(table), "abcmin", entry);
  gtk_widget_set_sensitive(entry, FALSE);
  entry = gtk_entry_new();
  set_width_chars(GTK_ENTRY(entry), 10);
  table_attach(table, entry, 4, 5, irow+1, irow+2);
  g_object_set_data(G_OBJECT(table), "abcmax", entry);
}

void page_ntreor (GtkWidget *father, GtkWidget *notebook)
{
  GtkWidget *parentbox = vbox_new (FALSE, 2);
  gtk_widget_show (parentbox);
  gtk_container_add (GTK_CONTAINER (notebook), parentbox);
#ifdef IN_GTK3
  GtkWidget *align = new_alignment(GTK_ALIGN_START, GTK_ALIGN_START, 0, 0);
#else
  GtkWidget *align = gtk_alignment_new(0.2f, 0.2f, 0, 0);
#endif

  gtk_box_pack_start(GTK_BOX(parentbox), align, TRUE, TRUE, 10);
  GtkWidget *table = table_new(5, 6, FALSE);
  gtk_container_add (GTK_CONTAINER (align), table);
  table_set_row_spacings(table, 10);
  table_set_col_spacings(table, 8);
  gtk_container_set_border_width(GTK_CONTAINER(table), 8);
  std::string labspec;
  labspec.assign("Maximum Volume (");
  labspec.append(special_car(Aring));
  labspec.append("<sup>3</sup>)");
  GtkWidget *label = LABEL_NEW(labspec.c_str(), NULL);
  set_alignment(label, 0, 0.5);
  table_attach(table, label, 0, 1, 0, 1, TRUE);
  labspec.assign("Maximum a,b,c (");
  labspec.append(special_car(Aring));
  labspec.append(")");
  label = LABEL_NEW(labspec.c_str(), NULL);
  set_alignment(label, 0, 0.5);
  table_attach(table, label, 0, 1, 1, 2, TRUE);
  GtkWidget *entry = gtk_entry_new();
  set_width_chars(GTK_ENTRY(entry), 10);
  table_attach(table, entry, 1, 2, 0, 1);
  g_object_set_data(G_OBJECT(father), "volmax1", entry);
  entry = gtk_entry_new();
  set_width_chars(GTK_ENTRY(entry), 10);
  table_attach(table, entry, 1, 2, 1, 2);
  g_object_set_data(G_OBJECT(father), "abcmax1", entry);
  label = LABEL_NEW("Max impurity lines", NULL);
  set_alignment(label, 0, 0.5);
  table_attach(table, label, 0, 1, 2, 3, TRUE);
  label = SPIN_NEW(0, 100, 0, 1, 0);
  g_object_set_data(G_OBJECT(father), "n_imp1", label);
  table_attach(table, label, 1, 2, 2, 3, TRUE);
  entry = gtk_entry_new();
  set_width_chars(GTK_ENTRY(entry), 10);
  table_attach(table, entry, 1, 2, 3, 4, TRUE);
  g_object_set_data(G_OBJECT(father), "fom1", entry);
  label = LABEL_NEW("Minimum figure-of-merit", NULL);
  set_alignment(label, 0, 0.5);
  table_attach(table, label, 0, 1, 3, 4, TRUE);

  int irow = 4;
  label = LABEL_NEW("Molecular weight", NULL);
  set_alignment(label, 0, 0.5);
  table_attach(table, label, 0, 1, irow, irow+1, TRUE);
  label = LABEL_NEW("Measured density", NULL);
  set_alignment(label, 0, 0.5);
  table_attach(table, label, 2, 3, irow, irow+1, TRUE);
  label = LABEL_NEW("Error on density", NULL);
  set_alignment(label, 0, 0.5);
  table_attach(table, label, 4, 5, irow, irow+1, TRUE);
  entry = gtk_entry_new();
  set_width_chars(GTK_ENTRY(entry), 10);
  table_attach(table, entry, 1, 2, irow, irow+1);
  g_object_set_data(G_OBJECT(father), "mol_weight_nt", entry);
  entry = gtk_entry_new();
  set_width_chars(GTK_ENTRY(entry), 10);
  table_attach(table, entry, 3, 4, irow, irow+1);
  g_object_set_data(G_OBJECT(father), "dens_nt", entry);
  entry = gtk_entry_new();
  set_width_chars(GTK_ENTRY(entry), 10);
  table_attach(table, entry, 5, 6, irow, irow+1);
  g_object_set_data(G_OBJECT(father), "err_dens_nt", entry);

  set_tab_label (GTK_NOTEBOOK (notebook), page_NTREOR, radios[page_NTREOR]);
}

void page_dicvol (GtkWidget *father, GtkWidget *notebook)
{
  const char *max_str = "Max";
  const char *min_str = "Min";
  GtkWidget *parentbox = vbox_new (FALSE, 2);
  gtk_widget_show (parentbox);
  gtk_container_add (GTK_CONTAINER (notebook), parentbox);

  GtkWidget *spin  = SPIN_WITH_LABEL("Number of lines used for searching solutions ", NULL, 1, 50, 1, 1, 0);
  g_object_set_data(G_OBJECT(father), "nlines", get_widget(spin, "spin"));
//  gtk_box_pack_start(GTK_BOX(parentbox), spin, FALSE, FALSE, 6);
//  gtk_container_set_border_width(GTK_CONTAINER(spin), 8);

  GtkWidget *table = table_new(5, 8, FALSE);
  table_set_row_spacings(table, 10);
  table_set_col_spacings(table, 8);
  gtk_container_set_border_width(GTK_CONTAINER(table), 8);
  gtk_box_pack_start(GTK_BOX(parentbox), table, TRUE, TRUE, 6);
  int irow = 1;

  table_attach(table, spin, 0, 5, 0, 1, TRUE);
  GtkWidget *label = LABEL_NEW("Peak position error", NULL);
  set_alignment(label, 0, 0.5);
  table_attach(table, label, 6, 7, 0, 1, TRUE);
  abc_and_vol(table, irow);
  g_object_set_data(G_OBJECT(father), "volmin", get_widget(table, "volmin"));
  g_object_set_data(G_OBJECT(father), "volmax", get_widget(table, "volmax"));
  g_object_set_data(G_OBJECT(father), "abcmin", get_widget(table, "abcmin"));
  g_object_set_data(G_OBJECT(father), "abcmax", get_widget(table, "abcmax"));
  std::string labspec;
  labspec.assign("B (");
  labspec.append(special_car(Degree));
  labspec.append(")");
  label = LABEL_NEW(labspec.c_str(), NULL);
  set_alignment(label, 0, 0.5);
  table_attach(table, label, 0, 1, irow+2, irow+3, TRUE);
  label = LABEL_NEW(min_str, NULL);
  set_alignment(label, 0, 0.5);
  table_attach(table, label, 1, 2, irow+2, irow+3, TRUE);
  label = LABEL_NEW(max_str, NULL);
  set_alignment(label, 0, 0.5);
  table_attach(table, label, 3, 4, irow+2, irow+3, TRUE);

  label = LABEL_NEW("Minimum figure-of-merit", NULL);
  set_alignment(label, 0, 0.5);
  table_attach(table, label, 3, 5, irow+3, irow+4, TRUE);
  label = LABEL_NEW("Max impurity lines", NULL);
  set_alignment(label, 0, 0.5);
  table_attach(table, label, 0, 2, irow+3, irow+4, TRUE);
  label = LABEL_NEW("Molecular weight", NULL);
  set_alignment(label, 0, 0.5);
  table_attach(table, label, 0, 2, irow+4, irow+5, TRUE);
  label = LABEL_NEW("Measured density", NULL);
  set_alignment(label, 0, 0.5);
  table_attach(table, label, 3, 5, irow+4, irow+5, TRUE);
  label = LABEL_NEW("Error on density", NULL);
  set_alignment(label, 0, 0.5);
  table_attach(table, label, 6, 7, irow+4, irow+5, TRUE);
  label = SPIN_NEW(0, 100, 0, 1, 0);
  g_object_set_data(G_OBJECT(father), "n_imp", label);
  table_attach(table, label, 2, 3, irow+3, irow+4, TRUE);
  GtkWidget *entry = gtk_entry_new();
  set_width_chars(GTK_ENTRY(entry), 10);
  table_attach(table, entry, 7, 8, 0, 1);
  g_object_set_data(G_OBJECT(father), "eps", entry);
  entry = gtk_entry_new();
  set_width_chars(GTK_ENTRY(entry), 10);
  table_attach(table, entry, 2, 3, irow+2, irow+3);
  g_object_set_data(G_OBJECT(father), "betamin", entry);
  entry = gtk_entry_new();
  set_width_chars(GTK_ENTRY(entry), 10);
  table_attach(table, entry, 4, 5, irow+2, irow+3);
  g_object_set_data(G_OBJECT(father), "betamax", entry);
  entry = gtk_entry_new();
  set_width_chars(GTK_ENTRY(entry), 10);
  table_attach(table, entry, 5, 6, irow+3, irow+4, TRUE);
  g_object_set_data(G_OBJECT(father), "fom", entry);
  entry = gtk_entry_new();
  set_width_chars(GTK_ENTRY(entry), 10);
  table_attach(table, entry, 2, 3, irow+4, irow+5);
  g_object_set_data(G_OBJECT(father), "mol_weight", entry);
  entry = gtk_entry_new();
  set_width_chars(GTK_ENTRY(entry), 10);
  table_attach(table, entry, 5, 6, irow+4, irow+5);
  g_object_set_data(G_OBJECT(father), "dens", entry);
  entry = gtk_entry_new();
  set_width_chars(GTK_ENTRY(entry), 10);
  table_attach(table, entry, 7, 8, irow+4, irow+5);
  g_object_set_data(G_OBJECT(father), "err_dens", entry);
  GtkWidget *frame = NEW_FRAME("Crystal System", &styleF);
  gtk_box_pack_start(GTK_BOX(parentbox), frame, TRUE, TRUE, 8);
  table = table_new(2, 3, TRUE);
  add_to_frame(frame, table);
  GtkWidget *check;
  for(int i=0; i<6; i++)
  {
      check = CHECK_BUTTON_WITH_LABEL (crysts[i], NULL);
      g_object_set_data(G_OBJECT(father), crysts[i], check);
      table_attach(table, check, i/2, i/2+1, i%2, i%2+1, TRUE);
  }
  frame = NEW_FRAME("Zero-point", &styleF);
  gtk_box_pack_start(GTK_BOX(parentbox), frame, TRUE, TRUE, 8);
  table = table_new(1, 4, FALSE);
  add_to_frame(frame, table);
  check = CHECK_BUTTON_WITH_LABEL ("Estimate zero-point", NULL);
  g_object_set_data(G_OBJECT(father), "zero_s", check);
  table_attach(table, check, 1, 2, 0, 1, TRUE);
  check = CHECK_BUTTON_WITH_LABEL ("Refine zero-point", NULL);
  g_object_set_data(G_OBJECT(father), "zero_ref", check);
  table_attach(table, check, 0, 1, 0, 1, TRUE);
  label = LABEL_NEW("Experimental zero-point  ", NULL);
  set_alignment(label, 1, 0.5);
  table_attach(table, label, 2, 3, 0, 1, TRUE);
  check = get_widget(father, "zero_s");
  entry = gtk_entry_new();
  set_width_chars(GTK_ENTRY(entry), 10);
  table_attach(table, entry, 3, 4, 0, 1);
  g_object_set_data(G_OBJECT(father), "zero", entry);
  g_signal_connect(G_OBJECT(check), "toggled", G_CALLBACK(on_cb_disattiva),
         entry);
  check = CHECK_BUTTON_WITH_LABEL ("Exhaustive search", NULL);
  g_object_set_data(G_OBJECT(father), "search_type", check);
  gtk_box_pack_start(GTK_BOX(parentbox), check, FALSE, FALSE, 8);

  set_tab_label (GTK_NOTEBOOK (notebook), page_DICVOL, radios[page_DICVOL]);
}

void page_mcmaille (GtkWidget *father, GtkWidget *notebook)
{
  GtkWidget *parentbox = vbox_new (FALSE, 2);
  gtk_widget_show (parentbox);
  gtk_container_add (GTK_CONTAINER (notebook), parentbox);

  GtkWidget *spin  = SPIN_WITH_LABEL("Number of lines used for searching solutions ", NULL, 1, 50, 1, 1, 0);
  g_object_set_data(G_OBJECT(father), "nlines_mcm", get_widget(spin, "spin"));
  gtk_box_pack_start (GTK_BOX (parentbox), spin, FALSE, FALSE, 8);

  GtkWidget *frame = NEW_FRAME("Process to be applied", &styleF);
  gtk_box_pack_start(GTK_BOX(parentbox), frame, FALSE, TRUE, 8);
  GtkWidget *vbox = vbox_new (FALSE, 0);
  add_to_frame(frame, vbox);

  GSList *group = NULL;
  for (int i=0; i<Nopt_bb; i++)
  {
    GtkWidget *radio = gtk_radio_button_new_with_label (group, opt_bb[i]);
    group = gtk_radio_button_get_group (GTK_RADIO_BUTTON (radio));
    gtk_box_pack_start (GTK_BOX (vbox), radio, FALSE, FALSE, 0);
    g_object_set_data (G_OBJECT (father), g_strdup_printf("%d",search_t[i]), radio);
  }

  GtkWidget *hbox2 = hbox_new(FALSE, 0);
  gtk_box_pack_start(GTK_BOX(parentbox), hbox2, FALSE, FALSE, 8);
  GtkWidget *label = LABEL_NEW("Experimental zero-point  ", NULL);
  set_alignment(label, 1, 0.5);
  gtk_box_pack_start(GTK_BOX(hbox2), label, FALSE, FALSE, 2);
  GtkWidget *entry = gtk_entry_new();
  set_width_chars(GTK_ENTRY(entry), 10);
  gtk_box_pack_start(GTK_BOX(hbox2), entry, FALSE, FALSE, 2);
  g_object_set_data(G_OBJECT(father), "zero_mcm", entry);
  GtkWidget *hbox = hbox_new(FALSE, 0);
  gtk_box_pack_start(GTK_BOX(hbox2), hbox, TRUE, TRUE, 0);
  g_object_set_data(G_OBJECT(father), "hbox_mcm", hbox);
  label = LABEL_NEW("W", NULL);
  set_alignment(label, 1, 0.5);
  gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 20);
  entry = gtk_entry_new();
  set_width_chars(GTK_ENTRY(entry), 10);
  gtk_box_pack_start(GTK_BOX(hbox), entry, FALSE, FALSE, 2);
  g_object_set_data(G_OBJECT(father), "width", entry);
  label = LABEL_NEW("Max impurity lines", NULL);
  set_alignment(label, 0, 0.5);
  gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 20);
  label = SPIN_NEW(0, 100, 0, 1, 0);
  g_object_set_data(G_OBJECT(father), "n_imp2", label);
  gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 2);
  gtk_widget_set_sensitive(hbox, FALSE);

  frame = NEW_FRAME("Crystal System", &styleF);
  gtk_box_pack_start(GTK_BOX(parentbox), frame, FALSE, FALSE, 8);
  g_object_set_data(G_OBJECT(father), "framesys", frame);
  GtkWidget *table = table_new(2, 3, TRUE);
  add_to_frame(frame, table);
  GtkWidget *check;
  for(int i=0; i<6; i++)
  {
      check = CHECK_BUTTON_WITH_LABEL (crysts[i], NULL);
      g_object_set_data(G_OBJECT(father), 
           g_strdup_printf("%s_mm", crysts[i]), check);
      table_attach(table, check, i/2, i/2+1, i%2, i%2+1, TRUE);
  }
  gtk_widget_set_sensitive(frame, FALSE);
#ifdef IN_GTK3
  GtkWidget *align = new_alignment(GTK_ALIGN_START, GTK_ALIGN_START, 0, 0);
#else
  GtkWidget *align = gtk_alignment_new(0, 0, 0.2f, 0);
#endif
  gtk_widget_set_sensitive(align, FALSE);
  gtk_box_pack_start(GTK_BOX(parentbox), align, TRUE, TRUE, 0);

  table = table_new(4, 5, FALSE);
  table_set_row_spacings(table, 10);
  table_set_col_spacings(table, 8);
  gtk_container_set_border_width(GTK_CONTAINER(table), 8);
  gtk_container_add(GTK_CONTAINER(align), table);
  abc_and_vol(table, 0);

  g_object_set_data(G_OBJECT(father), "volmin2", get_widget(table, "volmin"));
  g_object_set_data(G_OBJECT(father), "volmax2", get_widget(table, "volmax"));
  g_object_set_data(G_OBJECT(father), "abcmin2", get_widget(table, "abcmin"));
  g_object_set_data(G_OBJECT(father), "abcmax2", get_widget(table, "abcmax"));

  for (int i=0; i<3; i++)
  {
    GtkWidget *radio = get_widget(father,  g_strdup_printf("%d",search_t[i]));
    g_signal_connect(G_OBJECT(radio), "toggled", G_CALLBACK(on_cb_attiva), frame);
    g_signal_connect(G_OBJECT(radio), "toggled", G_CALLBACK(on_cb_attiva), align);
    g_signal_connect(G_OBJECT(radio), "toggled", G_CALLBACK(on_cb_attiva), hbox);
  }
  
  set_tab_label (GTK_NOTEBOOK (notebook), page_MCMAILLE, radios[page_MCMAILLE]);
}

void selected_prog(GtkToggleButton *radio, gpointer data)
{
  if(!gtk_toggle_button_get_active(radio))
     return;
  int page = GPOINTER_TO_INT(data);
  GtkWidget *notebook = get_widget(gtk_widget_get_toplevel(GTK_WIDGET(radio)),
       "nb");
  gtk_notebook_set_current_page (GTK_NOTEBOOK (notebook), page);
}

void FillFormIndexing(GtkWidget *dlg, index_settings iset[])
{
// N-TREOR
  GtkWidget *w = get_widget(dlg, "volmax1");
  gtk_entry_set_text(GTK_ENTRY(w), fToStr2(iset[0].volmax, 3).c_str());
  w = get_widget(dlg, "abcmax1");
  gtk_entry_set_text(GTK_ENTRY(w), fToStr2(iset[0].abcmax, 3).c_str());
  w = get_widget(dlg, "fom1");
  gtk_entry_set_text(GTK_ENTRY(w), fToStr2(iset[0].fom, 3).c_str());
  w = get_widget(dlg, "n_imp1");
  spin_button_set_value(w, iset[0].n_imp);
  w = get_widget(dlg, "dens_nt");
  gtk_entry_set_text(GTK_ENTRY(w), fToStr2(iset[0].dens, 3).c_str());
  w = get_widget(dlg, "err_dens_nt");
  gtk_entry_set_text(GTK_ENTRY(w), fToStr2(iset[0].delden, 3).c_str());
  w = get_widget(dlg, "mol_weight_nt");
  gtk_entry_set_text(GTK_ENTRY(w), fToStr2(iset[0].mol_weight, 3).c_str());
// DICVOL
  w = get_widget(dlg, "nlines");
  spin_button_set_value(w, iset[1].nlines);
  w = get_widget(dlg, "volmin");
  gtk_entry_set_text(GTK_ENTRY(w), fToStr2(iset[1].volmin, 3).c_str());
  w = get_widget(dlg, "volmax");
  gtk_entry_set_text(GTK_ENTRY(w), fToStr2(iset[1].volmax, 3).c_str());
  w = get_widget(dlg, "abcmax");
  gtk_entry_set_text(GTK_ENTRY(w), fToStr2(iset[1].abcmax, 3).c_str());
  w = get_widget(dlg, "betamin");
  gtk_entry_set_text(GTK_ENTRY(w), fToStr2(iset[1].betamin, 3).c_str());
  w = get_widget(dlg, "betamax");
  gtk_entry_set_text(GTK_ENTRY(w), fToStr2(iset[1].betamax, 3).c_str());
  w = get_widget(dlg, "zero");
  gtk_entry_set_text(GTK_ENTRY(w), fToStr2(iset[1].zero, 3).c_str());
  w = get_widget(dlg, "eps");
  gtk_entry_set_text(GTK_ENTRY(w), fToStr2(iset[1].eps, 3).c_str());
  w = get_widget(dlg, "dens");
  gtk_entry_set_text(GTK_ENTRY(w), fToStr2(iset[1].dens, 3).c_str());
  w = get_widget(dlg, "err_dens");
  gtk_entry_set_text(GTK_ENTRY(w), fToStr2(iset[1].delden, 3).c_str());
  w = get_widget(dlg, "mol_weight");
  gtk_entry_set_text(GTK_ENTRY(w), fToStr2(iset[1].mol_weight, 3).c_str());
  w = get_widget(dlg, "fom");
  gtk_entry_set_text(GTK_ENTRY(w), fToStr2(iset[1].fom, 3).c_str());
  w = get_widget(dlg, "n_imp");
  spin_button_set_value(w, iset[1].n_imp);
  for(int i=0; i<6; i++)
  {
      w = get_widget(dlg, crysts[i]);
      gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(w), iset[1].syst[i]);
  }
  w = get_widget(dlg, "zero_s");
  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(w), iset[1].zero_s);
  w = get_widget(dlg, "zero_ref");
  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(w), iset[1].zero_ref);
  w = get_widget(dlg, "search_type");
  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(w), iset[1].search_type);

//McMaille
  w = get_widget(dlg, "nlines_mcm");
  spin_button_set_value(w, iset[2].nlines);
  w = get_widget(dlg, "zero_mcm");
  gtk_entry_set_text(GTK_ENTRY(w), fToStr2(iset[2].zero, 3).c_str());
  w = get_widget(dlg, "n_imp2");
  spin_button_set_value(w, iset[2].n_imp);
  w = get_widget(dlg, "width");
  gtk_entry_set_text(GTK_ENTRY(w), fToStr2(iset[2].width, 3).c_str());
  w = get_widget(dlg, "volmin2");
  gtk_entry_set_text(GTK_ENTRY(w), fToStr2(iset[2].volmin, 3).c_str());
  w = get_widget(dlg, "volmax2");
  gtk_entry_set_text(GTK_ENTRY(w), fToStr2(iset[2].volmax, 3).c_str());
  w = get_widget(dlg, "abcmax2");
  gtk_entry_set_text(GTK_ENTRY(w), fToStr2(iset[2].abcmax, 3).c_str());
  w = get_widget(dlg, g_strdup_printf("%d",iset[2].search_type));
  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(w), TRUE);
  w = get_widget(dlg, "hbox_mcm");
  gtk_widget_set_sensitive(w, 
     ((iset[2].search_type >= 0) && (iset[2].search_type <3)));
  w = get_widget(dlg, "framesys");
  gtk_widget_set_sensitive(w, 
     ((iset[2].search_type >= 0) && (iset[2].search_type <3)));
  for(int i=0; i<6; i++)
  {
      w = get_widget(dlg, g_strdup_printf("%s_mm", crysts[i]));
      gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(w), iset[2].syst[i]);
  }
}

void ReadFormIndexing(GtkWidget *dlg, index_settings iset[], int sel[])
{
  GtkWidget *w;
  for (int i=0; i<nradios; i++)
  {
       w = get_widget(dlg, radios[i]);
       sel[i] = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(w));
  }
// N-TREOR
  w = get_widget(dlg, "volmax1");
  iset[0].volmax = (float) atof(gtk_entry_get_text(GTK_ENTRY(w)));
  w = get_widget(dlg, "abcmax1");
  iset[0].abcmax = (float) atof(gtk_entry_get_text(GTK_ENTRY(w)));
  w = get_widget(dlg, "n_imp1");
  iset[0].n_imp = spin_button_get_value_as_int(w);
  w = get_widget(dlg, "fom1");
  iset[0].fom = (float) atof(gtk_entry_get_text(GTK_ENTRY(w)));
  w = get_widget(dlg, "dens_nt");
  iset[0].dens = (float) atof(gtk_entry_get_text(GTK_ENTRY(w)));
  w = get_widget(dlg, "err_dens_nt");
  iset[0].delden = (float) atof(gtk_entry_get_text(GTK_ENTRY(w)));
  w = get_widget(dlg, "mol_weight_nt");
  iset[0].mol_weight = (float) atof(gtk_entry_get_text(GTK_ENTRY(w)));
// DICVOL
  w = get_widget(dlg, "nlines");
  iset[1].nlines = spin_button_get_value_as_int(w);
  w = get_widget(dlg, "volmin");
  iset[1].volmin = (float) atof(gtk_entry_get_text(GTK_ENTRY(w)));
  w = get_widget(dlg, "volmax");
  iset[1].volmax = (float) atof(gtk_entry_get_text(GTK_ENTRY(w)));
  w = get_widget(dlg, "abcmax");
  iset[1].abcmax = (float) atof(gtk_entry_get_text(GTK_ENTRY(w)));
  w = get_widget(dlg, "betamin");
  iset[1].betamin = (float) atof(gtk_entry_get_text(GTK_ENTRY(w)));
  w = get_widget(dlg, "betamax");
  iset[1].betamax = (float) atof(gtk_entry_get_text(GTK_ENTRY(w)));
  w = get_widget(dlg, "zero");
  iset[1].zero = (float) atof(gtk_entry_get_text(GTK_ENTRY(w)));
  w = get_widget(dlg, "eps");
  iset[1].eps = (float) atof(gtk_entry_get_text(GTK_ENTRY(w)));
  w = get_widget(dlg, "dens");
  iset[1].dens = (float) atof(gtk_entry_get_text(GTK_ENTRY(w)));
  w = get_widget(dlg, "err_dens");
  iset[1].delden = (float) atof(gtk_entry_get_text(GTK_ENTRY(w)));
  w = get_widget(dlg, "mol_weight");
  iset[1].mol_weight = (float) atof(gtk_entry_get_text(GTK_ENTRY(w)));
  w = get_widget(dlg, "fom");
  iset[1].fom = (float) atof(gtk_entry_get_text(GTK_ENTRY(w)));
  w = get_widget(dlg, "n_imp");
  iset[1].n_imp = spin_button_get_value_as_int(w);
  for(int i=0; i<6; i++)
  {
      w = get_widget(dlg, crysts[i]);
      iset[1].syst[i] = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(w));
  }
  w = get_widget(dlg, "zero_s");
  iset[1].zero_s = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(w));
  w = get_widget(dlg, "zero_ref");
  iset[1].zero_ref = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(w));
  w = get_widget(dlg, "search_type");
  iset[1].search_type = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(w));
//McMaille
  w = get_widget(dlg, "nlines_mcm");
  iset[2].nlines = spin_button_get_value_as_int(w);
  w = get_widget(dlg, "zero_mcm");
  iset[2].zero = (float) atof(gtk_entry_get_text(GTK_ENTRY(w)));
  w = get_widget(dlg, "n_imp2");
  iset[2].n_imp = spin_button_get_value_as_int(w);
  w = get_widget(dlg, "width");
  iset[2].width = (float) atof(gtk_entry_get_text(GTK_ENTRY(w)));
  w = get_widget(dlg, "volmin2");
  iset[2].volmin = (float) atof(gtk_entry_get_text(GTK_ENTRY(w)));
  w = get_widget(dlg, "volmax2");
  iset[2].volmax = (float) atof(gtk_entry_get_text(GTK_ENTRY(w)));
  w = get_widget(dlg, "abcmax2");
  iset[2].abcmax = (float) atof(gtk_entry_get_text(GTK_ENTRY(w)));
  for(int i=0; i<Nopt_bb; i++)
  {
      w = get_widget(dlg, g_strdup_printf("%d",search_t[i]));
      if(gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(w)))
         iset[2].search_type = search_t[i];
  }
  for(int i=0; i<6; i++)
  {
      w = get_widget(dlg, g_strdup_printf("%s_mm", crysts[i]));
      iset[2].syst[i] = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(w));
  }
}

static void response(GtkWidget *but, gpointer data)
{
using namespace C_Indexing;
    int responseId = GPOINTER_TO_INT(data);
    GtkWidget *dlg = gtk_widget_get_toplevel(but);
    //gtk_dialog_response(GTK_DIALOG(dlg), responseId);
    int run_dialog = 0;
    switch(responseId)
    {
         case 1:         // defaults
              FillFormIndexing(dlg, indx_set0);
              break;
         case 3:         // run
              run_dialog = 1;
              ReadFormIndexing(dlg, indx_set, select_pro);
              break;
         default:
         case 2:         // cancel
              run_dialog = -1;
              break;
    }
    if(run_dialog)
       gtk_widget_destroy(dlg);
     if(run_dialog > 0)
     {
        finish_index_(indx_set, select_pro);
        fine_elaborazione();
     }
     else
        close_indexation_win_();
}

GtkWidget *create_indexing_settings()
{
  GtkWidget *dialog1 = gtk_dialog_new ();
  gtk_window_set_title (GTK_WINDOW (dialog1), "Indexing Settings");
  gtk_container_set_border_width(GTK_CONTAINER(dialog1), 8);
  GtkWidget *vbox1 = gtk_dialog_get_content_area(GTK_DIALOG(dialog1));
  gtk_window_set_destroy_with_parent(GTK_WINDOW(dialog1), TRUE);
  gtk_window_set_transient_for(GTK_WINDOW(dialog1), GTK_WINDOW(finestre[0]));

  GtkWidget *frame = NEW_FRAME("Indexing Programs", true);
  gtk_box_pack_start(GTK_BOX(vbox1), frame, FALSE, FALSE, 0);
  GtkWidget *hbox = hbox_new(TRUE, 0);
  add_to_frame(frame, hbox);

  GtkWidget *notebook = NewNotebook (vbox1);
  //gtk_box_pack_start (GTK_BOX (vbox1), notebook, TRUE, TRUE, 0);
  g_object_set_data(G_OBJECT(dialog1), "nb", notebook);
  gtk_widget_set_name(notebook, "mynotebook");
  page_ntreor(dialog1, notebook);
  page_dicvol(dialog1, notebook);
  page_mcmaille(dialog1, notebook);

  for (int i=0; i<nradios; i++)
  {
    GtkWidget *radio = CHECK_BUTTON_WITH_LABEL (radios[i], NULL);
    gtk_box_pack_start (GTK_BOX (hbox), radio, TRUE, TRUE, 0);
    g_object_set_data (G_OBJECT (dialog1), radios[i], radio);
    g_signal_connect(G_OBJECT(radio), "toggled", G_CALLBACK(selected_prog),
          GINT_TO_POINTER(i));
  }

  GtkWidget *box = hbox_new(FALSE, 0);
  gtk_box_pack_end(GTK_BOX(vbox1), box, FALSE, FALSE, 10);
  GtkSizeGroup *szgroup = gtk_size_group_new(GTK_SIZE_GROUP_HORIZONTAL);
  GtkWidget *button = DEFAULTS_button();
  gtk_size_group_add_widget(szgroup, button);
  g_signal_connect(G_OBJECT(button), "clicked", G_CALLBACK(response),
            GINT_TO_POINTER(1));
  gtk_box_pack_start(GTK_BOX(box), button, FALSE, FALSE, 0);
  button = CANCEL_button();
  gtk_size_group_add_widget(szgroup, button);
  g_signal_connect(G_OBJECT(button), "clicked", G_CALLBACK(response),
            GINT_TO_POINTER(2));
  gtk_box_pack_end(GTK_BOX(box), button, FALSE, FALSE, 0);
  g_signal_connect_swapped(G_OBJECT(dialog1), "delete_event",
        G_CALLBACK(gtk_button_clicked), G_OBJECT(button));
  g_signal_connect (G_OBJECT (dialog1), "delete_event",
                      G_CALLBACK(true_cb), NULL);
  button = RUN_button();
  gtk_size_group_add_widget(szgroup, button);
  g_signal_connect(G_OBJECT(button), "clicked", G_CALLBACK(response),
            GINT_TO_POINTER(3));
  gtk_box_pack_end(GTK_BOX(box), button, FALSE, FALSE, 10);
  gtk_widget_show_all(dialog1);
  return dialog1;
}

void c_indexing(gpointer item, gpointer data)
{
using namespace C_Indexing;
  int sel_pro = GPOINTER_TO_INT(data);
  GtkWidget *dlg = create_indexing_settings();
  for (int i=1; i<=nradios; i++)
  {
       GtkWidget *radio = get_widget(dlg, radios[i-1]);
       gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(radio), (i==sel_pro));
  }


// get values
  open_indexation_win_(indx_set0, indx_set);

// fill form
  FillFormIndexing(dlg, indx_set);

  gtk_widget_show_all(dlg);
#if 0
  int run_dialog = 0;
  while (run_dialog == 0) 
  {
         int result = gtk_dialog_run(GTK_DIALOG(dlg));
         switch(result)
         {
         case 1:         // defaults
              FillFormIndexing(dlg, indx_set0);
              break;
         case 3:         // run
              run_dialog = 1;
              ReadFormIndexing(dlg, indx_set, select_pro);
              break;
         default:
         case 2:         // cancel
              run_dialog = -1;
              break;
         }
  }
  gtk_widget_destroy(dlg);
  if(run_dialog > 0)
  {
     //elaborazione_in_corso();
     finish_index_(indx_set, select_pro);
     fine_elaborazione();
  }
#endif
}

//  ----------------------- P E A K S ------------------

typedef struct  {
                  float x;
                  float xd;
                  float y;
                  float yint;
                  float fwhm;
                  float asym;
                  int   sel;
                  int   rcod;
                } peak_type;

typedef struct  {
                  int   posi;
                  float x;
                  float xd;
                  float y;
                  float fwhm;
                } peaklist_type;

extern "C" int  peak_list_window_(int *code);
extern "C" void get_peak_list_(peak_type peakvec[]);
extern "C" void peak_list_action_(int sel_vet[], int *nPeaks, int *code);
extern "C" void peak_list_change_(peak_type *peak, int *irow, int *icol, int *ier);
extern "C" void hide_peaks_(int *stato);

namespace peak_list_space {
          std::vector<peak_type> peakvect;
          int nPeaks;
                          }

void on_hide_peaks(gpointer w, gpointer data)
{
   int stato = obj_get_active(w);
   hide_peaks_(&stato);
}

void endPeakList(GtkWidget *w, gpointer data)
{
  g_object_set_data(G_OBJECT(finestre[0]), "peak_list_win", NULL);
  gtk_widget_destroy(gtk_widget_get_toplevel(w));
  int code = 2;
  (void) peak_list_window_(&code);
}

void fill_peak_list(GtkWidget *dlg, int npeaks)
{
using namespace peak_list_space;
  GtkWidget *list = get_widget(dlg, "lista_picchi");
  int alt = get_int(dlg, "alt");
  gtk_widget_set_size_request(get_widget(GTK_WIDGET(list), "scrollwin"), -1,
         (int) MRmin(1.5f*(npeaks+1)*alt, 18*alt));
  clear_list(list);
  GtkTreeIter iter;
  GtkListStore *list_store = GTK_LIST_STORE(gtk_tree_view_get_model(GTK_TREE_VIEW(list)));
  for (int i=0;i<npeaks;i++)
  {
       gtk_list_store_append (list_store, &iter);
       gtk_list_store_set (list_store, &iter,
           0, i,
           1, peakvect.at(i).x, 2, peakvect.at(i).xd,
           3, peakvect.at(i).y, 4, peakvect.at(i).fwhm, -1);
  }
}

static void on_savepeak(GtkWidget *, gpointer data)
{
     SavePeaks(4);
}

static void on_addpeak(GtkWidget *, gpointer data)
{
using namespace peak_list_space;
    GtkTreeIter iter;
    GtkTreeView *treeview = (GtkTreeView *)data;
    GtkTreeModel *model = gtk_tree_view_get_model(treeview);
    peaklist_type Row;
    Row.x = 0;
    Row.xd = 0;
    Row.y = 0;
    Row.fwhm = 0;
    Row.posi = n_children(GTK_WIDGET(data));
    gtk_list_store_append (GTK_LIST_STORE (model), &iter);
    gtk_list_store_set (GTK_LIST_STORE (model), &iter,
              0, Row.posi, 1, Row.x, 2, Row.xd,
              3, Row.y, 4, Row.fwhm, -1);
    GtkTreePath *path = gtk_tree_model_get_path(model, &iter);
    GtkTreeViewColumn *column = gtk_tree_view_get_column(treeview, 1);
    gtk_tree_view_scroll_to_cell(treeview, path, column, FALSE, 0, 0);
    gtk_tree_view_set_cursor(treeview, path, column, TRUE);
/*
    peak_type newpeak;
    int row = Row.posi + 1;
    int col = 1;
    int ier = 0;
    newpeak.x =  Row.x;
    newpeak.xd =  Row.xd;
    newpeak.y =  Row.y;
    newpeak.fwhm =  Row.fwhm;
    nPeaks++;
    peakvect.push_back(newpeak);
    peak_list_change_(&newpeak, &row, &col, &ier);
    select_row(GTK_WIDGET(treeview), Row.posi);
*/
}

static bool sort_by_pos_d( peaklist_type elem1, peaklist_type elem2 )
{
   return (elem1.posi > elem2.posi);
}

static bool sort_by_pos( peaklist_type elem1, peaklist_type elem2 )
{
   return (elem1.posi < elem2.posi);
}

static bool sort_by_x_d( peaklist_type elem1, peaklist_type elem2 )
{
   return (elem1.x > elem2.x);
}

static bool sort_by_x( peaklist_type elem1, peaklist_type elem2 )
{
   return (elem1.x < elem2.x);
}

static bool sort_by_xd_d( peaklist_type elem1, peaklist_type elem2 )
{
   return (elem1.xd > elem2.xd);
}

static bool sort_by_xd( peaklist_type elem1, peaklist_type elem2 )
{
   return (elem1.xd < elem2.xd);
}

static bool sort_by_y_d( peaklist_type elem1, peaklist_type elem2 )
{
   return (elem1.y > elem2.y);
}

static bool sort_by_y( peaklist_type elem1, peaklist_type elem2 )
{
   return (elem1.y < elem2.y);
}

static bool sort_by_fwhm_d( peaklist_type elem1, peaklist_type elem2 )
{
   return (elem1.fwhm > elem2.fwhm);
}

static bool sort_by_fwhm( peaklist_type elem1, peaklist_type elem2 )
{
   return (elem1.fwhm < elem2.fwhm);
}

void OrdinaColonna(std::vector<peaklist_type> peak_infov, 
                   GtkTreeModel *model, int ncol, int ordin)
{
    GtkTreeIter iter;
    GtkListStore *list_store = GTK_LIST_STORE(model);

    bool (*sort_by_col)(peaklist_type, peaklist_type);
    switch (ordin*(ncol+1))
    {
    case -1:
         sort_by_col = sort_by_pos_d;
         break;
    case -2:
         sort_by_col = sort_by_x_d;
         break;
    case -3:
         sort_by_col = sort_by_xd_d;
         break;
    case -4:
         sort_by_col = sort_by_y_d;
         break;
    case -5:
         sort_by_col = sort_by_fwhm_d;
         break;
    default:
    case 1:
         sort_by_col = sort_by_pos;
         break;
    case 2:
         sort_by_col = sort_by_x;
         break;
    case 3:
         sort_by_col = sort_by_xd;
         break;
    case 4:
         sort_by_col = sort_by_y;
         break;
    case 5:
         sort_by_col = sort_by_fwhm;
         break;
    }
    sort(peak_infov.begin(), peak_infov.end(), sort_by_col);
    gboolean valid = gtk_tree_model_get_iter_first (model, &iter);
    int k = 0;
    while(valid)
    {
       gtk_list_store_set (list_store, &iter,
           0, peak_infov[k].posi, 
           1, peak_infov[k].x, 
           2, peak_infov[k].xd, 
           3, peak_infov[k].y,
           4, peak_infov[k].fwhm,
          -1);
       k++;
       valid = gtk_tree_model_iter_next (model, &iter);
    }
}

void riassegna_posi(GtkWidget *list, int action)
{
using namespace peak_list_space;
    std::vector<peaklist_type> peak_infov;

    GtkTreeModel *model = gtk_tree_view_get_model(GTK_TREE_VIEW(list));
    GtkTreeIter iter;
    gboolean valid = gtk_tree_model_get_iter_first (model, &iter);
    while (valid)
    {
       peaklist_type peak_info;
       gtk_tree_model_get (model, &iter,
           0, &(peak_info.posi),
           1, &(peak_info.x),
           2, &(peak_info.xd),
           3, &(peak_info.y),
           4, &(peak_info.fwhm),
          -1);
       valid = gtk_tree_model_iter_next (model, &iter);
       peak_infov.push_back(peak_info);
    }
    if(action == 0)
       sort(peak_infov.begin(), peak_infov.end(), sort_by_pos);
    else
       sort(peak_infov.begin(), peak_infov.end(), sort_by_x);
    if(peak_infov.empty())
       return;
    for(int i=0; i<(int)peak_infov.size(); i++)
        peak_infov.at(i).posi = i;
    
    peakvect.clear();

    for(int i=0; i<(int)peak_infov.size(); i++)
    {
        peak_type peak0;
        peak0.x = peak_infov.at(i).x;
        peak0.xd = peak_infov.at(i).xd;
        peak0.y = peak_infov.at(i).y;
        peak0.fwhm = peak_infov.at(i).fwhm;
        peakvect.push_back(peak0);
    }
    int ncol = get_int(list, "col_order");
    GtkTreeViewColumn *column = gtk_tree_view_get_column(GTK_TREE_VIEW(list),
        ncol);
    int ordin = GPOINTER_TO_INT(g_object_get_data(G_OBJECT(column), "set_order"));
    if(ordin < -1)
       ordin = 1;
    OrdinaColonna(peak_infov, model, ncol, ordin);
    peak_infov.clear();
}

void print_peak_list()
{
using namespace std;
using namespace peak_list_space;
    if(peakvect.empty())
       return;
    for(size_t i=0; i<peakvect.size(); i++)
        cout << setiosflags( ios::fixed )
             << setiosflags( ios::right )
             << setw(3) << i+1 << ") "
             << setprecision(4)
             << setw(15)
             << peakvect.at(i).x    << "   "
             << peakvect.at(i).xd   << "   "
             << peakvect.at(i).fwhm << "   "
             << setprecision(0)
             << setw(8)
             << peakvect.at(i).y  << "\n";
}

static void on_edit_peak (GtkCellRendererText *cell, const gchar *path_string,
                          const gchar *new_text, gpointer data)
{
using namespace peak_list_space;
  GtkTreeModel *model = (GtkTreeModel *)data;
  GtkTreePath *path = gtk_tree_path_new_from_string (path_string);
  GtkTreeIter iter;
  GtkWidget *dlg = GTK_WIDGET(g_object_get_data(G_OBJECT(data), "father"));
  GtkWidget *list = get_widget(dlg, "lista_picchi");

  gint column = GPOINTER_TO_INT (g_object_get_data (G_OBJECT (cell), "column"));
  gint gt = GPOINTER_TO_INT (g_object_get_data (G_OBJECT (cell), "gtype"));
  gtk_tree_model_get_iter (model, &iter, path);
  if (gt == G_TYPE_FLOAT)
  {
       peaklist_type peak_info;
       peak_type newpeak = {0, 0, 0, 0, 0, 0};
       gtk_tree_model_get (model, &iter,
           0, &(peak_info.posi), 
           1, &(peak_info.x),
           2, &(peak_info.xd),
           3, &(peak_info.y),
           4, &(peak_info.fwhm),
          -1);
      int row = peak_info.posi + 1;
      int col = column;
      if(row < nPeaks)
         memcpy(&newpeak, &(peakvect[row-1]), sizeof(peak_type));
      float val = (float) atof(new_text);
      if(column == 1)
         newpeak.x = val;
      else if(column == 2)
         newpeak.xd = val;
      int ier =  0;
      peak_list_change_(&newpeak, &row, &col, &ier);
      if(!ier)
      {
         gtk_list_store_set (GTK_LIST_STORE (model), &iter, 
             0, row-1, 1, newpeak.x, 2, newpeak.xd, 
             3, newpeak.y, 4, newpeak.fwhm, -1);
         if(row >= nPeaks)
         {
            nPeaks++;
            peakvect.push_back(newpeak);
         }
         //print_peak_list();
         riassegna_posi(list, 1);
      }
      else                   // errore
      {
         if(row >= nPeaks)    // devo cancellare la riga se l'avevo aggiunta
            gtk_list_store_remove (GTK_LIST_STORE (model), &iter);
      }
  }
}

static void selected_peak(GtkTreeSelection  *sele, gpointer data)
{
using namespace peak_list_space;
     GtkWidget *dlg = (GtkWidget *)data;
     GtkTreeView *tree_view = gtk_tree_selection_get_tree_view(sele);
     GtkTreeModel *md = gtk_tree_view_get_model(tree_view);
     GList *rowsel = NULL;
     rowsel = get_selections(GTK_WIDGET(tree_view));
     GtkWidget *but = get_widget(dlg, "delete_b");
     gtk_widget_set_sensitive(but, (rowsel != NULL));
     if(rowsel && (nPeaks > 0))
     {
        int code = 1;
        int *sel_vet = g_new0(int, nPeaks);
        int num_sel = g_list_length(rowsel);
        for(int i=0; i<num_sel; i++)
        {
            int row;
            GtkTreePath *path = (GtkTreePath *) g_list_nth_data (rowsel, i);
            GtkTreeIter iter;
            if(gtk_tree_model_get_iter(md, &iter, path))
            {
                gtk_tree_model_get(md, &iter, 0, &row, -1);
                if((row >= 0) && (row < nPeaks))
                   sel_vet[row] = 10;
            }
        }
        g_list_foreach (rowsel, (GFunc)gtk_tree_path_free, NULL);
        g_list_free (rowsel);
        peak_list_action_(sel_vet, &nPeaks, &code);
        g_free(sel_vet);
     }
}

static void sort_peak_list(GtkWidget *column, gpointer data)
{
    std::vector<peaklist_type> peak_infov;

    int ncol = GPOINTER_TO_INT(data);
    int ordin = get_int(column, "set_order");
    GtkWidget *treeview = get_widget(column, "father");
    GtkTreeModel *model = gtk_tree_view_get_model(GTK_TREE_VIEW(treeview));
    GtkTreeIter iter;
    g_object_set_data(G_OBJECT(treeview), "col_order", GINT_TO_POINTER(ncol));
    gboolean valid = gtk_tree_model_get_iter_first (model, &iter);
    while (valid)
    {
       peaklist_type peak_info;
       gtk_tree_model_get (model, &iter,
           0, &(peak_info.posi), 
           1, &(peak_info.x), 
           2, &(peak_info.xd), 
           3, &(peak_info.y),
           4, &(peak_info.fwhm),
          -1);
       valid = gtk_tree_model_iter_next (model, &iter);
       peak_infov.push_back(peak_info);
    }
    if(ordin > 0)
       ordin = -1;
    else
       ordin = 1;
    g_object_set_data(G_OBJECT(column), "set_order", GINT_TO_POINTER(ordin));

    if(peak_infov.empty())
       return;
    OrdinaColonna(peak_infov, model, ncol, ordin);
    peak_infov.clear();
}

static void on_delpeak(GtkWidget *, gpointer data)
{
using namespace peak_list_space;
     GtkWidget *list = (GtkWidget *)data;
     GList *rowsel = NULL;
     rowsel = get_selections(list);
     if(rowsel)
     {
        int code = 2;
        int *sel_vet = g_new0(int, nPeaks);
        int num_sel = g_list_length(rowsel);
        std::vector <GtkTreeIter> Iters;
        GtkTreeIter iter;
        GtkTreeModel *md = gtk_tree_view_get_model(GTK_TREE_VIEW(list));
        for(int i=0; i<num_sel; i++)
        {
             GtkTreePath *path = (GtkTreePath *) g_list_nth_data (rowsel, i);
             if(gtk_tree_model_get_iter(md, &iter, path))
             {
                int row;
                gtk_tree_model_get(md, &iter, 0, &row, -1);
                sel_vet[row] = -999;
                Iters.push_back(iter);
             }
        }
        peak_list_action_(sel_vet, &nPeaks, &code);
        for(int i=nPeaks-1; i>=0; i--)
        {
            if(sel_vet[i] == -999)
               peakvect.erase(peakvect.begin() + i);
        }
        g_free(sel_vet);
        g_list_foreach (rowsel, (GFunc)gtk_tree_path_free, NULL);
        g_list_free (rowsel);
        nPeaks = (int) peakvect.size();

        if(!Iters.empty())
        {
           int hnd = get_int(list, "hnd");
           GtkTreeSelection *selec =
              gtk_tree_view_get_selection(GTK_TREE_VIEW(list));
           g_signal_handler_block(G_OBJECT(selec), hnd);
           for(int i=0; i<(int)Iters.size(); i++)
           {
               GtkTreeIter iter0 = Iters.at(i);
               gtk_list_store_remove (GTK_LIST_STORE (md), &iter0);
           }
           Iters.clear();
           g_signal_handler_unblock(G_OBJECT(selec), hnd);
        }
        riassegna_posi(list, 0);
        redrawpattern(1);
     }
}

GtkWidget *create_peak_list()
{
  GtkWidget *window1;
  GtkWidget *vbox1;
  GtkWidget *list;
  int lar, alt;
  PangoFontDescription *fontlist = get_fontN(MONOSPACE_FONT);
  get_metrics(fontlist, &lar, &alt);

  const char *colonne[] = {"num", "2Theta", "d", "Intensity", "Fwhm"};
  GType ty[] = { G_TYPE_INT, G_TYPE_FLOAT, G_TYPE_FLOAT, G_TYPE_FLOAT,
                 G_TYPE_FLOAT};
  int sizes[] = {-1, 0, 0, 0, 0};
  int precs[] = { 0, 5, 5, 0, 5};
  bool editable[] = {false, true, true, false, false};

  int ncolonne = (int) G_N_ELEMENTS(colonne);
  window1 = gtk_dialog_new ();
  gtk_window_set_title (GTK_WINDOW (window1), "Peak list");
  g_object_set_data(G_OBJECT(window1), "alt", GINT_TO_POINTER(alt));

  vbox1 = gtk_dialog_get_content_area(GTK_DIALOG(window1));
  gtk_container_set_border_width(GTK_CONTAINER(vbox1), 6);

  GtkWidget *hbox = hbox_new(FALSE, 0);
  gtk_box_pack_start(GTK_BOX(vbox1), hbox, TRUE, TRUE, 5);

  list = new_selection(window1, hbox, "lista_picchi", ncolonne, colonne, 
                        sizes, ty, precs, editable, G_CALLBACK(on_edit_peak));
  widget_modify_font(list, fontlist);

  g_object_set_data(G_OBJECT(list), "col_order", GINT_TO_POINTER(0));
   for(int i=0; i<ncolonne; i++)
  {
      GtkTreeViewColumn *column = gtk_tree_view_get_column(GTK_TREE_VIEW(list), i);
      GtkWidget *label = gtk_tree_view_column_get_widget(column);
      if(label)
         widget_modify_font(label, fontlist);
      gtk_tree_view_column_set_clickable(column, TRUE);
      g_object_set_data(G_OBJECT(column), "set_order", GINT_TO_POINTER(-2));
      g_object_set_data(G_OBJECT(column), "father", list);
      g_signal_connect(G_OBJECT(column), "clicked", G_CALLBACK(sort_peak_list),
                 GINT_TO_POINTER(i));
  }

  GtkTreeSelection *selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(list));
  int hnd = g_signal_connect (G_OBJECT (selection), "changed",
                      G_CALLBACK (selected_peak), window1);
  g_object_set_data(G_OBJECT(selection), "hnd", GINT_TO_POINTER(hnd));
  g_object_set_data(G_OBJECT(list), "hnd", GINT_TO_POINTER(hnd));
  set_selection_mode(list, GTK_SELECTION_MULTIPLE);

  GtkWidget *vvbox = vbox_new(TRUE, 0);
  gtk_box_pack_start(GTK_BOX(hbox), vvbox, FALSE, FALSE, 5);

  GtkWidget *button = myimage_button("      Add", "add");
  gtk_box_pack_start(GTK_BOX(vvbox), button, FALSE, FALSE, 5);
  (void) g_signal_connect(G_OBJECT (button), "clicked",
         G_CALLBACK(on_addpeak), (gpointer) list);

  button = myimage_button("Delete", "remove");
  gtk_box_pack_start(GTK_BOX(vvbox), button, FALSE, FALSE, 5);
  g_object_set_data(G_OBJECT(window1), "delete_b", button);
  (void) g_signal_connect(G_OBJECT (button), "clicked",
         G_CALLBACK(on_delpeak), (gpointer) list);
  gtk_widget_set_sensitive(button, FALSE);

  button = myimage_button("Export", "save");
  gtk_box_pack_start(GTK_BOX(vvbox), button, FALSE, FALSE, 5);
  (void) g_signal_connect(G_OBJECT (button), "clicked",
         G_CALLBACK(on_savepeak), (gpointer) list);

  button = OK_button();
  gtk_dialog_add_action_widget (GTK_DIALOG (window1), button, 1);
  g_signal_connect(G_OBJECT(button), "clicked",
         G_CALLBACK(endPeakList), GINT_TO_POINTER(1));
  gtk_button_box_set_layout(GTK_BUTTON_BOX(gtk_widget_get_parent(button)),
         GTK_BUTTONBOX_SPREAD);

  return window1;
}

void peak_list(gpointer , gpointer)
{
using namespace peak_list_space;
   peak_type *peaklist = NULL;
   int code = 1;
   int result = peak_list_window_(&code);
   if(result < 0)
      return;
   nPeaks = result;
   if(!peakvect.empty())
      peakvect.clear();
   if(nPeaks > 0)
   {
      peaklist = g_new0(peak_type, nPeaks);
      get_peak_list_(peaklist);
      for(int i=0; i<nPeaks; i++)
         peakvect.push_back(peaklist[i]);
      g_free(peaklist);
   }
   //g_print("nPeaks in C %d\n", nPeaks);
   GtkWidget *dlg;
   if(get_widget(finestre[0], "peak_list_win") == NULL)
   {
      if(nPeaks == 0)
         return;
      dlg = create_peak_list();
      g_object_set_data(G_OBJECT(finestre[0]), "peak_list_win", dlg);
      gtk_widget_show_all (dlg);
   }
   else
      dlg = get_widget(finestre[0], "peak_list_win");
   fill_peak_list(dlg, result);
}

extern "C" void update_peak_list_()
{
   if(get_widget(finestre[0], "peak_list_win") != NULL)
      peak_list(NULL, NULL);
}

void out_indexing(gpointer item, gpointer data)
{
using namespace gtkext;

  int sel_pro = GPOINTER_TO_INT(data);
  std::string filnam, ext;
  switch (sel_pro)
  {
  default:
  case 1:
       filnam.assign("condens.out");
       ext.assign("*.out");
       break;
  case 2:
       filnam.assign("dicvol_out.ord");
       ext.assign("*.ord");
       break;
  case 3:
       filnam.assign("mcmaille.imp");
       ext.assign("*.imp");
       break;
  }
  if(!g_file_test(filnam.c_str(), G_FILE_TEST_EXISTS))
  {
      dlg_errmsg(GTK_WINDOW(finestre[0]),
          g_strdup_printf("Required file '%s' does not exist.",filnam.c_str()));
      return;
  }
  int flags = IFREADONLY;
  text_window (GTK_WINDOW(finestre[0]), filnam.c_str(), ext.c_str(), flags,
       filnam.c_str());
}

void cell_list(gpointer item, gpointer data)
{
    open_cell_list_();
}

