module calculate_pdp

implicit none

contains

   subroutine calculate_powder_pattern(rfc,xczero,yc,ref,scal,nwave,ratioint,pear,pvoi,tch,ier)
   USE progtype
   USE pearsonf
   USE psvoigtf
   USE tchzf
   USE profile_function
   USE reflection_type_util
!
   implicit none
   type(refine_condition_type), intent(in)                         :: rfc       ! settings for powder pattern
   real, dimension(:), intent(in)                                  :: xczero    ! 2-theta with zero correction
   real, dimension(:), intent(out)                                 :: yc        ! calculated pattern
   type(reflection_type), dimension(:), allocatable, intent(inout) :: ref       ! reflections
   real, intent(in)                                                :: scal      ! scale
   integer, intent(in)                                             :: nwave     ! 1 or 2 (for second wavelength)
   real, intent(in)                                                :: ratioint  ! ratio of intensity between 2 wavel.
   type(pearson_function_type), intent(in)                         :: pear
   type(pvoigt_function_type), intent(in)                          :: pvoi
   type(tch_function_type), intent(in)                             :: tch
   real, dimension(rfc%lim%nrefu)                                  :: fcstr
   integer                                                         :: posfc,posfr
   integer, intent(out)                                            :: ier
!
   posfc = rfc%lim%ncu
   posfr = rfc%lim%nrefu
   if (rfc%raction > 0) then
       fcstr(:) = ref(:posfr)%fo
   else
       fcstr(:) = ref(:posfr)%fc
   endif
!
   select case (rfc%typefun)
     case (PEARSON7)
       call pearson(pear,ref(:posfr),xczero(:posfc),fcstr,scal,yc,nwave,ratioint,ier)
     case (PVOIG)
       call psvoigt(pvoi,ref(:posfr),xczero(:posfc),fcstr,scal,yc,nwave,ratioint,ier)
     case (TCHZ)
       call psvoigt_tchz(tch,ref(:posfr),xczero(:posfc),fcstr,scal,yc,nwave,ratioint,ier)
     case default
       ier = 1
   end select
!
   end subroutine calculate_powder_pattern

!--------------------------------------------------------------------------------------

   subroutine init_function_param(pears,psvoig,tch,fwhm)
!  
!  Initialize profile functions
!  
   USE pearsonf
   USE psvoigtf
   USE tchzf
   USE profile_function
!!!!FIXME - test this and try to add the refine of W in setstageprof
!!!!!test
!     USE General, only:alambda
!     USE counts
!!!!!test
   type(pearson_function_type), intent(out), optional :: pears
   type(pvoigt_function_type), intent(out), optional  :: psvoig
   type(tch_function_type), intent(out), optional     :: tch
   real, intent(in), optional                         :: fwhm
!  
   if (present(pears)) then
       pears = init_pearson_param()
       if (present(fwhm)) pears%w%val = fwhm*fwhm
!test
!          valuew =  delta_from_lambda(0.1,1.54056,alambda,4.0)
!               !write(0,*)'valuew=',valuew
!          pears%w = rparam_type(val=valuew*valuew,str='W')
!test
!           write(0,*)'VAL=',pears%w%val

   endif
!  
   if (present(psvoig)) then
       psvoig = init_psvoigt_param()
       if (present(fwhm)) psvoig%w%val = fwhm*fwhm
   endif
!  
   if (present(tch)) then
       tch = init_tchz_param()
       if (present(fwhm)) tch%w%val = fwhm*fwhm
   endif
!  
   end subroutine init_function_param

end module calculate_pdp
