#include "mrconfig.h"
#include "forcomm2.h"
#include "aliasx.h"

extern GtkWidget *finestre[6];
extern plotok *cplotok;
extern "C" void save_cell_list_(char *filenam, size_t lung);

static int cell_selected = 1;
static bool cancel = false;

typedef struct
             {
               float  par[6];     // cell parameters
               float  sd[6];      // st. dev. on parameters
               float  vol;        // cell volume
               float  m20;        // m20
               float  alt_fom;    // alternative f.o.m
               float  mc20;       // f.o.m used in mcmaille
               float  zshift;     // zero shift
               int    nund;       // num. of unindexed lines
               char   sys[2];     // crystal system
               char   syminfo[20];// additional symmetry information
               int    pcode;      // indexing program code
             } cell_info_type;

typedef struct
             {
               int    num;        // indice di colonna
               float  par[6];     // cell parameters
               float  vol;        // cell volume
               float  m20;        // m20
               float  alt_fom;    // alternative f.o.m
               float  mc20;       // mc20
               float  zshift;     // zero shift
               int    nund;       // num. of unindexed lines
               char   syminfo[21];// additional symmetry information
               int    pcode;      // indexing program code
             } mycell_info_type;

static const char *progs[] = {"N", "D", "M"};

void endCelldialog(GtkButton *button, gpointer data)
{
  int code = GPOINTER_TO_INT(data);
  cancel = (code == 2);
//  if(code == 2)
//     cplotok->kexitrun = 2;
  gtk_widget_destroy(gtk_widget_get_toplevel(GTK_WIDGET(button)));
  gtk_main_quit();
}

static void onSelectCell (GtkTreeSelection *selec, gpointer data)
{
  GtkTreeIter iter;
  GtkWidget *treeview = (GtkWidget *) data;
  GtkTreeModel *model = gtk_tree_view_get_model(GTK_TREE_VIEW(treeview));
  int num;

  if (gtk_tree_selection_get_selected (selec, &model, &iter))
      gtk_tree_model_get (model, &iter, 0, &num, -1);
  else
      num = 0;

  if(num > 0)
  {
     cell_selected = num;
     viewrefcell_(&num);
  }
}

void export_cells(GtkButton *button, GtkWidget *dialog)
{
using namespace gtkext;
      FilselItem local_item;
      std::string curdir = getPWD();
      local_item = gtkext::saveas(dialog, "*.txt,*.exp", NULL);
      dir_set_current(curdir.c_str());
      if(local_item.exitCode > 0)
      {
          char *f1 = g_build_filename(local_item.cwd, local_item.fileinp, NULL);
          size_t l1 = strlen(f1)+1;
          save_cell_list_(f1, l1);
      }
}

static bool Alphacompare(char *str1, char *str2)
{
   gchar *temp1, *temp2;
   temp1 = g_utf8_casefold(str1, -1);
   temp2 = g_utf8_casefold(str2, -1);
   int result = strcmp(temp1, temp2);
   g_free(temp1);
   g_free(temp2);
   return (result < 0);
}

static bool sort_by_numd( mycell_info_type elem1, mycell_info_type elem2 )
{
   return (elem1.num > elem2.num);
}

static bool sort_by_num( mycell_info_type elem1, mycell_info_type elem2 )
{
   return (elem1.num < elem2.num);
}

static bool sort_by_progd( mycell_info_type elem1, mycell_info_type elem2 )
{
   return (elem1.pcode > elem2.pcode);
}

static bool sort_by_prog( mycell_info_type elem1, mycell_info_type elem2 )
{
   return (elem1.pcode < elem2.pcode);
}

static bool sort_by_ad( mycell_info_type elem1, mycell_info_type elem2 )
{
   return (elem1.par[0] > elem2.par[0]);
}

static bool sort_by_a( mycell_info_type elem1, mycell_info_type elem2 )
{
   return (elem1.par[0] < elem2.par[0]);
}

static bool sort_by_bd( mycell_info_type elem1, mycell_info_type elem2 )
{
   return (elem1.par[1] > elem2.par[1]);
}

static bool sort_by_b( mycell_info_type elem1, mycell_info_type elem2 )
{
   return (elem1.par[1] < elem2.par[1]);
}

static bool sort_by_cd( mycell_info_type elem1, mycell_info_type elem2 )
{
   return (elem1.par[2] > elem2.par[2]);
}

static bool sort_by_c( mycell_info_type elem1, mycell_info_type elem2 )
{
   return (elem1.par[2] < elem2.par[2]);
}

static bool sort_by_alfad( mycell_info_type elem1, mycell_info_type elem2 )
{
   return (elem1.par[3] > elem2.par[3]);
}

static bool sort_by_alfa( mycell_info_type elem1, mycell_info_type elem2 )
{
   return (elem1.par[3] < elem2.par[3]);
}

static bool sort_by_betad( mycell_info_type elem1, mycell_info_type elem2 )
{
   return (elem1.par[4] > elem2.par[4]);
}

static bool sort_by_beta( mycell_info_type elem1, mycell_info_type elem2 )
{
   return (elem1.par[4] < elem2.par[4]);
}

static bool sort_by_gammad( mycell_info_type elem1, mycell_info_type elem2 )
{
   return (elem1.par[5] > elem2.par[5]);
}

static bool sort_by_gamma( mycell_info_type elem1, mycell_info_type elem2 )
{
   return (elem1.par[5] < elem2.par[5]);
}

static bool sort_by_vold( mycell_info_type elem1, mycell_info_type elem2 )
{
   return (elem1.vol > elem2.vol);
}

static bool sort_by_vol( mycell_info_type elem1, mycell_info_type elem2 )
{
   return (elem1.vol < elem2.vol);
}

static bool sort_by_fomd( mycell_info_type elem1, mycell_info_type elem2 )
{
   return (elem1.alt_fom > elem2.alt_fom);
}

static bool sort_by_fom( mycell_info_type elem1, mycell_info_type elem2 )
{
   return (elem1.alt_fom < elem2.alt_fom);
}

static bool sort_by_m20d( mycell_info_type elem1, mycell_info_type elem2 )
{
   return (elem1.m20 > elem2.m20);
}

static bool sort_by_m20( mycell_info_type elem1, mycell_info_type elem2 )
{
   return (elem1.m20 < elem2.m20);
}

static bool sort_by_mc20d( mycell_info_type elem1, mycell_info_type elem2 )
{
   return (elem1.mc20 > elem2.mc20);
}

static bool sort_by_mc20( mycell_info_type elem1, mycell_info_type elem2 )
{
   return (elem1.mc20 < elem2.mc20);
}

static bool sort_by_shiftd( mycell_info_type elem1, mycell_info_type elem2 )
{
   return (elem1.zshift > elem2.zshift);
}

static bool sort_by_shift( mycell_info_type elem1, mycell_info_type elem2 )
{
   return (elem1.zshift < elem2.zshift);
}

static bool sort_by_nundd( mycell_info_type elem1, mycell_info_type elem2 )
{
   return (elem1.nund > elem2.nund);
}

static bool sort_by_nund( mycell_info_type elem1, mycell_info_type elem2 )
{
   return (elem1.nund < elem2.nund);
}

static bool sort_by_syminfod( mycell_info_type elem1, mycell_info_type elem2 )
{
   return Alphacompare(elem1.syminfo, elem2.syminfo);
}

static bool sort_by_syminfo( mycell_info_type elem1, mycell_info_type elem2 )
{
   return Alphacompare(elem2.syminfo, elem1.syminfo);
}

static void sort_cell_list(GtkWidget *column, gpointer data)
{
    bool (*sort_by_col)(mycell_info_type, mycell_info_type);
    std::vector<mycell_info_type> cell_infov;

    int ncol = GPOINTER_TO_INT(data);
    int ordin = get_int(column, "set_order");
    GtkWidget *treeview = get_widget(column, "father");
    GtkTreeModel *model = gtk_tree_view_get_model(GTK_TREE_VIEW(treeview));
    GtkListStore *list_store = GTK_LIST_STORE(model);
    GtkTreeIter iter;
    gboolean valid = gtk_tree_model_get_iter_first (model, &iter);
    while (valid)
    {
       char *prog_code = g_new0(char, 2);
       mycell_info_type cell_info;
       int dim = sizeof(cell_info.syminfo);
       char *symm_info = g_new0(char, dim);
       gtk_tree_model_get (model, &iter,
           0, &(cell_info.num), 1, &prog_code,
           2, &(cell_info.par[0]), 3, &(cell_info.par[1]),
           4, &(cell_info.par[2]), 5, &(cell_info.par[3]),
           6, &(cell_info.par[4]), 7, &(cell_info.par[5]),
           8, &(cell_info.vol), 9, &(cell_info.m20),
          10, &(cell_info.alt_fom), 11, &(cell_info.mc20),
          12, &(cell_info.zshift),
          13, &(cell_info.nund), 14, &symm_info, -1);
       CopiaStringa(cell_info.syminfo, dim, symm_info);
       valid = gtk_tree_model_iter_next (model, &iter);
       for (int i=0; i<3; i++)
       {
            if(prog_code[0] == progs[i][0])
               cell_info.pcode = i+1;
       }
       cell_infov.push_back(cell_info);
       g_free(symm_info);
       g_free(prog_code);
    }
    if(ordin > 0)
       ordin = -1;
    else
       ordin = 1;
    g_object_set_data(G_OBJECT(column), "set_order", GINT_TO_POINTER(ordin));

    switch (ordin*(ncol+1))
    {
    case -1:
         sort_by_col = sort_by_numd;
         break;
    case -2:
         sort_by_col = sort_by_progd;
         break;
    case -3:
         sort_by_col = sort_by_ad;
         break;
    case -4:
         sort_by_col = sort_by_bd;
         break;
    case -5:
         sort_by_col = sort_by_cd;
         break;
    case -6:
         sort_by_col = sort_by_alfad;
         break;
    case -7:
         sort_by_col = sort_by_betad;
         break;
    case -8:
         sort_by_col = sort_by_gammad;
         break;
    case -9:
         sort_by_col = sort_by_vold;
         break;
    case -10:
         sort_by_col = sort_by_m20d;
         break;
    case -11:
         sort_by_col = sort_by_fomd;
         break;
    case -12:
         sort_by_col = sort_by_mc20d;
         break;
    case -13:
         sort_by_col = sort_by_shiftd;
         break;
    case -14:
         sort_by_col = sort_by_nundd;
         break;
    case -15:
         sort_by_col = sort_by_syminfod;
         break;
    default:
    case 1:
         sort_by_col = sort_by_num;
         break;
    case 2:
         sort_by_col = sort_by_prog;
         break;
    case 3:
         sort_by_col = sort_by_a;
         break;
    case 4:
         sort_by_col = sort_by_b;
         break;
    case 5:
         sort_by_col = sort_by_c;
         break;
    case 6:
         sort_by_col = sort_by_alfa;
         break;
    case 7:
         sort_by_col = sort_by_beta;
         break;
    case 8:
         sort_by_col = sort_by_gamma;
         break;
    case 9:
         sort_by_col = sort_by_vol;
         break;
    case 10:
         sort_by_col = sort_by_m20;
         break;
    case 11:
         sort_by_col = sort_by_fom;
         break;
    case 12:
         sort_by_col = sort_by_mc20;
         break;
    case 13:
         sort_by_col = sort_by_shift;
         break;
    case 14:
         sort_by_col = sort_by_nund;
         break;
    case 15:
         sort_by_col = sort_by_syminfo;
         break;
    }
    sort(cell_infov.begin(), cell_infov.end(), sort_by_col);
    valid = gtk_tree_model_get_iter_first (model, &iter);
    int k = 0;
    while(valid)
    {
       char *label = g_strdup(cell_infov[k].syminfo);
       gtk_list_store_set (list_store, &iter,
           0, cell_infov[k].num, 1, progs[cell_infov[k].pcode-1],
           2, cell_infov[k].par[0], 3, cell_infov[k].par[1],
           4, cell_infov[k].par[2], 5, cell_infov[k].par[3],
           6, cell_infov[k].par[4], 7, cell_infov[k].par[5],
           8, cell_infov[k].vol, 9, cell_infov[k].m20,
          10, cell_infov[k].alt_fom, 11, cell_infov[k].mc20,
          12, cell_infov[k].zshift,
          13, cell_infov[k].nund, 14, g_strstrip(label), -1);
       g_free(label);
       k++;
       valid = gtk_tree_model_iter_next (model, &iter);
    }
}

GtkWidget *createCelldialog ()
{
  GtkWidget *window1;
  GtkWidget *vbox1;
  GtkWidget *frame1;
  GtkWidget *list;
  GtkWidget *ok_space;
  GtkWidget *cancel_space;
  PangoFontDescription *fontlist = get_fontN(MONOSPACE_FONT);

  const char *space_labels[] = { " Nr. ", "Prog.", "a", "b", "c", 
                                 "alpha", "beta",
                                 "gamma", "  Vol. ", "M20", " FOMnew ", "Mc20",
                                 "shift", "NIX", "Symmetry Info"};
  GType ty[] = { G_TYPE_INT, G_TYPE_STRING,
                 G_TYPE_FLOAT, G_TYPE_FLOAT, G_TYPE_FLOAT,
                 G_TYPE_FLOAT, G_TYPE_FLOAT, G_TYPE_FLOAT,
                 G_TYPE_FLOAT, G_TYPE_FLOAT, G_TYPE_FLOAT, G_TYPE_FLOAT, 
                 G_TYPE_FLOAT, G_TYPE_INT, G_TYPE_STRING };
  int sizes[] = {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0};
  int precs[] = {0, 0, 5, 5, 5, 3, 3, 3, 1, -2, -3, -2, 3, 0, 0};

  int NSPACELABELS = (int) G_N_ELEMENTS(space_labels);


  window1 = gtk_dialog_new ();
  gtk_window_set_title (GTK_WINDOW (window1), "    Plausible cell parameters    ");

  vbox1 = gtk_dialog_get_content_area(GTK_DIALOG(window1));
  gtk_container_set_border_width(GTK_CONTAINER(vbox1), 6);

  frame1 = NEW_FRAME ("Select cell");
  g_object_set_data (G_OBJECT (window1), "frame_lista", frame1);
  gtk_widget_show (frame1);
  gtk_box_pack_start (GTK_BOX (vbox1), frame1, TRUE, TRUE, 0);
  gtk_container_set_border_width(GTK_CONTAINER(frame1), 6);

  list = new_selection(window1, NULL, "lista_celle", NSPACELABELS, space_labels, 
                        sizes, ty, precs);
  widget_modify_font(list, fontlist);
  add_to_frame(frame1, get_widget(list, "frame"));
  gtk_widget_show (list);

   for(int i=0; i<NSPACELABELS; i++)
  {
      GtkTreeViewColumn *column = gtk_tree_view_get_column(GTK_TREE_VIEW(list), i);
      GtkWidget *label = gtk_tree_view_column_get_widget(column);
      if(label)
         widget_modify_font(label, fontlist);
      gtk_tree_view_column_set_clickable(column, TRUE);
      g_object_set_data(G_OBJECT(column), "set_order", GINT_TO_POINTER(1));
      g_object_set_data(G_OBJECT(column), "father", list);
      g_signal_connect(G_OBJECT(column), "clicked", G_CALLBACK(sort_cell_list),
                 GINT_TO_POINTER(i));
  }

  ok_space = OK_button();
  gtk_dialog_add_action_widget (GTK_DIALOG (window1), ok_space, 1);
  gtk_widget_show(ok_space);
  g_signal_connect(G_OBJECT(ok_space), "clicked",
         G_CALLBACK(endCelldialog), GINT_TO_POINTER(1));

  cancel_space = CANCEL_button();
  gtk_dialog_add_action_widget (GTK_DIALOG (window1), cancel_space, 2);
  gtk_widget_show(cancel_space);
  g_signal_connect(G_OBJECT(cancel_space), "clicked",
         G_CALLBACK(endCelldialog), GINT_TO_POINTER(2));
  g_signal_connect_swapped(G_OBJECT(window1),"delete_event",
     G_CALLBACK(gtk_button_clicked),G_OBJECT(cancel_space));

  GtkWidget *button = myimage_button("  _Export", "save.png");
  gtk_dialog_add_action_widget (GTK_DIALOG (window1), button, 3);
  gtk_widget_show(button);
  g_signal_connect(G_OBJECT(button), "clicked",
         G_CALLBACK(export_cells), window1);

  g_signal_connect(G_OBJECT(gtk_tree_view_get_selection(GTK_TREE_VIEW(list))),
         "changed", G_CALLBACK(onSelectCell), list);
  gtk_button_box_set_layout(GTK_BUTTON_BOX(gtk_widget_get_parent(ok_space)),
         GTK_BUTTONBOX_SPREAD);
  return window1;

}

extern "C" void open_cell_dialog_(int *ibest, cell_info_type cell_info[],
                                  int *icell)
{
  int nlines = *ibest;

  fine_elaborazione();
  GtkWidget *cell_dialog = createCelldialog();
  GtkWidget *list = get_widget(cell_dialog, "lista_celle");
  clear_list(list);
  GtkTreeIter iter;
  GtkListStore *list_store = GTK_LIST_STORE(gtk_tree_view_get_model(GTK_TREE_VIEW(list)));
  GtkWidget *scrollwin = get_widget(GTK_WIDGET(list), "scrollwin");
  int dim = sizeof(cell_info[0].syminfo);
  for (int i=0;i<nlines;i++)
  {
       char *label = g_new0(char, dim + 1);
       CopiaStringan(label, dim + 1, (const char *)cell_info[i].syminfo, dim);
       label[dim] = '\0';
       gtk_list_store_append (list_store, &iter);
       gtk_list_store_set (list_store, &iter,
           0, i+1, 1, progs[cell_info[i].pcode-1],
           2, cell_info[i].par[0], 3, cell_info[i].par[1],
           4, cell_info[i].par[2], 5, cell_info[i].par[3], 
           6, cell_info[i].par[4], 7, cell_info[i].par[5], 
           8, cell_info[i].vol, 9, cell_info[i].m20,
          10, cell_info[i].alt_fom, 11, cell_info[i].mc20,
          12, cell_info[i].zshift, 
          13, cell_info[i].nund, 14, g_strstrip(label), -1);
       g_free(label);
  }
  gtk_widget_set_size_request(scrollwin, 600, 250);
  gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(scrollwin), 
      GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
  select_row(get_widget(cell_dialog, "lista_celle"), 0);

  gtk_window_set_transient_for(GTK_WINDOW(cell_dialog), GTK_WINDOW(finestre[0]));
  gtk_widget_show(cell_dialog);
  gtk_main();
  *icell = cell_selected; 
  if(cancel)
     *icell = 0;
}


