module expo_main
   implicit none 

   type param_options_type
        integer            :: nogui = 0
        integer            :: auto = 0
        integer            :: indexing = 0
        real               :: wavel = 0.0
   end type param_options_type

contains                    

   subroutine mainloop(file1,ier,param_opt)
   USE ParametriMappa
   USE General
   USE Molcom
   USE PhaseModule
   USE fileutil
   USE filereading
   USE ilmviewmod
   USE importexport
   USE fileutil
   USE comandiexpo
   USE progproject
   USE variables, only:lconn,atm
   USE strutil
   USE indexing
   USE gen_frm
   USE errormod
   USE variables, only: cryst
   character(len=*), intent(in)         :: file1
   integer, intent(out)                 :: ier
   type(param_options_type), intent(in) :: param_opt
   integer                              :: i, newf, iEnd
   integer                              :: active_file
   integer, dimension(3)                :: indx
   character(len=3)                     :: sval
   real                                 :: wavelength
   type(error_type)                     :: err
!
   ier = 0
   active_file = POW_FILE
   if (ifFirstTime.eq.1.and.jscreen.ne.0) then
       ifFirstTime = 0
       do i=1,100
          ListaOK(i) = 0
       enddo
   endif
!
!  check if is a structure file
   if (code_fragfilen(file1) > 0) then
       StructureName = file_rem_ext(file_get_name(file1))
       call cryst(1)%set_name(StructureName)
       if (jscreen > 0) then
           call wmenu_set_state('Info'//char(0),1)
           call ilmview_open(file1)
           call jav_import_structure(trim(file1)//char(0),0,1)
           call jav_set_name(file_get_name(file1), 1)
                     !call export_file("pippo.cif",ier)
       else
           call import_structure(file1,0,atm,lconn,celem,cell,spacegr,.false.,.false.,err)
           if (err%signal) ier = 1
       endif
       return
   endif
!                                     
!  check if is a powder pattern file
   if (filetype_from_filename(file1) /= 0) then
!
!      Check for wavelength
       if (param_opt%wavel <= 0.0 .and. param_opt%indexing > 0) then
           wavelength = DEF_WAVE        
       else
           wavelength = param_opt%wavel
       endif
       
       call open_pattern(file1,wavelength,ier)
!
!      check for indexing
       if (param_opt%indexing > 0) then
           indx(:) = 0
           write(sval,'(i3.3)')param_opt%indexing
           if (sval(1:1) == '1') indx(DICVOL06_PROG) = 1
           if (sval(2:2) == '1') indx(MCMAILLE_PROG) = 1
           if (sval(3:3) == '1') indx(NTREOR09_PROG) = 1
           call data_for_index()
           call finish_index(indx_set, indx)
       endif
       return
   endif
!
!  check if is a expo project file
   if (s_eqi(get_extension(file1),PRJEXT)) then
       ier = readproject(file1)
       return
   endif
!
   call spoolr(active_file,newf,param_opt,ier,file1)
!
   ipdir = 0
   if (ListaOK(6).eq.1) then
       kuser3 = 3 * ( np(1) + 2 ) * ( np(3) + 2 )
   endif
!
   if (ier.eq.0) then
       call scr2()
       call gescom(active_file,iEnd,ier)
   endif
   if (ier /= 0) then
       iEnd = 1
   endif
   if (iEnd.eq.0) then
       if (ier.eq.99) then
           ier = 0
       endif
   endif
!
   return
   end subroutine mainloop

!-----------------------------------------------------------------------

   subroutine CheckPath(ier)
   USE General, only: exepath,iasun,lo,package_name,ERR_WINDOW
   USE Molcom, only: kscreen
   USE fileutil
   USE spginfom
   USE strutil, only: lower
   USE iso_fortran_env, only: ERROR_UNIT
!
!-- add path to data files and check if they exist
!
   integer, intent(out)                    :: ier
   integer                                 :: i, nExitCode, Missing
   integer, parameter                      :: NCFGFILE = 1    ! Change this number if add/remove cfgfiles
   character(len=256), dimension(NCFGFILE) :: cfgfile
!corr   character(len=len_trim(package_name))   :: progname
!
   ier = 0
!corr!
!corr!-- add extensions to the package name
!corr!
!corr   progname = lower(package_name)
!corr   iasun  = progname //'.spg'
!corr!
!corr!-- add path to data files
!corr!-- exepath path to "sir2002.exe directory" //  "sirdir"
!corr!                    directory where read-only files are stored
!corr!-- cfgpath path to .cfg & .pal files
!corr!                    directory where read/write files are stored
!corr!
!corr   if (len_trim(exepath) > 0) then
!corr       iasun  = trim(exepath) // trim(iasun)
!corr   endif
   iasun=trim(exepath)//lower(package_name)//'.spg'
!
   cfgfile = (/iasun/)
!
!  Check the existence of configuration files
   Missing = 0
   do i=1,NCFGFILE
      if (.not.file_exist(cfgfile(i))) then
          if (Missing == 0) then
             write(ERROR_UNIT,'(//," ***** ERROR *****",/," Missing files:")')
          endif
          Missing = Missing + 1
          write(ERROR_UNIT,'(a)') trim(cfgfile(i))
          if (kscreen == 1) then      
              call MsgWinErr('ERROR', ' File '//trim(cfgfile(i))//' does not exist.', ERR_WINDOW, nExitCode)
          else
              write(lo,'(//a)')'  *** WARNING ***  File '//trim(cfgfile(i))//' does not exist.'
          endif
      endif
   enddo
!
   if (Missing > 0) then
       ier = -2
   !else
   !    call spg_set_symmetry_file(syminfo)  
   endif
!
   return
   end subroutine CheckPath

!-----------------------------------------------------------------------

   subroutine spoolr(active_file,newf,param_opt,ier,file1)
   USE General
   USE Molcom
   USE exportfiles
   USE commandsmod
   USE RandomStaff,only:ichoice,ipdirfigure,numtrialrando,i_ordino_DM
   USE strutil,only:upper,write_title
   USE messagemod
   USE fileutil
   USE elements
   USE errormod
   USE variables, only: cryst
   USE crystal_phase

   integer, intent(out)                   :: active_file
   type(param_options_type), intent(in)   :: param_opt
   integer, intent(out)                   :: ier
   character(len=*), intent(in), optional :: file1
   common /newPatt/ ifPiero
   integer :: ifPiero
   common /labellare/distpp,distpl,natp,natp1(2),ilab,speclab,ialter
   character*8 speclab(2)
   real :: distpp,distpl
   integer :: natp,natp1,ilab,ialter
   common /directo/ kdirect,kdirectsav,jpol,ized,ifou
   integer :: kdirect, kdirectsav, jpol, ized, ifou
   common /comdir/ icomq(200,2),maxcom,ipcom,ipdir,icomat
   integer :: icomq,maxcom,ipcom,ipdir,icomat
   integer :: nfgg,jfour,nExitCode,newf
   integer :: irefl
   integer :: i,j,kusef,nallc,init
   integer :: ipcmd,ipcont
   integer, dimension(7) :: vcont = [EXTRA_CMD,NORM_CMD,INVAR_CMD,PHASE_CMD,FOUR_CMD,MENU_CMD,END_CMD]
   character(len=:), allocatable :: strcomm
   logical :: applycont
   type(error_type) :: errf
   integer :: posall
!
   active_file = POW_FILE
   ifPiero = 0
   ipdirfigure=0
   ichoice=0
   i_ordino_DM=0 
   nexportf = 0
   ReflFilePath = ' '
   nfgg = 0
   card(:) = ' '
   jfour=0
!
   if (jscreen == 1) then
       call AbilitaPaned(1)
       call init_messages()
   endif
   kusef=ln
   irefl=0
   ier=0
   init = 1
   nallc=size(allcom)
   ipcom=0
   ipdir=0
   numtrialrando=20
   icomq(:,:)=0
   ipdir  = 0
   icomat = 0
!
!  Read all commands and directives from input file
   if (present(file1)) then
       call read_commands_from_file(file1,commands,errf)
       if (errf%warning) call errf%print()
       if (errf%signal) then
           call errf%print()
           go to 9000
       endif
   endif
!
!  Set jobname
   strcomm = get_string_command(commands,'job',ipcmd)
   if (len_trim(strcomm) > 0) jobname = strcomm
!
!  Directives 'refl' and 'ref2'
   strcomm = get_string_directive(commands,'refl')
   if (len_trim(strcomm) == 0) then
       strcomm = get_string_directive(commands,'ref2')
   endif
   if (len_trim(strcomm) > 0) then
       irefl=1
       active_file = REFL_FILE
       fname(HKL_FILE)=strcomm
   endif
!
!  Set structurename
   strcomm = get_string_command(commands,'structure',ipcmd)
   if (len_trim(strcomm) == 0) then
       if (present(file1)) then
           strcomm = file_rem_ext(file_get_name(file1))
       endif
   endif
   if (len_trim(strcomm) == 0) then
       if (active_file == POW_FILE) then
           strcomm = get_string_directive(commands,'pattern')
       elseif (active_file == REFL_FILE) then
           strcomm = get_string_directive(commands,'ref')
       endif
       if (len_trim(strcomm) > 0) strcomm = file_rem_ext(file_get_name(strcomm))
   endif
   if (len_trim(strcomm) > 0) then
       do i=1,size(fname)
!
!         coranna per non cambiare nome al file di inp e out
          if (i==INP_FILE .or. i==OUT_FILE) cycle
          fname(i)=strcomm//ext(i)
       enddo
       StructureName = strcomm
       call cryst(1)%set_name(StructureName)
       call cryst(1)%set_name(strcomm)
   else
       if (ipcmd > 0) call MsgWinErr('Warning','No structure name supplied',WARN_WINDOW,nExitCode)
   endif

!
!  Manage all occurencies of command alltrials 
   posall = 1
   do 
      ipcont = command_position(commands,'alltrials',pos=posall)
      if (ipcont /= 0) then
          !ipcmd = command_position_vet(commands(posall:ipcont),vcont(1:5))
          ipcmd = command_position_vet(commands(posall:ipcont),vcont(1:4))
          call add_command(commands,ipcont-1,vcont(ipcmd+1:4))
          !call print_commands(commands,70)
          posall = ipcont + size(vcont(ipcmd+1:4)) + 1 ! = command_position(commands,'alltrials')+1
      else
          exit
      endif
   enddo
!
!  Manage command continue
   ipcont = command_position(commands,'continue')
   if (ipcont /= 0) then
       applycont = .false.
       ipcmd = 0
!
!      Apply continue sequence if prevous command is data or indexing or command in array vcont
       loop_cmd: do i=ipcont-1,1,-1
          if (commands(i)%code == SAVE_PRJ .or. commands(i)%code == EXPORT_CMD) cycle 
          do j=1,size(vcont)
             if (commands(i)%code == DATA_CMD .or. commands(i)%code == NTREOR_CMD .or.  &
                 commands(i)%code == DICVOL_CMD .or. commands(i)%code == MCMAILLE_CMD) then
                 applycont = .true.
                 exit loop_cmd
             elseif (commands(i)%code == PATTERSON_CMD) then
                 applycont = .true.
                 ipcmd = 2
                 exit loop_cmd
             elseif (commands(i)%code == vcont(j)) then
                 applycont = .true.
                 ipcmd = j
                 exit loop_cmd
             endif
          enddo
          exit loop_cmd
       enddo loop_cmd
       if (applycont) then
           !ipcmd = command_position_vet(commands(1:ipcont),vcont)   !!!!TODO: ipcont-1
           !write(0,*)'IPCOM=',ipcont,ipcmd
           if (active_file == REFL_FILE .and. ipcmd == 0) ipcmd = ipcmd+1 ! exclude extraction
           call replace_command(commands,ipcont,vcont(ipcmd+1:))
       else
           call replace_command(commands,ipcont,[END_CMD])
       endif
!!!TODO: %save after %continue
   endif
!   
!  Manage some variables and errors
   do i=1,ncommands(commands)
      select case(commands(i)%code)
         case (AUTO_CMD)
           ifAutomatic = 1

         case (INIT_CMD)
           init = 1

         case (NOWINDOW_CMD)
           if (jscreen.eq.1) then
               call GoOnWithoutGraphics()
               kscreen = 1
               call text_mode(0)
               kscreen = 0
           endif

         case (WINDOW_CMD)
           if (param_opt%nogui == 0) kscreen=1 ! Don't enable GUI, if disabled by command line (nogui=1)

         case (LOAD_PRJ,CALCXPD_CMD,FRAG_CMD,CRYSTAL_CMD) ! command with directives, followed by key
           if (commands(i)%ndir == 0) then
               call MsgWinErr('Warning','Error on reading command %'    &
               //commands(i)%get_name()//'.'//char(10)//                &
               'Expected a file name or key.',WARN_WINDOW,nExitCode)
               call commands(i)%set_as_deleted()   ! (*)
           endif

         case (NTREOR_CMD, DICVOL_CMD, MCMAILLE_CMD)   ! ntreor
           if (command_is_before(commands,DATA_CMD,i) == 0 .and. command_is_before(commands,LOAD_PRJ,i) == 0) then
               call MsgWinErr('WARNING','Check the input file.'//char(10)//'%'//commands(i)%get_name()//  &
                ' has been ignored.'//char(10)//'%'//commands(i)%get_name()//' must follow %data.',           &
               WARN_WINDOW,nExitCode)
               call commands(i)%set_as_deleted()   ! (*)
           endif

         case (PATTERSON_CMD)
           if (command_is_before(commands,NORM_CMD,i) == 0) then
               call MsgWinErr('Error','%patterson command should be at most after %normal command' &
               ,ERR_WINDOW,nExitCode)
               go to 9000
           endif

         case (POLY_CMD); jpol = 1
         case (CLAB_CMD); ilab = 1
         case (ZMAT_CMD); ized = 1
         case (FOUR_CMD); ifou = 1
      end select
   enddo
!
   call delete_commands(commands)  ! (*) process set_as_deleted
   call delete_commands_code(commands,STRUCT_CMD)
   call delete_commands_code(commands,AUTO_CMD)
   call delete_commands_code(commands,JOB_CMD)
   call delete_commands_code(commands,WINDOW_CMD)
   call delete_commands_code(commands,NOWINDOW_CMD)
   call delete_commands_code(commands,INIT_CMD)
   call create_expo_commands(commands,ipcom,icomq,card)
!
   call SucchiaDati(init,newf,ier)
   if (ier.lt.0) return
   call write_title(lo,(/jobname/))
   write(lo,'(/)')
   call wrhead()
   call print_commands(commands,lo)
   !call print_commands(commands,0)
   call write_file_info(active_file)
   write(lo,'(/)')
   if (newf.eq.1) then
       write(lo,330)
  330 format(6x,'Direct access structure file has been initialized'/)
   else
       write(lo,340)
       call summar
  340 format(6x,'Direct access structure file actually contains:'/)
   endif
!
   return
9000 ier=-1
!
   return
   end

!-----------------------------------------------------------------------

   subroutine write_file_info(active_file)
   USE General
   USE commandsmod
   USE elements
   integer, intent(in) :: active_file
!
   write(lo,290) ln,lo,jdir,trim(fname(BIN_FILE)),iscra,jrel
  290 format(//,' Files used in  expo :',//,                         &
           '  Channel                                  name',/,      &
                  i6,'    card reader',/,                            &
                  i6,'    line printer',/,                           &
                  i6,'    direct access structure file     ',a,/,    &
                  i6,'    direct access scratch file',/,             &
                  i6,'    sequential scratch file ')              

!corr   if (irefl /= 0) then
   if (active_file == REFL_FILE) then
       write(lo,300) jfile,trim(fname(HKL_FILE))
  300 format(     i6,'    reflection input file            ',a)      
   endif

   if (is_directive_in_command(commands,'data','content')) then
       write(lo,310) kform, filexen_name(exepath)//' (Release:'//filexen_release()//')'
  310  format(     i6,'    scattering factors file          ',a,/)  
   endif
!corr   if (jfour.eq.1)  then
!corr       write(lo,320) ifour
!corr  320 format(     i6,'    sequential scratch file ',//)
!corr   endif
!
   end subroutine write_file_info

end module expo_main

   subroutine expo2005(param_opt)
   USE General
   USE Molcom
   USE PatternRef
   USE strutil
   USE expo_main
   USE timer
   USE gen_frm
   USE errormod
#ifdef MPI
   USE mpi_prog
#endif
   implicit none
   character(len=256)       :: file1,file2,nome
   character(len=80)        :: line
   type(param_options_type) :: param_opt
   integer                  :: i,k
   integer                  :: ier 
   integer                  :: nkscrx,nkscry
   integer                  :: iz, ind, icol0
   type(timer_type)         :: timeprog
   type(error_type)         :: err
#ifdef MPI
   integer :: mpi_err
#endif
!
#ifdef MPI
   call mpi_prog_init()
   if (mpi_prog_err%signal) then
       call mpi_prog_err%print()
   endif
   if (mpi_prog_numprocs > 1) param_opt%nogui = 1 ! disable GUI
   call read_config()
#else
   call leggiconfig
#endif
   call PrimeCose(ier)
   if (ier /= 0) then
       call cstop('Program cannot start')
   endif
   call InitExpo2002(0)
   ifAutomatic = param_opt%auto   !Set automatic mode by command line
!
   call timeprog%set_name(package_alt_name)
   call timeprog%set_as_elapsed()
   call timeprog%tic()
!
   call BeccaSt(ier,file1,file2,nome) 
   if (ier /= 0) go to 10
!
!-- set up the default names for work files
!
   k = Lung(nome)
   if (k > 0) then
       do i=1,size(fname)
          fname(i) = s_blank_delete(trim(nome)//ext(i))
       enddo
       if (len_trim(file1) > 0) then
           fname(INP_FILE)=file1
           call CheckStructCommand(file1,ier)
           call updRecentProjects()                          !mr
       endif
       if (len_trim(file2) > 0) then
           fname(OUT_FILE)=file2
       endif
   else
       line = 'New Project'
   endif
!
   call CheckPath(ier)
   if (ier.lt.0) go to 10
!
   call load_chemical_tables(exepath,err)
   if (err%signal) then
       ier = 1
       go to 10
   endif
!
!-- check if %window is supplied in *.sir file
!
   if (file1(1:1).ne.' '.and.file1(1:7).ne.'newproj') then
       call CheckWindowCommand(file1,nkscrx,nkscry,line,ier)
       if (ier.lt.0) go to 10
   endif
!
   if (param_opt%nogui == 1) kscreen = 0 !Disable GUI if required by command line
!
   if (kscreen.ne.0) then
       call scr0()
       if (file1(1:1).ne.' ') then   ! modifica il titolo della window
           call SetWindowTitle (line)
       endif
   else
#ifdef MPI
       if (mpi_prog_rank == mpi_prog_master) then
#endif
           jscreen = 0
           write(0,*)
           write(0,*) ' '//package_alt_name//' will go on without graphics.  '
           write(0,*) ' Results will be written in the file   '
           write(0,*) '         ',fname(OUT_FILE)(1:Lung(fname(OUT_FILE)))
           write(0,*)
#ifdef MPI
       endif
#endif
   endif
!
   if (file1(1:1).ne.' ') then
       iz = 0
       ind = 998
       icol0 = 0
       call graphs(iz, ind, ' ', icol0, icol0)
       call mainloop(file1,ier,param_opt)
       if (ier.lt.0) go to 10
   endif
   if (jscreen.ne.0) then
       call StartMainLoop(ifAutomatic)
   endif
#ifdef MPI
   call MPI_Finalize(mpi_err)
   return
#endif
!--
10 if (ier /= 0) then
       write(line,'(1x,a)')package_alt_name//' ends in error'
   else
       write(line,'(1x,a)')package_alt_name//' ends ok'
   endif
!
   if(ier.ne.-2) call scrend(line)
!
   close (jdir)
   if (jscreen.eq.0.or.ifAutomatic.eq.1) then
       if (ier.lt.0) then
           write(0,'(3x,a,a)') trim(line),': check output file or expo.log !!!'
       else
           write(0,'(3x,a)') trim(line)
       endif
   endif
!
   call timeprog%toc()
   call timeprog%tprint(lo)
   write(lo,'(/1x,a)') trim(line)
!
   close (lo)
   call text_mode(1)
   end

!-----------------------------------------------------------------------

#ifdef MPI
   subroutine read_config()
   USE mpi
   USE mpi_prog
   implicit none
   integer :: mpi_err  !, np
   integer :: file_free = 0
   integer :: status(MPI_STATUS_SIZE)
!   
   if (mpi_prog_rank == mpi_prog_master) then
       file_free = 1
   else
       call MPI_Recv(file_free,1,MPI_INTEGER, mpi_prog_rank-1,1,MPI_COMM_WORLD,status,mpi_err)
   endif
   if (file_free == 1) call leggiconfig
   if (mpi_prog_rank /= mpi_prog_numprocs-1) call MPI_Send(file_free,1,MPI_INTEGER,mpi_prog_rank+1,1,MPI_COMM_WORLD,mpi_err)
!
   end subroutine read_config
#endif
