MODULE proginterface

!
! Questo modulo contiene funzioni di interfaccia tra vecchia gestione e nuova gestione
!

!
!                 Routine di interfaccia per la gestione del contenuto
!S nk_to_elem(elem)                      Trasferisco in elem le informazioni sul contenuto
!S elem_to_nk(elem)                      Prendo le informazioni sul contenuto da elem
!S fillcounts()                          Prendo informazioni sui conteggi da expo
!
implicit none

CONTAINS

!corr   subroutine nk_to_elem(elem)
!corr!
!corr!  Trasferisco in elem le informazioni sul contenuto
!corr!
!corr   USE elements
!corr   USE general
!corr   type(element_type), dimension(:), intent(out), allocatable :: elem
!corr   integer                                       :: i
!corr!
!corr   call reallocate_elem(elem,nk)
!corr   do i=1,nk
!corr!corr      elem(i)%z = no(i)
!corr      select case (neutro)
!corr         case (0)
!corr          elem(i)%ax(:) = al(:,i)
!corr          elem(i)%bx(:) = bs(:,i)
!corr          elem(i)%cx = cl(i)
!corr         case (1)
!corr          elem(i)%fact = cl(i)
!corr      end select
!corr!corr      elem(i)%zeff = zeff(i)
!corr!corr      elem(i)%nw = nw(i)
!corr      elem(i)%radtype = neutro
!corr      elem(i)%ptab = pxen_from_specie(elem(i)%lab)
!corr      elem(i)%name = eleminfo(elem(i)%ptab)%name
!corr   enddo
!corr   call elem_set_scatt(elem)
!corr!
!corr   end subroutine nk_to_elem
!corr
!---------------------------------------------------------------------      
!corr
!corr   subroutine elem_to_nk(elem,norm)
!corr!
!corr!  Prendo le informazioni sul contenuto da elem
!corr!
!corr   USE general
!corr   USE elements
!corr   USE strutil
!corr   USE variables, only: cryst
!corr   USE crystal_phase
!corr   type(element_type), dimension(:), allocatable, intent(in) :: elem
!corr   logical, intent(in), optional                :: norm  
!corr   integer                                      :: i, nel
!corr   character(len=4)                             :: spec
!
!corr   nel = numelem(elem)
!corr   if (nel == 0) return
!corr   nk = min(nel,NMAXSPECIE)
!corr   do i=1,nk
!corr   do i=1,numelem(elem)
!corr      no(i) = elem(i)%z
!corr              spec = lower(elem(i)%lab(1:4))
!corr              read(spec,'(a4)')nalf(i)
!corr      select case (neutro)
!corr         case (0)   ! Raggi X
!corr           al(:,i) = elem(i)%ax
!corr           bs(:,i) = elem(i)%bx
!corr           cl(i) = elem(i)%cx 
!corr         case (1)   ! Neutroni
!corr           al(:,i) = 0
!corr           bs(:,i) = 0
!corr           cl(i) = elem(i)%fact 
!corr         case (2)   ! Elettroni  -- da controllare -> Gianluca
!corr           al(:,i) = elem(i)%ae
!corr           bs(:,i) = elem(i)%be
!corr           cl(i) = elem(i)%ce 
!corr      end select
!corr      zeff(i) = elem(i)%zeff
!corr      nw(i) = nint(elem(i)%nw)
!corr   enddo
!
!  norm=.true. e' necessario se nalf viene passato alla yfrm03
!corr   if (present(norm)) then
!corr       if (norm) then
!corr           do i=1,numelem(celem)
!corr              spec = lower(celem(i)%lab(1:4))
!corr              read(spec,'(a4)')nalf(i)
!corr           enddo
!corr       endif
!corr   endif
!corr!
!corr   call cryst(1)%set_scattering(elem)
!corr!
!corr   end subroutine elem_to_nk

!-------------------------------------------------------------------

   subroutine fillcounts()
!
!  Prendo informazioni sui conteggi da expo
!
   USE Conteggi, only: npunti
!corr   USE counts
!corr   USE progtype, only:ALLOCA
   USE patternref, only:thmin,thmax
   USE variables, only: dataset
!corr   integer :: i
!
!corr   call alloca_counts(ALLOCA,npunti)
!corr   do i=1,npunti
!corr      ttetacount(i) = theta_int(i,1)
!corr      ycountoss(i) = theta_int(i,2)
!corr      ttetaczero(i) = dataset(1)%x(i)
!corr      backgr(i) = back_int(i,1)
!corr      ycounttot(i) = theta_int(i,3)
!corr   enddo
   thmin = dataset(1)%x0(1)
   thmax = dataset(1)%x0(npunti)
!
   end subroutine fillcounts

!-------------------------------------------------------------------

   subroutine fillref(refl)
!
!  Scarica refl nelle variabili expo
!
   USE General  , only : nrefl,dtor,alambda
   USE Riflessi, only : Mult,IndHKL,fxsav,WHM,SetFob,TThet,iphaz   !!!!,ipub
   USE xdataMod, only : clp
   USE reflection_type_util
!
   implicit none
   type(reflection_type), dimension(:), allocatable, intent(inout) :: refl
   integer                                                         :: i
!
   call new_reflections(refl,nrefl)
   do i=1,nrefl
      refl(i)%hkl = IndHKL(i,1:3)                           ! indici h,k,l
      refl(i)%m   = Mult(i)                                 ! molteplicita'
      refl(i)%tthd = TThet(i)                               ! 2-theta in gradi
      refl(i)%slaq = (sin(0.5*TThet(i)*dtor)/alambda)**2    ! (sen(theta)/lambda)**2
      refl(i)%fo  = sqrt(SetFob(i,4))                       ! fattori di struttura osservati
      refl(i)%fwhm = WHM(i)                                 ! FWHM
      refl(i)%lp = clp(i)                                   ! LP
      refl(i)%ph = iphaz(i)                                 ! fasi calcolate
      refl(i)%fv   = fxsav(i)                               ! fattori di struttura veri
      !refl(i)%phv = ipub(i)                                ! fasi vere
      refl(i)%phv = IndHKL(i,5)/32                           ! fasi vere
   enddo
!
   end subroutine fillref

!---------------------------------------------------------------------------------------

   subroutine ref_to_expo(refl)
   USE General  , only : nrefl
   USE Riflessi, only : Mult,IndHKL,fxsav,WHM,SetFob,TThet,ipub,iphaz
   USE xdataMod, only : clp
   USE reflection_type_util
   type(reflection_type), dimension(:), allocatable, intent(in) :: refl
   integer                                                      :: i
   nrefl = numrefl(refl)
   call AllocaRiflessi(2)
   if (nrefl == 0) return
   call AllocaRiflessi(1)
   do i=1,nrefl
      IndHKL(i,1:3) = refl(i)%hkl            ! indici h,k,l
      Mult(i)  = refl(i)%m                   ! molteplicit�
      TThet(i) = refl(i)%tthd                ! 2-theta in gradi
      SetFob(i,4) = refl(i)%fo*refl(i)%fo    ! fattori di struttura osservati
      WHM(i)  = refl(i)%fwhm                 ! FWHM
      clp(i) = refl(i)%lp                    ! LP
      iphaz(i) = refl(i)%ph                  ! fasi calcolate
      fxsav(i) = refl(i)%fv                  ! fattori di struttura veri
      ipub(i) = refl(i)%phv                  ! fasi vere
   enddo
   end subroutine ref_to_expo

!-------------------------------------------------------------------

   subroutine write_phase_error(ref)
   USE reflection_type_util
   USE General, only: lo
   type(reflection_type), dimension(:), intent(in) :: ref
   integer                                         :: nrefex
   integer                                         :: i
   nrefex = size(ref)
   write(lo,'(" Phase error:")')
   call examin(1,(/(ref(i)%hkl,i=1,nrefex)/),ref%phv,ref%ph,nrefex,0,(/(1.0,i=1,nrefex)/),1)
   end subroutine write_phase_error

!-------------------------------------------------------------------
!corr
!corr   subroutine generate_reflections(tthmin,tthmax,lambda)  ! ,err)
!corr   USE patternref, only:thmin,thmax
!corr   USE General, only:alambda
!corr   USE Conteggi, only:npunti
!corr   real, intent(in), optional :: tthmin,tthmax
!corr   real, intent(in), optional :: lambda
!corr   integer                    :: iflag
!corr!
!corr   if (present(tthmin) .and. present(tthmax)) then
!corr       thmin = tthmin
!corr       thmax = tthmax
!corr   else
!corr       if (npunti == 0) then
!corr           thmin=1; thmax=80   !!!!40;
!corr       endif
!corr   endif
!corr!
!corr   if (present(lambda)) then
!corr       alambda = lambda
!corr   else
!corr       if (alambda == 0.0) alambda = 1.54056
!corr   endif
!corr!
!corr   call gener(iflag,0)
!corr!
!corr   end subroutine generate_reflections
!corr
END MODULE proginterface
