MODULE importexport

implicit none

CONTAINS

   subroutine import_structure(filename,ftype,atom,legm,elem,cell,spg,gui,has_symmetry,err,at_info,nsec,sec_struct)
   USE atom_basic
   USE pdb_frm
   USE general, only: nrefl,neutro,structurename
   USE Patternref, only:thmin,thmax
   USE enable_amb
   USE view
   USE spginfom
   USE sinterface
   USE atom_type_util
   USE gen_frm
   USE connect_mod
   USE errormod
   USE unit_cell
   USE errormod
   USE constraints
   USE prog_constants
   USE elements
   !USE ochemdb
   character(len=*), intent(in)                                               :: filename
   integer, intent(in)                                                        :: ftype ! if 0 the file type is set from filename
   type(atom_type), dimension(:), allocatable, intent(inout)                  :: atom
   type(bond_type), dimension(:), allocatable, intent(inout)                  :: legm
   real, dimension(6), intent(inout)                                          :: cell
   type(element_type), dimension(:), allocatable, intent(inout)               :: elem
   type(spaceg_type), intent(inout)                                           :: spg
   logical, intent(in)                                                        :: gui
   logical, intent(in)                                                        :: has_symmetry
   type(error_type)                                                           :: err
   type(pdb_atom_info_type), dimension(:), allocatable, optional, intent(out) :: at_info   ! additional info for pdb file
   integer, intent(out), optional :: nsec
   type(secondary_struct_type), dimension(:), allocatable, optional, intent(out) :: sec_struct ! secondary structure from pdb
   type(cell_type) :: cellt
!
   cellt = set_cell_type(cell)
   call import_crystal(filename,ftype,atom,legm,elem,cellt,spg,has_symmetry,neutro,gui,err,at_info,nsec,sec_struct)
   if (.not.err%signal) then
       call action_after_import(filename,has_symmetry,elem,cellt,spg)
       !if (.not.err%warning) call export_for_ochemdb(atom,legm,elem,cellt,spg,structurename)
                  !call make_test(atom,legm,elem,cellt)
   endif
   end subroutine import_structure
 
!---------------------------------------------------------------------      

   subroutine action_after_import(filename,has_symmetry,elem,cellv,spg)
   USE Conteggi, only:npunti
   USE elements
   USE spginfom
   USE gen_frm
   USE general, only: nrefl, alambda, neutro, cell
   USE sinterface
   USE enable_amb
   USE view
   USE variables, only: cryst
   USE unit_cell
   character(len=*), intent(in)                              :: filename
   logical, intent(in)                                       :: has_symmetry
   type(element_type), dimension(:), allocatable, intent(in) :: elem
   type(cell_type), intent(in)                               :: cellv
   type(spaceg_type), intent(in)                             :: spg
   integer                                                   :: filetype
   integer                                                   :: iflag
!
   if (.not.has_symmetry) then
        cell(:6) = cellv%get_par()
        call set_spaceg_new(spg) 
        call cryst(1)%set_symmetry(spg,cellv)
   endif
   call cryst(1)%set_scattering(elem)
   if (npunti > 0) call abilita_tasti('structure',1)
!
!  Generate reflections for cif and pdb files
   filetype = code_fragfilen(filename)
   if (npunti > 0 .and. nrefl == 0 .and. (filetype == CIF_FILE .or. filetype == PDB_FILE)) then
!corr       if (alambda == 0) alambda = 1.54056
       call gener(iflag,0)
       !call generate_reflections(thmin,thmax,alambda)
       call vedinew(1)
   endif
!
!  save distances for database in a file
   !if (.not.err%warning) call export_for_db(atom,legm,set_cell_type(cell),spacegr,elem,structurename)
   end subroutine action_after_import

!---------------------------------------------------------------------      

   subroutine make_test(atom,legm,elem,cell)
   USE atom_type_util
   USE connect_mod
   USE elements
   USE pdfcalc
!corr   USE contacts
!corr   USE molpnew
!corr   USE hydrogens
!corr   USE polyhedra
!corr   USE centroids
   USE qe_frm
   USE unit_cell
   USE General, only:spacegr,celem,neutro,structurename
   USE symmgrid
   USE crystal_phase
   USE variables, only: cryst
   !USE testgrid
!corr   USE spginfom
!corr   USE crystal_frm
!corr   USE bond_valence
!
   type(atom_type), dimension(:), allocatable, intent(inout) :: atom
   type(bond_type), dimension(:), allocatable, intent(inout) :: legm
   type(element_type), dimension(:), allocatable, intent(in) :: elem
   type(cell_type), intent(in)                               :: cell
   type(crystal_phase_t) :: crystal
!  test bond valence
!bv   type(bond_valence_t), dimension(size(elem),size(elem)) :: bvtab
!bv   real, dimension(:,:,:), allocatable :: asymtab
!bv   real :: val
!bv   integer :: i
   integer :: ndinfo,i,cn
  type(bond_info_t), dimension(:), allocatable :: dinfo
  real, dimension(:), allocatable :: bvs
!corr   integer                                                   :: ier
   type(atom_type), dimension(:), allocatable                :: atoms
   type(bond_type), dimension(:), allocatable                :: legms
   type(bond_type), dimension(:), allocatable                :: legmHview,legmshort
!corr   logical :: hbond,shbond,lpacking,lsymm
   integer, dimension(2) :: kexpand
!corr   type(polyhedron_type), dimension(:), allocatable      :: poly
   integer :: npoly
!corr   type(centroid_type), dimension(:), allocatable :: centr
   integer :: ncentr
!   type(torsion_type), dimension(:), allocatable :: tors
!   logical :: chang
!   type(angle_type), dimension(:), allocatable :: angle
!   integer, dimension(size(atom)) :: veta
!   integer :: nd
!test bv
!
   if (numatoms(atom) == 0) return
!
!  test for qe file
   !call write_qe_file(trim(structurename)//'_qe.in',atom,cell,spacegr,structurename)
!
!  test symm
   crystal = cryst(1)
   call crystal%set_atoms(atom)
   !call test_grid(crystal,atom,legm,cell,spacegr,elem)
   !call detect_spg(atom,cell)
!
!  test for pdf
   !call test_pdf(atom,set_cell_type(cell),spacegr,celem,neutro)
!
!  test bond valence
!c   call crystal%load_bvparam()
!c   !call crystal%print_bvparam(0)
!c   call crystal%make_symmetry_cell()
!c   call crystal%allocate_grid(cutoff=maxval(crystal%bvpar(:,:)%rmax))
!c   call crystal%fill_grid()
!c      write(0,*)'NAT=',crystal%natoms()
!c   allocate(bvs(crystal%natoms()))
!c   do i=1,crystal%natoms()
!c      call crystal%compute_bvs(i,bvs(i))
!c      write(70,*)'VAL=',crystal%at(i)%glab(),bvs(i)
!c      ndinfo = crystal%valence_atom(i)
!c      !write(0,*)'NB_'//crystal%at(i)%glab()//'=',ndinfo
!c      if (ndinfo > 0) then
!c          if (allocated(dinfo)) deallocate(dinfo)
!c          allocate(dinfo(ndinfo))
!c          call crystal%valence_atom_info(i,dinfo,bvs(i),cn)
!c          call print_bond_info(crystal%at,crystal%spg,dinfo,70)
!c          write(70,*)'NC_'//crystal%at(i)%glab()//'=',count(dinfo%dupl == 0),ndinfo
!c      endif
!c   enddo
!
!  test grow frag
   !call apply_symmetry(atom,atoms,legm,legms,infos,level=(/1.0,1.0,1.0/)-0.5,fitopt=2)
   !call grow_fragments(atom,atoms,legm,legms,infos,set_cell_type(cell),lsym=.true.,kexp=(/EXPAND_ALL,0/))
   !call grow_fragments(atom,atoms,legm,legms,infos,set_cell_type(cell),spacegr,lsym=.false.,kexp=(/EXPAND_ALL,0/))
   !call copy_atoms(atom,atoms)
   !call copy_bonds(legm,legms)
!
!  test bond perception
   !call bond_type_perception(atom,legm,cell)
!
!  test hydrogens
   !call remove_atoms_from_list(atom,atom%ptab,H_at,legm=legm)  !remove H
   !call add_hydrogens(atom,legm,celem,set_cell_type(cell),spacegr,ier=ier)
!
!  test change bond distance
   !do i=1,numbonds(legm)
   !   call change_bond_distance(atom,legm,8,4.5, matrice_metrica(cell))
   !   call change_bond_distance(atom,legm,11,4.5, matrice_metrica(cell))
   !enddo
!
!  test for change angle
   !call legm_to_ang(legm,atom,angle)
   !call change_bond_angle(3,angle,atom,legm,170.,set_cell_type(cell))
!
!  test csys
   !write(0,'(1x,a,a,1x,a)')'CRY SYS: ',trim(cry_sys(spacegr%csys_code)), trim(cry_sys(csys_from_cell(cell)))
!
!  test for change torsion
   !call legm_to_tors(legm,atom,tors)
   !call change_torsion_angle(tors(10),atom,legm,150.,set_cell_type(cell),chang)
   !return
!
!  test centroids
   !call find_centroids(atom,legm,centr,ncentr)
!
!  test contacts
   !hbond = .false.; shbond = .true.; lpacking = .false.
   !kexpand = [0,0]
   !call find_contacts(atom,legm,legmHview,legmshort,atoms,legms,lpacking,kexpand,hbond,shbond,set_cell_type(cell),spacegr)
   !call copy_atoms(atom,atoms)
   !call copy_bonds(legm,legms)
!
!  test polyhedra
   !lsymm = .false.
   !call find_polyhedra_sym(atom,legm,set_cell_type(cell),spacegr,atoms,legms,poly,npoly,lsym=lsymm)
!
!  test valence-bond
!
   end subroutine make_test

!----------------------------------------------------------------------------------------------------------------      
!corr
!corr   subroutine export_structure(filename,atom,cell,spg,elem,ftype,sname,radtype,wavel,legm,comm,iscell)
!corr   USE prog_constants
!corr   USE errormod
!corr   USE connect_mod
!corr   USE elements
!corr   USE spginfom
!corr   USE cif_frm
!corr   USE pdb_frm
!corr   USE shelx_frm
!corr   USE OBmodule
!corr   USE unit_cell
!corr   USE atom_type_util
!corr   USE nwchem_frm
!corr   USE gamess_frm
!corr   USE abinit_frm
!corr   USE xyz_frm
!corr   USE zmt_frm
!corr   USE fra_frm
!corr   USE mop_frm
!corr   USE gen_frm
!corr   USE crystal_frm
!corr   USE qe_frm
!corr   USE povray_frm
!corr   character(len=*), intent(in)                                       :: filename
!corr   type(atom_type), dimension(:), intent(in), allocatable             :: atom
!corr   type(cell_type), intent(in)                                        :: cell
!corr   type(spaceg_type), intent(in)                                      :: spg
!corr   type(element_type), dimension(:), allocatable                      :: elem
!corr   integer, intent(in)                                                :: ftype
!corr   character(len=*), intent(in)                                       :: sname
!corr   integer, intent(in)                                                :: radtype
!corr   real, intent(in)                                                   :: wavel
!corr   type(bond_type), dimension(:), allocatable, intent(inout),optional :: legm
!corr   character(len=*), intent(in), optional                             :: comm
!corr   logical, intent(in), optional                                      :: iscell
!corr   type(error_type)                                                   :: err_exp
!corr   real, dimension(:,:), allocatable                                  :: cols
!corr   real, dimension(3)                                                 :: colcell
!corr   real, dimension(2)                                                 :: vfact
!corr   real, dimension(3,3)                                               :: bmat
!corr   integer                                                            :: filetype
!corr   type(atom_type), dimension(:), allocatable                         :: atomc
!corr!
!corr   if (ftype == ALL_FILES) then
!corr       filetype = code_fragfilen(filename)
!corr   else
!corr       filetype = ftype
!corr   endif
!corr!
!corr   select case (filetype)
!corr      case (SHELX_FILE)    ! .res file
!corr        call create_shelxfile(filename,atom,legm,cell,spg,wavel,elem,sname,package_alt_name)
!corr
!corr      case (PDB_FILE)     ! pdb file
!corr        call create_pdbfile(filename,atom,cell,legm,spg)
!corr
!corr      case (FRA_FILE)     ! fra file
!corr        call create_frafile(filename,atom,cell,'Structure '//trim(sname))
!corr
!corr      case (XYZ_FILE)    ! cartesian xyz file
!corr        call create_xyzfile(filename,atom,cell,'Structure '//trim(sname))
!corr
!corr      case (POV_FILE)    ! POV file
!corr        allocate(cols(numelem(elem),3))
!corr        call jav_get_style(cols, colcell, vfact, bmat)
!corr        call create_povrayfile(filename,atom,legm,cell,elem,cols,colcell,iscell,bmat)
!corr#if _WIN32
!corr        call Csystem('C:\PROGRA~1\POV-RA~1.5\bin\pvengine.exe '//'+I'//trim(filename)//char(0))
!corr#else
!corr        call Csystem('povray +P +W800 +H600'//' +I'//trim(filename)//' &'//char(0))
!corr#endif
!corr
!corr      case (CIF_FILE)    ! CIF file
!corr        if (present(comm)) then
!corr            if (present(legm)) then
!corr                call create_ciffile(filename,atom,cell,spg,elem,package_alt_name,    &
!corr                comm,legm=legm,wave=wavel,source_type=radtype)
!corr            else
!corr                call create_ciffile(filename,atom,cell,spg,elem,package_alt_name,    &
!corr                comm,wave=wavel,source_type=radtype)
!corr            endif
!corr        else
!corr            if (present(legm)) then
!corr                call create_ciffile(filename,atom,cell,spg,elem,package_alt_name,    &
!corr                legm=legm,wave=wavel,source_type=radtype)
!corr            else
!corr                call create_ciffile(filename,atom,cell,spg,elem,package_alt_name,    &
!corr                wave=wavel,source_type=radtype)
!corr            endif
!corr        endif
!corr
!corr      case (ZMT_FILE)    ! zmt file (Fenske-Hall Z-Matrix)
!corr        call create_zmtfile(filename,atom,legm,cell)
!corr
!corr      case (MOL_FILE)    ! mol file
!corr        call bond_type_perception(atom,legm,cell)
!corr        call frac_to_cart_copy(atom,atomc,cell%get_ortom())
!corr        call OBConversionWriteFile(filename,atomc,cell,spg,legm)
!corr
!corr      case (NWCHEM_IN_DFT)
!corr        call write_nw_file(filename,atom,cell,spg,sname,nwtheory=1)
!corr
!corr      case (NWCHEM_IN_DFTD)
!corr        call write_nw_file(filename,atom,cell,spg,sname,nwtheory=2)
!corr
!corr      case (ABINIT_IN)
!corr        call write_ab_file(filename,atom,cell,spg)
!corr
!corr      case (CRYSTAL_IN)
!corr        call write_crystal_file(filename,atom,cell,spg,sname)
!corr
!corr      case (GAMESS_IN)
!corr        call write_gmfile(filename,atom,cell,sname)
!corr
!corr      case (GAUSSIAN_CART_IN, GAUSSIAN_ZMAT_IN)
!corr        call frac_to_cart_copy(atom,atomc,cell%get_ortom())
!corr        call OBConversionWriteFile(filename,atomc,cell,spg,legm)
!corr
!corr      case (MOP_FILE)    ! cartesian mop file 
!corr        call create_mopcrtfile(filename,atom,cell,sname)
!corr
!corr      case (QE_IN)    
!corr        call write_qe_file(filename,atom,cell,spg,sname)
!corr
!corr      case default 
!corr        call err_exp%set('Unknown file format. File '//trim(filename))
!corr   end select
!corr!
!corr   if (err_exp%signal) then
!corr       call err_exp%print()
!corr   endif
!corr!
!corr   end subroutine export_structure
!corr
!---------------------------------------------------------------------      

   subroutine export_info(filename,ftype,vet,atom,legm,legmc,adp_type)
   USE connect_mod
   USE fileutil
   USE atom_type_util
   USE general, only:spacegr,cell
   USE unit_cell
   USE progtype, only: structure_info_type
   character(len=*), intent(in)                           :: filename
   type(atom_type), dimension(:), allocatable, intent(in) :: atom
   type(bond_type), dimension(:), allocatable, intent(in) :: legm
   integer, intent(in)                                    :: ftype ! che tipo di informazioni
   integer, dimension(:), intent(in)                      :: vet   ! quali informazioni
   integer, intent(in), optional                          :: adp_type ! adp type: 1 for B, 2 for U
   integer                                                :: j_in
   integer                                                :: ier
   type(angle_type), dimension(:), allocatable            :: angle
   type(torsion_type), dimension(:), allocatable          :: tors
   type(structure_info_type)                              :: info
   type(bond_type), dimension(:), allocatable, intent(in), optional :: legmc
   type(cell_type) :: cellt
!
!  Open file
   call get_unit(j_in)
   open(unit=j_in,file=filename,iostat=ier,status='unknown')
!
   if (ier == 0) then
       select case (ftype)
          case (1)    ! general info
            info = structure_info(atom,1)
            call print_structure_info(info,j_in)

          case (2)    ! atomi
            if (present(adp_type)) then
                call print_atoms_vet(atom,kpr=j_in,vet=vet,adp_type=adp_type,spg=spacegr)
            else
                call print_atoms_vet(atom,kpr=j_in,vet=vet,spg=spacegr)
            endif

          case (3)    ! legami
            call print_connect(atom%lab,bond=legm,kpri=j_in,vet=vet)

          case (4)    ! angoli
            cellt = set_cell_type(cell)
            call legm_to_ang(legm,atom,cellt%get_g(),angle)
            call print_connect(atom%lab,angle=angle,kpri=j_in,vet=vet)

          case (5)    ! torsioni
            call legm_to_tors(legm,atom,cellt,tors)
            call print_connect(atom%lab,tors=tors,kpri=j_in,vet=vet)
        
          case (7)
            call print_connect(atom%lab,bond=legmc,kpri=j_in,vet=vet)

       end select

       close (j_in)
   endif
!
   end subroutine export_info

!---------------------------------------------------------------------      

   function structure_info(atom,icalc) result(info)
   USE General, only:cell,nrefl,spacegr,celem
   USE unit_cell
   USE variables, only:ref
   USE reflection_type_util
   USE atom_type_util
   USE stat
   USE elements
   USE proginterface
   USE progtype, only: structure_info_type
   type(structure_info_type)                              :: info
   type(atom_type), dimension(:), allocatable, intent(in) :: atom
   integer, intent(in)                                    :: icalc
   integer                                                :: i,ks
   real                                                   :: numat
   real, allocatable, dimension(:) :: vat
!
   info%nspec = numelem(celem)
   info%cell = cell(1:6)
   if (numatoms(atom) > 0) then
       allocate(vat(numelem(celem)),source=0.)
!
!      compute cell content from atoms
       info%vspec(:) = 0
       do i=1,numatoms(atom)
          ks = atom(i)%kscatt()
          if(ks == 0) cycle
          numat = atom(i)%och*atom(i)%ocry*spacegr%nsymop
          vat(ks) = vat(ks) + numat
       enddo
       do i=1,info%nspec
          info%zspec(i) = celem(i)%z 
          info%vspec(i) = nint(vat(i))
       enddo
   else
       do i=1,info%nspec
          info%vspec(i) = celem(i)%nw  
          info%zspec(i) = celem(i)%z   
       enddo
   endif
   info%spaceg = trim(spacegr%symbol_xhm)   !!!!trim(spaceg_string())
   if (spacegr%symcent == 1) then
       info%icent = 1
   else
       info%icent = 0
   endif
   info%nsymmstr = spacegr%nsymop
!
!  Se icalc = 0 non vengono riempite quelle informazioni che richiedono un calcolo
   if (icalc == 1) then
       info%cell_volume = cell_volume(cell(1:6))
       if (numatoms(atom) > 0) then
!corr           info%volume_per_atom = volume_per_atom(atom, nsymtot, info%cell_volume)
           info%volume_per_atom = volume_per_atom(atom, spacegr%nsymop, info%cell_volume)
       else
           info%volume_per_atom = volume_per_atom(celem,info%cell_volume)
       endif
       if (.not.allocated(ref)) call fillref(ref)
       if (numatoms(atom) > 0 .and. nrefl > 0) then
           call fcalcang(ref,atom,spacegr,celem)
           info%r_factor = Rvalue(ref%fo,ref%fc)
       else
           info%r_factor = 0.0
       endif
   endif
!
   end function structure_info

!---------------------------------------------------------------------      

   subroutine print_structure_info(info,kpr)
   USE General, only:StructureName,cell,spacegr,celem
   USE progtype, only: structure_info_type
   type(structure_info_type), intent(in) :: info
   integer, intent(in)                   :: kpr
   integer                               :: i
!
   write(kpr,'("Name                : ",a)')trim(StructureName)
   write(kpr,'("Cell Lengths        : ","a ",f0.5," b ",f0.5," c ",f0.5)')cell(1:3)
   write(kpr,'("Cell Angles         : ", "alpha ",f0.5," beta ",f0.5," gamma ",f0.5)')cell(4:6)
   if (info%icent == 1) then
       write(kpr,'("Space Group         : ",a)')trim(info%spaceg)//' (centrosymmetric)'
   else
       write(kpr,'("Space Group         : ",a)')trim(info%spaceg)//' (noncentrosymmetric)'
   endif
   call spacegr%prn_symop(2,kpr)
!corr   write(kpr,'("Symmetry-operations   ")')
!corr   write(kpr,'(4x,i0,a,a)')(i,')',trim(SymmStrings(i)),i=1,nSymmStrings)
!corr   write(kpr,'(4x,i0,a,a)')(i,')',trim(spacegr%symopstr(i)),i=1,spacegr%nsymop)
   write(kpr,'("Cell Volume         : ",f0.3)')info%cell_volume
   write(kpr,'("Cell Content        : ",18(a,1x,i0,1x))')(trim(celem(i)%lab),info%vspec(i),i=1,info%nspec)
   write(kpr,'("Volume per atom     : ",f0.3)')info%volume_per_atom
   write(kpr,'("R-Factor            : ",f0.2)')info%r_factor
!
   end subroutine print_structure_info

!---------------------------------------------------------------------      
   subroutine test_bond_perception(atom,legmv)
   USE fileutil
   USE atom_type_util
   USE atom_basic
   USE General, only: cell
   USE unit_cell
   USE connect_mod
   type(atom_type), dimension(:), allocatable, intent(in) :: atom
   type(bond_type), dimension(:), allocatable, intent(in) :: legmv
   type(bond_type), dimension(:), allocatable             :: legm
   type(file_handle)                                      :: fileh
   integer                                                :: ncorr
   integer                                                :: i
!
   call copy_bonds(legm,legmv)
   call bond_type_perception(atom,legm,set_cell_type(cell))   ! create new bond order
!
!  Compare legm%ord with legmv%ord e save the result in a external file
   call fileh%fopen('result_bond.txt','w')
   if (fileh%good()) then
       ncorr = 0
       do i=1,numbonds(legmv)
          if (legmv(i)%ord /= legm(i)%ord) then
              ncorr = ncorr + 1
              write(fileh%handle(),'(a,a,i0,a,i0)')'legame sbagliato: ',    &
              trim(slabnum(atom(legm(i)%n1)%lab,legm(i)%n1))//'-'//trim(slabnum(atom(legm(i)%n2)%lab,legm(i)%n2))   &
              //' ord=', legm(i)%ord,' -> ordine corretto=',legmv(i)%ord
              write(0,'(a,a,i0,a,i0)')'legame sbagliato: ',    &
              trim(slabnum(atom(legm(i)%n1)%lab,legm(i)%n1))//'-'//trim(slabnum(atom(legm(i)%n2)%lab,legm(i)%n2))   &
              //' ord=', legm(i)%ord,' -> ordine corretto=',legmv(i)%ord
          endif
       enddo
       if (ncorr == 0) then
           write(fileh%handle(),'("No errors in the bond order!")')
           write(0,'("No errors in the bond order!")')
       endif
       call fileh%fclose()
   endif
   !stop   !uncomment for shell
   end subroutine test_bond_perception

!----------------------------------------------------------------------------------------------------

   subroutine set_reflection_table(table,vet,ftype,ier)
!
!  Usata per generare una tabella di informazioni sui riflessi
!
   USE General, only:cell,spacegr,nrefl,alambda,celem
   USE variables, only:atm,ref
   USE Atomi
   USE counts
   USE prog_constants
   USE reflection_type_util
   USE proginterface
   type(reflection_type), intent(inout), dimension(:), allocatable :: table
   integer, parameter                                              :: NCOL = 8
   integer, dimension(NCOL)                                        :: vet
   integer, intent(in)                                             :: ftype
   integer, intent(out)                                            :: ier
   integer :: nref
!
   vet(:) = 0
   ier = 0
!
!corr   if (nrefl == 0 .and. nats > 0 .and. npunti == 0)  then
   if (alambda == 0.0) alambda = CU_WAVE
   if (nrefl == 0 .and. nats > 0) then
       call create_reflections(1.0,100.,cell,spacegr,alambda,table,nref)
       ! do i=1,nref
       ! !   if (any(table(i)%hkl - ref(i)%hkl /= 0)) write(0,*)'WARNING'
       !    write(0,*)'hkl=',table(i)%hkl,table(i)%tthd
       !    !write(0,*)'hkl=',ref(i)%hkl,ref(i)%tthd
       ! enddo
   else
       if (allocated(ref)) then
           !call reallocate(table,size(ref),.false.)
           call new_reflections(table,size(ref))
           table(:) = ref(:)
       else
           call fillref(table)
       endif
       nref = nrefl
   endif
!
   if (nref > 0) then  ! ci sono i riflessi?
       if (nats > 0 .and. allocated(atm)) then
           vet(6:7) = 1
           call fcalcang(table,atm,spacegr,celem)  
           if (ftype == 2) then
              table(:)%fc = table(:)%fc**2      ! col. 6: Fc^2
           endif
       else
           table(:)%fc = 0
           table(:)%ph = 0
       endif
       vet(1:5) = 1     
       vet(8) = 1     ! multeplicity
       table(:)%slaq = dvalue(table(:)%tthd,alambda) ! col. 4: d
       if (ftype == 2) then
           table(:)%fo = table(:)%fo**2          ! col. 5: Fo^2
       endif
   else
       ier = 1
   endif
!
   end subroutine set_reflection_table

!----------------------------------------------------------------------------------------------------

   subroutine write_reflection_table(table,vcol,kpri,ftype)
   USE strutil
   USE reflection_type_util, only: reflection_type
   type(reflection_type), intent(in), dimension(:) :: table  ! tabella dei riflessi
   integer, dimension(:), intent(in)               :: vcol   ! quali colonne stampare
   integer, intent(in)                             :: kpri
   integer, intent(in)                             :: ftype  ! 1 -> F; 2 -> F^2
   integer, dimension(size(vcol))                  :: ktype
   integer, parameter                              :: NCOL = 8
   integer, dimension(NCOL)                        :: ival
   real, dimension(NCOL)                           :: rval
   character(len=20), dimension(7)                 :: sval
   integer                                         :: i
!
!  Titoli delle colonne
   ktype(:) = 3
   if (ftype == 1) then
       call set_row_list(vcol,ktype,sval=(/'n.     ','h k l  ','2-theta',    &
         'd      ','Fo     ','Fc     ','phase  ','m      '/),kpr=kpri)
   else
       call set_row_list(vcol,ktype,sval=(/'n.     ','h k l  ','2-theta',    &
         'd      ','Fo^2   ','Fc^2   ','phase  ','m      '/),kpr=kpri)
   endif
!
!  Scrivi la tabella dei riflessi
   ktype = (/1,3,23,23,23,23,1,1/)   ! formato delle colonne
   do i=1, size(table)
      ival(1) = i
      write(sval(2),'(3(i0,1x))')table(i)%hkl(:)
      rval(3) = table(i)%tthd
      rval(4) = table(i)%slaq
      rval(5) = table(i)%fo
      rval(6) = table(i)%fc
      ival(7) = table(i)%ph
      ival(8) = table(i)%m
      call set_row_list(vcol,ktype,ival,rval,sval,kpri)
   enddo
!
   end subroutine write_reflection_table

!-------------------------------------------------------------------------------------------------------  
#if 0
   subroutine export_profile(fname,x,yoss,ycalc,yback,string)
   USE fileutil
   character(len=*), intent(in), optional   :: fname
   real, dimension(:), intent(in)           :: x,yoss
   real, dimension(:), intent(in), optional :: ycalc,yback
   character(len=*), intent(in), optional   :: string
   integer                                  :: i
   integer                                  :: j_in
   type(file_handle)                        :: fpatt
!
   call fpatt%fopen(fname,'w')
!
   if (fpatt%good()) then
       j_in = fpatt%handle()
       if (present(string)) write(j_in,'(a)')trim(string)
       if (present(ycalc) .and. present(yback)) then
          write(j_in,'(f15.3,1x,f15.3,f15.6,1x,f15.6,1x,f15.6)')   &
               (x(i),yoss(i),ycalc(i),yback(i),yoss(i)-ycalc(i),i=1,size(x))
       else
          write(j_in,'(f15.3,1x,f15.3)')(x(i),yoss(i),i=1,size(x))
       endif
       call fpatt%fclose()
   endif
!
   end subroutine export_profile
#endif
END MODULE importexport

   real function GetBondDistance(ispec1, ispec2)
!
!  Restituisce la distanza di legame date due specie
!      
   USE bondtmod
   USE connect_mod
   USE General, only: celem
   implicit none
   integer :: ispec1, ispec2
   integer :: z1,z2
!
   if (ispec1 < 0) then
       z1 = abs(ispec1)
   else
       z1 = celem(ispec1)%z
   endif
   if (ispec2 < 0) then
       z2 = abs(ispec2)
   else
       z2 = celem(ispec2)%z
   endif
   GetBondDistance = bond_distanceZ(z1,z2)
!
   end function GetBondDistance


MODULE exportfiles

implicit none

integer                       :: nexportf    ! number of files to export
character(250), dimension(10) :: export_name ! name of file
logical, dimension(10)        :: is_export   ! se vero il file e stato gia esportato 

CONTAINS
   
   subroutine export_file(filename,ier)
   USE Atomi
   USE General, only:nrefl,spacegr,celem,cell,StructureName,neutro,alambda
   USE reflection_type_util
   USE fileutil
   USE variables, only:atm, lconn
   USE importexport
   USE atom_type_util
   USE strutil
   USE proginterface
   USE unit_cell
   USE gen_frm
   character(len=*), intent(in)                     :: filename
   integer, intent(out)                             :: ier
   character(len=len_trim(filename))                :: ext
   type(reflection_type), dimension(:), allocatable :: reflb
!
   ext = get_extension(filename)
   if (len_trim(ext) > 0) then
       !select case (upper(ext(1:4)))
       select case (upper(trim(ext)))

          case ('HKL')      ! h k l   Fo^2 sigma in formato shelX
              if (nrefl > 0) then
                  allocate(reflb(nrefl))
                  call fillref(reflb)
                  call write_reflections(fname=filename,refl=reflb,code=2)
              else
                  ier = 2
              endif

          case ('FC')       ! h k l  Fo   Fc  phase
              if (nrefl > 0) then
                  allocate(reflb(nrefl))
                  call fillref(reflb)
                  if (nats > 0) then
                      call fcalcang(reflb,atm,spacegr,celem)
                  else
                      reflb(:)%fc = 0
                  endif
                  call write_reflections(fname=filename,refl=reflb,code=4)
              else
                  ier = 2
              endif

          case ('INT')
              call intensity_file_for_database(filename,ier)

          case default
              if (numatoms(atm) > 0) then
                  call export_structure(filename,atm,set_cell_type(cell),spacegr,celem,ALL_FILES,StructureName,neutro,alambda,lconn)
              endif
              
       end select
   endif
!
   end subroutine export_file

!-------------------------------------------------------------------------------------------------------  

   subroutine intensity_file_for_database(filename,ier)
   USE General, only:nrefl,lo,cell,spacegr,celem,neutro,nsyn,alambda
   USE patternref, only:thmin,thmax
   USE atom_type_util
   USE reflection_type_util
   USE variables, only:atm
   USE proginterface
   USE fileutil
   USE elements
   USE spginfom
   USE strutil
   USE ccryst, only: aniso_adp_is_ok
   USE unit_cell
   character(len=*), intent(in)                     :: filename
   integer, intent(out)                             :: ier
   type(reflection_type), dimension(:), allocatable :: reflb
   integer                                          :: nat,nghost
   type(file_handle)                                :: fileh
   character(len=*), dimension(0:2), parameter      :: subf = ['inorganic    ','organic      ','metallorganic']
   character(len=200)                               :: sform
   real                                             :: mu, dens, maxf, vol
   integer                                          :: zval,i,ng
   integer, dimension(6)                            :: icell
   real                                             :: rapp_I,rapp_rho_v,rir
   type(cell_type)                                  :: cellt
   real, parameter                                  :: maxf_Cu_corundum = 346889.6562  ! Calc. Intens. Corundum (from COD cif n. 1000017) b-iso=1.
   real, parameter                                  :: rho_vsq_Cu_corundum = 259076.6250 ! Calc. rho*vol*vol  Corundum  (from COD cif n. 1000017)
   integer, dimension(6)                            :: code
   type(element_type), dimension(:), allocatable    :: chform
   character(len=3)                                 :: adv
   integer                                          :: ierc,iflag
   integer, parameter                               :: MAXBISO = 20
   real, parameter                                  :: TTHETAMAX = 150
   integer, parameter                               :: NMAXREFLD = 500
!   
   icell(:) = 1
   ier = 0
   if (nrefl == 0) then
       if (alambda == 0) alambda = 1.54056
       thmin = 1.0; thmax = TTHETAMAX
       call gener(iflag,0)
!corr       call generate_reflections(1.0,TTHETAMAX,1.54056)
   endif
   if (nrefl > 0) then
       nghost = numatomspec(atm,0)  ! this is the number of ghost atoms
       nat = numatoms(atm) - nghost
       if (nghost > 0) then
           adv = 'no'
           write(lo,'(a)',advance=adv)'BIG TROUBLE: some atoms with Z=0: '
           ng = 0
           do i=1,numatoms(atm)
              if (atm(i)%get_nz() == 0) then
                  ng = ng + 1
                  if (ng == nghost) then
                      adv = 'yes'
                      write(lo,'(a)',advance=adv)trim(atm(i)%lab)
                  else
                      write(lo,'(a)',advance=adv)trim(atm(i)%lab)//', '
                  endif
              endif
           enddo
           if (real(nghost) / nat > 0.7.and.nat> 10) nat = 0
       endif
       allocate(reflb(nrefl))
       call fillref(reflb)
       if (nat > 0) then
           cellt = set_cell_type(cell)
           call calcola_occ_new(atm,spacegr,cellt%get_g(),0.3)
!          check for adp
           do i=1,nat
              if (atm(i)%ptab == H_at) cycle
                !write(0,*) 'ADP=',i,aniso_adp_is_ok(atm(i)%bij)
              !if (any(abs(atm(i)%bij(:)) > 10)) then
              !    write(0,*)'BISO=',i
              !    atm(i)%bij(1) = 0.0
              !endif
              if (atm(i)%biso <= 0 .or. atm(i)%biso > MAXBISO) then
                  atm(i)%bij = 0
                  call set_biso(atm(i))
              endif
           enddo
           call fcalcang(reflb,atm,spacegr,celem,anis=.true.)
       else
           !ier = 1
           !return
           reflb%fc = 0; reflb%ph = 0;
       endif
       reflb(:)%lp = LPcorrection(reflb%tthd,neutro,nsyn)
       call fileh%fopen(filename,ios='w')
       if (fileh%good()) then
           vol = cell_volume(cell)
!corr           spg = init_spaceg_type(spaceg_string())
           if (.not. spg_check_cell(spacegr,cell)) then
               write(lo,'(a)')'Cell dimensions and spacegroup are non consistent'
           endif
           if (nat > 0) then
               dens = density_value(molecular_weight(atm%ptab,atm%och*atm%ocry),vol,spacegr%nsymop)
               mu = linear_abs_coeff(atm,dens)
!
               maxf = maxval(reflb(:)%m*reflb(:)%lp*reflb(:)%fc**2)
               rapp_rho_v = (rho_vsq_Cu_corundum )/(vol*vol*dens)
               rapp_I = maxf/maxf_Cu_corundum
               write(6,'(a,f0.8)')'Rho_v2=     ',vol*vol*dens
               rir = rapp_rho_v * rapp_I
               !write(0,'(a,f0.3)')'RIR=   ',rir
!
               call formula_from_atom(atm,chform)
               zval = nint(molecular_weight(atm%ptab,spacegr%nsymop*atm%och*atm%ocry)/molecular_weight(chform%ptab,chform%nw))
               call chemical_formula(chform, sform, fform=2, hide1=.true.)
!               
               write(fileh%handle(),'(1x,a,a)')'FORMULA: ',trim(sform)
               write(fileh%handle(),'(1x,a,a)')'SUBFILE: ',subf(is_organic(celem%z,nint(celem%nw)))
           else
               rir = 0
               dens = 0
               zval = 0
               mu = 0
               write(fileh%handle(),*) 'SUBFILE: undefined'
           endif
           if (spacegr%undef()) then
               spacegr%csys_code = csys_from_cell(cell)
           endif
           call spg_get_cell_code(spacegr,cell,code,ierc)
           do i=1,6
              if(code(i).ne.0) icell(i) = 0
           enddo
           write(fileh%handle(),'(1x,a,3(1x,f0.4),3(1x,f0.3))')    'CELL:    ',cell(1:6)
           write(fileh%handle(),'(1x,a,6i6)')    'WRITE CELL: ',icell(1:6)
           write(fileh%handle(),'(1x,a,a)')    'SPG:     ',trim(spacegr%symbol_xhm)
           !if (spacegr%undef()) then
           !    write(fileh%handle(),'(1x,a,a)')    'CRY SYS: ',trim(cry_sys(csys_from_cell(cell)))
           !else
           if (spacegr%csys_code == CS_Trigonal) then
               if (uc_is_rhombohedral(cell,0.01)) then
                   write(fileh%handle(),'(1x,a,a)')    'CRY SYS: ',trim(cry_sys(spacegr%csys_code))//' (rhombohedral axes)'
               elseif (uc_is_hexagonal(cell,0.01)) then
                   write(fileh%handle(),'(1x,a,a)')    'CRY SYS: ',trim(cry_sys(spacegr%csys_code))//' (hexagonal axes)'
               endif
           else
               write(fileh%handle(),'(1x,a,a)')    'CRY SYS: ',trim(cry_sys(spacegr%csys_code))
           endif
           !endif
           write(fileh%handle(),'(1x,a,f0.3)') 'VOLUME : ',vol
           write(fileh%handle(),'(1x,a,f0.3,a)') 'Density: ',dens,' g cm-3'
           write(fileh%handle(),'(1x,a,i5)') 'Z : ',zval
           write(fileh%handle(),'(1x,a,f0.3,a)') 'mu(CuKa):',mu,' cm-1'
           write(fileh%handle(),'(1x,a,f0.3)') 'RIR     :',rir
           write(fileh%handle(),'(1x,a)') '----------------------------------------------------------------------------- '
           write(fileh%handle(),'(1x,a)') '  Remarks: Diffraction pattern calculated by EXPO from COD database cif file'
           write(fileh%handle(),'(1x,a)') '  Remarks: RIR calculated by EXPO     '
           write(fileh%handle(),'(1x,a)') '----------------------------------------------------------------------------- '
!!!!!!
!FIXME - test this to remove reflections in overlap and compute group intensity
!corr           reflb(:)%fwhm = 1.0
!corr           call make_ref_groups(reflb,gref,0.00005)
!corr           call print_refg(gref,0)
!corr           do i=1,size(gref)
!corr              if (gref(i)%code > 1) then
!corr                  reflb(gref(i)%vref(1))%fc = sum(reflb(gref(i)%vref(:))%fc)
!corr                  reflb(gref(i)%vref(2:))%fc = 0
!corr              endif
!corr           enddo
!!!!!!
           call write_reflections(jfile=fileh%handle(),refl=reflb,code=5,nref=min(NMAXREFLD,nrefl))
           call fileh%fclose()
       endif
   else
       ier = 2
   endif

   end subroutine intensity_file_for_database


!-------------------------------------------------------------------------------------------------------  

   subroutine export_files()
   USE commandsmod
   integer :: ierrex
   integer :: i
!
!  esporta i file non esportati in precedenza
!old   do i=1,nexportf
!old      call export_file(export_name(i),ierrex)
!old   enddo
   do i=1,ncommands(commands)
      if (commands(i)%code == EXPORT_CMD) then
          if (commands(i)%ndir > 0) then
              call export_file(commands(i)%strdir(1),ierrex)
          endif
      endif
   enddo
!
   end subroutine export_files

!-------------------------------------------------------------------------------------------------------  

   subroutine update_model_and_export(atom,legm,kmodel)
   USE connect_mod, only: bond_type
   USE atom_basic, only: atom_type
   USE sinterface
   type(atom_type), dimension(:), allocatable, intent(in) :: atom
   type(bond_type), dimension(:), allocatable, intent(in) :: legm
   integer, intent(in)                                    :: kmodel
!
   if (kmodel == 1) then
       call update_model(atom,legm)
       call export_files()
   endif
!
   end subroutine update_model_and_export

END MODULE exportfiles

