!#define FFTW

MODULE fouriermap

implicit none

integer, parameter             :: FOUR_FO=1, FOUR_2FO_FC=2, FOUR_FO_FC=3, FOUR_FC=4
character(len=8), dimension(4) :: four_name = [ 'Fo      ', '2Fo - Fc', 'Fo - Fc ', 'Fc      ' ] 

real :: maxfmap

type map_info_type 
   logical :: active   ! map visualization active?
   integer :: ftype    ! map type? 
end type map_info_type

type(map_info_type) :: mapinfo

CONTAINS

    subroutine init_fourier_map()
!
!   Initialize information about map
    mapinfo%active = .false.
    mapinfo%ftype = 0
    end subroutine init_fourier_map

!----------------------------------------------------------------------

    subroutine jav_show_map(itypef,ref,map_name,sogmap,fctype,force)
    USE mappa, only:xmap
!corr    USE progtype
    USE fourier_mod
    USE reflection_type_util, only: reflection_type
    type(reflection_type), dimension(:), intent(in), optional :: ref        ! reflections
    character(len=*), intent(in), optional                    :: map_name   ! name of map
    real, intent(in), optional                                :: sogmap     ! show only intensity > sogmap
    integer, intent(in), optional                             :: fctype     ! ref%fc id Fc (fctype=1) or Fc^2 (fctype=2)
    logical, intent(in), optional                             :: force
    integer, intent(in)                                       :: itypef     ! fourier type
    integer                                                   :: ftype
    logical                                                   :: lforce
!
    if (itypef == 0 .and. .not.allocated(xmap)) then
        mapinfo = map_info_type(.true.,itypef)   ! segnala che l'utente ha richiesto la fourier
        return
    endif
!
    if (present(fctype)) then
        ftype = fctype
    else
        ftype = 1
    endif
!
    if (present(force)) then
        lforce = force
    else
        lforce = .false.
    endif
!
    if (itypef /= 0) then
        if (.not.allocated(xmap)) then
            call CheckGrid()
            call AllocaMappa(1)   
        endif
        call create_map(itypef,ref,size(ref),ftype)
    endif
!
    if (present(sogmap)) then
        call VediMappa(sogmap,namem=map_name,force=lforce)
    else
        call VediMappa(namem=map_name)
    endif
    mapinfo = map_info_type(.true.,itypef)
!   
    end subroutine jav_show_map
    
!----------------------------------------------------------------------

    logical function is_map_enabled()
!
!   Check if user has required a map visualization
!
    integer :: is_map_visible
!
    is_map_enabled = mapinfo%active .or. is_map_visible(1) == 1
!
    end function is_map_enabled

!----------------------------------------------------------------------

    integer function map_type_selected()
!
!   Which map has been required. 
!
    if (mapinfo%ftype == 0) then
        map_type_selected = FOUR_FO_FC  ! if unknown force the difference map
    else
        map_type_selected = mapinfo%ftype
    endif
    end function map_type_selected

!----------------------------------------------------------------------

    subroutine sogmap_init(itypef,refl,atom,sog,umapel)
    USE mappa, only:xmap
    !USE progtype
    USE atom_type_util
    USE fourier_mod
    USE reflection_type_util, only: reflection_type
    integer, intent(in)                             :: itypef
    type(reflection_type), dimension(:), intent(in) :: refl
    type(atom_type), dimension(:), intent(in)       :: atom
    real, intent(out)                               :: sog
    real, intent(out), optional                     :: umapel ! number of map unit per electrons
    integer, dimension(size(atom))                  :: zval
!
    if (.not.allocated(xmap)) then
        call CheckGrid()
        call AllocaMappa(1)   
    endif
    call create_map(itypef,refl,size(refl),1)
    sog = maxval(xmap) * 0.8
    if (present(umapel)) then
!corr        zval(:) = atomic_number(atom)
        zval(:) = atom%z()
        umapel = maxval(xmap) / maxval(zval)
          write(0,*)'umapel=',umapel, maxval(xmap), maxval(zval), sog
    endif
    !if (code == 1) then
    !elseif (code == 2) then
    !    call create_map(FOUR_FC,refl,size(refl),1)
    !    zval(:) = atomic_number(atom)
    !    sog = maxval(xmap) / maxval(zval)
    !    sog = maxval(xmap) * 0.08
    !endif
       !write(0,*)'sog=',maxval(xmap),sog
!
    end subroutine sogmap_init

!----------------------------------------------------------------------

    subroutine create_map(itypef,refl,nrefsize,fctype)
    USE mappa
    !USE progtype
    USE fourier_mod
    USE reflection_type_util, only: reflection_type
    integer, intent(in)                             :: itypef
    type(reflection_type), dimension(:), intent(in) :: refl
    integer, intent(in)                             :: nrefsize
    integer, intent(in)                             :: fctype
    integer, dimension(3,nrefsize)                  :: hkl
    real, dimension(nrefsize)                       :: wei
    real, allocatable, dimension(:)                 :: fcoef
    integer                                         :: i
!
    do i=1,nrefsize
       hkl(:,i) = refl(i)%hkl(:)
    enddo
    wei(:) = 1.0
    call fourier_coef(fcoef,refl,nrefsize,itypef,fctype)
#ifdef FFTW
    call MakeMap(hkl,fcoef,refl(:)%ph,nrefsize,0,xmap,FORTH,BACK)
      !       call invmap(hkl, iphi2, fop2, nrefsize, np(1), np(2), np(3), BACK)
      !   do i=1,nrefsize
      !      write(70,*)fop2(i), fcoef(i),iphi2(i),refl(i)%ph
      !   enddo
#else
    call MakeMap(hkl,fcoef,refl(:)%ph,wei,nrefsize,0,xmap) 
#endif
!
    end subroutine create_map

!----------------------------------------------------------------------

    subroutine VediMappa(sogmap,namem,force)
    USE Mappa
    USE stat
    USE General, only:nrefl
                 USE stat
!   
    implicit none
    real, intent(in), optional    :: sogmap
    character(len=*), intent(in)  :: namem
!!!FIXME - spiegare force
    logical, intent(in), optional :: force
    real                          :: denmin, denmax, avden, stden
    integer                       :: npunt
    real                          :: resm
    real                          :: sog
!   
    call maxmin3d(denmin,denmax,avden,stden,npunt,0)
    !call maxmin3d(denmin,denmax,avden,stden,npunt,1)
    !stden = sqrt(stden)
    !denmin = minval(xmap)
    !denmax = maxval(xmap)
    resm = 2.0
!FIXME - il risultato grafico sembra indipendente dal valore del min e del max (chiedere a Rosi)
!        write(0,'(a,5(a,f10.3))')'mappa:','min=',denmin,' max=',denmax,' ave=',avden,' std=',stden,'sog=',sogmap
!            denmin = - 200
!            denmax = 200
!            stden = 60
!        call PassaMappaParz(trim(namem)//char(0), 1, 1, np, nrefl, resm, denmin, denmax, avden, stden, 1)
!                return
!!!!    sogmap = 100  ! mostra tutto cio' che è al di sopra di sogmap (stden*1.5)
    if (present(sogmap)) then
        !write(0,'(a,5(a,f10.3))')'mappa:','min=',denmin,' max=',denmax,' ave=',avden,' std=',stden,'sog=',sogmap
        sog = sogmap
        if (force) then
            if (sogmap > denmax) sog = denmax*0.5
        endif
        call PassaMappaParz(trim(namem)//char(0), 1, 1, np, nrefl, resm, denmin, denmax, avden, sog/1.5, 1)
    else
        call PassaMappaParz(trim(namem)//char(0), 1, 1, np, nrefl, resm, denmin, denmax, avden, stden, 1)
    endif
        maxfmap = denmax
!   
    return
    end subroutine VediMappa


!----------------------------------------------------------------------

      subroutine MakeFour(icalfo,npikfo,iweifo,iselect_ref,preff,showfourier,ier)
      USE General
      !USE Riflessi
      USE Atomi
      USE Molcom      
      USE nr
      USE PhaseModule
      USE Mappa
      USE variables, only:atm,ref
      USE atom_type_util
      USE fourier_mod
      USE reflection_type_util
      implicit none
      integer, intent(in) :: showfourier
      common /searchfour/ isearchfo
      integer :: isearchfo
      integer, intent(in)                              :: icalfo
      integer, intent(in)                              :: npikfo
      integer, intent(in)                              :: iweifo
      integer, intent(in)                              :: iselect_ref
      real, intent(in)                                 :: preff
      integer, intent(out)                             :: ier
      type(reflection_type), allocatable, dimension(:) :: refl
      real, dimension(nrefl)                           :: wwt
      integer, dimension(nrefl)                        :: iorder
      real, allocatable, dimension(:)                  :: frvet
      integer, allocatable, dimension(:,:)             :: hkl
      integer                                          :: i 
      integer                                          :: mode,jpatt
      integer                                          :: numbMax
      integer                                          :: nExitCode
      type(atom_type), allocatable, dimension(:)       :: atmold
      integer :: nat_old
      real :: sogfmap, umapel    !!!!, maxmap1
!
      ier = 0
!!!!!!!!!!!!!only for test
                 !is_extraction = .true.
                 !ref(:)%fo = ref(:)%fv
!!!!!!!!!!!!!only for test
      if (is_extraction) then
          call fcalcang(ref,atm,spacegr,celem)
          allocate(refl(nrefl))
          refl(:) = ref(:) ! si puo' evitare - da sistemare
      else
          call MsgWinErr('ERROR','F(obs) are not available!',ERR_WINDOW,nExitCode)
          return
      endif
!
!     Calcolo fattori di struttura fcv e fasi iphb dal modello
!corr      if (knw == 1) call AllocaTestStuff(1)
!
!     Salvo modello corrente per confronto
      nat_old = nats !!!!numatoms(atm)
      call copy_atoms(atmold,atm)
!
!     Calcolo peso wwt
!     iweifo = 0 --> wwt = 1.
!     iweifo = 1 --> wwt = peso di tipo Sim
      if (iweifo == 1.or.iselect_ref == 2) then
         call simweight(refl,wwt)
      else
         wwt(:) = 1.0
      endif
!
!     Ordina riflessi e pesi
      select case (iselect_ref) 
         case (1)          ! ordina in base all'Fo
           call indexx(refl(:)%fo,iorder)
           refl(:) = refl(iorder)
           if (iweifo == 1) then
               wwt(:) = wwt(iorder)
           else
               wwt(:) = 1.0
           endif

         case (2)          ! ordina in base al peso
           call indexx(wwt,iorder)
           refl(:) = refl(iorder)
           if (iweifo == 1) wwt(:) = wwt(iorder)

      end select 
!
!     Seleziona una percentuale di riflessi pari a preff
      numbMax = nint(nrefl*preff/100.)
      if (numbMax.lt.1) numbMax = 1
      if (numbMax.gt.nrefl) numbMax = nrefl
!
      call AllocaMappa(1)
!
      write(lo,'(10x,a)')'***   A Fourier map with ('//trim(four_name(icalfo))//') coefficients ***'
      select case (icalfo) 
          case (FOUR_FO)        ! Fo
            aperc(1) = nat_old + npikfo
          case (FOUR_2FO_FC)        ! 2Fo - Fc 
            aperc(1) = nat_old + npikfo
          case (FOUR_FO_FC)        ! Fo - Fc
            aperc(1) = npikfo
      end select
      write(lo,'(20x,a)') " ***   has been required   ***"
      write(lo,'(/7x,a,i0,a,i0,a)')'Number of used reflections: ',numbMax,' reflections (on ',nrefl,')'
                 umapel = 1.0    !!!!FIXME-non funziona
      if (showfourier == 1) then
          call  sogmap_init(FOUR_FC,refl,atm,sogfmap,umapel) 
   !!!!test       maxmap1 = maxval(xmap)
      endif
!
!     Esegue Fourier
      jpatt = 0
      krecy = 0
      allocate(hkl(3,numbMax))
      do i=1,numbMax
         hkl(:,i) = refl(i)%hkl
      enddo
      call fourier_coef(frvet,refl,numbMax,icalfo,1)
#ifdef FFTW
      call MakeMap(hkl,frvet,refl(:)%ph,numbMax,jpatt,xmap,FORTH,BACK)
#else
      call MakeMap(hkl,frvet,refl(:)%ph,wwt,numbMax,jpatt,xmap)
#endif
          !write(0,*)'mapmax=',maxval(xmap)
      !xmap = xmap * maxmap1 / maxval(xmap)
!
      if (showfourier == 1) then
          call map_set_visible(1,1)   ! force map visualization
          call jav_show_map(0, map_name = trim(four_name(icalfo))//' map',sogmap=sogfmap,force=.true.)
      endif
!
!     Esegue peak search
      isearchfo = 1
      mode = 0
!corr      call Search99(xmap,mode,ier,numbMax)
      call Search99(mode,ier,numbMax)
      isearchfo = 0
!
!     Confronta vecchio e nuovo modello
      !call labelx(icalfo,atmold,nint(aperc(1)),umapel,ier)
      call labelx(icalfo,atmold,nint(aperc(1)),ier)
      call to_coord(1)
!
!corr      if (knw == 1) call AllocaTestStuff(2)
!
      end subroutine makefour

!----------------------------------------------------------------------

      subroutine fourier_coef(fcoef,refl,nrefuse,ftype,fctype)
!
!     Calculate coefficients for Fourier
!
      USE arrayutil
      USE reflection_type_util, only: reflection_type
      real, dimension(:), allocatable, intent(out)    :: fcoef   ! coefficients
      type(reflection_type), dimension(:), intent(in) :: refl    ! reflections
      integer, intent(in)                             :: ftype   ! Fourier type
      integer, intent(in)                             :: fctype  ! 1 -> Fc, 2 -> Fc^2
      integer, intent(in)                             :: nrefuse ! number of used reflections for fourier
      real                                            :: scal
      real, dimension(size(refl))                     :: fq 
!
!     compute scale
      if (fctype == 1) then
          scal =  sum(refl(:)%fc) / sum(refl(:)%fo)  
      else
          fq(:) = sqrt(refl(:)%fc)                    
          scal =  sum(fq(:)) / sum(refl(:)%fo)
      endif
!
      call resize_array(fcoef,nrefuse)
!
      select case(ftype)
         case (FOUR_FO)
           fcoef(:) = scal*refl(:nrefuse)%fo
         case (FOUR_2FO_FC)
           if (fctype == 1) then
               fcoef(:) = 2*scal*refl(:nrefuse)%fo - refl(:nrefuse)%fc
           else
               fcoef(:) = 2*scal*refl(:nrefuse)%fo - fq(:nrefuse)
           endif
         case (FOUR_FO_FC)
           if (fctype == 1) then
               fcoef(:) = scal*refl(:nrefuse)%fo - refl(:nrefuse)%fc
           else
               fcoef(:) = scal*refl(:nrefuse)%fo - fq(:nrefuse)
           endif
         case (FOUR_FC)
           if (fctype == 1) then
               fcoef(:) = refl(:nrefuse)%fc
           else
               fcoef(:) = fq(:nrefuse)
           endif
      end select
!
      end subroutine fourier_coef

!----------------------------------------------------------------------

   subroutine labelx(icalfo,atmold,nPeaksToFind,ier)
   USE General, only: lo, StructureName, cell, celem, spacegr
   USE Symmetry, only: nsym, ddmin, icent
   USE Atomi
   USE Molcom, only: It
   USE sinterface
   USE ilmviewmod
   USE variables
   USE atom_type_util
   USE fragmentmod
   USE connect_mod
   USE unit_cell
   USE PhaseModule, only:biso
   USE strutil
   USE cgeom
!
   implicit none
   integer, intent(in)                            :: icalfo
   type(atom_type), dimension(:), intent(in)      :: atmold
   integer, intent(in)                            :: nPeaksToFind
!corr      real, intent(in)                               :: umapel
   type(fragment_type), dimension(:), allocatable :: fragm
   integer                                        :: nfrag
   type(atom_type), dimension(:), allocatable     :: atmc
   real, dimension(3)                             :: xn,xo,xeq,s
   real, dimension(2)                             :: vet
   real, dimension(:), allocatable                :: tempdist
   real, allocatable, dimension(:)                :: distminv
   character(len=16), dimension(:), allocatable   :: labmin
   integer, allocatable, dimension(:)             :: kminv,selected
   integer                                        :: i, j, k, ifi, kt, jpunt, ifin
   integer                                        :: Naccept
   integer                                        :: ier
   integer                                        :: iExitC,MultiSel,NumCan
   real                                           :: distmin, d, distm
   integer                                        :: kmin, iat
   integer                                        :: nequal, nnzo
   real                                           :: scalatom, sumnew, sumold
   integer                                        :: nat_old, nat_new
   type(atom_type), dimension(:), allocatable     :: atmnew
   logical                                        :: kpr = .true.
   integer                                        :: new_peaks
   real                                           :: coef
   character(len=:), allocatable                  :: msg
   type(cell_type) :: cellt
!
   nat_old = size(atmold)
   call xyzs_to_atom(atmnew)
   nat_new = numatoms(atmnew)
   call atom_string(atmnew,celem)  ! peaks will be as Q1, Q2, ...
   atmnew(:)%biso=biso
   if (kpr) then
       call print_atoms(atmold,title='Old atoms list')
       call print_atoms(atmnew,title='Fourier peaks')
   endif
   allocate(tempdist(nat_new))
   vet(:) = (/1.0,-1.0/)
   ifin=icent+1
   qq(:) = 0.0
   cellt = set_cell_type(cell)
   do i=1,nat_old
      !nnzo=iabs(atmold(i)%nz)/nzConst
      nnzo=atmold(i)%kscatt()
      xo(:) = atmold(i)%xc(:)
      distm=9999
      jpunt=0
      do j=1,nat_new
         if (qq(j).lt.0.5) then
             xn(:) = atmnew(j)%xc(:)
             do kt=1,spacegr%ncoper
                do ifi=1,ifin
                   coef=vet(ifi)
                   do k=1,nsym
                      call xequi(xo,k,coef,xeq,kt)
!                      call xdist(xn,xeq,d,s)
                      call xdisteqs(xn,xeq,cellt%get_g(),d,s)
                      if (d.lt.ddmin) then
                          distm=d
                          jpunt=j
                          qq(jpunt) = i
                          tempdist(jpunt) = d
                      endif
                   enddo
                enddo
             enddo
         endif
      enddo
   enddo
!
   if (kpr) then
       do j=1,nat_new
          i=nint(qq(j))
          if (i.gt.0) then
              write(lo,'(2x,a,i5,4f10.3,10x,a8,i4,4f10.3,1x,f10.3)')             &
              atmnew(j)%lab,nint(atmnew(j)%inte),atmnew(j)%xc,atmnew(j)%biso,    &
              atmold(i)%lab,i,atmold(i)%xc(:),bfac(i),tempdist(j)
          else
              write(lo,'(2x,a,i5,4f10.3)') atmnew(j)%lab,nint(atmnew(j)%inte),atmnew(j)%xc,atmnew(j)%biso
          endif
       enddo
   endif
   new_peaks = 0
   nequal = 0
   sumold = 0.0
   sumnew = 0.0
!
   do j=1,nat_new
!
!-- accept the new peak only if has no counterpart
!-- in the atom list
!
      i=nint(qq(j))
      if (i.le.0) then
          new_peaks = new_peaks + 1
          atmnew(new_peaks)%xc = atmnew(j)%xc
          atmnew(new_peaks)%biso = biso
      elseif (icalfo.ne.3) then
          nequal = nequal + 1
          sumold = sumold + atmold(i)%inte
          sumnew = sumnew + atmnew(j)%inte
      endif
   enddo
   scalatom = 1.0
!
! Riscala intensita
!
  !!!!FIXME-non funziona                     atmnew(:)%inte = atmnew(:)%inte / umapel
   if (nequal.gt.0.and.icalfo.ne.3) then
       if (sumnew.gt.0.0) scalatom = sumold/sumnew
!test          do j=1,new_peaks
!test             atmnew(j)%inte = scalatom * atmnew(j)%inte
!test          enddo
   endif
   if (new_peaks > nPeaksToFind) new_peaks = nPeaksToFind
!
   nat_new = new_peaks + nat_old
   call resize_atoms(atmnew,nat_new)
   if (new_peaks > 0) then
       write(lo,265) new_peaks
  265     format(  10x,i5, '  new peak(s) found')
       call calcola_occ(atmnew(:new_peaks),spacegr,cellt)
!
!         move atoms at the end of arrays
!         to make room for old atoms
       atmnew(nat_old+1:) = atmnew(:new_peaks)
   endif
!
!  copy old atoms at the beginning of arrays
   atmnew(:nat_old) = atmold(:)
!
   if (new_peaks > 0) then
       nats = nat_old + nats
       call copy_atoms(atm,atmnew)
!
!      Frammenta il modello
       call get_fragments(atm,cellt,lconn,nfrag,fragm)
!
!      Raggruppa i frammenti 
       call regroup_fragment(fragm,atm,cellt,spacegr)
!
!      Disegna il modello e i nuovi atomi come ghost
       if (.not.jav_opened())call ilmview_open(StructureName)
       call jav_set_modify(0)
       call ilmview_draw(atm,lconn)
!
!      Calcola distanza minime
       call copy_atoms(atmc,atm)
       call frac_to_cart(atmc,orthomatrix(cell))
       allocate(distminv(new_peaks),labmin(new_peaks), kminv(new_peaks))
       do i=1,new_peaks
          iat = nat_old+i
          call get_minimum_distance(atmc(iat),atmc(:nat_old),kmin,distmin)
          distminv(i) = distmin
          kminv(i) = kmin          !!mr
          labmin(i) = atmc(kmin)%lab
          write(It(i),'(a,i0,4x,a,f0.3)')trim(atmc(iat)%lab)//'  I=', int(atmc(iat)%inte),  &
            'dmin('//trim(atmc(iat)%lab)//'-'//trim(atmc(kmin)%lab)//')=' ,distmin
           It(i) = trim(It(i))//char(0)
       enddo
!
       call ClearMessage()
! --
       msg = trim(i_to_s(new_peaks))//' new peaks found.'//char(10)//  &
             'Check Intensities &amp; distances and select peaks to retain.'//char(0)
       NumCan = new_peaks
       MultiSel = 1
       allocate (selected(NumCan))
       selected(:) = 0
       call c_NewPeekList('Select peaks to accept'//char(0), new_peaks,     &
                          MultiSel, selected,iExitC, nat_old, msg, atmc, distminv, kminv)
       if (iExitC == 3) then
           selected(:) = 0
       elseif (iExitC.eq.2) then
           selected(:) = 1
       endif
       Naccept = 0
       do i=1, NumCan
          if (selected(i).eq.1) then
              Naccept = Naccept + 1
              atm(nat_old+Naccept) = atm(nat_old+i)
          endif
       enddo
       if (Naccept == 0) ier = 1 ! segnala che non c'e' modifica
       nats = nat_old + Naccept
       call resize_atoms(atm,nats)
       call ilmview_draw(atm,lconn)
       call jav_set_modify(1)
   else
       nats=nat_old
       ier = -1
       call OpenWarning('No new peaks found !!!')
   endif
   call atom_to_xyzs(atm)
   !call print_connect(atm%lab,lconn,kpri=0)
!
   end subroutine labelx

!----------------------------------------------------------------------------------------------------

   subroutine simweight(refl,sim,eoo,ecc)
   USE General, only : ff,bt,celem,neutro !!,al,cl,bs
   USE Symmetry, only : pts
   USE reflection_type_util, only: reflection_type
   USE elements, only: numelem,at_scatt
!
   type(reflection_type), dimension(:), intent(in) :: refl
   real, dimension(:),intent(out)                  :: sim
   real, dimension(:),intent(out), optional        :: eoo,ecc
   real, dimension(size(refl))                     :: eo,ec
   integer                                         :: nr
   real                                            :: sumff,den,bes
   real                                            :: eps
   integer                                         :: multi
   integer                                         :: i,ij
!
   nr = size(refl)
   do i=1,nr
!corr      do ij=1,numelem(celem)
!corr         ff(ij)=cl(ij) + sum(al(:,ij)*exp(-bs(:,ij)*refl(i)%slaq))
!corr      enddo
      ff(:numelem(celem)) = at_scatt(celem,refl(i)%slaq,neutro)
      sumff = 0.0
      do ij=1,numelem(celem)
         sumff = sumff + celem(ij)%nw*(ff(ij)**2)
      enddo
      call cmult2002(refl(i)%hkl,eps,multi)
      den = sqrt((exp(-2.0*bt*refl(i)%slaq))*sumff*eps)
      ec(i)=refl(i)%fc/(sqrt(pts)*den)
      eo(i)=refl(i)%fo/(sqrt(pts)*den)
      sim(i)=bes(2.0*eo(i)*ec(i))
   enddo
   where(eo < 0.1 .and. ec < 0.1)sim=0.0
   if (present(eoo) .and. present(ecc)) then
       eoo(:) = eo(:)
       ecc(:) = ec(:)
   endif
!
   end subroutine simweight

END MODULE fouriermap

!----------------------------------------------------------------------

    subroutine Lanciajav(itypef, iAction, ierf)
    USE variables
    USE ilmviewmod
!corr    USE factor
    USE fouriermap
    USE atom_type_util
    USE General, only:cell, nrefl, spacegr,celem
    USE unit_cell
!corr    USE proginterface
    USE reflection_type_util
    USE prog_constants
!corr    USE sfacmod
    implicit none
    integer                                    :: itypef, iAction, ierf
    type(atom_type), dimension(:), allocatable :: atomc
    real                                       :: radmap
    real                                       :: sogfmap
!corr    integer :: nref
!corr    integer :: ier
!   
    ierf = 0
    if (.not.allocated(atm)) then
        ierf = 1
        return
    endif
    if (.not. allocated(ref)) then
        if (itypef == FOUR_FC) then
            call create_reflections(0.,80.,cell,spacegr,CU_WAVE,ref,nrefl)
            if (nrefl == 0) then
                ierf = 2
                return
            !else
            !    !call fillref(ref)
            !    call ref_to_expo(ref)
            endif
        else
            ierf = 2
            return
        endif
    endif
!  
!   Disable 'Show Fourier Map' required?
    if (itypef == 0 .and. iAction == 0) then
        call map_set_visible(1,0)  
        return
    endif
!
    if (ierf == 0) then
        call ilmview_open()
!!!!!!!!!!!!!only for test
           !      ref(:)%fo = ref(:)%fv
!!!!!!!!!!!!!only for test
        call fcalcang(ref,atm,spacegr,celem)
        call frac_to_cart_copy(atm,atomc,orthomatrix(cell))
        call get_radius_molecule(atomc,radmap)
        call map_set_centre(baricentro(atm))
        call map_set_rad(radmap)
        call map_set_visible(1,1)   ! force map visualization
        if (itypef /= 0) then
!corr            sogfmap =  sogmap_init(FOUR_FC,ref,atm,code=2,sogfmap) 
            call  sogmap_init(FOUR_FC,ref,atm,sogfmap) 
        !cor    write(0,*)'radmap=',radmap
            call jav_show_map(itypef,ref, 'Fourier map',sogmap=sogfmap,force=.true.)
        else
            call jav_show_map(itypef,ref, 'Fourier map')
        endif
    endif
!   
    end subroutine Lanciajav

!----------------------------------------------------------------------
!
!    real function GetXMap(i, j, k)
!    USE Mappa
!!   
!    implicit none
!    integer       :: i, j, k
!!   
!    GetXMap = xmap(i,j,k)
!!   
!    return
!    end
!
!----------------------------------------------------------------------
