module gen_frm

USE prog_constants, only: CU_WAVE

implicit none

enum, bind(c)
  enumerator :: ALL_FILES, CIF_FILE, PDB_FILE, SHELX_FILE, FRA_FILE,   &
                XYZ_FILE, ZMT_FILE, MOL_FILE, MOL2_FILE, MOP_FILE,     &
                POV_FILE, GAMESS_OUT, NWCHEM_OUT, ABINIT_OUT, GAMESS_IN, NWCHEM_IN_DFT, &
                GAUSSIAN_CART_IN, GAUSSIAN_ZMAT_IN, NWCHEM_IN_DFTD, ABINIT_IN, CRYSTAL_IN, &
                CRYSTAL_OUT,QE_IN,QE_OUT
endenum

integer, parameter :: TETRA=1,OCTA=2,SQUARE=3,CUBE=4,TRIG=5, APRISM_TETRA=6,&
                      PRISM_TRIGONAL=7,ICOSAHEDRON=8,ISOLATED=9,SMILES=10

integer, private, dimension(:), allocatable :: vmodel_cif
integer, private                            :: selected_cif

type xpd_option_type
  real :: wave = CU_WAVE
  real :: ttmin = 5
  real :: ttmax = 70
  real :: ttstep = 0.02
  real :: fwhm = 0.1
  real :: scal = 1000.
end type xpd_option_type

private :: make_powder_pattern_var, make_powder_pattern_xpd
interface make_powder_pattern
  module procedure make_powder_pattern_var, make_powder_pattern_xpd
end interface

contains
   subroutine import_crystal(filename,ftype,atom,legm,elem,cell,spgin,has_symmetry,radtype,   &
                             gui_enabled,err_read,at_info,nsec,sec_struct)
!
!  If has_symmetry is true structure will be imported in symmetry (cell and spgin) defined in input
!  If has_symmetry is false cell and spgin will contain the symmetry of the file or default if symmetry is absent in file
!
   USE fileutil
   USE errormod
   USE connect_mod
   USE molpnew
   USE atom_type_util
   USE strutil
   USE elements
   USE spginfom
   USE unit_cell
   USE cif_frm
   USE pdb_frm
   USE fra_frm
   USE xyz_frm
   USE zmt_frm
   USE shelx_frm
   USE mol2_frm
   USE mop_frm
   USE qe_frm
   USE wlist
   USE OBmodule
   USE crystal_phase
   USE crystal_frm
   character(len=*), intent(in)                                 :: filename
   integer, intent(in)                                          :: ftype ! if 0 the file type is set from filename
   type(atom_type), dimension(:), allocatable, intent(inout)    :: atom
   type(bond_type), dimension(:), allocatable, intent(inout)    :: legm
   type(element_type), dimension(:), allocatable, intent(inout) :: elem
   type(cell_type), intent(inout)                               :: cell
   logical, intent(in)                                          :: has_symmetry  ! if true structure will be imported in cell and spgin
   type(spaceg_type), intent(inout)                             :: spgin
   integer, intent(in)                                          :: radtype
   logical, intent(in)                                          :: gui_enabled    ! if true GUI is used to chose between more models
   type(error_type), intent(out)                                :: err_read
   type(bond_type), dimension(:), allocatable                   :: legmc
   type(atom_type), dimension(:), allocatable                   :: atomc
   type(pdb_atom_info_type), dimension(:), allocatable, optional, intent(out) :: at_info   ! additional info for pdb file
   integer, intent(out), optional :: nsec
   type(secondary_struct_type), dimension(:), allocatable, optional, intent(out) :: sec_struct ! secondary structure from pdb
   type(spaceg_type)                                         :: spgfile
   logical                                                   :: is_bfac,is_occ
   logical                                                   :: make_conn
   logical                                                   :: trasl
   type(error_type), dimension(:), allocatable               :: errcif
   integer                                                   :: i
   logical                                                   :: make_label
   logical                                                   :: make_occ
   logical                                                   :: is_conect
   character(len=16)                                         :: spacepdb
   integer                                                   :: nmodel
   logical, dimension(2)                                     :: optconn
   type(cell_type)                                           :: cellfile
   integer                                                   :: filetype
   integer, parameter                                        :: NMAXATOM = 500
   integer                                                   :: nblockcif
   type(crystal_phase_t), dimension(:), allocatable          :: phase_file
   integer, parameter                                        :: CODE_CIFLIST = 101
   logical :: usecov = .false.
!
   make_conn = .false.     
   is_bfac = .false.
   trasl = .false.
   make_label = .false.
   make_occ = .false.
   optconn = (/.true.,.true./)
   if (ftype == ALL_FILES) then
       filetype = code_fragfilen(filename)
   else
       filetype = ftype
   endif

   select case (filetype)

     case (FRA_FILE)    ! fra or frac file
       spgfile = init_spaceg_type()
       call read_fra_file(filename,atomc,is_bfac,is_occ,err_read)
!  
!      se si genera errore leggi il file come standard fractional file
       if (err_read%signal) then
           call clear_atoms(atomc)
           call read_fractional(filename,cell,has_symmetry,atomc,err_read)
           is_bfac = .false.
           is_occ = .false.
       else
           if (.not.has_symmetry) cell =  cell_init()
       endif
       if (.not.err_read%signal) then
           make_conn = .true.
           make_occ = .not.is_occ
           make_label = .true.
       endif

     case (XYZ_FILE)    ! cartesian file
       if (.not.has_symmetry) cell =  cell_init()
       spgfile = init_spaceg_type()
       call read_xyzfile(filename,cell,atomc,err_read)
       if (.not.err_read%signal) then
           make_conn = .true.
           make_occ = .true.
           trasl = .true.
           make_label = .true.
       endif

     case (SHELX_FILE)    ! ShelX file (.ins,.res)
       call read_shelxfile(filename,atomc,cell,has_symmetry,spgfile,err_read)
       if (.not.err_read%signal) then
           make_conn = .true.
           is_bfac = .true.
       endif

     case (PDB_FILE)    ! pdb file
       if (present(at_info) .and. present(nsec) .and. present(sec_struct)) then
           call read_PDBfile(filename,cell,has_symmetry,atomc,legmc,is_conect,spacepdb,err_read,   &
                             at_info,nsec=nsec,sec_struct=sec_struct)
       else
           call read_PDBfile(filename,cell,has_symmetry,atomc,legmc,is_conect,spacepdb,err_read)
       endif
       !cell = set_cell_type(cellpar)
       if (.not.err_read%signal) then
           spgfile = init_spaceg_type(spacepdb)
           if (is_conect)call copy_bonds(legm,legmc)
           make_conn = .not.is_conect
           is_bfac = .true.
           trasl = .true.
           make_occ = .true.
       endif

     case (CIF_FILE)    ! cif file
       call read_CIFfile(filename,phase_file,nblockcif,has_symmetry,cell,errcif)
!       call read_CIFfile(j_in,phase_file,nblockcif,has_symmetry,cellpar,errcif,etype='S',dummy=.true.)  ! for INT file generation
       if (any(errcif(:nblockcif)%signal .eqv. .false.)) then
           nmodel = 0
           allocate(vmodel_cif(nblockcif))
           vmodel_cif(:) = 0
           do i=1,nblockcif
              if (phase_file(i)%natoms() > 0) then
                  nmodel = nmodel + 1
                  vmodel_cif(nmodel) = i
              endif
           enddo
           if (nmodel > 0) then
               selected_cif = vmodel_cif(1)
               !if (nmodel > 1 .and. jscreen > 0 .and. ifAutomatic == 0) then
               if (nmodel > 1 .and. gui_enabled) then
                   call wlist_create(CODE_CIFLIST,1,nmodel,['Data block name'],'CIF file',   &
                   trim(i_to_s(nmodel))//' crystal structures. Select one of them.',2,1,cif_func,     &
                   [set_column((/(trim(phase_file(vmodel_cif(i))%cr_name)//' ('//    &
                   trim(phase_file(vmodel_cif(i))%spg%symbol_xhm)//')',i=1,nmodel)/))])
               endif
           else
               selected_cif = 1
           endif
           deallocate(vmodel_cif)
           err_read = errcif(selected_cif)
!
!          controlla la cella 
           if (.not.has_symmetry) then
               cell = phase_file(selected_cif)%cell
           endif
           if (phase_file(selected_cif)%is_spg()) then
               spgfile = phase_file(selected_cif)%spg
           else
               spgfile = init_spaceg_type()   ! force 'P 1'
           endif
           if (phase_file(selected_cif)%natoms() > 0) then
               call copy_atoms(atomc,phase_file(selected_cif)%at)
               make_conn = .true.
               is_bfac = .true.
               trasl = .false.
               make_occ = .false.
           endif
       else
           err_read = errcif(1)
       endif

     case (ZMT_FILE)    ! z-matrix
       spgfile = init_spaceg_type() 
       if (.not.has_symmetry) cell = cell_init()
       call read_zmtfile(filename,cell,atomc,err_read)
       if (.not.err_read%signal) then
           make_conn = .true.
           make_label = .true.
           make_occ = .true.
       endif

     case (MOP_FILE)    ! mop file
       spgfile = init_spaceg_type() 
       if (.not.has_symmetry) cell = cell_init()
!
!      Prova a leggere il file come mop in formato coordinate interne
       call read_mopintfile(filename,cell,atomc,err_read)
!    
!      se si genera errore leggi il file come mop a coordinate cartesiane
       if (err_read%signal) then
           call clear_atoms(atomc)
           call read_mopcartfile(filename,cell,atomc,err_read)
       endif
       if (.not.err_read%signal) then
           make_conn = .true.
           make_occ = .true.
           make_label = .true.
           trasl = .true.
       endif

     case (MOL_FILE,GAMESS_OUT,NWCHEM_OUT,ABINIT_OUT,CRYSTAL_OUT)    ! mol file, gamess output file
       if (is_espresso_file(filename)) then
           call read_qe_file(filename,atomc,cellfile,spgfile,err_read)
           if (.not.err_read%signal) then
               if (.not.has_symmetry) cell = cellfile
               make_conn = .true.
               make_label = .true.
               make_occ = .true.
           endif
       elseif (is_crystal_file(filename) > 0) then
           call read_crystal_file(filename,atomc,cellfile,spgfile,err_read)
           if (.not.err_read%signal) then
               if (.not.has_symmetry) cell = cellfile
               make_conn = .true.
               make_label = .true.
               make_occ = .true.
           endif
       else
           call OBConversionReadFile(filename,atomc,legmc,err_read)
           if (.not.err_read%signal) then
               if (.not.has_symmetry) then
                   if (has_cell) then
                       cell = set_cell_type(MyCell)
                   else
                       cell = cell_init()
                   endif
                   if(has_spg) then
                      spgfile = init_spaceg_type(spgname)
                   else
                      spgfile = init_spaceg_type() 
                   endif
               endif
               call copy_bonds(legm,legmc)
               call cart_to_frac(atomc,cell%get_ortoi())
               trasl = .true.
               make_occ = .true.
               make_label = .true.
           endif
       endif

     case (MOL2_FILE)    ! mol2 file
       call read_mol2file(filename,atomc,legmc,spgfile,cellfile,err_read)
       if (.not.err_read%signal) then
           if (.not.has_symmetry) then
               cell = cellfile
           else
               call cart_to_frac(atomc,cell%get_ortoi())
           endif
           call copy_bonds(legm,legmc)
           trasl = .true.
           make_occ = .true.
       endif

     case default 
         call err_read%set('Unknown file format. File '//trim(filename))

   end select
!
   if (err_read%signal) then
       call err_read%print()
   else
!
!      Update space group in input if was not set by user
       if (.not.has_symmetry) spgin = spgfile
!
       if (numatoms(atomc) > 0) then  ! cif files can't contain atom coordinates
           call copy_atoms(atom,atomc)
!
!          Trasla il frammento in cella: azione eseguita solo se il file contiene coord. cart.
           if (trasl) call translate_in_cell(atom)
!
!          Calcola occupanza cristallog. senza modificare le coordinate     
           call calcola_occ(atom,spgin,cell,modcoor=.false.)
           if(make_occ) atom%och = 1.0  ! inizializza occ. chimica
!
!          Update content
           call update_elem(elem,atom(:)%ptab,radtype,spgin%nsymop,atom%ocry*atom%och)
           call specie_from_ptab(atom,elem)
!
!          Creare le label
           if (make_label) call atom_string(atom,elem)  
!
!          Genera connettivita' se non esiste nel file importato
           if (make_conn) then
!              disable because coordinates could be wrong or for too atoms
               !if (err_read%warning .or. size(atom) > NMAXATOM) optconn(:) = .false.   !!FIXME - potresti usare un code error
                                                                     !optconn(:) = .false.
               if (size(atom) > NMAXATOM) optconn(:) = .false. 
               if (((filetype == PDB_FILE .or. filetype == CIF_FILE)) .and. any(atom%rcod(1) /= 0)) then  ! special connectivity for duplicate residues
                   optconn(:) = .false.
                   call create_connectivity(atom,legm,cell,spgin,check=optconn(1),move=optconn(2), &
                        vet=atom(:)%rcod(1),code=2,usecov=usecov)
                   atom%rcod(1) = 0
               else
                   call create_connectivity(atom,legm,cell,spgin,check=optconn(1),move=optconn(2),usecov=usecov)
               endif
           endif
!
!          Genera fattori termici se non esistono nel file
           if(.not.is_bfac)call set_biso(atom)
!!!
!!!          Calcola occupanza cristallog. senza modificare le coordinate     
!!           call calcola_occ(atom,spgin,cellt,modcoor=.false.)
!!           if(make_occ) atom%och = 1.0  ! inizializza occ. chimica
!
           call set_intensity(atom)
!
       else
           call clear_atoms(atom)
           call clear_bonds(legm)
       endif
!
       if (err_read%warning) call err_read%print()
   endif
!
   end subroutine import_crystal

!----------------------------------------------------------------------------------------------------------------      

   subroutine export_structure(filename,atom,cell,spg,elem,ftype,sname,radtype,wavel,legm,comm,iscell)
   USE prog_constants
   USE errormod
   USE connect_mod
   USE elements
   USE spginfom
   USE cif_frm
   USE pdb_frm
   USE shelx_frm
   USE OBmodule
   USE unit_cell
   USE atom_type_util
   USE nwchem_frm
   USE gamess_frm
   USE abinit_frm
   USE xyz_frm
   USE zmt_frm
   USE fra_frm
   USE mop_frm
   USE crystal_frm
   USE qe_frm
   USE povray_frm
   character(len=*), intent(in)                                       :: filename
   type(atom_type), dimension(:), intent(in), allocatable             :: atom
   type(cell_type), intent(in)                                        :: cell
   type(spaceg_type), intent(in)                                      :: spg
   type(element_type), dimension(:), allocatable                      :: elem
   integer, intent(in)                                                :: ftype
   character(len=*), intent(in)                                       :: sname
   integer, intent(in)                                                :: radtype
   real, intent(in)                                                   :: wavel
   type(bond_type), dimension(:), allocatable, intent(inout),optional :: legm
   character(len=*), intent(in), optional                             :: comm
   logical, intent(in), optional                                      :: iscell
   type(error_type)                                                   :: err_exp
   real, dimension(:,:), allocatable                                  :: cols
   real, dimension(3)                                                 :: colcell
   real, dimension(2)                                                 :: vfact
   real, dimension(3,3)                                               :: bmat
   integer                                                            :: filetype
   type(atom_type), dimension(:), allocatable                         :: atomc
!
   if (ftype == ALL_FILES) then
       filetype = code_fragfilen(filename)
   else
       filetype = ftype
   endif
!
   select case (filetype)
      case (SHELX_FILE)    ! .res file
        call create_shelxfile(filename,atom,legm,cell,spg,wavel,elem,sname,package_alt_name)

      case (PDB_FILE)     ! pdb file
        call create_pdbfile(filename,atom,cell,legm,spg)

      case (FRA_FILE)     ! fra file
        call create_frafile(filename,atom,cell,'Structure '//trim(sname))

      case (XYZ_FILE)    ! cartesian xyz file
        call create_xyzfile(filename,atom,cell,'Structure '//trim(sname))

      case (POV_FILE)    ! POV file
        allocate(cols(numelem(elem),3))
        call jav_get_style(cols, colcell, vfact, bmat)
        call create_povrayfile(filename,atom,legm,cell,elem,cols,colcell,iscell,bmat)
#if _WIN32
        call Csystem('C:\PROGRA~1\POV-RA~1.5\bin\pvengine.exe '//'+I'//trim(filename)//char(0))
#else
        call Csystem('povray +P +W800 +H600'//' +I'//trim(filename)//' &'//char(0))
#endif

      case (CIF_FILE)    ! CIF file
        if (present(comm)) then
            if (present(legm)) then
                call create_ciffile(filename,atom,cell,spg,elem,package_alt_name,    &
                comm,legm=legm,wave=wavel,source_type=radtype)
            else
                call create_ciffile(filename,atom,cell,spg,elem,package_alt_name,    &
                comm,wave=wavel,source_type=radtype)
            endif
        else
            if (present(legm)) then
                call create_ciffile(filename,atom,cell,spg,elem,package_alt_name,    &
                legm=legm,wave=wavel,source_type=radtype)
            else
                call create_ciffile(filename,atom,cell,spg,elem,package_alt_name,    &
                wave=wavel,source_type=radtype)
            endif
        endif

      case (ZMT_FILE)    ! zmt file (Fenske-Hall Z-Matrix)
        call create_zmtfile(filename,atom,legm,cell)

      case (MOL_FILE)    ! mol file
        call bond_type_perception(atom,legm,cell)
        call frac_to_cart_copy(atom,atomc,cell%get_ortom())
        call OBConversionWriteFile(filename,atomc,legm,cell,spg)

      case (NWCHEM_IN_DFT)
        call write_nw_file(filename,atom,cell,spg,sname,nwtheory=1)

      case (NWCHEM_IN_DFTD)
        call write_nw_file(filename,atom,cell,spg,sname,nwtheory=2)

      case (ABINIT_IN)
        call write_ab_file(filename,atom,cell,spg)

      case (CRYSTAL_IN)
        call write_crystal_file(filename,atom,cell,spg,sname)

      case (GAMESS_IN)
        call write_gmfile(filename,atom,cell,sname)

      case (GAUSSIAN_CART_IN, GAUSSIAN_ZMAT_IN)
        call frac_to_cart_copy(atom,atomc,cell%get_ortom())
        call OBConversionWriteFile(filename,atomc,legm,cell,spg)

      case (MOP_FILE)    ! cartesian mop file 
        call create_mopcrtfile(filename,atom,cell,sname)

      case (QE_IN)    
        call write_qe_file(filename,atom,cell,spg,sname)

      case default 
        call err_exp%set('Unknown file format. File '//trim(filename))
   end select
!
   if (err_exp%signal) then
       call err_exp%print()
   endif
!
   end subroutine export_structure

!----------------------------------------------------------------------------------------------------------------      

   subroutine cif_func(code)
   integer, intent(in) :: code
   select case (code)
      case (:-1)
        selected_cif = vmodel_cif(-code)
      case (0)  ! open list
      case (1)  ! Ok on list
      case (2)  ! Cancel on list
   end select
   end subroutine cif_func

!---------------------------------------------------------------------      

   integer function code_fragfilen(filedirname)   result(code)
!
!  Utilizza l'estensione del nome del file per individuare il tipo
!
   USE strutil
   USE fileutil
   character(len=*), intent(in)         :: filedirname
!
   select case (remc0(upper(get_extension(filedirname))))
     case ('FRA','FRAC')
       code = FRA_FILE

     case ('XYZ')
       code = XYZ_FILE

     case ('RES','INS')
       code = SHELX_FILE

     case ('PDB','ENT')
       code = PDB_FILE

     case ('CIF')
       code = CIF_FILE

     case ('ZMT','FH','FHZ')
       code = ZMT_FILE

     case ('MOP')
       code = MOP_FILE

     case ('MOL','MDL','SDF','SD')
       code = MOL_FILE

     case ('MOL2','ML2')
       code = MOL2_FILE

     case ('NW')
       code = NWCHEM_IN_DFT

     case ('POV')
       code = POV_FILE

     case ('OUT','LOG')
       code = GAMESS_OUT

     case ('INP','GAMIN')
       code = GAMESS_IN

     case ('COM','GAU')
       code = GAUSSIAN_CART_IN

     case ('GZMAT')
       code = GAUSSIAN_ZMAT_IN

     case ('IN')
       code = ABINIT_IN

     case ('D12')
       code = CRYSTAL_IN

     case default 
       code = 0

   end select
!
   end function code_fragfilen

!-------------------------------------------------------------------------------------------------------  

   subroutine crystal_file_import(crystal,filename,ftype,radtype,has_symmetry,err)
   USE crystal_phase
   USE spginfom
   USE unit_cell
   USE errormod
   USE fileutil
   type(crystal_phase_t), intent(inout) :: crystal
   character(len=*), intent(in)         :: filename
   integer, intent(in), optional        :: ftype
   integer, intent(in), optional        :: radtype
   logical, intent(in), optional        :: has_symmetry
   type(error_type), intent(out)        :: err
   integer                              :: ftypef,radtypef
   type(cell_type)                      :: cell
   type(spaceg_type)                    :: spg
   logical                              :: has_symmetryf
!
   if (present(ftype)) then
       ftypef = ftype
   else
       ftypef = ALL_FILES
   endif
   if (present(radtype)) then
       radtypef = radtype
   else
       radtypef = RX_SOURCE
   endif
   if (present(has_symmetry)) then
       has_symmetryf = has_symmetry
   else
       has_symmetryf = .false.
   endif
   if (has_symmetryf) then
       call import_crystal(filename,ftypef,crystal%at,crystal%bond,crystal%elem,crystal%cell,     &
                           crystal%spg,has_symmetryf,radtypef,.false.,err)
   else
       call import_crystal(filename,ftypef,crystal%at,crystal%bond,crystal%elem,                  &
                           cell,spg,has_symmetryf,radtypef,.false.,err)
   endif
   if (.not.err%signal) then
       if (.not.has_symmetryf)call crystal%set_symmetry(spg,cell)
       call crystal%set_name(file_rem_ext(file_get_name(filename)))
   endif
!
   end subroutine crystal_file_import

!----------------------------------------------------------------------------------------------------------------      

   subroutine crystal_file_export(crystal,filename,ftype,radtype,wave,comm)
   USE crystal_phase
   USE prog_constants
   type(crystal_phase_t), intent(inout)   :: crystal
   character(len=*), intent(in)           :: filename
   integer, intent(in), optional          :: ftype
   integer, intent(in), optional          :: radtype
   real, intent(in), optional             :: wave
   character(len=*), intent(in), optional :: comm
   integer                                :: ftypef,radtypef
   real                                   :: wavef
!
   if (present(ftype)) then
       ftypef = ftype
   else
       ftypef = ALL_FILES
   endif
   if (present(radtype)) then
       radtypef = radtype
   else
       radtypef = RX_SOURCE
   endif
   if (present(wave)) then
       wavef = wave
   else
       wavef = CU_WAVE
   endif
!
   if (present(comm)) then
       call export_structure(filename,crystal%at,crystal%cell,crystal%spg,crystal%elem,ftypef,  &
                             crystal%cr_name,radtypef,wavef,crystal%bond,comm,.false.)
   else
       call export_structure(filename,crystal%at,crystal%cell,crystal%spg,crystal%elem,ftypef,  &
                             crystal%cr_name,radtypef,wavef,crystal%bond,iscell=.false.)
   endif
!
   end subroutine crystal_file_export

!----------------------------------------------------------------------------------------------------------------      

   subroutine import_from_file(filename,atom,legm,elem,cell,spg,add,gui,has_symmetry,radtype,err)
   USE connect_mod
   USE atom_type_util
   USE errormod
   USE spginfom
   USE elements
   USE unit_cell
   character(len=*), intent(in)                                 :: filename
   type(atom_type), dimension(:), allocatable, intent(inout)    :: atom
   type(bond_type), dimension(:), allocatable, intent(inout)    :: legm
   type(element_type), dimension(:), allocatable, intent(inout) :: elem
!   real, dimension(6), intent(inout)                            :: cell
   type(cell_type), intent(inout)                               :: cell
   type(spaceg_type), intent(inout)                             :: spg
   logical, intent(in)                                          :: add
   logical, intent(in)                                          :: gui
   logical, intent(in)                                          :: has_symmetry
   integer, intent(in)                                          :: radtype
   type(error_type), intent(out)                                :: err
   type(atom_type), dimension(:), allocatable                   :: atom1
   type(bond_type), dimension(:), allocatable                   :: legm1
   integer                                                      :: natnew
!
   call import_crystal(filename,ALL_FILES,atom1,legm1,elem,cell,spg,has_symmetry,radtype,gui,err)
   if (err%signal) return
!
   if (add) then
       call add_atoms_to_list(atom,atom1,natnew,legm1=legm,legm2=legm1)
       call remove_duplicate_labels(atom)
   else
       call copy_atoms(atom,atom1)
       call copy_bonds(legm,legm1)
   endif
!
   !call action_after_import(filename,has_symmetry,elem,cell,spg)
!
   end subroutine import_from_file

!-------------------------------------------------------------------------------------------------

   subroutine import_fragment(strdir,atom,legm,cell,spg,elem,radtype,err,kadd,gui,has_symmetry)
!
!  Import model parsing string from command '%fragment'
!
   USE errormod
   USE atom_type_util
   USE unit_cell
   USE elements
   USE spginfom
   USE connect_mod
   character(len=*), intent(in)                                 :: strdir
   type(atom_type), dimension(:), allocatable, intent(inout)    :: atom
   type(bond_type), dimension(:), allocatable, intent(inout)    :: legm
   type(cell_type), intent(inout)                               :: cell
   type(spaceg_type), intent(inout)                             :: spg
   type(element_type), dimension(:), allocatable, intent(inout) :: elem
   integer, intent(in)                                          :: radtype
   type(error_type), intent(out)                                :: err
   logical, intent(in)                                          :: kadd,gui,has_symmetry
   type(element_type), dimension(:), allocatable                :: cform
   integer                                                      :: codef
   integer                                                      :: i
   real                                                         :: dist
   logical                                                      :: kadd1
   character(len=len_trim(strdir))                              :: smiles_string
!
   call parse_fragment_string(strdir,codef,cform,dist,smiles_string,err)
   if (.not.err%signal) then
       select case(codef)
         case (TETRA,OCTA,SQUARE,CUBE,TRIG,APRISM_TETRA,PRISM_TRIGONAL,ICOSAHEDRON)
           call make_fragment(atom,legm,cell,spg,elem,codef,0,-cform(1)%z,-cform(2)%z,kadd,dist,radtype)
         case (ISOLATED)
           kadd1 = kadd
           do i=1,numelem(cform)
              if (i > 1) kadd1 = .true.  ! add next elements
              call make_fragment(atom,legm,cell,spg,elem,ISOLATED,0,-cform(i)%z,nint(cform(i)%nw),kadd1,dist,radtype)
           enddo
         case (SMILES) 
           call get_smiles(smiles_string,atom,legm,elem,spg%nsymop,radtype,cell,kadd,err)
           if (err%signal) call err%print()
         case (0) 
           call import_from_file(strdir,atom,legm,elem,cell,spg,kadd,gui,has_symmetry,radtype,err)
       end select
   else
       call err%print()
   endif
!
   end subroutine import_fragment

!-------------------------------------------------------------------------------------------------------  

   subroutine crystal_fragment_import(crystal,str,radtype,err)
   USE crystal_phase
   USE errormod
   character(len=*), intent(in)         :: str
   type(crystal_phase_t), intent(inout) :: crystal
   integer, intent(in)                  :: radtype
   type(error_type), intent(out)        :: err
   type(cell_type)                      :: cell
   type(spaceg_type)                    :: spg
   logical                              :: kadd,gui,has_symmetry
!
   gui = .false.
   has_symmetry = .false.
   if (crystal%is_cell()) then
       has_symmetry=.true.
       cell = crystal%cell
       spg = crystal%spg
   endif
   kadd = crystal%natoms() > 0
   call import_fragment(str,crystal%at,crystal%bond,cell,spg,crystal%elem,radtype,err,kadd,gui,has_symmetry)
!corr       call crystal%print(0,'E')
!corr         return
!corr   if (.not.err%signal) then
!corr       call crystal%add_atoms(atom,bonds)
!corr       call crystal%print(0,'E')
!corr!!!!TOFIX: use method add to add atom and bonds, then check elem if new elem are added
!corr   endif
!
   end subroutine crystal_fragment_import

!-------------------------------------------------------------------------------------------------------  

   subroutine import_crystal_dir(crystal,dir,ndir,radtype,thmin,thmax,wave,err)
   USE crystal_phase
   USE spginfom
   USE unit_cell
   USE errormod
   USE strutil
   USE spginfom
   USE atom_type_util
   type(crystal_phase_t), intent(out)      :: crystal
   character(len=*), dimension(:), intent(in) :: dir
   integer, intent(in)                        :: ndir
   integer, intent(in)                        :: radtype
   real, intent(in)                           :: thmin,thmax
   real, intent(in)                           :: wave
   type(error_type), intent(out)              :: err
   integer :: i,j,zval
   character(len=:), allocatable :: sdir,word,file_known
   real, dimension(10) :: vet
   integer :: iv
   type(cell_type) :: cellnew,cellpub
   type(spaceg_type) :: spgpub
   type(atom_type), dimension(:), allocatable :: atemp
   type(bond_type), dimension(:), allocatable :: btemp
!   logical :: process_after
   integer :: nfrag,ncopies,nc
!
   file_known = ' '
!corr   process_after = .false.
   nfrag = 0
   do i=1,ndir
      sdir = trim(dir(i))
      call cutsta(sdir,line2=word)
      if (match_word(word,'autoz')) then
          zval = nint(number_of_molecules(crystal%at,crystal%cell,crystal%spg))
          if (zval > 1) then
              call crystal%get_atoms(atemp)
              call crystal%get_bonds(btemp)
              do j=1,zval-1
                 call crystal%add_atoms(atemp,btemp)
              enddo
          endif
          !write(0,*)'ZVAL=',number_of_molecules(crystal%at,crystal%cell,crystal%spg)

      elseif (match_word(word,'cell')) then
         call getnum(sdir,vet,iv=iv)
         if (iv == 6) then
             cellnew = set_cell_type(vet(:6))
             if (crystal%natoms() > 0) then
                 call coord_in_newcell(crystal%at,crystal%cell,cellnew)
             endif
             call crystal%set_unit_cell(cellnew)
         else
             call err%set('Error reading directive '//trim(dir(i)))
         endif

      elseif (match_word(word,'file')) then
         call crystal_file_import(crystal,sdir,ALL_FILES,radtype,err=err)
         if (err%signal) exit  ! error already printed

      elseif (match_word(word,'frag')) then
         nfrag = nfrag + 1
         call read_dir_numfrag(dir,ndir,nfrag,ncopies,err)
         if (.not.err%signal) then
             do nc=1,ncopies
                call crystal_fragment_import(crystal,sdir,radtype,err)
                if (err%signal) exit
             enddo
         endif

      elseif (match_word(word,'know')) then
         file_known = sdir
         !write(0,*)'KNOWN STRUCT:',sdir

      elseif (match_word(word,'name')) then
         call crystal%set_name(sdir)

      elseif (match_word(word,'space')) then
         call crystal%set_spg(sdir,err)

      elseif (match_word(word,'numfrag')) then
         cycle

      else
         if (i == 1) then
             call crystal_file_import(crystal,dir(i),ALL_FILES,radtype,err=err)
             if (err%signal) exit  ! error already printed
         else
             call err%set('Error reading directive '//trim(dir(i)))
         endif

      endif

      if (err%signal) then
          call err%print()
          return
      endif
   enddo
!
   call crystal%make_reflections(thmin,thmax,wave)
!
!corr   if (process_after) then
!corr       call read_dir_numfrag(dir,ndir,err)
!corr   endif
!
!  Process directive known
   if (len_trim(file_known) > 0) then
       call import_crystal(file_known,ALL_FILES,atemp,btemp,crystal%elem,cellpub,spgpub,.false.,radtype,.false.,err)
       call crystal%set_as_known(atemp,radtype)
   endif
!
   end subroutine import_crystal_dir 

!-------------------------------------------------------------------------------------------------

   subroutine read_dir_numfrag(dir,ndir,kfrag,ncopies,err)
!
!  Process directive that modify the number of atoms
!
   USE crystal_phase
   USE strutil
   USE fragmentmod
   USE errormod
   character(len=*), dimension(:), intent(in) :: dir
   integer, intent(in)                        :: ndir
   integer, intent(in)                        :: kfrag
   integer, intent(out)                       :: ncopies
   type(error_type), intent(out)              :: err
   integer                                    :: i,iv
   character(len=:), allocatable              :: sdir,word
   integer, dimension(:), allocatable         :: ivet
!
   ncopies = 1
   do i=1,ndir
      sdir = trim(dir(i))
      call cutsta(sdir,line2=word)
      if (match_word(word,'numfrag')) then
          call getnum1(sdir,ivet=ivet,iv=iv)
          if (iv /= 2) go to 10
          if (ivet(1) == kfrag) then
              !if (ivet(2) == 1) cycle
              ncopies = ivet(2)
              !write(0,*)'Repeat frag. ',kfrag,ivet(2)
          endif
!corr          if (crystal%numfrag() == 0) call crystal%make_fragments()
!corr          if (ivet(1) > crystal%numfrag()) go to 10
!corr          if (ivet(2) == 0) then
!corr              crystal%frag(ivet(1))%nat = 0 ! set 0 to delete after
!corr          else
!corr              call duplicate_fragment(crystal%at,crystal%bond,crystal%elem,crystal%frag,ivet(1),ivet(2)-1)
!corr          endif
      endif
   enddo
!
!  delete fragment
!corr   i=0
!corr   do 
!corr      i = i + 1
!corr      if (crystal%frag(i)%nat == 0) call delete_fragment(crystal%at,crystal%bond,crystal%frag,i)
!corr      if (i == crystal%numfrag()) exit
!corr   enddo
!
   return
!
10 call err%set('Error reading directive '//sdir)
!
   end subroutine read_dir_numfrag

!-------------------------------------------------------------------------------------------------

   subroutine make_fragment(atom,legm,cell,spg,elem,itipo,kpos,kspec1i,kspec2i,kadd,bd,radtype)
   USE atom_type_util
   USE unit_cell
   USE spginfom
   USE connect_mod
   USE elements
   USE unit_cell
!
   type(atom_type), dimension(:), allocatable, intent(inout)    :: atom   
   type(bond_type), dimension(:), allocatable, intent(inout)    :: legm
   type(cell_type), intent(in)                                  :: cell
   type(spaceg_type), intent(in)                                :: spg
   type(element_type), dimension(:), allocatable, intent(inout) :: elem
   integer, intent(in)                                          :: itipo         ! tipo di frammento
   integer, intent(in)                                          :: kpos          ! posizione del frammento
   integer, intent(in)                                          :: kspec1i,kspec2i ! specie nel frammento
   logical                                                      :: kadd          ! true se il frammento va aggiunto in atom
   real, intent(in)                                             :: bd            ! distanza 
   integer, intent(in)                                          :: radtype
   type(atom_type), allocatable, dimension(:)                   :: atom1
   integer                                                      :: i
   real, dimension(3)                                           :: xpos
   integer                                                      :: kkpos
   logical                                                      :: centr
   integer                                                      :: n1
   integer                                                      :: nat
   integer                                                      :: nadd,nlegadd
   real, dimension(3)                                           :: xrand
   logical                                                      :: addnew1,addnew2
   integer                                                      :: kspec1,kspec2
   integer                                                      :: newsize
!
   nat = numatoms(atom)
!
!  If kspec is negative add new elements to the content (Z number = -kspec)
   if (kspec1i < 0 .or. kspec2i < 0) then
       addnew1 = .false.
       if (kspec1i < 0) then
!
!          set how many atoms to add
           select case (itipo)
               case (TETRA,SQUARE)     
                 newsize = 5
               case(OCTA,PRISM_TRIGONAL)
                 newsize = 7
               case(CUBE,APRISM_TETRA)
                 newsize = 9
               case(TRIG)
                 newsize = 4
               case(ICOSAHEDRON)
                 newsize = 13
               case(ISOLATED)
                 newsize = kspec2i
           end select
           if (kpos > 0 .and. itipo /= ISOLATED) newsize = newsize - 1
!
           call add_element(elem,abs(kspec1i),nw=newsize*spg%nsymop,add=addnew1,wave=radtype)
           kspec1 = is_element(elem,abs(kspec1i)) 
       endif
       addnew2 = .false.
       if (kspec2i < 0) then
           call add_element(elem,abs(kspec2i),nw=spg%nsymop,add=addnew1,wave=radtype)
           kspec2 = is_element(elem,abs(kspec2i))  
       endif
   endif
   if (kspec1i > 0) kspec1 = kspec1i
   if (kspec2i > 0) kspec2 = kspec2i
!
!  Posizione del frammento
   if (kpos == 0) then
       call random_number(xpos)  ! in posizione random
       kkpos = -1
       centr = .true.
   else
       xpos = atom(kpos)%xc     ! nella posizione dell'atom kpos
       kkpos = kpos - nat      ! serve ad allineare la connettivita
       centr = .false.
   endif
!
!  crea il frammento
   select case(itipo)
       case(TETRA)              ! Tetrahedron
         call make_tetrahedron(atom1,bd,cell,xpos=xpos,kcentr=kkpos,centro=centr)

       case(OCTA)               ! Octahedron
         call make_octahedron(atom1,bd,cell,xpos=xpos,kcentr=kkpos,centro=centr)

       case(SQUARE)             ! Square Plane
         call make_square(atom1,bd,cell,xpos=xpos,kcentr=kkpos,centro=centr)

       case(ISOLATED)           ! Isolated Atoms
         if (kadd) then
             n1 = nat + 1
             nat = kspec2 + nat
         else
             n1 = 1
             nat = kspec2
             call clear_bonds(legm)
         endif
         call resize_atoms(atom,nat)
         do i=n1,nat
            call random_number(atom(i)%xc)
            call atom(i)%set_specie_from_el(kspec1,elem)
         enddo
 
       case (CUBE)                ! Cube
         call make_cube(atom1,bd,cell,xpos)

       case (TRIG)                ! Trigonale Plane
         call make_trigonal(atom1,bd,cell,xpos)

       case (APRISM_TETRA)        ! Antiprism Tetragonal
         call make_anti_prism_tetragonal(atom1,bd,cell,xpos)

       case (PRISM_TRIGONAL)      ! Prism Trigonal
         call make_prism_trigonal(atom1,bd,cell,xpos)

       case (ICOSAHEDRON)         ! Icosahedron
         call make_icosahedron(atom1,bd,cell,xpos)

   end select
!
!  assegna nz
   if (itipo /= ISOLATED) then
       if (kpos == 0) then
           call atom1(1)%set_specie_from_el(kspec1,elem)
           call specie_from_el(atom1,kspec2,elem,2)
       else
           call specie_from_el(atom1,kspec2,elem)
       endif
!
!      combina i 2 modelli aggiungendo il nuovo frammento in fondo alla lista
       if (kadd) then
           nadd = size(atom1)   ! numero atomi da aggiungere
!
!          Applica una rotazione random intorno al baricentro per kpos/=0 
!          In questo modo frammenti centrati sullo stesso atomo (kpos/=0) non vanno in sovrapposizione
           if (itipo /= 4 .and. kpos /=0) then
               call random_number(xrand)
               call rand_rotate_atoms(atom1,xrand,cell)
           endif
!
           call add_atoms_to_list(atom,atom1,nat)
           n1 = nat - nadd + 1  ! puntatore al primo atomo aggiunto
           if (kpos == 0) then
               nlegadd = nadd - 1   ! numero legami da aggiungere
               call add_bonds(legm,atom,(/(n1,i=1,nlegadd)/),(/(i+n1,i=1,nlegadd)/),cell)
           else
               nlegadd = nadd       ! numero legami da aggiungere
               call add_bonds(legm,atom,(/(kpos,i=1,nlegadd)/),(/(i+n1-1,i=1,nlegadd)/),cell)
           endif
       else
           call copy_atoms(atom,atom1)
           call clear_bonds(legm)
           nat = size(atom)
           n1 = 1
           call add_bonds(legm,atom,(/(n1,i=1,nat-1)/),(/(i+n1,i=1,nat-1)/),cell)
       endif
   endif
!
!  Calcola occupanza cristallog. senza modificare le coordinate     
   call calcola_occ(atom(n1:nat),spg,cell,modcoor=.false.)
   atom(n1:nat)%och = 1.0  ! inizializza occ. chimica
!
   call set_biso(atom(n1:nat))
!
!  Assegna intensita in base a Z
   call set_intensity(atom(n1:nat))
!          
   call atom_string(atom,elem,vet=(/(i,i=n1,nat)/))
!
   end subroutine make_fragment

!-------------------------------------------------------------------------------------------------
 
   subroutine parse_fragment_string(str,code,cform,dist,strf,err) 
!
!  Parse string associated to command '%fragment'
!
   USE strutil
   USE elements
   USE errormod
   USE connect_mod
   character(len=*), intent(in)                               :: str
   integer, intent(out)                                       :: code
   real, intent(out)                                          :: dist
   character(len=*), intent(out)                              :: strf
   type(error_type)                                           :: err
   type(element_type), dimension(:), allocatable, intent(out) :: cform
   character(len=len_trim(str))                               :: str1,word
   integer                                                    :: ier,nlen2
   logical                                                    :: poly
   integer                                                    :: spec1,spec2
!
   code = 0
   spec1 = 0
   spec2 = 0
   dist = 0.0
!
   poly = .false.
   str1 = trim(str)
   call cutst(str1,line2=word,nlong2=nlen2)
   if (nlen2 == 0) return
   select case(lower(word))
      case ('tetra')
         code = TETRA
         poly = .true.
      case ('octa')
         code = OCTA
         poly = .true.
      case ('square')
         code = SQUARE
         poly = .true.
      case ('cube')
         code = CUBE
         poly = .true.
      case ('trigonal')
         code = TRIG
         poly = .true.
      case ('prism_tetra')
         code = APRISM_TETRA
         poly = .true.
      case ('prism_trig')
         code = PRISM_TRIGONAL
         poly = .true.
      case ('icosa')
         code = ICOSAHEDRON
         poly = .true.
      case ('atoms')
         code = ISOLATED
         call read_chemformula(str1,ier,cform)
         if (ier /= 0) goto 10
      case ('smiles')
         code = SMILES
         strf = trim(str1)
   end select
!
   if (poly) then
       call cutst(str1,line2=word)
       spec1 = z_from_specie(word)
       if (spec1 == 0) goto 10
       call add_element(cform,spec1)
       call cutst(str1,line2=word)
       spec2 = z_from_specie(word)
       if (spec2 == 0) goto 10
       call add_element(cform,spec2)
       if (len_trim(str1) > 0) then
           ier = s_to_r(str1,dist)
           if (ier /= 0) go to 10
       endif
       if (dist == 0.0) dist = bond_distanceZ(spec1,spec2)
   endif

   return

10 call err%set("Error reading directive: '"//trim(str)//"'")
!
   end subroutine parse_fragment_string

!-------------------------------------------------------------------------------------------------

   subroutine import_from_smiles(smiles_string, atom, legm, elem, nsymtot, radtype, cell, err)
   USE atom_type_util
   USE OBmodule
   USE elements
   USE errormod
   USE unit_cell
   character(len=*), intent(in)                                 :: smiles_string
   type(atom_type), dimension(:), allocatable, intent(out)      :: atom
   type(bond_type), dimension(:), allocatable, intent(out)      :: legm
   type(element_type), dimension(:), allocatable, intent(inout) :: elem  
   integer, intent(in)                                          :: nsymtot, radtype
   type(cell_type), intent(in)                                  :: cell
   type(error_type), intent(out)                                :: err
!
   call OBConversionFromSmiles(smiles_string,atom,legm,err)
   if (err%signal) return
!
   call update_elem(elem,atom(:)%ptab,radtype,nsymtot)  ! update content
   call specie_from_ptab(atom,elem)
   call atom_string(atom,elem)                         ! label
   call set_biso(atom)                                 ! adp
   call set_intensity(atom)                            ! int.
   call cart_to_frac(atom,cell)
!
   end subroutine import_from_smiles

!-------------------------------------------------------------------------------------------------

   subroutine get_smiles(smiles_string,atom,legm,elem,nsymtot,radtype,cell,add,err)
   USE atom_type_util
   USE elements
   USE errormod
   USE unit_cell
   USE connect_mod
   character(len=*), intent(in)                                 :: smiles_string
   type(atom_type), dimension(:), allocatable, intent(inout)    :: atom
   type(bond_type), dimension(:), allocatable, intent(inout)    :: legm
   type(element_type), dimension(:), allocatable, intent(inout) :: elem  
   integer, intent(in)                                          :: nsymtot, radtype
   type(cell_type), intent(in)                                  :: cell
   logical, intent(in)                                          :: add
   type(error_type), intent(out)                                :: err
   type(atom_type), dimension(:), allocatable                   :: atom1
   type(bond_type), dimension(:), allocatable                   :: legm1
   integer                                                      :: natnew,natcurr
!
   call import_from_smiles(smiles_string,atom1,legm1,elem,nsymtot,radtype,cell,err)
   if (err%signal) return
   if (add) then  ! add smiles to current model
       natcurr = numatoms(atom)
       call add_atoms_to_list(atom,atom1,natnew,legm1=legm,legm2=legm1)
       call remove_duplicate_labels(atom)
   else
       call copy_atoms(atom,atom1)
       call copy_bonds(legm,legm1)
   endif
!
   end subroutine get_smiles

!-------------------------------------------------------------------------------------------------------  

   subroutine make_powder_pattern_var(xc,yc,ref,atom,spg,cell,elem,thmin,thmax,thstep,wave,fwhm,scal)
   USE atom_basic
   USE unit_cell
   USE crystal_phase
   USE calculate_pdp
   USE reflection_type_util
   USE pearsonf
   USE psvoigtf
   USE tchzf
   USE profile_function
   USE arrayutil
   USE spginfom
   USE progtype, only: refine_condition_type
   real, dimension(:), allocatable, intent(inout)                  :: xc,yc
   type(reflection_type), dimension(:), allocatable, intent(inout) :: ref
   type(atom_type), dimension(:), allocatable, intent(in)          :: atom
   type(spaceg_type), intent(in)                                   :: spg
   type(cell_type), intent(in)                                     :: cell
   type(element_type), dimension(:), allocatable, intent(in)       :: elem
   real, intent(in)                                                :: thmin,thmax,thstep
   real, intent(in)                                                :: wave,fwhm,scal
   type(crystal_phase_t)                                        :: crystal
   integer                                                         :: nc,i
   real                                                            :: tmax
   type(refine_condition_type)                                     :: rfc
   type(pearson_function_type)                                     :: pear
   type(pvoigt_function_type)                                      :: pvoi
   type(tch_function_type)                                         :: tch
   integer                                                         :: ier
   type(spaceg_type)                                               :: p1
!
   if (spg%undef()) then
       call p1%set_p1()  ! force undefined space group to P1
       call crystal%set(atom,spg=p1,cell=cell,elem=elem)
   else
       call crystal%set(atom,spg=spg,cell=cell,elem=elem)
   endif
!
!  Make series tmin,tstep 
!!!!!FIXME - use a make_series from arrayutil or from NR
   nc = floor((thmax - thmin) / thstep + 1)   ! number of counts
   call new_array(xc,nc)
   xc(:) = thmin + [(i*thstep,i=0,nc-1)]     
   tmax = xc(nc)
   call new_array(yc,nc)
   !write(0,*)'NC=',nc,' thmin=',thmin,' thmax=',thmax,' step=',thstep
!
   call crystal%make_reflections(thmin,tmax,wave)
   call crystal%sfactor()
   call crystal%LPcorr(0,0)
   !call crystal%print(0,'SCAR')
!
!!!!!FIXME - use a calculate_powder_pattern_crystal
   rfc%lim%ncu = nc;  rfc%lim%nrefu=numrefl(crystal%ref); rfc%typefun = PEARSON7
!corr   call AllocaRietnew(1,rfc%lim%nrefu)
   call new_array(rapIcalc,rfc%lim%nrefu)
   rapIcalc(:) = 1.0
   call init_function_param(pear,pvoi,tch,fwhm)
   call calculate_powder_pattern(rfc,xc,yc,crystal%ref,crystal%scal,1,0.0,pear,pvoi,tch,ier)
   yc = scal * yc / maxval(yc)
   call delete_array(rapIcalc)
!corr   call AllocaRietnew(0,rfc%lim%nrefu)
   call crystal%get_reflections(ref)

   !call spg%prn(70)
   !call crystal%print(70,'SCARE')
   !call write_reflections(70,refl=ref,code=5)
   !do i=1,nc
   !   write(71,*)xc(i),yc(i)
   !enddo
!
   end subroutine make_powder_pattern_var

!--------------------------------------------------------------------------------------

   subroutine make_powder_pattern_xpd(xc,yc,ref,atom,spg,cell,elem,xpd)
   USE unit_cell
   USE spginfom
   USE elements
   USE atom_basic
   USE reflection_type_util
   real, dimension(:), allocatable, intent(inout)                  :: xc,yc
   type(reflection_type), dimension(:), allocatable, intent(inout) :: ref
   type(atom_type), dimension(:), allocatable, intent(in)          :: atom
   type(spaceg_type), intent(in)                                   :: spg
   type(cell_type), intent(in)                                     :: cell
   type(element_type), dimension(:), allocatable, intent(in)       :: elem
   type(xpd_option_type), intent(in)                               :: xpd

   call make_powder_pattern_var(xc,yc,ref,atom,spg,cell,elem,xpd%ttmin,xpd%ttmax,xpd%ttstep,xpd%wave,xpd%fwhm,xpd%scal)

   end subroutine make_powder_pattern_xpd

!--------------------------------------------------------------------------------------

   subroutine make_powder_pattern_cmd(cmd,atom,spg,cell,elem)
   USE commandsmod
   USE unit_cell
   USE spginfom
   USE atom_basic
   USE elements
   USE reflection_type_util
   USE prog_constants
   type(command_type), intent(in)                            :: cmd
   type(atom_type), dimension(:), allocatable, intent(in)    :: atom
   type(spaceg_type), intent(in)                             :: spg
   type(cell_type), intent(in)                               :: cell
   type(element_type), dimension(:), allocatable, intent(in) :: elem
   real, dimension(:), allocatable                           :: xpd,ypd
   type(reflection_type), dimension(:), allocatable          :: refpd
   !real :: ttmin=5., ttmax=70., ttstep=0.02, wave=CU_WAVE, fwhm=0.1
   type(xpd_option_type) :: xpdopt
!
   call cmd%prn(0)

   if (cmd%ndir == 0) return

   if (cmd%ndir > 0) then
       !corrcall make_powder_pattern(xpd,ypd,refpd,atom,spg,   &
       !corr                     cell,elem,ttmin,ttmax,ttstep,wave,fwhm)
       call make_powder_pattern(xpd,ypd,refpd,atom,spg,cell,elem,xpdopt)
!!!FIXME - use testc.exp
!!!!!FIXME - use a make_series from arrayutil in make_powder_pattern
!!!!!FIXME - use a calculate_powder_pattern_crystal in make_powder_pattern
!!!FIXME - meglio non sovrascivere file esistenti
       call export_profile(cmd%strdir(1),xpd,ypd,string='#      2theta    yoss')  
!!!!!FIXME - con piu' cif crea prima il metodi from_file per la classe cristal e poi overloda make_powder_pattern
!!!!!FIXME - con crystal. la vecchia make_powder_patter chiamera la prima
   endif
!
   end subroutine make_powder_pattern_cmd

!-------------------------------------------------------------------------------------------------------  

   subroutine export_profile(fname,x,yoss,ycalc,yback,string)
   USE fileutil
   character(len=*), intent(in), optional   :: fname
   real, dimension(:), intent(in)           :: x,yoss
   real, dimension(:), intent(in), optional :: ycalc,yback
   character(len=*), intent(in), optional   :: string
   integer                                  :: i
   integer                                  :: j_in
   type(file_handle)                        :: fpatt
!
   call fpatt%fopen(fname,'w')
!
   if (fpatt%good()) then
       j_in = fpatt%handle()
       if (present(string)) write(j_in,'(a)')trim(string)
       if (present(ycalc) .and. present(yback)) then
          write(j_in,'(f15.3,1x,f15.3,f15.6,1x,f15.6,1x,f15.6)')   &
               (x(i),yoss(i),ycalc(i),yback(i),yoss(i)-ycalc(i),i=1,size(x))
       else
          write(j_in,'(f15.3,1x,f15.3)')(x(i),yoss(i),i=1,size(x))
       endif
       call fpatt%fclose()
   endif
!
   end subroutine export_profile

!----------------------------------------------------------------------------------------------------

   subroutine load_chemical_tables(fpath,err)
   USE elements
   USE bondtmod
   USE bond_valence
   USE errormod
   USE spginfom
   character(len=*), intent(in)  :: fpath
   type(error_type), intent(out) :: err
!
   call read_chemical_elements(filexen_name(fpath),err)
   if (err%signal) return
   call spg_set_symmetry_file(trim(fpath)//'syminfo.lib',err)
   if (err%signal) return
   call load_bond_table()
   call load_bval_table()
!
   end subroutine load_chemical_tables

!----------------------------------------------------------------------------------------------------

   subroutine crystal_find_contacts(crystal,atoms,bonds,lsym,expand,hbond,shbond)
   USE crystal_phase
   USE contacts
   type(crystal_phase_t), intent(in)                                    :: crystal
   type(atom_type), dimension(:), allocatable, intent(inout)            :: atoms  ! atoms with symmetry
   type(bond_type), dimension(:), allocatable, intent(inout)            :: bonds  ! bonds with simmetria
   logical, intent(in)                                                  :: lsym   ! packing 
   integer, dimension(2), intent(in)                                    :: expand ! expand contacts
   type(bond_type), dimension(:), allocatable, intent(inout), optional  :: hbond  ! H-bonds
   type(bond_type), dimension(:), allocatable, intent(inout), optional  :: shbond ! short contacts
   type(bond_type), dimension(:), allocatable                           :: shbond1,hbond1
   logical                                                              :: lhbond,lshbond
!
   lhbond = present(hbond)
   lshbond = present(shbond)
   if (lhbond .and. lshbond) then
       call find_contacts(crystal%at,crystal%bond,crystal%cell,crystal%spg,hbond,shbond,atoms,bonds,lsym,expand,lhbond,lshbond)
   else
       if (lhbond) then
           call find_contacts(crystal%at,crystal%bond,crystal%cell,crystal%spg,hbond,shbond1,atoms,bonds,lsym,expand,lhbond,lshbond)
       elseif (lshbond) then
           call find_contacts(crystal%at,crystal%bond,crystal%cell,crystal%spg,hbond1,shbond,atoms,bonds,lsym,expand,lhbond,lshbond)
       endif
   endif
!
   end subroutine crystal_find_contacts
end module gen_frm
