#include <sstream>
#include <fstream>
#include <iostream> 

#ifndef USE_OPENBABEL
#define USE_OPENBABEL 1
#endif
#ifdef HAVE_GTKGLEXT
#ifdef GTK_DISABLE_SINGLE_INCLUDES
#undef GTK_DISABLE_SINGLE_INCLUDES
#include <gtk/gtkgl.h>
#include <gdk/gdkglglext.h>
#define GTK_DISABLE_SINGLE_INCLUDES 1
#else
#include <gtk/gtkgl.h>
#include <gdk/gdkglglext.h>
#endif
#endif

#ifdef HAVE_OPENGL_GL_H
#include <OpenGL/gl.h>
#elif defined (HAVE_GL_GL_H)
#include <GL/gl.h>
#endif
#ifdef GDK_WINDOWING_X11
#ifdef HAVE_OPENGL_GLX_H
#include <OpenGL/glx.h>
#elif defined(HAVE_GL_GLX_H)
#include <GL/glx.h>
#endif
#endif

#if defined (_WIN32)
#include <GL/glew.h>
#include <GL/wglew.h>
#define WM_MOUSEWHEEL                   0x020A
#define GET_WHEEL_DELTA_WPARAM(wParam)  ((short)HIWORD(wParam))
#include <windowsx.h>
#endif

#define USE_CAIRO_TEXTURES 1
#ifndef USE_CAIRO
#undef USE_CAIRO_TEXTURES
#endif

#include <gtkdefs.h>

#define jav_msgdefault    " Welcome to jav"
#define SettingsWindow    "settings_window"
#define SequenceWindow    "sequence_window"
#define MapInfoWindow     "mapinfo_window"
#define InfoWindow        "info_window"
#define InfoAtomsW        "info_atoms"
#define InfoOverlay       "info_overlay"
#define EditAtomsW        "edit_atoms"
#define EditBondsW        "edit_bonds"
#define EditAnglesW       "edit_angles"
#define EditTorsW         "edit_tors"
#define TransformW        "transform_model"
#define HBOptWindow       "edit_hbopt"
#define BondSettingWindow "bond_setting"
#define EditHDist         "edit_hdist"
#define BondValenceSumWindow "bond_vs"

#define SCALA        1.2f
#define MAX_SPECIES  40
#define MAX_LABLEN   25
#define MAX_NUM_MODELS   5
#define MAX_NUM_MAPS     5
#define NATS_LIM         1000
enum plotstyles {WIRES, SIMPLE, BONDSTYLE, RODS, STICK_AND_BALLS, ELLIPSOIDS,
                 VANDERWAALS, POLYPEPTIDE, C_ALPHA, RIBBONS, STRANDS,
                 CARTOONS, PLOT_STYLES};
enum colormodes {BY_SPECIES, BY_B, BY_SOF, BY_SYMMETRY, MONOC, BY_RESIDUE,
                  BY_CHAIN, BY_STRUCT, BY_GROUP, COLOR_MODES};
enum glasses_types {REDBLUE, REDGREEN, REDCYAN, BLUERED, GREENRED, CYANRED, GLASSES_TYPES};
enum popup_types {ATOMS_POPUP, BONDS_POPUP, CALPHA_POPUP, BB_POPUP, LIG_POPUP, RESID_POPUP, LEGEND_POPUP};
enum bg_types {TYPE_BLACK, TYPE_WHITE, TYPE_FLAT, TYPE_HGRAD, TYPE_VGRAD, TYPE_DGRAD, TYPE_RGRAD};

typedef float myreal;

typedef struct {
                 int   nsymop, latt, icent;
                 char  spg[17];
               }_SymmInfo;

typedef struct {
                 unsigned long ulDimensionX;
                 unsigned long ulDimensionY;
                 int            n_channels;
                 int            rowstride;
                 GLenum         format;
                 guchar         *pcData;
               } Texture ;

typedef struct {
                 bool enabled;
                 float diffuse[4];
                 float specular[4];
                 float position[4];
               } _LightInfo;

enum lights {JAV_LIGHT0=0, JAV_LIGHT1, JAV_AMBIENT};

typedef float atom_color[4];

typedef struct {
                 bool  visible;
                 int   transparency;
                 bool  ifBorder;
                 float colore[4];
                 float Bcolore[4];
                 float line_width;
                 int   vert;
               } polyhedron_style;

typedef struct {
    int                 nspec;
    int                 zspec[MAX_SPECIES],vspec[MAX_SPECIES];
    float               cell[6];
    char                spaceg[16];
    float               cell_volume;
    float               volume_per_atom;
    float               r_factor;
    int                 icent;
    int                 nsymmstr;
               }  structure_info_type;

typedef struct {
      int                        z             ;// atomic number Z
      char                       name[20]      ;// name
      char                       lab[8]        ;// label
      float                      weight        ;// atomic weight
      float                      c_radius      ;// covalent radius
      float                      w_radius      ;// van der Waals radius
      float                      ax[4]         ;// coefficients a per calcolo scattering factors (x-ray)
      float                      bx[4]         ;// coefficients b per calcolo scattering factors (x-ray)
      float                      cx            ;// coefficients c per calcolo scattering factors (x-ray)
      float                      fMo[3]        ;// f',f'',mu/rho del Molibdeno
      float                      fCu[3]        ;// f',f'',mu/rho del Rame
      float                      ae[4]         ;// coefficients a per calcolo scattering factors (electron)
      float                      be[4]         ;// coefficients b per calcolo scattering factors (electron)
      float                      ce            ;// coefficients c per calcolo scattering factors (electron)
      float                      fact          ;// fattore di scattering dei neutroni
      float                      al[4]         ;// used scattering factors
      float                      bs[4]         ;// used scattering factors
      float                      cl            ;// used scattering factors
      float                      nw            ;// number of elements
      int                        radtype       ;// tipo di radiazione usata
      int                        charge        ;// charge 
      int                        ptab          ;// pointer to table of elements
      float                      zeff;
      float                      ifMottFormula;
               } element_type;

typedef struct {
                  int op;
                  int wM[4], wA[4];
                  float result;
               }  DistAng_type;

namespace GLextmr
{
   class GLWindow {
   public:
      GLWindow(GtkWidget *wid, int w, int h);
      void Destroy(int Flag);
      size_t truesize();
      void freeStructs();
      void Save() ;
      void Save(std::string &strfn);
      void Save(std::string &strfn, bool binary);
      void SaveSurfaces(std::ofstream &f);
      void SaveSurfaces(FILE *fp);
      void SaveMaps(std::ofstream &f);
      void SaveMaps(FILE *fp);
      void SaveModels(std::ofstream &f);
      void SaveModels(FILE *fp);
      void SaveLegends(std::ofstream &f);
      void SaveLegends(FILE *fp);
      bool Restore();
      bool Restore(std::string &strfn);
      bool Restore(std::string &strfn, bool binary);
      bool RestoreMaps(std::ifstream &f);
      bool RestoreMaps(FILE *fp);
      bool RestoreSurfaces(std::ifstream &f);
      bool RestoreSurfaces(FILE *fp);
      bool RestoreModels(std::ifstream &f);
      bool RestoreModels(FILE *fp);
      bool RestoreLegends(std::ifstream &f);
      bool RestoreLegends(FILE *fp);
      void Redraw();
      void DrawMaps();
      void DrawModels();
      void DrawSurface(int k);
      void DrawDistAng();
      void CheckDistAng();
      void RunMCAll();
      void DrawGrid();
      void DrawCell();
      void DrawAxes();
      void DrawLegend();
      void DrawSelection();
      void UpdateTitle();
      void UpdateSigma();
      void EnableClip();
      void DisableClip();
      void PrintMess(const char *stringa);
      vet3 Trasforma(vet3 num);
      void NormalizzasuModello(int k, bool fResize);
      void NormalizzasuMappa(int k, bool ifFromC=false);
      void NormalizzasuCella();
      void RiNormalizzasuModello(bool fResize);
      void RiNormalizzasuMappa();
      void RiNormalizza(bool fResize=false);
      void NormalizzaModelli();
      void calcolaO(std::vector<double> mini, std::vector<double> maxi);
      void CalcolaMatrici();
      void SetBgColor(bool aggiorna=true);
      void SetOrtho(int c);
      void ResetView(bool azzera=false, bool redraw=true);
      void DrawSplash();
      void RenderScene();
      void ResetModelMatrix(int sign);
      void ResetProjectionMatrix(float factor);
      void ResetProjectionMatrix(float *pos1, float *pos2);
      void SetViewpoint(float factor);
      void SetViewpoint(float *pos1, float *pos2);
      void ViewPlane(int whichPlane, bool redraw=true);
      void ToggleCell(int flag);
      void ToggleGrid(int flag);
      void ToggleFog(int flag);
      void ToggleStereo(int flag);
      void ToggleTb(int flag);
      void ToggleLegend(int flag);
      void DestroyLegend();
      void change_clip_value();
      void on_rotate(int axis, float sign);
      void on_move(int axis, float val);
      void on_move_center(int axis, int sign);
      void on_move_centerf(int axis, float sign);
      float *FromPixel(int x, int y);
      float *FromPixel3D(int x, int y);
      void SetSphereCentre(int wm, int wa, bool redr=true);
      void SetSphereRadius(float r, bool redr=true);
      void SetLightForSelected(int b=0, float *mat_e = NULL, float alpha=0.4f);
      void ResetLightForSelected();
      void drawSphere();
      bool ManageClickLegend(int x, int y, int button);
      void DrawBackground();
      bool MakeCurrent();
      void EndCurrent();
      int  GetCap(std::vector<int> &ns);
      void SetCap(GtkWidget *w);
      int  GetCurNSamples();
      void mySwapBuffers();
      void SetLights();
      void SetLightsAndDraw();
      void ReleaseWinStuff();
      void DefaultLights(_LightInfo l_info[]);
      void CreateCairoTexture(Texture *pTexture);
      gboolean ManageMousePress(int x, int y, int butt,
                                 bool ifCtrl, bool ifShift);
      gboolean ManageMouseRelease(int x, int y, int butt);
      gboolean ManageMouseScroll(bool ifScrollUp);
      gboolean ManageMouseMove (int x, int y, int butt);
      bool RenderingInfo(std::string &rendering_info);
      void AggiornaListaSpecie();
      bool poliedri();
      void CalcPar(vet3 v1, vet3 abc);
      void DeleteAllMapModels();
      void IsAtom(int x, int y);
      vet3 FractToCart(float coord[3]);
      vet3 CartToFract(float coord[3]);
      vet3 FractToCart(vet3 coord);
      vet3 CartToFract(vet3 coord);

      GtkWidget *Widget;
      int inModify;
      int Normalized;
      int border;
      float aspectRatio;
      float Scale;
      double ClipFactor;
      float  FogDensity;
      int subdivisions;
      int Height;
      int Width;
      int smooth;                // line smooth for antialiased lines
      bool CenterOnMap;
      bool Clip;
      bool Stereo;
      bool ShowTb;
      int  glasses_type;
      int Xstart;
      int Ystart;
      bool ifOrtho;
      bool ifCell;
      bool ifGrid;
      int  nDivisions;
      bool normalToV;
      int  GridPattern;
      bool fog;
      float xmin;
      float xmax;
      float ymin;
      float ymax;
      float zmin;
      float zmax;
      float deltav;
      vet3  Xd;
      vet3  obj_pos;
      vet4  view_quat_diff;
      vet4  view_quat;
      mat4x4 Rotm;
      mat4x4 RotB;  //3][3];
      int   nModels;
      int   nMaps;
      int   activeMap;
      float Cell[6];
      float CellR[6];
      _SymmInfo sy; 
      float CellN[6];
      float Radius;
      bool  ifWhole;
      double cylinder;
      double sc_atoms;
      double sc_bonds;
      bool   ShowMultBonds;
      float  scale_atoms[N_ELEMENTS+1];
      int    ell_tr;
      int    RadType;
      vet3  vc[8];    // vertici cella
      vet3  vcN[8];
      vet3  xyzsN[4];
      mat3x3 rmC;
      mat3x3 rmI;
      mat3x3 MMRec;
      mat3x3 om;
      mat3x3 omI;
      vet3  xyzC;
      vet3  abcPar;
      vet3  VertPar[8];
      vet3  VertParN[8];
      bool  ifDrawPar;
      double eq0[4];
      double eq1[4];
      bool  DrawCenter;
      int   ifAxes;
      int   ifDrawAxes;
      float CellColor[3];
      float AxesColor[3];
      float CentreColor[3];
      float BgColor[4];
      float BgColor1[4];
      float BgColor2[4];
      float ColorHB[4];
      float ColorSC[4];
      float DistColor[4];
      int   BgType;
      _LightInfo lightInfo[3];
      int usedC[MAX_NUM_MAPS];
      bool mapvisSav[MAX_NUM_MAPS];
      bool modelvisSav[MAX_NUM_MODELS];
      bool ShowLegend;
      int  nLegends;
      int  activeLegend;
      int  ListaSpecies[MAX_NUM_MODELS*MAX_SPECIES];
      int  NSpecies;
      int  ifGrow;
      int  ifHBonds;
      int  ifShortc;
      int  ifDelHang;
      float xoffset;
      float yoffset;
      char *fontname;
      std::vector <DistAng_type> DistAng_vet;
   };
   class Legenda {
   public:
      Legenda();
      GdkRectangle *LegendRect();
      void  SetText(const char *text);
      void  SetFont(const char *fontname);
      void  SetPos(int x, int y);
      void  GetPos(int *x, int *y);
      void  SetRect(int x, int y, int w, int h);
      void  SetShow(bool visible);
      void  SetBackground(float color[]);
      void  SetForeground(float color[]);
      void  freeLegend();
      void  Save(std::ofstream &f);
      bool  Restore(std::ifstream &f);
      void  Save(FILE *fp);
      bool  Restore(FILE *fp);
      bool  IsVisible() const {return visible;}
      void  Draw(vet3 xy);
      void  LegendDialog(GtkWidget *window);
      char  *GetText() const {return testo;}
   private:
      float colore[4];
      float bcolore[4];
      GdkRectangle rect;
      int   xpos;
      int   ypos;
      bool  visible;
      char *testo;
      char *fontname;
   };

   GtkWidget *create_GLwindow(int width, int height, const char *title,
                              int ifSmooth,int x, int y);
   void Sfera(gboolean solid, double radius, int div);
   void TestMultiSampling();
}                            // end namespace
bool Okfb();

/**************************************************************************/
/*
           8_______ 2       1 {1, 0, 1}
          /|      /|        2 {1, 1, 1}
       6 /_|_____/4|        3 {1, 0, 0}
         | |     | |        4 {1, 1, 0}
         | |_7___|_| 1      5 {0, 0, 0}
         | /     | /        6 {0, 1, 0}
       5 |/______|/3        7 {0, 0, 1}
                            8 {0, 1, 1}
*/
