#include <utils.h>
#include "grafico.h"

#define SEC_TICS 5
#define EPS 1e-6

namespace GraphicSpace 
{
    bool Cfrx(_Point p1, _Point p2) {return (p1.x < p2.x); }
    bool Cfry(_Point p1, _Point p2) {return (p1.y < p2.y); }

    Plot::Plot(int n, float x[], float y[])
    {
        if(!points.empty())
            points.clear();
        if(!ipoints.empty())
            ipoints.clear();
        ifLines = true;
        ifMarkers = false;
        ifRefl = false;
        _Color blue = {0, 0, 0.9f, 1};
        _Color violet = {0.7f, 0, 0.7f, 1};
        memcpy(&color_line, &blue, sizeof(_Color));
        memcpy(&color_marker, &blue, sizeof(_Color));
        memcpy(&color_refl, &violet, sizeof(_Color));

        numPoints = n;
        for(int i=0; i<n; i++)
        {
            _Point p;
            p.x = x[i];
            if(y != NULL)
               p.y = y[i];
            else
            {
               p.y = 0;
               ifRefl = true;
            }
            points.push_back(p);
        }
        xmin = (*min_element(points.begin(), points.end(), Cfrx)).x;
        xmax = (*max_element(points.begin(), points.end(), Cfrx)).x;
        ymin = (*min_element(points.begin(), points.end(), Cfry)).y;
        ymax = (*max_element(points.begin(), points.end(), Cfry)).y;
    }

    void Plot::GetLim(float *x1, float *x2, float *y1, float *y2)
    {
        *x1 = xmin;
        *x2 = xmax;
        *y1 = ymin;
        *y2 = ymax;
    }

    void Plot::DisegnaRiflessi(gpointer context, gpointer pixmap, int bottom, gpointer graf1)
    {
        GdkColor *c1 = ToGdkColor(color_refl);
        set_foreground(context, c1);
        std::vector<GdkSegment> ss;
        for(int i=0; i<numPoints; i++)
        {
            if(InLimits(ipoints.at(i).x, graf1))
            {
               GdkSegment s;
               s.x1 = s.x2 = ipoints.at(i).x;
               s.y1 = bottom+5;
               s.y2 = bottom+15;
               ss.push_back(s);
            }
        }
        if(ss.empty())
           return;
        int nss = (int)ss.size();
        GdkSegment *segs = g_new0(GdkSegment, nss);
        for(int i=0; i<nss; i++)
        {
            segs[i].x1 = ss.at(i).x1;
            segs[i].x2 = ss.at(i).x2;
            segs[i].y1 = ss.at(i).y1;
            segs[i].y2 = ss.at(i).y2;
        }
        ss.clear();
        draw_segments(pixmap, context, segs, nss);
        g_free(segs);
    }

    bool Plot::InLimits(_iPoint p1, gpointer graf1)
    {
        Grafico *gr = (Grafico *) graf1;
        _Point pp1, pp2;
        gr->GetLim(&(pp1.x), &(pp2.x), &(pp1.y), &(pp2.y));
        _iPoint minlim = gr->ToScreen(pp1);
        _iPoint maxlim = gr->ToScreen(pp2);
        return ((p1.x >= minlim.x) && (p1.x <= maxlim.x) );//&&
                //(p1.y <= minlim.y) && (p1.y >= maxlim.y));
    }

    bool Plot::InLimits(int x1, gpointer graf1)
    {
        Grafico *gr = (Grafico *) graf1;
        _Point pp1, pp2;
        gr->GetLim(&(pp1.x), &(pp2.x), &(pp1.y), &(pp2.y));
        _iPoint minlim = gr->ToScreen(pp1);
        _iPoint maxlim = gr->ToScreen(pp2);
        return ((x1 >= minlim.x) && (x1 <= maxlim.x));
    }

    void Plot::Disegna(gpointer context, gpointer pixmap, gpointer graf1)
    {
        GdkColor *c1 = ToGdkColor(color_line);
        set_foreground(context, c1);
        std::vector<GdkPoint> pps;
        for(int i=0; i<numPoints; i++)
        {
            if(InLimits(ipoints.at(i), graf1))
            {
               GdkPoint pp = {ipoints.at(i).x, ipoints.at(i).y};
               pps.push_back(pp);
            }
        }
        if(pps.empty())
           return;
        int npp=(int) pps.size();
        GdkPoint *gdkpoints = g_new0(GdkPoint, npp);
        for(int i=0; i<npp; i++)
        {
            gdkpoints[i].x = pps.at(i).x;
            gdkpoints[i].y = pps.at(i).y;
        }
        pps.clear();
        draw_lines_aa( pixmap, context, gdkpoints, npp);
        g_free(gdkpoints);
    }

void CalcolaTics(float vmin, float vmax, double *step_t, int *ntics,
                  double *primotic, int *disp, int *nci)
{
   int    espo, i1, i2;
   double delta, step_tic, pristep, primopri, secstep, primosec;

   delta = (double) (vmax - vmin);
   if(MRabs(delta) < EPS)
   {
      g_print("Warning: vmin = vmax\n");
      delta = 10;
   }
   espo = 0;
   while(((int) delta) >= 10)
   {
       delta /= 10.;
       espo--;
   }
   while(((int) delta) == 0)
   {
       delta *= 10.;
       espo++;
   }
   if(delta <= 1.5)
      step_tic = 0.1/SEC_TICS;
   else if(delta <= 3.)
      step_tic = 0.2/SEC_TICS;
   else if(delta <= 7.)
      step_tic = 0.5/SEC_TICS;
   else 
      step_tic = 1./SEC_TICS;

   step_tic *= 2;

   *ntics  = (int)(delta / step_tic) + 1;
   *step_t = step_tic / pow(10., (double)(espo)); 
   secstep = *step_t;
   pristep = step_tic * SEC_TICS / pow(10., (double)(espo));

   i1 = (int)(vmin*pow(10., (double)(espo+1))+0.5);
   i2 = (int)(pristep*pow(10., (double)(espo+1))+0.5);
   while((i1 % i2) != 0)
      i1++;
   primopri = (double)i1 / pow(10., (double)(espo+1));
   if(primopri < vmin)
      primopri += pristep;

   i1 = (int)(SEC_TICS*vmin*pow(10., (double)(espo+1))+0.5);
   i2 = (int)(SEC_TICS*secstep*pow(10., (double)(espo+1))+0.5);
   while((i1 % i2) != 0)
      i1++;
   primosec = (double)i1 / (SEC_TICS * pow(10., (double)(espo+1)));
   if(primosec < vmin)
   {
      primosec += secstep;
      (*ntics)--;
   }
   *primotic = primosec;
   *disp = (int)((primopri - primosec)/secstep);

   delta = pristep;
   espo = 0;
   while(((int) delta) >= 10)
   {
       delta /= 10.;
       espo--;
   }
   while(((int) delta) == 0)
   {
       delta *= 10.;
       espo++;
   }
   if(espo > 0)
      *nci = espo+1;
   else
      *nci = espo;
}

     Grafico::Grafico()
     {
         if(!Plots.empty())
            Plots.clear();
         _Color black = {0.2f, 0.2f, 0.2f, 1};
         memcpy(&color_axes, &black, sizeof(_Color));
         Gfont = get_fontN(MONOSPACE_FONT);
         get_metrics(Gfont, &wfont, &hfont);
         xmin = xmax = ymin = ymax = 0;
         start_move = false;
         Xiniz = Yiniz = -1;
         Wrect = Hrect = 0;
     }
    
     void Grafico::SetArea(int w, int h)
     {
         Width  = w;
         Height = h;
     }

     void Grafico::SetLim(float x1, float x2, float y1, float y2, bool force)
     {
         xmin = x1;
         xmax = x2;
         if(force)
         {
            ymin = y1;
            ymax = y2;
         }
         else
         {
            ymin = MRmin(ymin, y1);
            ymax = MRmax(ymax, y2);
         }
     }

     void Grafico::CalcolaScala()
     {
         int dispx, dispy, ncix, nciy, xtics, ytics;
         double tic1x, stepx, tic1y, stepy;

         CalcolaTics(xmin, xmax, &stepx, &xtics, &tic1x, &dispx, &ncix);
         CalcolaTics(ymin, ymax, &stepy, &ytics, &tic1y, &dispy, &nciy);

         right = left = top = bottom = hfont;
         bottom += 4*hfont;
         left += (MRabs(nciy)+6)*wfont;//+10)*wfont;
         top    += 2*hfont;
         XScale = (float)(Width - left - right)/(xmax -xmin);
         YScale = (float)(Height - top - bottom)/(ymax -ymin);
         dx = (float) left / XScale - xmin;
         dy = (float) top / YScale + ymax;
         assex.valmin = xmin;
         assex.valmax = xmax;
         assex.bottom = bottom;
         assex.top    = top;
         assex.right  = right;
         assex.left   = left;
         assex.deltax = Width-right;
         assex.deltay = Height-bottom;
         assex.scale  = XScale;
         assex.start  = tic1x;
         assex.step   = stepx;
         assex.tics   = xtics;
         assex.fprim  = dispx;
         assex.fsec   = (int) ((tic1x - xmin) * XScale) + left;
         assex.ori    = 'h';
         assex.label  = labelx;
         assex.ncifre = ncix;
         assex.ifSect = true;

         assey.valmin = ymin;
         assey.valmax = ymax;
         assey.bottom = bottom;
         assey.left   = left;
         assey.top    = top;
         assey.right  = right;
         assey.deltax = Width-right;
         assey.deltay = Height-bottom;
         assey.scale  = YScale;
         assey.start  = tic1y;
         assey.step   = stepy;
         assey.tics   = ytics;
         assey.fprim  = dispy;
         assey.fsec   = (int) ((tic1y - ymin) * YScale) + top;
         assey.ori    = 'v';
         assey.label  = labely;
         assey.ncifre = nciy;
         assey.ifSect = true;

         labx.x = (assex.deltax+assex.left)/2;
         labx.y = assex.deltay+(int)(2.5*hfont);
         if(nciy > 0)
            laby.x = 4;
         else
            laby.x = (int)(wfont*1.8);
         laby.y = assey.deltay/2;

     }

     _Point Grafico::FromScreen(_iPoint p)
     {
         _Point p1 = {(float)p.x / XScale - dx,
                      dy - (float)p.y / YScale};
         return p1;
     }

     _iPoint Grafico::ToScreen(_Point p)
     {
         _iPoint p1 = {(int)((p.x + dx) * XScale),
                       (int)((dy - p.y) * YScale)};
         return p1;
     }

     
     void Grafico::DrawAxes(Axes axe)
     {
          int k, x0, x1, y0, y1, ntics;
          GdkSegment *segs;
       
          GdkColor *c1 = ToGdkColor(color_axes);
          switch(axe.ori)
          {
          case 'h':
                if(!axe.label.empty())
                   draw_string(Gpixmap, Gfont, Gcontext, labx.x, labx.y,
                      axe.label.c_str(), c1, NULL);
                
                x0 = axe.left;
                x1 = axe.deltax;
                y0 = axe.deltay;
                y1 = y0;
                break;
          case 'H':
                x0 = axe.left;
                x1 = axe.deltax;
                y0 = axe.top;
                y1 = y0;
                break;
          case 'v':
                if(!axe.label.empty())
                   draw_Vstring(Gpixmap, Gfont, Gcontext, laby.x, laby.y,
                      axe.label.c_str(), c1, 0);
                x0 = axe.left;
                x1 = x0;
                y0 = axe.top; 
                y1 = axe.deltay;
                break;
          case 'V':
                x0 = axe.deltax;
                x1 = x0;
                y0 = axe.top; 
                y1 = axe.deltay;
                break;
          }
          set_foreground( Gcontext, c1);
          draw_line(Gpixmap, Gcontext, x0,y0, x1, y1);
          ntics = axe.tics;
          segs = g_new0(GdkSegment, ntics);
          vector<GdkSegment> Segs;
          for(int i=0; i<ntics; i++)
          {
              switch(axe.ori)
              {
              case 'h':
                    k = 1;
                    segs[i].x1 = axe.fsec + (int)((axe.step*(double)i)*axe.scale);
                    if(((i-axe.fprim) % SEC_TICS) == 0)
                    {
                        k = 2;
                        float duetheta = (float) (axe.start+i*axe.step);
                        int nc = MRmax(0, axe.ncifre);
                        std::string dt = fToStr2(duetheta, nc);
                        trim(dt);
                        int lu = wfont*(int)dt.length();
                        draw_string(Gpixmap, Gfont, Gcontext, 
                           segs[i].x1-lu, axe.deltay+hfont, dt.c_str(),
                           c1, NULL, PANGO_ALIGN_CENTER);
                    }
                    segs[i].x2 = segs[i].x1;
                    segs[i].y1 = axe.deltay;// - 5*k ;
                    segs[i].y2 = axe.deltay + 5*k;
                    if(k == 2)
                       Segs.push_back(segs[i]);
                    break;
              case 'v':
                    segs[i].y1 = axe.top + axe.deltay - axe.fsec
                                  - (int)((axe.step*(double)i)*axe.scale);
                    k = 1;
                    if(((i-axe.fprim) % SEC_TICS) == 0)
                    {
                        k = 2;
                        int nc = MRmax(0, axe.ncifre);
                        draw_string(Gpixmap, Gfont, Gcontext, 
                           axe.left-10*wfont, segs[i].y1-hfont/2,
                           //laby.x+4*wfont, segs[i].y1-hfont/2,
                           fToStr2((float)(axe.start+i*axe.step), nc).c_str(),
                           c1, NULL, PANGO_ALIGN_RIGHT);
                    }
                    segs[i].y2 = segs[i].y1;
                    segs[i].x1 = axe.left - 5 * k;
                    segs[i].x2 = axe.left;
                    if(k == 2)
                       Segs.push_back(segs[i]);
                    break;
              }
          }
          if(axe.ifSect)
             draw_segments(Gpixmap, Gcontext, segs, axe.tics);
          else
          {
            g_free(segs);
            segs = g_new0(GdkSegment, Segs.size());
            for(size_t i=0; i<Segs.size(); i++)
                segs[i] = Segs.at(i);
            draw_segments(Gpixmap, Gcontext, segs, (int)Segs.size());
          }
          g_free(segs);
          Segs.clear();
     }

     void Grafico::AddPlot(float x[], float y[], int num, float tmin,
                           float tmax, bool uset)
     {
         float x1, y1, x2, y2;
         Plot plotn = Plot(num, x, y);
         plotn.GetLim(&x1, &x2, &y1, &y2);
         if(!uset)
         {
            tmin = x1;
            tmax = x2;
         }
         SetLim(tmin, tmax, y1, y2);
         xmin0 = tmin;
         xmax0 = tmax;
         if(y != NULL)
         {
            ymin0 = y1;
            ymax0 = y2;
         }
         Plots.push_back(plotn);
     }

     void Grafico::Disegna()
     {
         for(size_t i=0; i<Plots.size(); i++)
         {
             Plots[i].ToScreen((gpointer) this);
             if(Plots[i].IsRefl())
                Plots[i].DisegnaRiflessi(Gcontext, Gpixmap, assey.deltay, this);
             else
                Plots[i].Disegna(Gcontext, Gpixmap, this);
         }
         _Point pp1, pp2;
         GetLim(&(pp1.x), &(pp2.x), &(pp1.y), &(pp2.y));
        _iPoint minlim = ToScreen(pp1);
        _iPoint maxlim = ToScreen(pp2);

         ClearArea(minlim.x, 0, maxlim.x-minlim.x, maxlim.y);
         ClearArea(minlim.x, minlim.y, maxlim.x-minlim.x, Height-minlim.y);

         DrawAxes(assex);
         DrawAxes(assey);

         assex.ori = 'H';
         assey.ori = 'V';
         DrawAxes(assex);
         DrawAxes(assey);
         assex.ori = 'h';
         assey.ori = 'v';

     }

     void Grafico::ClearArea(int x, int y, int w, int h) 
     {
          GdkColor colorw = {0, 65535, 65535, 65535};
          set_foreground(Gcontext, &colorw);
          draw_rectangle (Gpixmap, Gcontext, TRUE, x, y, w, h);
     }

     void Grafico::Aggiorna(GtkWidget *widget)
     {
          ClearArea(0, 0, Width, Height);
          //GdkColor colorw = {0, 65535, 65535, 65535};
          //set_foreground(Gcontext, &colorw);
          //draw_rectangle (Gpixmap, Gcontext, TRUE, 0, 0, Width, Height);

          CalcolaScala();
          Disegna();
          if(gtk_widget_get_window(widget))
             draw_drawable(gtk_widget_get_window(widget), Gcontext, Gpixmap, 0, 0);
          SaveRestore(widget, 1);   // save on savewindow
     }

     void Plot::ToScreen(gpointer data)
     {
         Grafico *graf1 = ( Grafico * ) data;
         if(!ipoints.empty())
            ipoints.clear();
         for(int i=0; i<numPoints; i++)
             ipoints.push_back(graf1->ToScreen(points.at(i)));            
     }

     void Grafico::Restore(GtkWidget *widget)
     {
         if(gtk_widget_get_window(widget))
            flush_os(Gpixmap, gtk_widget_get_window(widget), Gcontext);
     }

void Grafico::SaveRestore (GtkWidget *widget, int action)
{
    GtkAllocation allocation;
    gtk_widget_get_allocation(widget, &allocation);
    gpointer saveWindow = get_object(widget,"savescreen");
    if(saveWindow == NULL)
       return;

    if (action == 1)   // Save
    {
         draw_drawable(saveWindow, Gcontext, Gpixmap, 0, 0, 0, 0,
                  allocation.width, allocation.height);
    }
    else     // Restore
    {
         draw_drawable(Gpixmap, Gcontext, saveWindow, 0, 0, 0, 0,
                  allocation.width, allocation.height);
    }
}

     void Grafico::DrawDashRect (GtkWidget *wi, int ix, int iy, int w, int h)
     {
        if(!gtk_widget_get_window(wi))
           return;
        GdkColor colorw = {0, 0, 0, 0};
        set_foreground(Gcontext, &colorw);

        gpointer outWindow = (gpointer) gtk_widget_get_window(wi);
        gpointer lcontext = Gcontext;

        for(int x=ix; x<ix+w; x+=10)
        {
            draw_line( outWindow, lcontext, x, iy, x+3, iy);
            draw_line( outWindow, lcontext, x, iy+h-1, x+3, iy+h-1);
        }

        for(int y=iy; y<iy+h; y+=10)
        {
            draw_line( outWindow, lcontext, ix, y, ix, y+3);
            draw_line( outWindow, lcontext, ix+w-1, y, ix+w-1, y+3);
        }
     }

     void Grafico::GetLim(float *x1, float *x2, float *y1, float *y2)
     {
        *x1 = xmin;
        *x2 = xmax;
        *y1 = ymin;
        *y2 = ymax;
     }

     void Grafico::ResetLim()
     {
         xmin = xmin0;
         xmax = xmax0;
         ymin = ymin0;
         ymax = ymax0;
     }

     void Grafico::Configure(GtkWidget *widget)
     {
         gpointer pixmap, pixmap1;
         gpointer context, context1;
         os_and_gc *rt, *rt1;
         GtkAllocation allocation;
         gtk_widget_get_allocation(widget, &allocation);
     
         int w = (int)allocation.width;
         int h = (int)allocation.height;
     
         if (get_object(widget, "context") != NULL)
             context = get_object(widget, "context");
         else
             context = NULL;
     
         if (get_object(widget, "offscreen") != NULL)
             pixmap = get_object(widget, "offscreen");
         else
             pixmap = NULL;
     
         destroy_os_and_gc(pixmap, context);
         rt = get_os_and_gc(widget);
         pixmap = rt->os;
         context = rt->gc;
         g_free(rt);
         if (get_object(widget, "sc_context") != NULL)
             context1 = get_object(widget, "sc_context");
         else
             context1 = NULL;
     
         if (get_object(widget, "savescreen") != NULL)
             pixmap1 = get_object(widget, "savescreen");
         else
             pixmap1 = NULL;
     
         destroy_os_and_gc(pixmap1, context1);
         rt1 = get_os_and_gc(widget);
         context1 = rt1->gc;
         pixmap1 = rt1->os;
         g_free(rt1);
         g_object_set_data( G_OBJECT(widget), "savescreen", pixmap1);
         g_object_set_data(G_OBJECT(widget), "sc_context", context1);
     
         g_object_set_data(G_OBJECT(widget), "context", context);
         g_object_set_data( G_OBJECT(widget), "offscreen", pixmap);
         SetArea(w, h);
         SetContext(context, pixmap);
         Aggiorna(widget);
     }

     void Grafico::MouseRelease(GtkWidget *widget, GtkWidget *but)
     {
       if(start_move)
       {
          if((Wrect * Hrect) != 0)
          {
              _iPoint ip1 = { MRmin(Xiniz, Xiniz+Wrect), MRmax(Yiniz, Yiniz+Hrect)};
              _iPoint ip2 = { MRmax(Xiniz, Xiniz+Wrect), MRmin(Yiniz, Yiniz+Hrect)};
              _Point p1 = FromScreen(ip1);
              _Point p2 = FromScreen(ip2);
              float x1, y1, x2, y2;
              GetLim(&x1, &x2, &y1, &y2);
     
              if(x1 < p1.x)
                 x1 = p1.x;
              if(x1 > p2.x)
                 x1 = p2.x;
              if(x2 < p1.x)
                 x2 = p1.x;
              if(x2 > p2.x)
                 x2 = p2.x;
              if(y1 < p1.y)
                 y1 = p1.y;
              if(y1 > p2.y)
                 y1 = p2.y;
              if(y2 < p1.y)
                 y2 = p1.y;
              if(y2 > p2.y)
                 y2 = p2.y;
              if((MRabs(y1-y2) > 1e-8) && (MRabs(x1-x2) >1e-8))
              { 
                  SetLim(x1, x2, y1, y2, true);
                  gtk_widget_set_sensitive(but, TRUE);
              }
              Aggiorna(widget);
          }
          Xiniz = -1;
          Yiniz = -1;
          Wrect = 0;
          Hrect = 0;
          start_move = false;
          return;
       }
       Restore(widget);
     }
     
     void Grafico::MouseMove(GtkWidget *w, int x, int y)
     {
       if(start_move)
       {
          if (Xiniz < 0 )
          {
              Xiniz = x;
              Yiniz = y;
     
          }
          else
          {
              SaveRestore(w, 0);        // put on screen savewindow content
              Wrect = x - Xiniz + 1;
              Hrect = y - Yiniz + 1;
              DrawDashRect(w, (int) MRmin(Xiniz, x), (int) MRmin(Yiniz, y),
                 (int) MRabs(Wrect),(int) MRabs(Hrect));
              Restore(w);               // put on screen offscreen content
          }
     
       }
     }
     
     void Grafico::ResetZoom(GtkWidget *button, GtkWidget *widget)
     {
         ResetLim();
         Aggiorna(widget);
         gtk_widget_set_sensitive(button, FALSE);
     }

}               // end namespace
