#ifndef __GRAFICO_H
#define __GRAFICO_H

using namespace std;

namespace GraphicSpace 
{
typedef struct {float x, y;} _Point;
typedef struct {int x, y;} _iPoint;
typedef float _Color[4];
typedef struct
               {
                double valmin;
                double valmax;
                int    bottom;
                int    left;
                int    right;
                int    top;
                int    deltax;
                int    deltay;
                double scale;
                double start;
                double step;
                int    tics;
                int    fprim;
                int    fsec;
                int    ncifre;
                char   ori;
                bool   ifSect;
                string label;
               } Axes;


class Plot {
public:
    Plot(int n, float x[], float y[]);
    void GetLim(float *x1, float *x2, float *y1, float *y2);
    void ToScreen(gpointer p);
    void Disegna(gpointer context, gpointer pixmap, gpointer graf1);
    void DisegnaRiflessi(gpointer context, gpointer pixmap, int bottom, gpointer gr);
    bool IsRefl() {return ifRefl;}
    bool InLimits(_iPoint p1, gpointer graf1);
    bool InLimits(int p1, gpointer graf1);
private:
    int numPoints;
    vector<_Point> points;
    vector<_iPoint> ipoints;
    float xmin, ymin, xmax, ymax;
    bool ifLines, ifMarkers, ifRefl;
    _Color color_line, color_marker, color_refl;
};

class Grafico {
public:
     Grafico();
     void SetArea(int w, int h);
     void ClearArea(int x, int y, int w, int h);
     void SetLim(float x1, float x2, float y1, float y2, bool force=false);
     _iPoint ToScreen(_Point p);
     _Point FromScreen(_iPoint p);
     void Aggiorna(GtkWidget *w);
     void CalcolaScala();
     void AddPlot(float x[], float y[], int num, float tmin, float tmax,
                  bool use=true);
     void SetContext(gpointer context, gpointer pixmap) {Gcontext = context;
                                                         Gpixmap = pixmap; }
     void Disegna();
     void DrawAxes(Axes axe);
     void SetLabels(const char *lx, const char *ly) {labelx = lx; labely = ly;}
     void Restore(GtkWidget *widget);
     void DrawDashRect(GtkWidget *widget, int x, int y, int w, int h);
     void GetLim(float *x1, float *x2, float *y1, float *y2);
     void ResetLim();
     void SaveRestore (GtkWidget *widget, int action);
     void Configure(GtkWidget *widget);
     void SetStartMove(bool val) {start_move=val;}
     void MouseRelease(GtkWidget *widget, GtkWidget *but);
     void MouseMove(GtkWidget *w, int x, int y);
     void ResetZoom(GtkWidget *button, GtkWidget *widget);
private:
     int Width, Height;
     float xmin0, xmax0, ymin0, ymax0;
     float xmin, xmax, ymin, ymax;
     float dx, dy, XScale, YScale;
     int top, right, bottom, left;
     int hfont, wfont;
     Axes assex, assey;
     vector <Plot> Plots;
     string labelx, labely;
     _iPoint labx, laby;
     gpointer Gcontext, Gpixmap;
     PangoFontDescription *Gfont;
    _Color color_axes;
    bool start_move;
    int Xiniz;
    int Yiniz;
    int Wrect;
    int Hrect;
};
}               // end namespace
#endif
