  MODULE ilmviewmod

! S ilmview_open(stname)                ! apre il viewer
! S ilmview_close()                     ! chiude il viewer
! F ilmview_is_opened()                 ! il viewer e' aperto?
! S ilmview_draw(atom, legm, infos)     ! disegna il modello
! S ilmview_set_title(title)            ! Definisce il titolo della finestra di ilmview
! S jav_set_message(msg)            ! Definisce il messaggio sulla statusbar di ilmview
!
  USE progtype, only: bond_setting_type, DEF_BONDSET
  USE atom_basic, only: atom_type
  USE elements, only: element_type, RX_SOURCE
  USE connect_mod, only: bond_type
  USE compare_struct, only:compare_info_type
  USE polyhedra, only: polyhedron_type
  USE model_util, only: model_type
  USE reflection_type_util, only: reflection_type
  USE spginfom, only: spaceg_type

  implicit none
!
  integer, parameter :: NMAXMODEL = 5 ! numero massimo di modelli
  integer            :: num_models    ! numero di modelli
  type(model_type), dimension(NMAXMODEL), save :: modelview
  type(model_type), dimension(NMAXMODEL), save :: modelviews
!
  type(polyhedron_type), dimension(:), allocatable      :: polyview
  type(bond_type), dimension(:), allocatable            :: legmHview,legmshort
  type(compare_info_type), save                         :: infoc
  type(bond_setting_type)                               :: bondset = DEF_BONDSET

  type expand_option_type
    integer :: code  = 0
    integer :: atom  = 0
    integer :: level = 0
  end type
  type(expand_option_type), dimension(NMAXMODEL) :: expandc !corr     = expand_option_type(0,0,0)
  logical, dimension(NMAXMODEL)                  :: sym_atoms = .false.

  character(len=*), dimension(35), parameter, private :: ustr =                                     &
  (/'Delete           ', 'Add Bonds        ', 'Change Species   ', 'Build Bonds      ',             &
    'Assemble Residues', 'Rename           ', 'Sort by Specie   ', 'Renumber         ',             &        
    'Complete         ', 'Regularize       ', 'Enantiomorph     ', 'Add Hydrogens    ',             &
    'Add Hydrogens    ', 'Add Hydrogens    ', 'Add Hydrogens    ', 'Add Hydrogens    ',             &
    'Add Hydrogens    ', 'Add Hydrogens    ', 'Add Hydrogens    ', 'Add Hydrogens    ',             &
    'Import           ', 'Add Fragments    ', 'Modify Bonds     ', 'Move Atoms       ',             &
    'Edit Structure   ', 'RBM              ', 'Fourier          ', 'Normalising      ',             &
    'Duplicate        ', '                 ', 'Delete Duplicates', 'Add Atom         ',             &
    'Transform        ', 'Optimize Geometry', 'SMILES           '/)

  integer, private, parameter :: ID_UNDO = 0
  integer, private, parameter :: ID_REDO = 1
  
  integer, parameter :: DELOBJ = 1 
  integer, parameter :: ADD_BONDS = 2 
  integer, parameter :: CHANGE_SPEC = 3 
  integer, parameter :: BUILD_BONDS = 4 
  integer, parameter :: ASSEMBLE_RESIDUES = 5 
  integer, parameter :: REN_ATOMS = 6 
  integer, parameter :: SORT_BY_SPEC = 7 
  integer, parameter :: RENUMBER = 8 
  integer, parameter :: COMPLETE = 9 
  integer, parameter :: REGULARIZE = 10
  integer, parameter :: ENANT = 11
  integer, parameter :: H_AUTOMATIC = 12
  integer, parameter :: H_METHINE = 13
  integer, parameter :: H_METHYLENE = 14
  integer, parameter :: H_METHYL = 15
  integer, parameter :: H_AROMATIC = 16
  integer, parameter :: H_HYDROXYL = 17
  integer, parameter :: H_TERMINAL_CH2 = 18
  integer, parameter :: H_TERMINAL_BH = 19
  integer, parameter :: H_ACETYLENIC = 20
  integer, parameter :: IMPORT = 21
  integer, parameter :: ADD_FRAG = 22
  integer, parameter :: BOND_SETTINGS = 23  
  integer, parameter :: MOVE_ATOMS = 24
  integer, parameter :: EDIT_STRUCT = 25
  integer, parameter :: RBM = 26
  integer, parameter :: FOURIER = 27
  integer, parameter :: NORMALISEH = 28
  integer, parameter :: DUPLICATE = 29
  integer, parameter :: DEL_DUPLICATE_ATOMS = 31
  integer, parameter :: ADD_ATOM = 32
  integer, parameter :: TRANSFORM_AT = 33
  integer, parameter :: OPTIMIZE_GEOMETRY = 34
  integer, parameter :: ADD_SMILES = 35
  integer, parameter :: DISPLAY_CONTACTS = 101

  integer :: nuract           ! numero di azioni nello stack undo/redo
  integer :: posur            ! posizione nello stack

  type urtype
    integer                                               :: azione
    type(atom_type), dimension(:), allocatable            :: atom
    type(bond_type), dimension(:), allocatable            :: legm
    character(len=80)                                     :: stract
    type(element_type), dimension(:), allocatable         :: elem
    integer                                               :: active
    real, dimension(6)                                    :: cell
    type(spaceg_type)                                     :: spg
  end type 

  type pdf_option_type
    integer :: rad  = RX_SOURCE
    real    :: rmax = 10.0
    real    :: qmax = 25.0
  end type

  type(urtype), allocatable, dimension(:) :: uract
  type(urtype), save                      :: uract_temp

  real, dimension(:), allocatable                  :: xjav_pd,yjav_pd
  type(reflection_type), dimension(:), allocatable :: refjav

  CONTAINS

   subroutine jav_init()
   num_models = 1
   end subroutine jav_init
!--------------------------------------------------------------------
   subroutine ilmview_open(stname)
!
   character(len=*), optional, intent(in) :: stname
   integer                                :: Lung
   call Open_GLWindow()
   call init_undoredo()
   if(present(stname)) then
      call ilmview_set_title(stname)
      call SetName(stname(1:Lung(stname)) // char(0), 1)
   endif
!
   end subroutine ilmview_open
!--------------------------------------------------------------------
   subroutine ilmview_close()
!
   call Close_GLWindow()
!
   end subroutine ilmview_close
!-----------------------------------------------------------------------
   logical function jav_opened()
   integer :: IsOpened_GLWindow
!
   jav_opened = IsOpened_GLWindow() == 1
!
   end function jav_opened
!--------------------------------------------------------------------
   subroutine jav_set_modify(modify)
   integer, intent(in) :: modify
!
   call SetModify(modify)
!
   end subroutine jav_set_modify
!--------------------------------------------------------------------
   subroutine jav_redraw(kmod,ifResize,kaction,symm_enabledd)
   USE connect_mod
   USE atom_type_util
   USE polyhedra
   USE contacts
   USE molpnew
   USE unit_cell
   USE General, only: cell,spacegr
!
   integer, intent(in)           :: kmod
   integer, intent(in), optional :: ifResize
   integer, intent(in), optional :: kaction
   logical, intent(in), optional :: symm_enabledd ! when active symmetry operatations are applyed again
   real, dimension(3)            :: klevelv
   integer                       :: kfitopt
   integer                       :: npoly
   logical                       :: symactive
   integer                       :: nlegH,nlegsh, flag_resize
   logical                       :: lsym
   logical                       :: lgrow                       
   logical                       :: hbond,shbond
   logical                       :: is_contacts
   integer, dimension(2)         :: kexpand
   logical                       :: lpacking
   integer                       :: action_required
   logical                       :: symm_enabled
   type(cell_type)               :: cellt
!
   flag_resize = 2   !! mr 1
   if (present(ifResize)) then
       flag_resize = ifResize
   endif
   action_required = 0
   if (present(kaction)) then
       action_required = kaction
   endif
   symm_enabled = .true.
   if (present(symm_enabledd)) then
       symm_enabled = symm_enabledd
   endif
   hbond = is_hydrogen_bonds(kmod)
   shbond = is_short_contacts(kmod)
   is_contacts = hbond .or. shbond
   symactive = symatoms(kmod)
   cellt = set_cell_type(cell)
   if (symactive .or. expandc(kmod)%atom > 0) then           ! atomi con equivalenti per simmetria
                    if (symm_enabled) then
       lgrow = is_grow_fragment(kmod) .or. expandc(kmod)%code == 2
       nlegH = 0
       nlegsh = 0
       if (is_packing(kmod)) then    ! packing
           if (expandc(kmod)%atom == 0) then
               call get_packing_info(klevelv, kfitopt, kmod)
               call apply_symmetry(modelview(kmod)%at,modelviews(kmod)%at,   &
                modelview(kmod)%leg,modelviews(kmod)%leg,klevelv,kfitopt,cellt,spacegr)
           endif
           lpacking = .true.
       else
           lpacking = .false.
       endif
       if (is_contacts) then
           if(expandc(kmod)%atom > 0) then
              kexpand = (/EXPAND_ATOM,expandc(kmod)%atom/)
           elseif (lgrow) then
              kexpand = (/EXPAND_ALL,0/)
           else
              kexpand = (/0,0/)
           endif
           if (action_required == DISPLAY_CONTACTS .and. expandc(kmod)%atom > 0) then  ! the action required is display contacts
               kexpand = (/0,0/)                                                       ! after the expand
               call find_contacts(modelview(kmod)%at,modelview(kmod)%leg,cellt,spacegr,legmHview,legmshort,  &
                    modelviews(kmod)%at,modelviews(kmod)%leg,.true.,kexpand,hbond,shbond)
           else         
               call find_contacts(modelview(kmod)%at,modelview(kmod)%leg,cellt,spacegr,legmHview,legmshort,  &
                    modelviews(kmod)%at,modelviews(kmod)%leg,lpacking,kexpand,hbond,shbond)
           endif
       else
           if (expandc(kmod)%atom > 0) then
               lsym = lpacking .or. expandc(kmod)%level > 1 .or. lgrow
               call grow_fragments(modelview(kmod)%at,modelviews(kmod)%at, &
               modelview(kmod)%leg,modelviews(kmod)%leg,cellt,spacegr,lsym=lsym,      &
               kexp=(/EXPAND_ATOM,expandc(kmod)%atom/)) 
           elseif (lgrow) then
               call grow_fragments(modelview(kmod)%at,modelviews(kmod)%at, &
               modelview(kmod)%leg,modelviews(kmod)%leg,cellt,spacegr,  &
               lsym=lpacking,kexp=(/EXPAND_ALL,0/)) 
           endif
       endif
       if (is_polyhedra(kmod)) then
           lsym = lpacking .or. is_grow_fragment(kmod) .or. is_hydrogen_bonds(kmod) .or. expandc(kmod)%atom > 0
           call find_polyhedra_sym(modelview(kmod)%at,modelview(kmod)%leg, cellt, spacegr, &
           modelviews(kmod)%at,modelviews(kmod)%leg,polyview,npoly,lsym=lsym)
       endif
                       endif
!
       call jav_passa_modello(modelviews(kmod),kmod, flag_resize)
       if (is_polyhedra(kmod)) call jav_passa_poliedri()
       if (hbond) then
           nlegH = numbonds(legmHview)
           if (nlegH > 0) call jav_passa_legami(legmHview, nLegH, 1)
       endif
       if (shbond) then
           nlegsh = numbonds(legmshort)
           if (nlegsh > 0) call jav_passa_legami(legmshort, nlegsh, 2)
       endif
   else
       call jav_passa_modello(modelview(kmod),kmod, flag_resize)
   endif
   call jav_draw()
!
   end subroutine jav_redraw
!----------------------------------------------------------------------
   subroutine jav_passa_modello(model,kmodel, ifResize)
   USE connect_mod
   USE atom_type_util
   USE javinterface
   type(model_type), intent(inout) :: model
   integer, intent(in)             :: kmodel
   integer, intent(in), optional   :: ifResize
   integer                         :: natom
   integer                         :: numlegs
   integer                         :: numau
   integer                         :: numlegsau
   integer                         :: flag_resize
!
   flag_resize = 2   !! mr 1
   if (present(ifResize)) then
       flag_resize = ifResize
   endif
   natom = numatoms(model%at)
   numlegs = numbonds(model%leg)
   numlegsau = numbonds(modelview(kmodel)%leg)
   numau = numatoms(modelview(kmodel)%at)    
!
!  Simmetria?
!!mr   if (.not.symatoms(kmodel)) then
!!mr       call init_infos(model%at)
!!mr   endif
!
   call real_jav_passa_modello(model%at,model%leg,numau,numlegsau,kmodel,num_models, flag_resize)
!
   end subroutine jav_passa_modello
!----------------------------------------------------------------------
      subroutine jav_passa_poliedri()
      USE polyhedra
      integer :: npoly 
      npoly = numpoly(polyview)
      if (npoly > 0) then
          call PassaPoliedri(npoly, polyview)
      else
          call PassaPoliedri(0, 0)
      endif
      end subroutine jav_passa_poliedri
!----------------------------------------------------------------------
      subroutine ilmview_draw(atom, legm, kmodel, resize)
      USE connect_mod
      USE atom_type_util
      USE model_util
!
      type(atom_type), dimension(:), allocatable, intent(in)           :: atom
      type(bond_type), dimension(:), allocatable, intent(in), optional :: legm
      integer, intent(in), optional                                    :: kmodel
      logical, intent(in), optional                                    :: resize
      integer                                                          :: kmod
      integer                                                          :: iresize
!
      if (present(kmodel)) then
          kmod = kmodel
      else
          kmod = 1
      endif
!
      iresize = 1
      if (present(resize)) then
          if (.not.resize) iresize = 0
      endif
!
      if (jav_opened()) then
          if (present(legm)) then
              call model_set(modelview(kmod),atom,legm)
          endif
!
          call ClearDistAng()
          call jav_redraw(kmod,iresize)
      endif
!
      end subroutine ilmview_draw
!-----------------------------------------------------------------------
      subroutine ilmview_set_title(title)
!
!     Definisce il titolo della finestra di ilmview
!
      character(len=*), intent(in)     :: title
      character(len=len_trim(title)+1) :: titlec
!
      titlec = trim(title)//char(0)     
      call set_title(titlec)
!
      end subroutine ilmview_set_title
!-----------------------------------------------------------------------
      subroutine jav_set_message(msg)
!
!     Definisce il messaggio sulla statusbar di ilmview
!
      character(len=*), intent(in)   :: msg
      character(len=len_trim(msg)+1) :: msgc
!
      msgc = trim(msg)//char(0)     
      call set_message(msgc)
!
      end subroutine jav_set_message
!-----------------------------------------------------------------------
      subroutine ilmview_set_cell(kaction)
!
!     Definisce se la cella deve essere visibile o meno
!     kaction == 1      cella visibile
!     kaction == 0      cella non visibile
!
      integer, intent(in) :: kaction
      call set_cell(kaction)
!
      end subroutine ilmview_set_cell
!-----------------------------------------------------------------------
      subroutine jav_set_labels(kaction,kmodel)
!
!     Definisce se le labels devono essere visibili o meno
!     kaction == 1     labels visibili
!     kaction == 0     labels non visibili
!
      integer, intent(in) :: kaction, kmodel
      call set_labels(kaction, kmodel)
!
      end subroutine jav_set_labels
!-----------------------------------------------------------------------
      subroutine jav_set_sym(kaction,klevel,kmodel)
!
!     Definisce se la simmetria deve essere visibile o meno
!
      integer, intent(in) :: kaction, klevel, kmodel
      call set_symm(kaction, klevel, kmodel)
!
      end subroutine jav_set_sym
!-----------------------------------------------------------------------
      subroutine jav_set_style(kstyle, kmodel)
!
!     Definisce il plotstyle
!     Codici per il plotstyle
!                   WIRES           = 1
!                   SIMPLE          = 2
!                   BONDS           = 3
!                   RODS            = 4
!                   STICK AND BALLS = 5
!                   VAN DER WAALS   = 6
      integer, intent(in) :: kstyle, kmodel
!
      call set_style(kstyle, kmodel)
!
      end subroutine jav_set_style
!-----------------------------------------------------------------------
      integer function ilmview_info(code, kmodel)
!
!     Prende informazioni sullo stato del viewer
!      1 - cella
!      2 - simmetria
!      3 - plotstyle
!      4 - labels
!      5 - klevel
!      6 - poliedri
!      7 - grow fragment
!      8 - hydrogen_bonds
!      9 - short contacts
!     10 - delete hangings
!
      integer, intent(in)             :: code
      integer, intent(in)             :: kmodel
      integer, dimension(10)          :: infov
!
      call get_info_fromC(infov, kmodel)
      ilmview_info = infov(code)
!
      end function ilmview_info

!-----------------------------------------------------------------------

      subroutine jav_delete_object(kmodel,kptra,nptra,kptrb,nptrb)
      USE atom_type_util
      USE connect_mod
      USE polyhedra
      integer, intent(in)                :: kmodel
      integer, dimension(*), intent(in)  :: kptra  ! atomi selezionati
      integer, intent(in)                :: nptra  ! num. di atomi
      integer, dimension(*), intent(in)  :: kptrb  ! legami selezionati
      integer, intent(in)                :: nptrb  ! num. di legami
      logical                            :: symactive
      integer, dimension(:), allocatable :: iorder
      integer                            :: nlegH
!
      symactive = symatoms(kmodel)
      if (nptrb > 0) then    ! legami da eliminare?
          if (symactive) then    
              call remove_bonds_sym(modelviews(kmodel)%leg,modelview(kmodel)%leg,  &
              modelviews(kmodel)%at,kptrb(:nptrb))
          else
              call remove_bonds(modelview(kmodel)%leg,kptrb(:nptrb))
          endif
      endif
!
      if (nptra > 0) then   ! atomi da eliminare?
          if (symactive) then    ! simmetria attiva
              call remove_atoms_sym(modelview(kmodel)%at,modelviews(kmodel)%at,   &
              modelview(kmodel)%leg, modelviews(kmodel)%leg,kptra(:nptra),iorder)
          else
              call remove_atoms_vet(modelview(kmodel)%at,modelview(kmodel)%leg, &
              kptra(:nptra),iorder)
          endif
!
!         Aggiorna i poliedri         
          if (is_polyhedra(kmodel)) call update_polyhedra(polyview,iorder)            
!
!         Aggiorna legami H
          if (is_hydrogen_bonds(kmodel)) call bonds_update(legmHview,iorder)            
      endif
!
!     Ridisegna se ho eliminato qualcosa
      if (nptra + nptrb > 0) then
          if (symactive) then
              call jav_passa_modello(modelviews(kmodel),kmodel)
          else
              call jav_passa_modello(modelview(kmodel),kmodel)
          endif
          if (is_polyhedra(kmodel)) call jav_passa_poliedri()
          nlegH = numbonds(legmHview)
          if (nlegH > 0) call jav_passa_legami(legmHview, nlegH, 1)
      endif
!
      end subroutine jav_delete_object

!-----------------------------------------------------------------------

      logical function symatoms(kmodel)
!
      integer, intent(in)                :: kmodel
!     Se vero gli atomi su cui operare contengono equivalenti per simmetria
!
      symatoms = ilmview_info(2,kmodel) == 1 .or. ilmview_info(7,kmodel) == 1 .or.  &
                 ilmview_info(8,kmodel) == 1 .or. ilmview_info(6,kmodel) == 1 .or.  &
                 ilmview_info(9,kmodel) == 1 .or. expandc(kmodel)%atom > 0 .or. sym_atoms(kmodel) 
!
      end function symatoms

!-----------------------------------------------------------------------

      subroutine jav_add_bonds(kmodel,kptra,nptra)
      USE connect_mod
      USE atom_type_util
      USE General, only: cell,spacegr
      USE unit_cell
      integer, intent(in)                        :: kmodel
      integer, dimension(*), intent(in)          :: kptra  ! atomi selezionati
      integer, intent(in)                        :: nptra  ! num. di atomi
      integer, allocatable, dimension(:)         :: vet1,vet2,vet1s,vet2s
      integer                                    :: i
      logical                                    :: symactive
      integer                                    :: n1,n2
      integer                                    :: nlegau,nlegs
      type(bond_type), dimension(:), allocatable :: legmsnew
      type(cell_type)                            :: cellt
!
      symactive = symatoms(kmodel)
!
!     Alloca in vet1-vet2 i legami da aggiungere
      allocate(vet1(nptra-1),vet2(nptra-1))
      do i=1,nptra-1
         vet1(i) = kptra(i)
         vet2(i) = kptra(i+1)
      enddo
!
      cellt = set_cell_type(cell)
      if (symactive) then                        ! simmetria non attiva
          nlegau = 0
          nlegs = 0
          allocate(vet1s(nptra-1),vet2s(nptra-1))
          do i=1,nptra-1
             n1 = vet1(i)
             n2 = vet2(i)
!corr             if (modelviews(kmodel)%symm(n1)%oper == modelviews(kmodel)%symm(n2)%oper) then  ! legame all'interno dell'u.a.
             if (modelviews(kmodel)%at(n1)%op%op == modelviews(kmodel)%at(n2)%op%op) then  ! legame all'interno dell'u.a.
                 nlegau = nlegau + 1
!corr                 vet1(nlegau) = modelviews(kmodel)%symm(n1)%asym
!corr                 vet1(nlegau) = modelviews(kmodel)%symm(n1)%asym
                 vet2(nlegau) = modelviews(kmodel)%at(n2)%asym
                 vet2(nlegau) = modelviews(kmodel)%at(n2)%asym
             else                                        ! legame tra 2 u.a.
                 nlegs = nlegs + 1
                 vet1s(nlegs) = vet1(i)
                 vet2s(nlegs) = vet2(i)
             endif
          enddo
          if (nlegau > 0) then
              call add_bonds(modelview(kmodel)%leg,modelview(kmodel)%at,vet1(:nlegau),vet2(:nlegau),cellt)
!
!             Estendi a tutta la cella i nuovi legami
              call apply_symmetry_legm(modelviews(kmodel)%at,modelview(kmodel)%leg,legmsnew,spacegr)
              call combine_legm(modelviews(kmodel)%leg,legmsnew)
          endif
!
!         Aggiungi legami tra u.a. (questi al momento non vengono estesi a tutta le cella)
          if (nlegs > 0) then
              call add_bonds(modelviews(kmodel)%leg,modelviews(kmodel)%at,  &
                             vet1s(:nlegs),vet2s(:nlegs),cellt)
          endif
      else
          call add_bonds(modelview(kmodel)%leg,modelview(kmodel)%at,vet1,vet2,cellt)
      endif
      call jav_redraw(kmodel)
!
      end subroutine jav_add_bonds

!-----------------------------------------------------------------------

      subroutine jav_change_specie(kmodel,ispeci,kptraa,nptra,ier)
      USE Atomi, only:nzConst
      USE atom_type_util
      USE elements
      USE proginterface
      USE molpnew
      USE connect_mod
      USE General, only:cell,celem,spacegr
      USE unit_cell
      USE atom_basic
      integer, intent(in)                           :: kmodel
      integer, intent(in)                           :: ispeci ! specie nuova
      integer, dimension(*), intent(in)             :: kptraa ! atomi selezionati
      integer, intent(in)                           :: nptra  ! num. di atomi
      integer, intent(out)                          :: ier
      integer, dimension(nptra)                     :: kptra
      integer                                       :: i,j
      integer                                       :: ispecold
      integer                                       :: kat
      integer                                       :: nchange
      logical                                       :: symactive
      integer                                       :: ispec
!
      ispec = check_element(ispeci,size(kptra))
      symactive = symatoms(kmodel)
!
      kptra(:) = selected_atoms(kptraa,nptra,kmodel)
!
      nchange = 0
      do i=1,nptra
         kat = kptra(i)
         !ispecold = iabs(modelview(kmodel)%at(kat)%nz)/nzConst
         ispecold = modelview(kmodel)%at(kat)%kscatt()
         if (ispecold /= ispec) then
             nchange = nchange + 1
             kptra(nchange) = kat   ! save modified atoms for connectivty
!
!            modifica l'nz
             !corr modelview(kmodel)%at(kat)%nz = ispec*nzConst  ! fregatene del segno e del resto
             call modelview(kmodel)%at(kat)%set_specie_from_el(ispec,celem)
!
!            genera stringa
             call atom_string(modelview(kmodel)%at,celem,vet=(/(kat)/))
!
!            Estendi le modifiche di nz e lab a tutta la cella se la simmetria e' attiva
             if (symactive) then           
                 do j=1,size(modelviews(kmodel)%at)
!corr                    if (modelviews(kmodel)%symm(j)%asym == kat) then
                    if (modelviews(kmodel)%at(j)%asym == kat) then
                        !modelviews(kmodel)%at(j)%nz = modelview(kmodel)%at(kat)%nz
                        call modelviews(kmodel)%at(j)%set_specie(modelview(kmodel)%at(kat)%get_nz(),celem)
                        modelviews(kmodel)%at(j)%lab = modelview(kmodel)%at(kat)%lab
                    endif
                 enddo
             endif
         endif
      enddo
!
!     Ridisegna
      if (nchange > 0)then
          ier = 0
!
!         recompute bonds only for modified atoms
          call remove_bond_from_atom(modelview(kmodel)%leg,kptra(1:nchange))
          call create_connectivity(modelview(kmodel)%at,modelview(kmodel)%leg,set_cell_type(cell),spacegr,vet=kptra(1:nchange))
!
          if (symactive) then
              call jav_redraw(kmodel) ! this force new connectivity between molecules in cell
              !call jav_passa_modello(modelviews(kmodel),kmodel)
          else
              call jav_passa_modello(modelview(kmodel),kmodel)
          endif
      else
          ier = 1
      endif
!
      end subroutine jav_change_specie

!-----------------------------------------------------------------------

   integer function check_element(ispeci,nptra)  result(ispec)
   USE general, only:celem, spacegr
   USE elements
   USE variables, only: cryst
   integer, intent(in) :: ispeci
   integer, intent(in) :: nptra
   logical             :: addnew
 
   if (ispeci < 0) then   ! new element was select
!
!      add element -z
       call add_element(celem,abs(ispeci),nw=nptra*spacegr%nsymop,add=addnew)
       if (addnew) call cryst(1)%set_scattering(celem)
       ispec = is_element(celem,abs(ispeci))  ! get position of ispeci
   else
       ispec = ispeci
   endif
 
   end function check_element

!-----------------------------------------------------------------------

      subroutine jav_reconnect(kmodel,kaction,change)
      USE molpnew
      USE atom_type_util, only:numatoms
      USE connect_mod
      USE unit_cell
      USE General, only: cell,spacegr
      integer, intent(in)  :: kmodel
      integer, intent(in)  :: kaction
      integer, intent(out) :: change  ! = 1 se cambia qualcosa
      logical              :: move_option
!
      if (kaction == 0) then       ! Build Bonds
          call save_action(BUILD_BONDS,0,kmodel)
          move_option = .false.
      else                         ! Assemble Residues
          call save_action(ASSEMBLE_RESIDUES,0,kmodel)
          move_option = .true.
      endif
!
      if (numatoms(modelview(kmodel)%at) > 1) then
          if (move_option) then
              call assemble_fragments(modelview(kmodel)%at,modelview(kmodel)%leg,   &
                set_cell_type(cell),spacegr,check=bondset%angle > 0, tolmin=bondset%tolmin, &
                tolmax=bondset%tolmax,angl=bondset%angle)
          else
              call create_connectivity(modelview(kmodel)%at,modelview(kmodel)%leg,  &
                set_cell_type(cell),spacegr,check=bondset%angle > 0, tolmin=bondset%tolmin, &
                tolmax=bondset%tolmax, angl=bondset%angle,move=move_option)
          endif
!
          change =  compare_legm(uract_temp%legm,modelview(kmodel)%leg)
!
          call jav_redraw(kmodel)  
      else
          change = 0 
      endif
!
      end subroutine jav_reconnect

!-----------------------------------------------------------------------

      subroutine jav_rename(kmodel,kptraa,nptra,newname)
      USE strutil
      USE atom_basic
      integer, intent(in)                :: kmodel
      integer, dimension(*), intent(in)  :: kptraa
      integer, intent(in)                :: nptra
      character(len=*), intent(in)       :: newname
      integer, dimension(nptra)          :: kptra
      logical                            :: is_name
      integer                            :: i,j
      logical                            :: symactive
      character(len=100)                 :: sprefix
      character(len=:), allocatable      :: slabold,slabnew
      integer                            :: numbmax
      integer                            :: numb
      integer                            :: ierror
!
      symactive = symatoms(kmodel)
!
      kptra(:) = selected_atoms(kptraa,nptra,kmodel)
!
      if (nptra == 1) then
!
!         Controlla che il nuovo  nome non  esista gia'
          is_name = string_locate(newname,modelview(kmodel)%at(:)%lab) > 0
!
!         Se non esiste applica la modifica
          if (.not.is_name) then
              call jav_set_message(trim(modelview(kmodel)%at(kptra(1))%lab)//' renamed in '//trim(newname))
              modelview(kmodel)%at(kptra(1))%lab = newname
              if (symactive) then
                  do j=1,size(modelviews(kmodel)%at)
!corr                     if (modelviews(kmodel)%symm(j)%asym == kptra(1)) then
                     if (modelviews(kmodel)%at(j)%asym == kptra(1)) then
                         modelviews(kmodel)%at(j)%lab = newname
                     endif
                  enddo
                  call jav_passa_modello(modelviews(kmodel),kmodel)
              else
                  call jav_passa_modello(modelview(kmodel),kmodel)
              endif
          else
              call jav_set_message('Duplicate name exists')
          endif
      else
!
!         Se clicco su piu' atomi il nuovo nome e' usato come prefisso
!
!         Identifica tra gli atomi non cliccati il numero seriale piu' grande 
!         che ha per prefisso newname
          numbmax = 0
          do i=1,size(modelview(kmodel)%at)
             if (any(kptra(:nptra) == i)) cycle  ! escludi gli atomi cliccati
             call get_from_label(modelview(kmodel)%at(i)%lab,sprefix,numb,ierror)
             if (ierror == 0) then
                 if(numb > numbmax .and. s_eqi(newname,sprefix)) numbmax = numb
             endif
          enddo
!
!         Genera stringa con le vecchie label separate da virgola
          !call slabvet(kptra(:nptra),modelview(kmodel)%at(:)%lab,slabold,ierror,showord=.false.)
          slabold = slabvet(kptra(:nptra),modelview(kmodel)%at(:)%lab)
!
!         Modifica la label
          do i=1,nptra
             write(modelview(kmodel)%at(kptra(i))%lab,'(a,i0)')trim(newname),numbmax+i
          enddo
!
!         Estendi le modifiche a tutta la cella se la simmetria e' attiva
          if (symactive) then           
              do i=1,nptra
                 do j=1,size(modelviews(kmodel)%at)
!corr                    if (modelviews(kmodel)%symm(j)%asym == kptra(i)) then
                    if (modelviews(kmodel)%at(j)%asym == kptra(i)) then
                        modelviews(kmodel)%at(j)%lab = modelview(kmodel)%at(kptra(i))%lab
                    endif
                 enddo
              enddo
          endif
!
!         Genera stringa con le vecchie label separate da virgola
          !call slabvet(kptra(:nptra),modelview(kmodel)%at(:)%lab,slabnew,ierror,showord=.false.)
          slabnew = slabvet(kptra(:nptra),modelview(kmodel)%at(:)%lab)
          call jav_set_message(trim(slabold)//' renamed in '//trim(slabnew))
          if (symactive) then
              call jav_passa_modello(modelviews(kmodel),kmodel)
          else
              call jav_passa_modello(modelview(kmodel),kmodel)
          endif
      endif
!
      end subroutine jav_rename

!-----------------------------------------------------------------------

      subroutine jav_sort_by_specie(kmodel)
      USE atom_type_util
      USE unit_cell
      USE General, only: cell
      integer, intent(in)                :: kmodel
      integer, dimension(:), allocatable :: iord
!
      allocate(iord(numatoms(modelview(kmodel)%at))) 
      call sort_by_specie(modelview(kmodel)%at,modelview(kmodel)%leg,set_cell_type(cell),iorder=iord)
      call jav_redraw(kmodel)
!
      end subroutine jav_sort_by_specie

!-----------------------------------------------------------------------

      subroutine jav_renumber(kmodel,kptraa,nptra,string,iser)
      USE atom_type_util
      USE General, only:WARN_WINDOW
      USE atom_basic
      integer, intent(in)               :: kmodel
      integer, dimension(*), intent(in) :: kptraa
      integer                           :: nptra
      integer, dimension(nptra)         :: kptra
      integer, dimension(nptra)         :: vetd
      character(len=*), intent(in)      :: string
      integer                           :: iser
      integer                           :: i,j
      character(len=100)                :: msgw
      character(len=:), allocatable     :: strwarn
      integer                           :: nd
      integer                           :: iExCode
      logical                           :: symactive
      integer, parameter                :: MAXLENS = 100
      integer                           :: posc
!
      symactive = symatoms(kmodel)
!
      kptra(:) = selected_atoms(kptraa,nptra,kmodel)
!
      call renumber_atoms(modelview(kmodel)%at,kptra(:nptra),string,iser,vetd,nd)
!
!     Messaggio di errore per stringhe che si duplicano
      if (nd > 0) then
          strwarn = slabvet(vetd(:nd),modelview(kmodel)%at%lab)  ! concatena le labels che si duplicano
          if (nd == 1) then
              msgw = 'There is a couple of atoms with the same name:'
          else
              msgw = 'There are couples of atoms with the same name:'
          endif
          if (len(strwarn) > MAXLENS) then
              posc = index(strwarn(1:MAXLENS),',',back=.true.)  ! cut at ','
              call MsgWinErr('W A R N I N G',trim(msgw)//' '//strwarn(1:posc)//' ...',WARN_WINDOW,iExCode)
          else
              call MsgWinErr('W A R N I N G',trim(msgw)//' '//trim(strwarn),WARN_WINDOW,iExCode)
          endif
      endif
!
!     Disegna
      if (symactive) then
!
!         Estendi l'informazione sulle label a tutta la cella
          do i=1,nptra
             do j=1,size(modelviews(kmodel)%at)
!corr                if (modelviews(kmodel)%symm(j)%asym == kptra(i)) then
                if (modelviews(kmodel)%at(j)%asym == kptra(i)) then
                    modelviews(kmodel)%at(j)%lab = modelview(kmodel)%at(kptra(i))%lab
                endif
             enddo
          enddo
          call jav_passa_modello(modelviews(kmodel),kmodel)
      else
          call jav_passa_modello(modelview(kmodel),kmodel)
      endif
!
      end subroutine jav_renumber

!-----------------------------------------------------------------------

      subroutine jav_enantiomorph(kmodel)
      USE Symmetry, only:icent
      integer :: i
      integer, intent(in) :: kmodel
!
      if (icent == 0) then
          do i=1,size(modelview(kmodel)%at)
             modelview(kmodel)%at(i)%xc = 1 - modelview(kmodel)%at(i)%xc
          enddo
          if (symatoms(kmodel)) then
              do i=1,size(modelviews(kmodel)%at)
                 modelviews(kmodel)%at(i)%xc = 1 - modelviews(kmodel)%at(i)%xc
              enddo
              call jav_passa_modello(modelviews(kmodel),kmodel)
          else
              call jav_passa_modello(modelview(kmodel),kmodel)
          endif
      endif
!
      end subroutine jav_enantiomorph

!-----------------------------------------------------------------------

      function selected_atoms(kptr,nptr,kmod)  result(vsel)
!
!     Determina gli atomi asimmetrici selezionati
!
      integer, dimension(nptr) :: kptr
      integer, intent(in)      :: nptr
      integer, intent(in)      :: kmod
      integer, dimension(nptr) :: vsel
      integer                  :: i
      logical :: symactive
!
      symactive = symatoms(kmod)
!
      if (symactive) then       
          do i=1,nptr
!corr             vsel(i) = modelviews(kmod)%symm(kptr(i))%asym 
             vsel(i) = modelviews(kmod)%at(kptr(i))%asym 
          enddo
      else
          vsel(:) = kptr(:nptr)
      endif
!
      end function selected_atoms

!-----------------------------------------------------------------------

      subroutine jav_complete(kmodel,kptraa,nptra,dist)
      USE molpnew
      USE General, only: cell,spacegr,celem
      USE unit_cell
      integer, intent(in)               :: kmodel
      integer, dimension(*), intent(in) :: kptraa
      integer, intent(in)               :: nptra
      real, intent(in)                  :: dist
      integer, dimension(nptra)         :: kptra
!
      kptra(:) = selected_atoms(kptraa,nptra,kmodel)
!
      call complete_atoms(modelview(kmodel)%at,set_cell_type(cell),spacegr,celem,kptra(:nptra),dist,modelview(kmodel)%leg)
!
      call jav_redraw(kmodel)
!
      end subroutine jav_complete

!-----------------------------------------------------------------------

      subroutine jav_regularize(kmodel,kptraa,nptra,dist,ier)
      USE molpnew
      USE general, only:spacegr,cell
      USE unit_cell
      integer, intent(in)               :: kmodel
      integer, dimension(*), intent(in) :: kptraa
      integer, intent(in)               :: nptra
      real, intent(in)                  :: dist
      integer, intent(out)              :: ier
      integer, dimension(nptra)         :: kptra
!
      kptra(:) = selected_atoms(kptraa,nptra,kmodel)
!
      call regularize_atoms(modelview(kmodel)%at,kptra(:nptra),dist,modelview(kmodel)%leg,spacegr,set_cell_type(cell),ier)
!
      if (ier == 0) call jav_redraw(kmodel)
!
      end subroutine jav_regularize

!-----------------------------------------------------------------------

      subroutine jav_add_hydrogens(kmodel,kptraa,nptra,htype,ier)
      USE hydrogens
      USE General, only:WARN_WINDOW,cell,spacegr,celem
      USE atom_type_util
      USE unit_cell
      USE variables, only: cryst
!corr      USE proginterface
      integer, intent(in)               :: kmodel
      integer, dimension(*), intent(in) :: kptraa
      integer, intent(in)               :: nptra
      integer, intent(in)               :: htype
      integer, intent(out)              :: ier
      integer, dimension(nptra)         :: kptra
      integer                           :: exitcode
!
      kptra(:) = selected_atoms(kptraa,nptra,kmodel)
!
      call add_hydrogens(modelview(kmodel)%at,modelview(kmodel)%leg,celem,set_cell_type(cell),spacegr,kptra(:nptra),htype,ier)
      if (ier > 0) then
          if (nptra == 1) then
              call MsgWinErr('Hydrogens procedure','This atom seems to have incorrect connectivity',WARN_WINDOW,exitcode)
          else
              call MsgWinErr('Hydrogens procedure','Some atom(s) seem to have incorrect connectivity',WARN_WINDOW,exitcode)
          endif
      endif
!
!     Segnale errore solo se tutti gli atomi hanno dato errore
      if (ier < nptra) ier = 0
!
      if (ier == 0) then
!corr          call elem_to_nk(celem)
          call cryst(1)%set_scattering(celem)
          call jav_redraw(kmodel)   
      endif
!
      end subroutine jav_add_hydrogens

!-----------------------------------------------------------------------

      subroutine jav_add_hydrogens_auto(kmodel,kptraa,nptra,ier)
      USE atom_type_util
      USE General, only:WARN_WINDOW,cell,spacegr,celem
      USE hydrogens
      USE unit_cell
      USE variables, only: cryst
!corr      USE proginterface
      integer, intent(in)               :: kmodel
      integer, dimension(*), intent(in) :: kptraa
      integer, intent(in)               :: nptra
      integer, intent(out)              :: ier
      integer, dimension(nptra)         :: kptra
!
      if (nptra > 0) then
          kptra(:) = selected_atoms(kptraa,nptra,kmodel)
          call add_hydrogens(modelview(kmodel)%at,modelview(kmodel)%leg,celem,set_cell_type(cell),spacegr,kptra(:nptra),ier=ier)
          if (ier < nptra) ier = 0 ! Segnale errore solo se tutti gli atomi hanno dato errore
      else
          call add_hydrogens(modelview(kmodel)%at,modelview(kmodel)%leg,celem,set_cell_type(cell),spacegr,ier=ier)
          if (ier < numatoms(modelview(kmodel)%at)) ier = 0 ! Segnale errore solo se tutti gli atomi hanno dato errore
      endif
!
      if (ier == 0) then
!corr          call elem_to_nk(celem)
          call cryst(1)%set_scattering(celem)
          call jav_redraw(kmodel)
      endif
!
      end subroutine jav_add_hydrogens_auto
      
!-----------------------------------------------------------------------

   subroutine jav_add_fragment(kmodel,filename,err)
   USE importexport, only: action_after_import
   USE errormod
   USE Molcom, only: jscreen
   USE General, only: ifAutomatic, celem, spacegr, cell, neutro
   USE atom_type_util
   USE gen_frm
   USE unit_cell
   integer, intent(in)                    :: kmodel
   character(len=*), intent(in), optional :: filename
   type(error_type), intent(out)          :: err
   logical                                :: gui,has_symmetry
   type(cell_type)                        :: cellt
!
   gui = (jscreen > 0 .and. ifAutomatic == 0)
   has_symmetry = numatoms(modelview(kmodel)%at) > 0
   cellt = set_cell_type(cell)
   call import_from_file(filename,modelview(kmodel)%at,modelview(kmodel)%leg,celem,cellt,spacegr,.true.,gui,has_symmetry,neutro,err)
   call action_after_import(filename,has_symmetry,celem,cellt,spacegr)
   if (err%signal) return
   call jav_redraw(kmodel)
!
   end subroutine jav_add_fragment

!-----------------------------------------------------------------------

   subroutine jav_overlay(filename,err)
   USE importexport
   USE compare_struct
   USE atom_type_util
   USE fileutil
   USE gen_frm
   USE unit_cell
   USE General, only: cell,spacegr,celem,spacegr,cell
   USE errormod
   character(len=*), intent(in)  :: filename
   type(error_type), intent(out) :: err
   integer                       :: kmodel
   integer                       :: tipo
   logical                       :: has_symmetry
!
   tipo = code_fragfilen(filename)
!
   num_models = num_models + 1
   kmodel = num_models
   has_symmetry = .true.
   call import_structure(filename,tipo,modelview(kmodel)%at,modelview(kmodel)%leg,celem, &
                         cell,spacegr,.false.,has_symmetry,err)
   if (.not.err%signal) then
       call overlay_models(modelview(1)%at,modelview(1)%leg,    &
            modelview(kmodel)%at,modelview(kmodel)%leg,set_cell_type(cell),spacegr,celem)
       call jav_redraw(kmodel)  
       call jav_set_name(file_get_name(filename), kmodel) 
   else
       num_models = num_models - 1
   endif
!
   end subroutine jav_overlay

!-----------------------------------------------------------------------

   subroutine jav_compare(distance, kmodel)
   USE compare_struct
   USE unit_cell
   USE spginfom
   USE general, only:cell,spacegr,celem
   !corr type(compare_info_type), intent(out) :: info_compare
   real, intent(in)                     :: distance
   integer                              :: kmodel
!
   call compare_models(modelview(1)%at,modelview(kmodel)%at,spacegr,set_cell_type(cell), &
                       celem,dist=distance,infoc=infoc,kpr=0)
!corr   infoc = info_compare
!
   end subroutine jav_compare

!-----------------------------------------------------------------------

      subroutine jav_move(kmodel,kptraa,nptra,ier)
      USE molpnew
      USE General, only: cell,spacegr
      USE unit_cell
      integer, intent(in)               :: kmodel
      integer, dimension(*), intent(in) :: kptraa
      integer, intent(in)               :: nptra
      integer, intent(out)              :: ier
      integer, dimension(nptra)         :: kptra
!
      kptra(:) = selected_atoms(kptraa,nptra,kmodel)
      call move_atoms(modelview(kmodel)%at,modelview(kmodel)%leg,spacegr,kptra(:nptra-1),kptra(nptra),set_cell_type(cell),ier)
      call jav_redraw(kmodel)  
!
      end subroutine jav_move

!-----------------------------------------------------------------------

      logical function is_polyhedra(kmodel)
!
!     Chiedi al C se e' attiva la funzione poliedri
!
      integer, intent(in) :: kmodel
      is_polyhedra = ilmview_info(6,kmodel) == 1 

      end function is_polyhedra

!-----------------------------------------------------------------------

      logical function is_grow_fragment(kmodel)
!
!     Chiedi al C se e' attiva la funzione grow fragment
!
      integer, intent(in) :: kmodel
      is_grow_fragment = ilmview_info(7,kmodel) == 1 

      end function is_grow_fragment

!-----------------------------------------------------------------------

      logical function is_packing(kmodel)
!
!     Chiedi al C se e' attiva la funzione grow fragment
!
      integer, intent(in) :: kmodel
      is_packing = ilmview_info(2,kmodel) == 1 

      end function is_packing

!-----------------------------------------------------------------------

      logical function is_hydrogen_bonds(kmodel)
!
!     Chiedi al C se e' attiva la funzione per visualizzare i legami H
!
      integer, intent(in) :: kmodel
      is_hydrogen_bonds = ilmview_info(8,kmodel) == 1 
!
      end function is_hydrogen_bonds

!-----------------------------------------------------------------------

      logical function is_short_contacts(kmodel)
!
!     Chiedi al C se e' attiva la funzione per visualizzare gli short contacts
!
      integer, intent(in) :: kmodel
      is_short_contacts = ilmview_info(9,kmodel) == 1

      end function is_short_contacts

!-----------------------------------------------------------------------

      subroutine reallocate_urtype(vetr,n,savevet)
!
!     Rialloca ad n un vettore di tipo urtype.
!     Se savevet = .true. o non esiste si salva il suo contenuto.
!
      type(urtype), allocatable, intent(inout) :: vetr(:)
      integer, intent(in)                      :: n
      logical, optional, intent(in)            :: savevet
      logical                                  :: savev
      integer                                  :: nv
      type(urtype), allocatable                :: vsav(:)
      integer                                  :: nsav
!
!     se n = 0 (riallocazione a 0): dealloca ed esci
      if (n == 0) then
          if (allocated(vetr)) deallocate(vetr)
          return
      endif
!
      if (.not.allocated(vetr)) then
          allocate(vetr(n))
      else
!
          nv = size(vetr)
          if (present(savevet)) then
              savev = savevet
          else
              savev = .true.
          endif
!
          if (savev) then
!
!             nsav contiene qual è la porzione di vetr da salvare
              select case(nv-n)
                case (1:)       ! compatta x ad n
                  nsav = n
                case (:-1)      ! espandi x ad n
                  nsav = nv
                case (0)
                  return        ! n=nv non fare niente
              end select
!
!             salva vetr fino a nsav
              allocate(vsav(nsav))
              vsav = vetr(:nsav)
!
!             dealloca vetr
              deallocate(vetr)
!
!             rialloca vetr su n
              allocate(vetr(n))
!
!             ripristina vetr fino a nsav
              vetr(:nsav) = vsav(:nsav)
          else
              !if (nv /= n) then
                  deallocate(vetr)
                  allocate(vetr(n))
              !endif
          endif
      endif
!
      end subroutine reallocate_urtype

!-----------------------------------------------------------------------

      subroutine init_undoredo()
!
      nuract = 0
      posur = 0
      call reallocate_urtype(uract,0)
      call jav_set_undoredo(ID_UNDO,0,' ')
      call jav_set_undoredo(ID_REDO,0,' ')
!
      end subroutine init_undoredo

!-----------------------------------------------------------------------

   subroutine save_action(azione,code,kmodel)
   implicit none
   integer, intent(in) :: azione
   integer, intent(in) :: code
   integer, intent(in) :: kmodel
!
   if (code == 0) then
       uract_temp%azione = azione
       call undor_azione(azione,0,uract_temp,kmodel)
   elseif (code == 1) then
!
!      Salva lo stato prima della modifica
       if (mod(posur,2) /= 0) then
           posur = posur - 1
       endif
       nuract = posur + 1
       call reallocate_urtype(uract,nuract+1)
       posur = nuract
       uract(posur) = uract_temp
!
!      Salva lo stato dopo la modifica
       nuract = posur + 1
       posur = nuract
       uract(nuract)%azione = azione
       if (mod(nuract,2) == 0) then
           call state_undoredo(posur,0)
       endif
       call undor_azione(azione,0,uract(posur),kmodel)
   endif
!
   end subroutine save_action

!-----------------------------------------------------------------------

   subroutine undor_azione(azione,code,uraction,kmodel)
   integer, intent(in)         :: azione 
   integer, intent(in)         :: code 
   type(urtype), intent(inout) :: uraction
   integer, intent(in)         :: kmodel
!
   select case(azione)
      case (DELOBJ,ASSEMBLE_RESIDUES,SORT_BY_SPEC,COMPLETE,REGULARIZE,      &
            MOVE_ATOMS,EDIT_STRUCT,RBM,FOURIER,DUPLICATE,CHANGE_SPEC,       &
            DEL_DUPLICATE_ATOMS,TRANSFORM_AT)
        call undor_model(code,uraction,kmodel)

      case (ADD_FRAG,H_AUTOMATIC, H_METHINE,H_METHYLENE,H_AROMATIC,  &
            H_HYDROXYL,H_TERMINAL_CH2,H_TERMINAL_BH,H_ACETYLENIC,H_METHYL,  &
            ADD_SMILES,ADD_ATOM)
        call undor_content(code,uraction)
        call undor_model(code,uraction,kmodel)

      case (IMPORT)
        call undor_symmetry(code,uraction)
        call undor_content(code,uraction)
        call undor_model(code,uraction,kmodel)

      case (ADD_BONDS,BUILD_BONDS,BOND_SETTINGS)
       call undor_bonds(code,uraction,kmodel)

      case (REN_ATOMS,RENUMBER,ENANT,OPTIMIZE_GEOMETRY)
       call undor_atoms(code,uraction,kmodel)

      case (NORMALISEH)
       call undor_hydrogens(code,uraction,kmodel)

   end select
!
   end subroutine undor_azione

!-----------------------------------------------------------------------

   subroutine undo_redo(code)
   USE exportfiles
   implicit none
   integer, intent(in) :: code  ! 1=redo; -1=undo
   integer             :: pos
   integer             :: kmodel
!
   if (code == -1) then
       if (mod(posur,2) == 0) then
           posur = posur - 1
       else
           posur = posur - 2
       endif
       pos = posur
       call state_undoredo(posur,-1)
   else
       if (mod(posur,2) == 0) then
           posur = posur + 2
       else
           posur = posur + 1
       endif
       pos = posur
       call state_undoredo(posur,+1)
   endif
!
   kmodel = uract(pos)%active
   call undor_azione(uract(pos)%azione,1,uract(pos),kmodel)
!
!  unset this variable becouse jav_redraw is unable to rebuild info from transform tool
   sym_atoms(kmodel) = .false. 
!
   call jav_redraw(kmodel)
   call update_model_and_export(modelview(kmodel)%at,modelview(kmodel)%leg,kmodel)
!
   end subroutine undo_redo

!-----------------------------------------------------------------------

   subroutine state_undoredo(pos,direction)
!   
!  Setting dello stato e delle stringhe dei tasti undo/redo
!
   integer, intent(in) :: pos
   integer, intent(in) :: direction
   integer             :: azione
!
   if (nuract == 0) then
       call jav_set_undoredo(ID_UNDO,0,' ')
       call jav_set_undoredo(ID_REDO,0,' ')
   else
       if (pos == nuract) then           ! eseguo una nuova azione
           azione = uract(pos)%azione
           call jav_set_undoredo(ID_UNDO,1,ustr(azione))
           call jav_set_undoredo(ID_REDO,0,' ')
       elseif (pos == 1) then            ! sono all'inizio dello stack delle azioni
           azione = uract(pos)%azione
           call jav_set_undoredo(ID_UNDO,0,' ')
           call jav_set_undoredo(ID_REDO,1,ustr(azione))
       else                              ! ho premuto undo/redo
           if (direction == 1) then      ! redo
               azione = uract(pos)%azione
               call jav_set_undoredo(ID_UNDO,1,ustr(azione))
               azione = uract(pos+1)%azione
               call jav_set_undoredo(ID_REDO,1,ustr(azione))
           elseif (direction == -1) then ! undo
               azione = uract(pos-1)%azione
               call jav_set_undoredo(ID_UNDO,1,ustr(azione))
               azione = uract(pos)%azione
               call jav_set_undoredo(ID_REDO,1,ustr(azione))
           endif
       endif
   endif
!
   end subroutine state_undoredo

!-----------------------------------------------------------------------

   subroutine undor_model(azione,uraction,kmodel)
!
!  azione = 0/1 -> save/restore modello strutturale
!  
   USE atom_type_util
   USE connect_mod
   integer, intent(in)         :: azione
   type(urtype), intent(inout) :: uraction
   integer, intent(in)         :: kmodel
   integer                     :: kmod
!
   if (azione == 0) then      ! salva
       uraction%active = kmodel
       call copy_atoms(uraction%atom,modelview(kmodel)%at)
       call copy_bonds(uraction%legm,modelview(kmodel)%leg)
   else                       ! ripristina
       kmod = uraction%active
!
!      undo/redo save only a.u., expansion level > 1 require symmetry of saved atoms
       if (expandc(kmod)%level > 1) expandc(kmod)%level = 1
!
       call copy_atoms(modelview(kmod)%at,uraction%atom)
       call copy_bonds(modelview(kmod)%leg,uraction%legm)
            call ClearDistAng()
   endif
!
   end subroutine undor_model

!-----------------------------------------------------------------------

   subroutine undor_bonds(azione,uraction,kmodel)
!
!  azione = 0/1 -> save/restore legami
!  
   USE connect_mod
   integer, intent(in)         :: azione
   type(urtype), intent(inout) :: uraction
   integer, intent(in)         :: kmodel
   integer                     :: kmod
!
   if (azione == 0) then      ! salva
       uraction%active = kmodel
       call copy_bonds(uraction%legm,modelview(kmodel)%leg)
   else                       ! ripristina
       kmod = uraction%active
       call copy_bonds(modelview(kmod)%leg,uraction%legm)
   endif
!
   end subroutine undor_bonds

!-----------------------------------------------------------------------

   subroutine undor_atoms(azione,uraction,kmodel)
!
!  azione = 0/1 -> save/restore atoms
!  
   USE atom_type_util
   USE General, only: cell
   USE unit_cell
   USE connect_mod
   integer, intent(in)         :: azione
   type(urtype), intent(inout) :: uraction
   integer, intent(in)         :: kmodel
   integer                     :: kmod
   type(cell_type)             :: cellt
!
   if (azione == 0) then     ! save
       uraction%active = kmodel
       call copy_atoms(uraction%atom,modelview(kmodel)%at)
   else                      ! restore
       kmod = uraction%active
!
!      undo/redo save only a.u., expansion level > 1 require symmetry of saved atoms
       if (expandc(kmod)%level > 1) expandc(kmod)%level = 1
!
       call copy_atoms(modelview(kmod)%at,uraction%atom)
!
!      ribuild bond distances
       cellt = set_cell_type(cell)
       call bond_distance_update(modelview(kmod)%at,modelview(kmod)%leg,cellt%get_g())
!
       call ClearDistAng()
   endif
!
   end subroutine undor_atoms

!-----------------------------------------------------------------------

   subroutine undor_hydrogens(azione,uraction,kmodel)
!
!  azione = 0/1 -> save/restore hydrogens
!  
   USE hydrogens
   integer, intent(in)         :: azione
   type(urtype), intent(inout) :: uraction
   integer, intent(in)         :: kmodel
   integer                     :: kmod
!
   if (azione == 0) then
       uraction%active = kmodel
       call save_hydrogens(modelview(kmodel)%at,uraction%atom)
   else
       kmod = uraction%active
       call update_hydrogens(modelview(kmod)%at,uraction%atom)
       call ClearDistAng()
   endif
!
   end subroutine undor_hydrogens

!-----------------------------------------------------------------------

   subroutine undor_content(azione,uraction)
!
!  azione = 0/1 -> save/restore cell content
!  
   USE general, only: celem
!corr   USE proginterface
   USE elements
   USE variables, only: cryst
   integer, intent(in)         :: azione
   type(urtype), intent(inout) :: uraction
!
   if (azione == 0) then
       call copy_elem(uraction%elem,celem)
   else
!corr       call elem_to_nk(uraction%elem)
       call cryst(1)%set_scattering(uraction%elem)
       call copy_elem(celem,uraction%elem)
   endif
!
   end subroutine undor_content
!-----------------------------------------------------------------------

   subroutine undor_symmetry(azione,uraction)
!
!  azione = 0/1 -> save/restore cell content
!  
   USE General, only: spacegr, cell
   USE sinterface
   USE variables, only: cryst
   integer, intent(in)         :: azione
   type(urtype), intent(inout) :: uraction
   if (azione == 0) then
       uraction%spg = spacegr
       uraction%cell = cell(:6)
   else
       spacegr = uraction%spg 
       call cryst(1)%set_spg(spacegr)
       cell(:6) =  uraction%cell
       call set_spaceg_new(spacegr)
   endif
    
   end subroutine undor_symmetry

!-----------------------------------------------------------------------
      subroutine ilmview_load_bond_table()
      USE BONDTMOD
!
      call load_bond_table()  !!aggiunta da mr
!
      end subroutine ilmview_load_bond_table
!-----------------------------------------------------------------------
      subroutine jav_set_num_models(nmodelli)
      implicit none
      integer::     nmodelli
!
      num_models = nmodelli
!
      end subroutine jav_set_num_models
!-----------------------------------------------------------------------
      subroutine jav_set_name(nome,kmodel)
      implicit none
      character(len=*), intent(in) :: nome
      integer, intent(in)          :: kmodel
      integer                      :: Lung
!
      call SetName(nome(1:Lung(nome)) // char(0), kmodel)
      end subroutine jav_set_name

!-----------------------------------------------------------------------

      subroutine jav_get_bond_table(table)
      USE bondtmod
      USE General, only:celem
      USE elements
!
      type(bond_table_type), dimension(:), allocatable :: table
      integer                                          :: numt
!     
      if (numelem(celem) > 0) then
          call get_bond_table(celem%z,table,numt)
      endif
!
      end subroutine jav_get_bond_table

!----------------------------------------------------------------------

      subroutine jav_duplicate(kmodel,kptraa,nptra,ier)
      USE atom_type_util
      USE General, only: celem
      integer, intent(in)                   :: kmodel
      integer, dimension(nptra), intent(in) :: kptraa
      integer, intent(in)                   :: nptra
      integer, intent(out)                  :: ier
      integer, dimension(nptra)             :: kptra
!
      ier = 0
      if (numatoms(modelview(kmodel)%at) == 0) then
          ier = 1
      else
          if (nptra == 0) then
              call duplicate_atoms(modelview(kmodel)%at,modelview(kmodel)%leg,celem)
          else
              kptra(:) = selected_atoms(kptraa,nptra,kmodel)
              call duplicate_atoms(modelview(kmodel)%at,modelview(kmodel)%leg,celem,kptra)
          endif
          call jav_redraw(kmodel)
      endif
      end subroutine jav_duplicate

!----------------------------------------------------------------------

   subroutine jav_delete_duplicate(kmodel,dist,ier)
   USE atom_type_util
   USE strutil
   USE unit_cell
   USE General, only: cell,spacegr
   integer, intent(in)                            :: kmodel
   real, intent(in)                               :: dist
   integer, intent(out)                           :: ier
   integer, dimension(size(modelview(kmodel)%at)) :: vet
   integer                                        :: nd
!
   ier = 1
   call find_duplicate_atoms(modelview(kmodel)%at,modelview(kmodel)%leg,set_cell_type(cell),spacegr,dist,vet,nd,-1)
   if (nd > 0) then
       ier = 0
       call remove_atoms_vet(modelview(kmodel)%at,modelview(kmodel)%leg,vet(:nd))
       call jav_set_message(trim(i_to_s(nd))//' atoms deleted')
       call jav_redraw(kmodel)
   endif
!
   end subroutine jav_delete_duplicate

!----------------------------------------------------------------------

   subroutine jav_add_atom(kmodel,kat,ispeci,btype,ier)
   USE hydrogens
   USE unit_cell
   USE general, only:cell, celem
   integer, intent(in)  :: kmodel,kat
   integer, intent(in)  :: ispeci,btype
   integer, intent(out) :: ier
   integer              :: ispec
!
   ispec = check_element(ispeci,1)
   call add_atom_bound_to(kat,modelview(kmodel)%at,modelview(kmodel)%leg,set_cell_type(cell),celem,ispec,btype,ier)
   call jav_redraw(kmodel)
!
   end subroutine jav_add_atom

!----------------------------------------------------------------------

   subroutine jav_smiles(kmodel,smiles_string,saction,ier)
   USE General, only: celem, neutro, cell, spacegr
   USE errormod
   USE unit_cell
   USE gen_frm
   USE variables, only: cryst
   integer, intent(in)          :: kmodel
   character(len=*), intent(in) :: smiles_string
   integer, intent(in)          :: saction
   integer, intent(out)         :: ier
   type(error_type)             :: err
!
   call get_smiles(smiles_string,modelview(kmodel)%at,modelview(kmodel)%leg,celem,spacegr%nsymop,  &
                   neutro,set_cell_type(cell),saction /= 0,err)
   if (err%signal) then
       ier = 1
   else
       call cryst(1)%set_scattering(celem)
       call jav_redraw(kmodel)
       ier = 0
   endif
!
   end subroutine jav_smiles

END module ilmviewmod

!-----------------------------------------------------------------------

   subroutine jav_sphere(kat,dist, kmodel) 
   USE ilmviewmod
   USE atom_type_util
   USE unit_cell
   USE General, only: cell, spacegr  !!!, celem
   USE symmgrid
   integer, intent(in)                      :: kat       ! atomo selezionato
   integer, intent(in)                      :: kmodel
   real, intent(in)                         :: dist      ! raggio della sfera
   integer                                  :: kfound    ! num. di atomi trovati
   integer, dimension(:), allocatable       :: vat       ! atomo trovato
   type(op_type), dimension(:), allocatable :: vsym      ! op. di simmetria
   real, dimension(:), allocatable          :: vdist     ! distanze da kat
   integer                                  :: symactive
!corr   integer                                  :: i
   type(cell_type)                          :: cellt
!corr   real                                     :: bvs
!
   symactive = ilmview_info(2, kmodel)
   cellt = set_cell_type(cell)

   if (symactive == 1 .or. kat > numatoms(modelview(kmodel)%at)) then  ! bug corrected: click on atom after expand
       call get_distance_radius(modelviews(kmodel)%at,cellt,spacegr,kat,dist,1,kfound,vat,vdist,vsym,modelviews(kmodel)%at)
       !do i=1,kfound
       !   vsym(i) = modelviews(kmodel)%at(vat(i))%op
       !enddo
       call put_found(kfound,vat,vdist,vsym)
   else
       call get_distance_radius(modelview(kmodel)%at,cellt,spacegr,kat,dist,2,kfound,vat,vdist,vsym)
       call put_found(kfound,vat,vdist,vsym)
       !call compute_bvs(modelview(kmodel)%at,celem,cellt,spacegr,kat,dist,bvs)
       ! write(0,*)'BVS=',modelview(kmodel)%at(kat)%glab(),bvs
   endif
!
   end subroutine jav_sphere

!-----------------------------------------------------------------------

   integer function jav_modify_object(kptra,nptra,kptrb,nptrb,icod,threshold,kmodel,ispec,newname)
   USE ilmviewmod
   USE exportfiles
   USE hydrogens
   USE errormod
   implicit none
   integer, dimension(*), intent(in)      :: kptra   ! atomi selezionati
   integer, intent(in)                    :: nptra   ! num. di atomi
   integer, dimension(*), intent(in)      :: kptrb   ! legami selezionati
   integer, intent(in)                    :: nptrb   ! num. di legami
   integer, intent(in)                    :: icod    ! quale azione eseguire
   integer, optional, intent(in)          :: ispec   ! nuova specie
   character(len=*), optional, intent(in) :: newname ! nuovo nome or smiles
   integer                                :: ier
   integer, intent(in)                    :: kmodel
   integer                                :: change
   real, intent(in)                       :: threshold
   type(error_type)                       :: err
!
   ier = 0
!
!  Prima di ogni azione memorizza nello stack quello che si modifica
   select case (icod)
     case (DELOBJ)    ! Delete 
       call save_action(icod,0,kmodel)
       if (nptra > 0 .or. nptrb > 0) then
           call jav_delete_object(kmodel,kptra,nptra,kptrb,nptrb)
       else
           ier = 1  ! selezione nulla, segnale l'errore
       endif

     case (ADD_BONDS)    ! Add Bonds 
       call save_action(icod,0,kmodel)
       call jav_add_bonds(kmodel,kptra,nptra)

     case (CHANGE_SPEC)    ! Change Specie
       call save_action(icod,0,kmodel)
       call jav_change_specie(kmodel,ispec,kptra,nptra,ier)
    
     case (BUILD_BONDS)    ! Build Bonds
       !call save_action(icod,0,kmodel)
       call jav_reconnect(kmodel,0,change)
       if (change == 0) ier = 1

     case (ASSEMBLE_RESIDUES)    ! Assemble Residues
       !call save_action(icod,kmodel)
       call jav_reconnect(kmodel,1,change)
       if (change == 0) ier = 1

     case (REN_ATOMS)         ! Rename
       call save_action(icod,0,kmodel)
       call jav_rename(kmodel,kptra,nptra,newname(1:len_trim(newname))) 

     case (SORT_BY_SPEC)    ! Riordina gli atomi
       call save_action(icod,0,kmodel)
       call jav_sort_by_specie(kmodel)

     case (RENUMBER)    ! Renumber
       call save_action(icod,0,kmodel)
       call jav_renumber(kmodel,kptra,nptra,newname(1:len_trim(newname)),ispec)
       
     case (COMPLETE)    ! Complete
       call save_action(icod,0,kmodel)
       call jav_complete(kmodel,kptra,nptra,dist=nptrb/1000.)

     case (REGULARIZE)    ! Regularize
       call save_action(icod,0,kmodel)
       call jav_regularize(kmodel,kptra,nptra,nptrb/1000.,ier)

     case (ENANT)    ! Enantiomorph
       call save_action(icod,0,kmodel)
       call jav_enantiomorph(kmodel)

     case (H_ACETYLENIC)  ! Add Acetylenic Hydrogens 
       call save_action(icod,0,kmodel)
       call jav_add_hydrogens(kmodel,kptra,nptra,C_ALCHINO,ier)

     case (H_METHYLENE)   ! Add Methilene Hydrogens
       call save_action(icod,0,kmodel)
       call jav_add_hydrogens(kmodel,kptra,nptra,CH2,ier)

     case (H_AUTOMATIC)   ! Add Hydrogens
       call save_action(icod,0,kmodel)
       call jav_add_hydrogens_auto(kmodel,kptra,nptra,ier)

     case (H_METHINE)     ! Add Methine Hydrogens
       call save_action(icod,0,kmodel)
       call jav_add_hydrogens(kmodel,kptra,nptra,CH,ier)

     case (H_METHYL)      ! Add Methyl group
       call save_action(icod,0,kmodel)
       call jav_add_hydrogens(kmodel,kptra,nptra,CH3,ier)

     case (H_AROMATIC)    ! Add Aromatic Hydrogens
       call save_action(icod,0,kmodel)
       call jav_add_hydrogens(kmodel,kptra,nptra,C_Ar,ier)

     case (H_HYDROXYL)    ! Add Aromatic Hydrogens
       call save_action(icod,0,kmodel)
       call jav_add_hydrogens(kmodel,kptra,nptra,O_H,ier)

     case (H_TERMINAL_CH2)    ! Add terminal X=CH2
       call save_action(icod,0,kmodel)
       call jav_add_hydrogens(kmodel,kptra,nptra,X_CH2,ier)

     case (H_TERMINAL_BH)    ! Add terminal BH
       call save_action(icod,0,kmodel)
       call jav_add_hydrogens(kmodel,kptra,nptra,B_H,ier)

     case (ADD_FRAG)
!!per Corrado: in questo caso nptrb contiene il tipo
       call save_action(icod,0,kmodel)
       call jav_add_fragment(kmodel,filename=newname(1:len_trim(newname)),err=err)
       if (err%signal) ier = 1
       
     case (MOVE_ATOMS)
       call save_action(icod,0,kmodel)
       call jav_move(kmodel,kptra,nptra,ier)

     case (DUPLICATE)
       call save_action(icod,0,kmodel)
       call jav_duplicate(kmodel,kptra,nptra,ier)

     case (DEL_DUPLICATE_ATOMS)
       call save_action(icod,0,kmodel)
       call jav_delete_duplicate(kmodel,threshold,ier)

     case (ADD_ATOM)
       call save_action(icod,0,kmodel)
       call jav_add_atom(kmodel,kptra(1),ispec,nptrb,ier)

     case (ADD_SMILES)
       call save_action(icod,0,kmodel)
       call jav_smiles(kmodel,newname(1:len_trim(newname)),nptra,ier)

     case default 
       ier = 1

   end select
!      
   if (ier == 0) then
       call save_action(icod,1,kmodel)     ! metti la modifica nello stack
       call update_model_and_export(modelview(kmodel)%at,modelview(kmodel)%leg,kmodel)
   endif
   jav_modify_object = ier
!
   end function jav_modify_object

!-----------------------------------------------------------------------

   subroutine jav_undo_redo(icod)
   USE ilmviewmod
   implicit none
   integer, intent(in) :: icod    ! -1 = undo, 1 = redo
   call undo_redo(icod)
   end subroutine jav_undo_redo

!-----------------------------------------------------------------------

      subroutine jav_set_undoredo(ibutton,istate,string)
      USE ilmviewmod
      implicit none
      integer                        :: ibutton    ! 0 = undo, 1 = redo
      integer                        :: istate     ! 0 = disable, 1 = enable
      character (len=*)              :: string 
      character (len=80)             :: string2
!
      string2 = string // char(0)
      call set_undoredo(ibutton,istate,string2)
!
      end subroutine jav_set_undoredo

!-----------------------------------------------------------------------

      subroutine GetB( B)
      USE Molcom, only:  BTot
!
      implicit none
      real            :: B(3, 3)
      B(:,:) = BTot(:,:)
      return
!
      end
!-----------------------------------------------------------------------

      subroutine ApplicaSimmetria(Which)
      USE ilmviewmod
      implicit none
      integer, intent(in) :: Which
!
      expandc = expand_option_type(0,0,0)
      sym_atoms(which) = .false.
      call jav_redraw(Which) 
!
      end subroutine ApplicaSimmetria

!-----------------------------------------------------------------------

      subroutine WriteMapXYZ_expo(fnamexyz)
      USE General
      USE Riflessi
      USE Mappa
      USE sinterface
!
      implicit none
      character(len=*), intent(in) :: fnamexyz
      integer                      :: jop, jopen, itype, iform, ier2, ix, iy, iz
!
      itype = 0
      iform = 1
      jop = jopen(jrel,fnamexyz,itype,iform,ier2)
      write(jrel,'(6f9.3)') cell(1:6)
      Inte = spaceg_string()
      call ucase(Inte)
      write(jrel,'(a)') Inte(1:16)
!
      do iz =0, np(3)-1
         do iy =0, np(2)-1
            do ix =0, np(1)-1
               write(jrel,'(3i6,f15.5)') ix, iy, iz, xmap(ix,iy,iz)
            enddo
         enddo
      enddo
      close (jrel)
!
      return
      end
!-----------------------------------------------------------------------
      subroutine GetAngle(kmodel)
      USE ilmviewmod
      USE connect_mod
      USE General, only: cell
      USE unit_cell
      implicit none
      type(angle_type), dimension(:), allocatable :: angle
      integer                                     :: numang
      integer                                     :: kmodel
      type(cell_type)                             :: cellt
!
      cellt = set_cell_type(cell)
      call legm_to_ang(modelview(kmodel)%leg,modelview(kmodel)%at,cellt%get_g(),angle)
      numang = numangles(angle)
      if (numang == 0) then
          call PassaAngle (kmodel, 0, numang)  ! non ci sono angoli di legame
      else
          call PassaAngle (kmodel, angle, numang)
      endif
!
      end subroutine GetAngle
!-----------------------------------------------------------------------
      subroutine GetTorsion(kmodel)
      USE ilmviewmod
      USE connect_mod
      USE atom_type_util
      USE General, only: cell
      USE unit_cell
      implicit none
      type(torsion_type), dimension(:), allocatable :: tors
      integer                                       :: ntors
      integer                                       :: kmodel
!
      call legm_to_tors(modelview(kmodel)%leg,modelview(kmodel)%at,set_cell_type(cell),tors)
      ntors = numtorsions(tors)
!
      if (ntors == 0) then
          call PassaTorsion (kmodel, 0, ntors)  ! non ci sono angoli di torsione
      else
          call PassaTorsion (kmodel, tors, ntors)
      endif
!
      end subroutine GetTorsion
!-----------------------------------------------------------------------
      subroutine jav_import_structure(filnam, tipo, kmodel)
      USE ilmviewmod
      USE importexport
      USE exportfiles
      USE errormod
      USE General, only: ifAutomatic,celem,spacegr,cell
      USE Molcom, only: jscreen
      USE sinterface
      implicit none
      character(len=*), intent(in) :: filnam
      type(error_type)             :: err
      integer                      :: kmodel, tipo
      logical                      :: gui,has_symmetry
!
      call save_action(IMPORT,0,kmodel)
      gui = (jscreen > 0 .and. ifAutomatic == 0)
      has_symmetry = is_cell_defined_list()
      call import_structure(filnam(1:len_trim(filnam)),tipo,modelview(kmodel)%at,modelview(kmodel)%leg, &
                            celem,cell,spacegr,gui,has_symmetry,err)
!
      if (.not.err%signal) then
          call save_action(IMPORT,1,kmodel)     ! metti la modifica nello stack
          call jav_redraw(kmodel)   
          call update_model_and_export(modelview(kmodel)%at,modelview(kmodel)%leg,kmodel)
      endif
!
      end subroutine jav_import_structure
!-----------------------------------------------------------------------
      subroutine jav_export_structure(filnam, tipo, kmodel)
      USE ilmviewmod
      USE gen_frm
      USE General, only: cell,spacegr,celem,StructureName,neutro,alambda
      USE unit_cell
      implicit none
      character(len=*), intent(in) :: filnam
      integer                      :: kmodel, tipo
!
      if (code_fragfilen(filnam(1:len_trim(filnam))) == POV_FILE .and. symatoms(kmodel)) then
          call export_structure(filnam(1:len_trim(filnam)),modelviews(kmodel)%at,set_cell_type(cell),   &
                                spacegr,celem,tipo,StructureName,neutro,alambda,modelviews(kmodel)%leg, &
                                iscell=ilmview_info(1,kmodel)==1)
      else
          call export_structure(filnam(1:len_trim(filnam)),modelview(kmodel)%at,set_cell_type(cell),   &
                                spacegr,celem,tipo,StructureName,neutro,alambda,modelview(kmodel)%leg, &
                                iscell=ilmview_info(1,kmodel)==1)
      endif
!
      end subroutine jav_export_structure
!-----------------------------------------------------------------------
      subroutine getSName(nome)
!
      USE General, only:StructureName, fname
      USE variables, only: cryst
      implicit none
      character (len=*) nome
      integer Lung
!
      if (Lung(StructureName) == 0) then
          StructureName = fname(1)(1:index(fname(1),'.bin')-1)
          call cryst(1)%set_name(StructureName)
      endif
      nome = StructureName(1:Lung(StructureName)) // char(0)
!
      end subroutine getSName
!----------------------------------------------------------------------
      subroutine jav_save_info(filnam, kmodel, page, ncol, vet, whichiso)
      USE ilmviewmod
      USE importexport
      USE connect_mod
      implicit none
      integer, intent(in)                        :: page, ncol, whichiso   
      integer, dimension(ncol), intent(in)       :: vet
      character(len=*), intent(in)               :: filnam
      logical                                    :: symactive
      logical                                    :: hbond,shbond
      type(bond_type), dimension(:), allocatable :: legmc
      integer                                    :: kmodel
!
      symactive = symatoms(kmodel)
      if (page == 7) then   ! esporta i contatti
          hbond = is_hydrogen_bonds(kmodel)
          shbond = is_short_contacts(kmodel)
          if (hbond .and. shbond) then
              call copy_bonds(legmc,legmHview) 
              call combine_legm(legmc,legmshort)
              call export_info(filnam,page,vet,modelviews(kmodel)%at,modelviews(kmodel)%leg,legmc)
          else
              if (hbond) then
                  call export_info(filnam,page,vet,modelviews(kmodel)%at,modelviews(kmodel)%leg,legmHview)
              else
                  call export_info(filnam,page,vet,modelviews(kmodel)%at,modelviews(kmodel)%leg,legmshort)
              endif
          endif
      else
          if (symactive) then
              call export_info(filnam,page,vet,modelviews(kmodel)%at,modelview(kmodel)%leg,adp_type=whichiso)
!corr                   infos=modelviews(kmodel)%symm)
          else
              call export_info(filnam,page,vet,modelview(kmodel)%at,modelview(kmodel)%leg,adp_type=whichiso)
          endif
      endif
!
      end subroutine jav_save_info
!----------------------------------------------------------------------
   subroutine jav_get_info_overlay1(distance, n_ass, nfound, n1, n2, distmin, distm, rmsd, rmsd_nonH, errph, nref, kmodel)
   USE ilmviewmod
   USE compare_struct
   implicit none
!
   real, intent(in)           :: distance
   integer, intent(out)       :: n_ass, nfound, n1, n2, nref
   real, intent(out)          :: distmin, distm, errph, rmsd, rmsd_nonH
!corr   type(compare_info_type)    :: info_compare
   integer                    :: kmodel
!
   call jav_compare(distance, kmodel)
   n_ass  = infoc%nass
   nfound = infoc%nfound
   n1     = size(infoc%at1)
   n2     = size(infoc%at2)
   distmin = infoc%distmin
   distm   = infoc%distm
   rmsd    = infoc%rmsd
   rmsd_nonH = infoc%rmsd_nonH
   errph   = infoc%errph
   nref    = infoc%nref 
!
   end subroutine jav_get_info_overlay1

!----------------------------------------------------------------------

   subroutine jav_get_info_overlay2(distance, nv1, nv2, vdist, at1, at2, shift, kmodel)
   USE ilmviewmod
   USE compare_struct
   implicit none
   real, intent(in)              :: distance
   integer, dimension(*)         :: nv1
   integer, dimension(*)         :: nv2
   real, dimension(*)            :: vdist 
   real, dimension(4)            :: shift
   type(atom_type), dimension(*) :: at1
   type(atom_type), dimension(*) :: at2
!corr   type(compare_info_type)       :: info_compare
   integer                       :: i, kmodel
!
!corr   call jav_compare(info_compare,distance, kmodel)
   do i=1, infoc%nass
      nv1(i) = infoc%nv1(i)
      nv2(i) = infoc%nv2(i)
      vdist(i) = infoc%vdist(i)
   end do
   do i=1, size(infoc%at1)
      at1(i) = infoc%at1(i)
   end do
   do i=1, size(infoc%at2)
      at2(i) = infoc%at2(i)
   end do
   do i=1, 4
      shift(i) = infoc%shift(i) 
   end do
!
   end subroutine jav_get_info_overlay2

!-----------------------------------------------------------------------

   subroutine Get_StringSymmF(koper, outStr)
   USE general, only: spacegr
   implicit none
   integer, intent(in)               :: koper
   character (len=*)                 :: outStr
!
   outStr = spacegr%symopstr(koper) // char (0)
!
   end subroutine Get_StringSymmF

!----------------------------------------------------------------------

   subroutine jav_edit_structure(icode,atom,natoms,kmodel)
   USE atom_type_util
   USE ilmviewmod
   USE exportfiles
   USE general, only:spacegr,cell,celem
   USE unit_cell
   implicit none
   integer, intent(in)                            :: icode
   integer, intent(in)                            :: natoms
   type(atom_type), dimension(natoms), intent(inout) :: atom  
   integer                                        :: nat1
   integer                                        :: kat
   integer                                        :: nmod1
   integer, save                                  :: nmod
   integer                                        :: kmodel
!
   select case (icode)
       case (:-1) ! cancella atomo 
         nat1 = numatoms(modelview(kmodel)%at)
         kat = abs(icode)        ! num. dell'atomo da eliminare
         if (kat <= nat1) then   ! l'atomo da eliminare potrebbe non essere stato aggiunto ad atomview
             call jav_delete_object(kmodel,(/kat/),1,(/0/),0)
             nmod = nmod + 1
         endif
         call ClearDistAng()

       case (0)   ! salva parametri all'apertura delle finestra
         call save_action(EDIT_STRUCT,0,kmodel)
         atom = modelview(kmodel)%at(:natoms)  !!! prova per Rosi
!corr         atom%ptab = pxen_from_nz(atom%nz)
         nmod = 0

       case (1,3)   ! OK/Apply
            write(0,*)'AT=',atom(1)%get_nz(),atom(1)%ptab
         call merge_model(modelview(kmodel)%at,atom,spacegr,set_cell_type(cell),celem,nmod1)
         nmod = nmod1 + nmod
         call ClearDistAng()
         call jav_redraw(kmodel)   
         if (icode == 1 .and. nmod > 0) then
             call save_action(EDIT_STRUCT,1,kmodel)  ! metti la modifica nello stack su OK
             call update_model_and_export(modelview(kmodel)%at,modelview(kmodel)%leg,kmodel)
         endif 

       case (2)   ! Cancel (annulla tutte le azioni fatte)
         nmod = 0
         call undor_model(1,uract_temp,kmodel)   ! ripristina il modello iniziale al cancel
         call jav_redraw(kmodel)  

   end select
!
   end subroutine jav_edit_structure

!----------------------------------------------------------------------

   subroutine jav_edit_bond_properties(icode,legm,kmodel,quale)
   USE atom_type_util
   USE ilmviewmod
   USE exportfiles
   USE connect_mod
   USE unit_cell
   USE general, only:cell
   implicit none
   integer, intent(in)                          :: icode
   integer, intent(in)                          :: quale
   type(bond_type), dimension(*), intent(inout) :: legm 
   integer, intent(in)                          :: kmodel
   integer                                      :: nleg
   integer, save                                :: nmod
   real, dimension(3,3)                         :: gmat
!
   select case (icode)
       case (0)   ! save parameters
         call save_action(EDIT_STRUCT,0,kmodel)
         nleg = numbonds(modelview(kmodel)%leg)
         legm(:nleg) = modelview(kmodel)%leg(:)
         nmod = 0

       case (1)
         if (nmod > 0) then
             call save_action(EDIT_STRUCT,1,kmodel)  ! metti la modifica nello stack su OK
             call update_model_and_export(modelview(kmodel)%at,modelview(kmodel)%leg,kmodel)
         endif

       case (3)   ! OK/Apply
         gmat = matrice_metrica(cell)
         call change_bond_distance(modelview(kmodel)%at,modelview(kmodel)%leg,quale,legm(quale)%dist,gmat)
         nleg = numbonds(modelview(kmodel)%leg)
         legm(:nleg)%dist = modelview(kmodel)%leg%dist  ! distance could be changed!
         call ClearDistAng()
         call jav_redraw(kmodel)   
         nmod = nmod + 1

       case (2)   ! Cancel (reset all actions)
         if (nmod > 0) call undor_model(1,uract_temp,kmodel)   ! ripristina il modello iniziale al cancel
         call jav_redraw(kmodel)  
         nmod = 0

   end select
!
   end subroutine jav_edit_bond_properties
!----------------------------------------------------------------------

   subroutine jav_edit_angle_properties(icode,angle,kmodel,quale)
   USE connect_mod
   USE ilmviewmod
   USE general, only:cell
   USE unit_cell
   USE atom_type_util
   USE exportfiles
   USE unit_cell
   implicit none
   integer, intent(in)                               :: icode
   integer, intent(in)                               :: quale
   type(angle_type), dimension(*), intent(inout)     :: angle 
   integer, intent(in)                               :: kmodel
   type(angle_type), dimension(:), allocatable, save :: ang
   type(cell_type)                                   :: cellm
   integer, save                                     :: nmod
   integer                                           :: nang
   type(cell_type)                                   :: cellt
!
   select case (icode)
       case (0)   ! save parameters
         call save_action(EDIT_STRUCT,0,kmodel)
         cellt = set_cell_type(cell)
         call legm_to_ang(modelview(kmodel)%leg,modelview(kmodel)%at,cellt%get_g(),ang)
         nmod = 0

       case (1)   ! OK
         if (nmod > 0) then
             call save_action(EDIT_STRUCT,1,kmodel)  ! metti la modifica nello stack su OK
             call update_model_and_export(modelview(kmodel)%at,modelview(kmodel)%leg,kmodel)
         endif

       case (3) ! Apply
         cellm = set_cell_type(cell)
         nang = numangles(ang)
         call change_bond_angle(quale,ang,modelview(kmodel)%at,modelview(kmodel)%leg,angle(quale)%val,cellm)
         nmod = nmod + 1
         angle(:nang) = ang(:)
         call ClearDistAng()
         call jav_redraw(kmodel)   

       case (2)   ! Cancel (reset all actions)
         if (nmod > 0) call undor_model(1,uract_temp,kmodel)   ! ripristina il modello iniziale al cancel
         call jav_redraw(kmodel)  
         call reallocate_angle(ang,0)
         nmod = 0
   end select
!
   end subroutine jav_edit_angle_properties

!----------------------------------------------------------------------

   subroutine jav_edit_torsion_properties(icode,tors,tval,kmodel)
   USE connect_mod
   USE ilmviewmod
   USE general, only:cell
   USE unit_cell
   USE atom_type_util
   USE exportfiles
   implicit none
   integer, intent(in)               :: icode
   type(torsion_type), intent(inout) :: tors 
   real, intent(in)                  :: tval
   integer, intent(in)               :: kmodel
   type(cell_type)                   :: cellm
   integer, save                     :: nmod
   logical                           :: changed
!
   select case (icode)
       case (0)   ! save parameters
         call save_action(EDIT_STRUCT,0,kmodel)
         nmod = 0

       case (1)   ! OK
         if (nmod > 0) then
             call save_action(EDIT_STRUCT,1,kmodel)  ! metti la modifica nello stack su OK
             call update_model_and_export(modelview(kmodel)%at,modelview(kmodel)%leg,kmodel)
         endif 

       case (3) ! Apply
         cellm = set_cell_type(cell)
         call change_torsion_angle(tors,modelview(kmodel)%at,modelview(kmodel)%leg,tval,set_cell_type(cell),changed)
         if (changed) then
             nmod = nmod + 1
             call ClearDistAng()
             call jav_redraw(kmodel)   
         endif

       case (2)   ! Cancel (reset all actions)
         if (nmod > 0) call undor_model(1,uract_temp,kmodel)   ! ripristina il modello iniziale al cancel
         call jav_redraw(kmodel)  
         nmod = 0
   end select
!
   end subroutine jav_edit_torsion_properties

!----------------------------------------------------------------------

      subroutine jav_normalize_structure(icode,distH,kmodel)
      USE hydrogens
      USE ilmviewmod
      USE exportfiles
      USE unit_cell
      USE General, only: cell
      implicit none
      integer, intent(in)                        :: icode
      real, dimension(NHDISTANCES),intent(inout) :: distH  ! distance da usare per la normalizzazione
      real, dimension(NHDISTANCES), save         :: distsav
      logical, save                              :: modif
      integer                                    :: ndd
      integer                                    :: ier
      integer                                    :: kmodel
!
!!      kmodel = active_model
      select case (icode)
         case (0)    ! Save all'apertura e passa distanze
            modif = .false.
            distsav(:) = distHn(:)  
            distH(:) = distHn(:)  
            call save_action(NORMALISEH,0,kmodel)

         case (1,2)  ! OK/Apply
            ndd = NHDISTANCES - 2
!
!           Applica la modifica se non e' stata gia' applicata (modif e' false) oppure
!           qualche distanza e' cambiata
            if (.not.modif .or. any(abs(distHn(:ndd) - distH(:ndd)) > 0.001)) then
                distHn(:ndd) = distH(:ndd)  
                call normalise_hydrogens(modelview(kmodel)%at,modelview(kmodel)%leg,set_cell_type(cell),ier)
                if (ier == 0) then
                    call jav_redraw(kmodel)  
                    call update_model_and_export(modelview(kmodel)%at,modelview(kmodel)%leg,kmodel)
                    modif = .true.
                endif
            endif
            if (icode == 1 .and. modif) call save_action(NORMALISEH,1,kmodel)  ! metti la modifica nello stack su OK
            
         case (3)    ! Cancel
            distHn = distsav(:)
            if (modif) then
                call undor_hydrogens(1,uract_temp,kmodel)   ! ripristina il modello iniziale al cancel
                call jav_redraw(kmodel)  
                call update_model_and_export(modelview(kmodel)%at,modelview(kmodel)%leg,kmodel)
            endif

      end select
!
      end subroutine jav_normalize_structure
!----------------------------------------------------------------------
      subroutine ges_add_fragments_win(code, kmodel, kgeom, kspec1, kspec2, kpos, dist, kadd)
      USE importexport
      USE ilmviewmod
      USE exportfiles
      USE elements
      USE General, only: celem,cell,spacegr,neutro
      USE unit_cell
      USE gen_frm
      USE variables, only: cryst
      implicit none
      integer, intent(in)           :: code
      integer, optional             :: kspec1, kspec2
      integer, intent(in), optional :: kgeom, kpos, kadd
      real, intent(in), optional    :: dist
      integer, save                 :: nmod
      integer                       :: kmodel
      type(cell_type)               :: cellt
      integer :: nk
!
      select case (code)
          case (0)      ! Salva il modello    
            nmod = 0
            call save_action(IMPORT,0,kmodel)
!
!           seleziona le specie che compongono il poliedro
            nk = numelem(celem)
            if (nk <= 1) then
                kspec1 = nk
                kspec2 = nk
            else
                kspec2 = is_element(celem,8) 
                if (kspec2 > 0) then ! force l'O ai vertici se esiste
                    kspec1 = nk
                else
                    kspec1 = nk
                    kspec2 = nk-1
                endif
            endif
            !write(0,*)'spec=',celem(ks1)%lab,celem(ks2)%lab

          case (1)      ! 'Quit'
            if (nmod > 0) then
                call save_action(IMPORT,1,kmodel)     ! metti la modifica nello stack sul 'Quit'
                call update_model_and_export(modelview(kmodel)%at,modelview(kmodel)%leg,kmodel)
            endif

          case (2)      ! Ripristina il modello sul 'Cancel'
            nmod = 0
            call undor_model(1,uract_temp,kmodel)   ! ripristina il modello iniziale al cancel
            call jav_redraw(kmodel) 
 
          case (3)
            nmod = nmod + 1
            cellt = set_cell_type(cell)
            call make_fragment(modelview(kmodel)%at,modelview(kmodel)%leg,cellt,spacegr,celem,   &
                               kgeom,kpos,kspec1,kspec2,kadd>0,dist,neutro)
            call cryst(1)%set_scattering(celem)
            call jav_redraw(kmodel) 
!
            if (kgeom == ISOLATED) then   ! atomi isolati?
!
!               undo/redo va gestito subito nel caso di atomi isolati perche' la finestra si chiude subito
                call save_action(IMPORT,1,kmodel) 
                call update_model_and_export(modelview(kmodel)%at,modelview(kmodel)%leg,kmodel)
            endif

      end select
!
      end subroutine ges_add_fragments_win
!-----------------------------------------------------------------------
      subroutine jav_polyhedra(kmodel)
      USE ilmviewmod
      USE polyhedra
      implicit none
      integer                                   :: kmodel
!
      call jav_redraw(kmodel)  
!
      end subroutine jav_polyhedra
!-----------------------------------------------------------------------
      integer function Presentnzo()
!corr      USE Molcom, only: RS
      USE javinterface
!
!corr      if(ALLOCATED(RS)) then
         Presentnzo = PRESENTNZ
!corr      else
!corr         Presentnzo = -1
!corr      endif
!
      end function Presentnzo
!-----------------------------------------------------------------------
      subroutine Getnzo(irs, inum)
!corr      USE Molcom, only: RS
!
      implicit none
      integer :: irs(*), inum
!corr      integer :: i
!
      irs(:inum) = 0
!corr      do i=1,inum
!corr         if (RS(i) > 0.0) then
!corr             irs(i) = 0              ! atomo visibile
!corr         elseif (RS(i) > -10.0) then
!corr             irs(i) = 1              ! atomo hidden
!corr         else
!corr             irs(i) = 2              ! atomo deleted
!corr         endif
!corr      enddo
!
      end subroutine Getnzo
!-----------------------------------------------------------------------
      integer function jav_grow_fragment(kmodel)
      USE ilmviewmod
      integer                       :: kmodel
!
      call jav_redraw(kmodel)  
      jav_grow_fragment = 1
!
      end function jav_grow_fragment
!-----------------------------------------------------------------------
      integer function jav_hydrogen_bonds(kmodel)
      USE ilmviewmod
      USE contacts
      integer                       :: kmodel
!
      hbtopt%hangat = 1
!
      if (.not.is_hydrogen_bonds(kmodel)) then    ! if action is disabling h-bonds
          expandc = expand_option_type(0,0,0)     ! per sicurezza esegui un reset di expandc perche' 
      endif                                       !  expandc%katom potrebbe essere un hang atom
!
      call jav_redraw(kmodel,kaction=DISPLAY_CONTACTS)
      jav_hydrogen_bonds = 1
!
      end function jav_hydrogen_bonds
!-----------------------------------------------------------------------
      integer function jav_short_contacts(kmodel)
      USE ilmviewmod
      integer                       :: kmodel
!
      call jav_redraw(kmodel,kaction=DISPLAY_CONTACTS)
      jav_short_contacts = 1
      end function jav_short_contacts
!-----------------------------------------------------------------------
      subroutine GetHBOptions(d, a, opt)
      USE contacts
      type(contact_type), dimension(NDONORS)    :: d
      type(contact_type), dimension(NACCEPTORS) :: a
      type(contact_option_type)                 :: opt
!
      d(1:NDONORS) = donor(1:NDONORS)
      a(1:NACCEPTORS) = acceptor(1:NACCEPTORS)
      opt = hbtopt
!
      end subroutine GetHBOptions
!-----------------------------------------------------------------------
      subroutine PutHBOptions(kmodel, d, a, opt)
      USE contacts
      USE ilmviewmod
      type(contact_type), dimension(NDONORS)    :: d
      type(contact_type), dimension(NACCEPTORS) :: a
      type(contact_option_type)                 :: opt
      integer                                   :: i
      logical                                   :: cambiato, hbond, redraw
      integer                                   :: kmodel
!
      cambiato = .FALSE.
      do i=1, NDONORS
         if(donor(i)%active /= d(i)%active) then
            cambiato = .TRUE.
            donor(i)%active = d(i)%active
         endif
      end do
      do i=1, NACCEPTORS
         if(acceptor(i)%active /= a(i)%active) then
            cambiato = .TRUE.
            acceptor(i)%active = a(i)%active
         endif
      end do
      if(hbtopt%hpresent /= opt%hpresent) then
         hbtopt%hpresent = opt%hpresent
         cambiato = .TRUE.
      endif
      if(hbtopt%inter /= opt%inter) then
         hbtopt%inter = opt%inter
         cambiato = .TRUE.
      endif
      if(hbtopt%intra /= opt%intra) then
         hbtopt%intra = opt%intra
         cambiato = .TRUE.
      endif
      if (hbtopt%intra /= 0) then
          if(hbtopt%nminbond /= opt%nminbond) then
             hbtopt%nminbond = opt%nminbond
             cambiato = .TRUE.
          endif
      endif
      if(hbtopt%tolmin /= opt%tolmin) then
         hbtopt%tolmin = opt%tolmin
         cambiato = .TRUE.
      endif
      if(hbtopt%tolmax /= opt%tolmax) then
         hbtopt%tolmax = opt%tolmax
         cambiato = .TRUE.
      endif
      hbond = is_hydrogen_bonds(kmodel)
      redraw = hbond .and. cambiato
      if(redraw) then
         call jav_redraw(kmodel)
      endif
!
      end subroutine PutHBOptions
!-----------------------------------------------------------------------
      subroutine GetStructureInfo(info, icalc, kmodel)
      USE importexport
      USE ilmviewmod
      USE progtype, only: structure_info_type
      implicit none
      type(structure_info_type), intent(out) :: info
      integer, intent(in)                    :: icalc
      integer                                :: kmodel!!mr = 1
!
      info = structure_info(modelview(kmodel)%at,icalc)
!
      end subroutine GetStructureInfo
!-----------------------------------------------------------------------
      subroutine jav_delete_model(Which)
      USE ilmviewmod
      USE model_util
!
      implicit none
      integer, intent(in) :: Which
!
      call model_delete(modelview,Which)
      num_models = num_models - 1
!
      end subroutine jav_delete_model
!-----------------------------------------------------------------------
      subroutine overlay_struct(newname)
      USE ilmviewmod
      USE errormod
      character(len=*), intent(in) :: newname ! nuovo nome + char(0)
      type(error_type)             :: err
!
      call jav_overlay(trim(newname(1:len_trim(newname))),err)
!
      end subroutine overlay_struct
!-----------------------------------------------------------------------
      subroutine jav_edit_bonds(icode,tableC,bsetting,numt,kmodel)
      USE bondtmod
      USE progtype
      USE General, only:cell,spacegr,celem
      USE ilmviewmod 
      USE connect_mod
      USE elements
      implicit none
      integer                                          :: icode
      type(bond_table_type), dimension(:), allocatable :: table
      type(bond_table_type), dimension(*)              :: tableC
      type(bond_setting_type)                          :: bsetting
      integer                                          :: numt, i
      integer                                          :: kmodel
      integer, save                                    :: kmod
      integer                                          :: diff
      
      select case (icode)
         case (0)  ! apertura della finestra, passa parametri al C e salva
           kmod = 0
           if (numelem(celem) > 0) then
               call get_bond_table(celem%z,table,numt)
               do i=1, numt
                  tableC(i) = table(i)
               enddo
           endif
           bsetting = bondset
           call save_action(BOND_SETTINGS,0,kmodel)

         case (1,3)  ! OK, Apply
           if (numelem(celem) > 0) call update_bond_table(tableC(:numt))
           call connect_and_draw()
           kmod = kmod + 1
           if (icode == 1) then   ! ho premuto OK
               bondset = bsetting
!
!              salva la modifiche nello stack
               diff =  compare_legm(uract_temp%legm,modelview(kmodel)%leg)
               if (diff > 0) call save_action(BOND_SETTINGS,1,kmodel)
           endif

         case (2)  ! Cancel
           if (kmod > 0) then    ! se ho fatto delle modifiche ripristino tutto
               bsetting = bondset
               call connect_and_draw()
           endif

         case (4)  ! Ripristina i valori di default
           call unload_bond_table()
           call load_bond_table()
           if (numelem(celem) > 0) then
               call get_bond_table(celem%z,table,numt)
               do i=1, numt
                  tableC(i) = table(i)
               enddo
           endif
           bsetting = DEF_BONDSET
           call connect_and_draw()

      end select

      CONTAINS

        subroutine connect_and_draw()
        USE molpnew
        USE atom_type_util
        USE unit_cell
        if (numatoms(modelview(kmodel)%at) > 1) then
            call create_connectivity(modelview(kmodel)%at,modelview(kmodel)%leg,set_cell_type(cell), &
                 spacegr,check=bsetting%angle > 0, tolmin=bsetting%tolmin,tolmax=bsetting%tolmax,   &
                 angl=bsetting%angle,move=.false.)
            call jav_redraw(kmodel)
        endif
        end subroutine connect_and_draw

      end subroutine jav_edit_bonds

!-----------------------------------------------------------------------

      subroutine jav_expand_selection(kptra,nptra,kmodel)
      USE ilmviewmod
      USE fragmentmod
      USE General, only: cell
      USE unit_cell
      USE atom_basic
      implicit none
      integer, dimension(*), intent(inout)   :: kptra   ! atomi selezionati
      integer, intent(inout)                 :: nptra   ! num. di atomi
      integer                                        :: nfrag
      type(fragment_type), dimension(:), allocatable :: frag
      integer :: kmodel
      integer, dimension(:), allocatable :: vfrag
      integer :: i,j
      integer :: kpos
      integer, dimension(nptra) :: kptraa
      logical                                                          :: symactive
      integer, dimension(nptra) :: kptrasav
      integer, dimension(1000) :: getsym
      integer :: nget
      integer :: nptrasav
      integer :: kat
      type(cell_type) :: cellt
!
      symactive = symatoms(kmodel)
      cellt = set_cell_type(cell)
!
      if (symactive) then   ! save if symmmetry is active
          kptrasav(:) = kptra(:nptra)
          nptrasav = nptra
      endif
!
!     Se e' attiva la simmetria conviene lavorare comunque sull'u.a.
      kptraa(:) = selected_atoms(kptra,nptra,kmodel)
      call get_fragments(modelview(kmodel)%at,cellt,modelview(kmodel)%leg,nfrag,frag)
!
!     segnala in vfrag i frammenti coinvolti
      allocate(vfrag(nfrag))
      vfrag(:) = 0
      do i=1,nptra
         kpos = fragment_pos(frag,kptraa(i))
         vfrag(kpos) = 1
      enddo
!
!     Aggiorna nptra e kptra
      nptra = 0
      do i=1,nfrag
         if (vfrag(i) == 1) then
             do j=1,frag(i)%nat
                nptra = nptra + 1
                kptra(nptra) = frag(i)%pos(j)
             enddo
         endif
      enddo
!
      if (symactive) then
!
!         se è attiva la simmetria seleziona gli atomi corrispondenti
!         isola in getsym tutti gli atomi con lo stesso opcode di kptrasav
          nget = 0
          do j=1,nptrasav
             do i=1,size(modelviews(kmodel)%at)
!corr                if (modelviews(kmodel)%symm(i)%opcode == modelviews(kmodel)%symm(kptrasav(j))%opcode) then
                if (modelviews(kmodel)%at(i)%op == modelviews(kmodel)%at(kptrasav(j))%op) then
                    nget = nget + 1
                    getsym(nget) = i
                endif
             enddo
          enddo
!
!         Prendi di getsym solo quelli corrispondenti a kptra
          do i=1,nptra
             do j=1,nget
                kat = getsym(j)
!corr                if (modelviews(kmodel)%symm(kat)%asym == kptra(i)) then
                if (modelviews(kmodel)%at(kat)%asym == kptra(i)) then
                    kptra(i) = kat
                    exit
                endif
             enddo
          enddo
      endif
!
      end subroutine jav_expand_selection

! -----------------------------------------------------------------------

      subroutine jav_expand_contacts(katom, kmodel, icode, ncontact)
      USE contacts
      USE ilmviewmod
      USE connect_mod
      implicit none
      integer, intent(in)  :: katom,kmodel
      integer, intent(in)  :: icode
      integer, intent(out) :: ncontact
!
      expandc(kmodel)%code = icode
      if (expandc(kmodel)%code == 1) then
          expandc(kmodel)%atom = katom
      endif
      expandc(kmodel)%level = expandc(kmodel)%level + 1
      call jav_redraw(kmodel, 0)
      if (size(modelviews(kmodel)%at) == size(modelview(kmodel)%at)) then
          expandc(kmodel)%code = 0
          expandc(kmodel)%atom = 0
      endif
      ncontact = 0  ! variabile non gestita al momento
      end subroutine jav_expand_contacts
!-----------------------------------------------------------------------

      subroutine jav_show_multbonds(kmodel)
      USE ilmviewmod
      USE atom_type_util
      USE unit_cell
      USE General, only: cell
      implicit none
      integer, intent(in)                    :: kmodel
!
      call bond_type_perception(modelview(kmodel)%at,modelview(kmodel)%leg,set_cell_type(cell))
      call jav_redraw(kmodel)
!
      end subroutine jav_show_multbonds

!-----------------------------------------------------------------------
      subroutine jav_get_au(atom,natoms,kmodel)
      USE atom_type_util
      USE ilmviewmod
      implicit none
      integer, intent(in)                            :: natoms, kmodel
      type(atom_type), dimension(natoms), intent(inout) :: atom
!
      atom = modelview(kmodel)%at(:natoms)
!corr      atom%ptab = pxen_from_nz(atom%nz)
!
      end subroutine jav_get_au

!-----------------------------------------------------------------------
      subroutine jav_find_centroids(kmodel) 
      USE atom_type_util
      USE ilmviewmod
      USE centroids
      implicit none
      integer, intent(in)                            :: kmodel
      type(centroid_type), dimension(:), allocatable :: centr
      integer                                        :: ncentr, i

!
      call find_centroids(modelview(kmodel)%at,modelview(kmodel)%leg, centr,ncentr)
      call start_centroids()
      do i=1, ncentr
         call passa_centroid(i, centr(i)%x, centr(i)%nat, centr(i)%at)
      enddo
      call end_centroids()
!
      end subroutine jav_find_centroids

!-----------------------------------------------------------------------

   subroutine jav_transform_model(iaction, koper, T, kmodel, nptra, kptra)
   USE ilmviewmod
   USE atom_type_util
   USE general, only: spacegr,cell
   USE connect_mod
   USE molpnew
   USE spginfom
   USE math_util
   USE unit_cell
   implicit none
   integer, intent(in)                        :: kmodel
   integer, intent(in)                        :: koper    ! operatore scelto
   integer, intent(in)                        :: iAction  ! 1=Transform, 2=Copy, 3=Undo, 4=Close
   integer, dimension(*), intent(in)          :: kptra    ! atomi selezionati
   integer, intent(in)                        :: nptra    ! num. di atomi
   real, dimension(3), intent(inout)          :: T        ! traslazione
   type(symop_type)                           :: symop
   real, dimension(3)                         :: trasl
   integer, dimension(3)                      :: tcell
   logical, save                              :: modif_applied
   type(cell_type)                            :: cellt
!
!  Get cell translation, approximate to the closest integer
   trasl(:) = T - spacegr%symop(koper)%trn
   tcell(:) = nint(trasl)
!
!  Set symop and add translation only along polar axis
   symop = spacegr%symop(koper)
   if (spacegr%polarx) symop%trn(1) = symop%trn(1) + (trasl(1) - tcell(1))
   if (spacegr%polary) symop%trn(2) = symop%trn(2) + (trasl(2) - tcell(2))
   if (spacegr%polarz) symop%trn(3) = symop%trn(3) + (trasl(3) - tcell(3))
   T = spacegr%symop(koper)%trn(:) + tcell(:)   ! update translation
!corr   write(0,*)'TCELL=',tcell
!corr   write(0,*)'TPOLAR=',trasl - tcell
!corr   write(0,*)'T=',spacegr%symop(koper)%trn(:) + tcell(:)
!
   cellt = set_cell_type(cell)
   select case(iaction)
      case (1,2)    ! Transform - Copy
         if (koper == 1 .and. equal_vector(symop%trn+tcell,[0.,0.,0.])) return ! identity mat
         if (iaction == 1) then  ! Transform
             if (symatoms(kmodel)) then
                 if (nptra > 0) then
                     call transform_atoms(modelviews(kmodel)%at,cellt,spacegr,symop,tcell,    &
                                          nptra,kptra(1:nptra),modelviews(kmodel)%leg)
                 else
                     call transform_atoms(modelviews(kmodel)%at,cellt,spacegr,symop,tcell)
                 endif
                 call jav_redraw(kmodel,symm_enabledd=.false.)
                 modif_applied = .true.
             else
                 if (nptra > 0) then
                     call transform_atoms(modelview(kmodel)%at,cellt,spacegr,symop,tcell,nptra,kptra(1:nptra),modelview(kmodel)%leg)
                 else
                     call transform_atoms(modelview(kmodel)%at,cellt,spacegr,symop,tcell)
                 endif
                 call jav_redraw(kmodel)
                 modif_applied = .true.
             endif
         else      ! Copy
             if (symatoms(kmodel)) then
                 call copy_atoms_sym(modelviews(kmodel)%at,modelviews(kmodel)%leg,koper,symop,tcell)
!test_symm                 call make_infos(modelviews(kmodel)%at,modelviews(kmodel)%symm)
                 call jav_redraw(kmodel,symm_enabledd=.false.)
             else
                 sym_atoms(kmodel) = .true.
                 call copy_atoms(modelviews(kmodel)%at, modelview(kmodel)%at)
                 call copy_bonds(modelviews(kmodel)%leg, modelview(kmodel)%leg)
                 call init_infos(modelviews(kmodel)%at)
                 if (nptra > 0) then
                     call copy_atoms_sym(modelviews(kmodel)%at,modelviews(kmodel)%leg,    &
                                         koper,symop,tcell,nptra,kptra(1:nptra))
                 else
                     call copy_atoms_sym(modelviews(kmodel)%at,modelviews(kmodel)%leg,    &
                                         koper,symop,tcell)
                 endif
!test_symm                 call make_infos(modelviews(kmodel)%at,modelviews(kmodel)%symm)
                 call jav_redraw(kmodel)
             endif
         endif

      case (3)    ! Open
         modif_applied = .false.
         call save_action(TRANSFORM_AT,0,kmodel)

      case (4)    ! Close
         !sym_atoms(kmodel) = .false.
         if (modif_applied) call save_action(TRANSFORM_AT,1,kmodel)
   end select
!
   end subroutine jav_transform_model

!-----------------------------------------------------------------------

   subroutine jav_optimize_geometry(kmodel,kptra,nptra)
   USE atom_type_util
   USE ilmviewmod
   USE OBmodule
   USE General, only: cell
   USE unit_cell
   USE errormod
   USE exportfiles
   implicit none
   integer, intent(in)                :: kmodel
   integer, dimension(*), intent(in)  :: kptra
   integer, intent(in)                :: nptra
   integer, dimension(:), allocatable :: vetfix
   integer                            :: nat,iat,i
   type(error_type)                   :: err

!
   nat = numatoms(modelview(kmodel)%at)
   call save_action(OPTIMIZE_GEOMETRY,0,kmodel)
      !   write(0,*)'NPTRA=',nptra,kptra(:nptra)
   if((nptra == 0) .or. (nptra == nat)) then
!
!     Optimize all atoms
      call WaitCursorOn()
      call OBOptimizeGeometry(modelview(kmodel)%at, modelview(kmodel)%leg, set_cell_type(cell), err)
      call WaitCursorOff()
     
   else
!
!     Optimize only selected atoms
!     Put unselected atoms in vetfix
      allocate (vetfix(nat),source=0)
      iat = 0
      do i=1,nat
         if (any(kptra(:nptra) == i)) cycle
         iat = iat + 1
         vetfix(iat) = i
      enddo
      !write(0,*)'vetfix=',vetfix(:iat)
      call WaitCursorOn()
      call OBOptimizeGeometry(modelview(kmodel)%at, modelview(kmodel)%leg, set_cell_type(cell), err, vetfix(:iat))
      call WaitCursorOff()
   endif
   if (err%signal) then
       call err%print()
   else
       call jav_redraw(kmodel)
       call save_action(OPTIMIZE_GEOMETRY,1,kmodel) 
       call update_model_and_export(modelview(kmodel)%at,modelview(kmodel)%leg,kmodel)
   endif
!
   end subroutine jav_optimize_geometry

!----------------------------------------------------------------------

   subroutine jav_optimize_geometry_mopac(icode,kmodel,atom,natoms)
   USE atom_type_util
   USE ilmviewmod
   USE exportfiles
   USE connect_mod
   USE General, only: cell
   USE unit_cell
   implicit none
   integer, intent(in)                               :: icode
   integer, intent(in)                               :: natoms
   type(atom_type), dimension(natoms), intent(inout) :: atom  
   integer                                           :: kmodel
   integer                                           :: i
   type(cell_type)                                   :: cellt
   type(atom_type), dimension(:), allocatable :: atomc
!
   select case (icode)
       case (0)     ! save parameters when open mopac window
         call save_action(OPTIMIZE_GEOMETRY,0,kmodel)
!check coordinates              
            call frac_to_cart_copy(modelview(kmodel)%at,atomc,orthomatrix_std(cell))
            call print_atoms(atomc,kpr=0)
!check coordinates              

       case (1)   ! Run Mopac
         if ((natoms /= numatoms(modelview(kmodel)%at)) .or. (natoms == 0)) return ! generally not possible
!xRosi         call cart_to_frac(atom,orthomatrixi_std(cell))
         do i=1,natoms
            modelview(kmodel)%at(i)%xc(1:3) = atom(i)%xc(1:3)
         enddo
         cellt = set_cell_type(cell)
         call bond_distance_update(modelview(kmodel)%at,modelview(kmodel)%leg,cellt%get_g())
         call jav_redraw(kmodel)   
         call save_action(OPTIMIZE_GEOMETRY,1,kmodel)  ! save action
         call update_model_and_export(modelview(kmodel)%at,modelview(kmodel)%leg,kmodel)

   end select
!
   end subroutine jav_optimize_geometry_mopac

!----------------------------------------------------------------------

   subroutine cartesianToZmatrix_forC(atom1, numat1, I_coor, conn, kmodel)
   USE ilmviewmod
   USE atom_type_util
!
   implicit none
   type(atom_type), dimension (numat1) :: atom1
   real, dimension(3,*)                :: I_coor
   integer, dimension(3,*)             :: conn
   real, dimension(3,numat1)           :: I_coor1
   integer, dimension(3,numat1)        :: conn1
   integer                             :: kmodel, numat1
!
   call cartesian_to_zmatrix(atom1,modelview(kmodel)%leg,I_coor1,conn1)
   I_coor(1:3, :numat1) = I_coor1
   conn (1:3, :numat1)= conn1
!
   end subroutine cartesianToZmatrix_forC

!----------------------------------------------------------------------

   subroutine jav_powder_pattern(kmodel,xpd)
   USE ilmviewmod
   USE unit_cell
   USE general, only: cell, spacegr, celem
   USE gen_frm
   implicit none
   integer, intent(in)               :: kmodel
   type(xpd_option_type), intent(in) :: xpd
!
   call make_powder_pattern(xjav_pd,yjav_pd,refjav,modelview(kmodel)%at,spacegr,   &
                            set_cell_type(cell),celem,xpd)
       !call jav_pdf(kmodel,pdf_option_type())
       !call do_powder_pattern(xjav_pd, yjav_pd, size(xjav_pd), 0, 0)
       !     return
   if (allocated(refjav)) then
       call do_powder_pattern(xjav_pd, yjav_pd, size(xjav_pd), refjav, size(refjav))
   else
       call do_powder_pattern(xjav_pd, yjav_pd, size(xjav_pd), 0, 0)
   endif
!
   end subroutine jav_powder_pattern

!----------------------------------------------------------------------

   subroutine jav_export_powder_pattern(filename,code)
   USE gen_frm
   USE ilmviewmod
   USE reflection_type_util
   implicit none
   character(len=*), intent(in) :: filename
   integer, intent(in)          :: code
!
   select case(code)
     case(1)   ! file .xy
       call export_profile(filename,xjav_pd,yjav_pd,string='#      2theta    yoss')
     case(2)   ! file .hkl
       call write_reflections(fname=filename,refl=refjav,code=3)
   end select
!
   end subroutine jav_export_powder_pattern

!----------------------------------------------------------------------

   subroutine jav_pdf(kmodel,pdfopt)
   USE general, only: cell, celem, spacegr
   USE ilmviewmod
   USE pdfcalc
   USE elements
   USE unit_cell
   USE atom_type_util
   implicit none
   integer, intent(in)                           :: kmodel
   type(pdf_option_type), intent(in)             :: pdfopt
   type(pdf_type)                                :: pdf
   type(element_type), dimension(:), allocatable :: elpdf
   type(atom_type), dimension(:), allocatable    :: atomcr
   type(cell_type)                               :: cellt
!
   pdf%qmax = pdfopt%qmax
   pdf%rmax = pdfopt%rmax
!
   pdf%rfmax = pdf%rmax
   pdf%rfmin = pdf%rmin
!
!  For pdf radiation
   call copy_elem(elpdf,celem)
   call elem_set_radiation(elpdf,pdfopt%rad)
!
   pdf%nscat = numatoms(modelview(kmodel)%at)
   cellt = set_cell_type(cell)
   pdf%cr_icc = [3,3,3]
   call make_crystal(modelview(kmodel)%at,cellt,spacegr,pdf%cr_icc,atomcr)
   !call print_atoms(atomcr,kpr=73)
!TOFIX   call thermal_displ(atomcr,cellt)
   !call print_atoms(atomcr,kpr=73)
   !call frac_to_cart(atomcr,cellt)
! 
   call pdf%setup(cellt,elpdf,atomcr,pdfopt%rad)
   call pdf%determine(atomcr,cellt)
   call pdf%save_file(xjav_pd,yjav_pd)
!
!xRosi   call do_powder_pattern(xjav_pd, yjav_pd, size(xjav_pd), 0, 0)
!
   call do_pdf(xjav_pd, yjav_pd, size(xjav_pd))
   end subroutine jav_pdf

!----------------------------------------------------------------------

   subroutine jav_bond_valence_sum(kmodel,nbvs,bvs, cn, gii)
   USE variables, only: cryst
   USE crystal_phase
   USE symmgrid
                use bond_valence
   implicit none
   integer, intent(in)                          :: kmodel
   integer                                      :: i
   integer, intent(in)                          :: nbvs
   real, dimension(nbvs)                        :: bvs
   integer, dimension(nbvs)                     :: cn
   type(bond_info_t), dimension(:), allocatable :: dinfo
   integer                                      :: ndinfo,jav_bvs_ndinfo
   real                                         :: gii
!corr     type(bond_valence_t), dimension(:), allocatable :: bvec
!
   if (cryst(1)%natoms() == 0 .or. cryst(1)%numelem() == 0) return
   call cryst(1)%load_bvparam() 
!corr   call cryst(1)%print_bvparam(0)
   call cryst(1)%make_symmetry_cell()
   call cryst(1)%allocate_grid(cutoff=maxval(cryst(1)%bvpar(:,:)%rmax))
   call cryst(1)%fill_grid()
   do i=1,cryst(1)%natoms()
      !call cryst(1)%compute_bvs(i,bvs(i))
      !write(0,*)'VAL=',modelview(kmodel)%at(i)%glab(),bvs(i)
      ndinfo = jav_bvs_ndinfo(kmodel,i)
      if (ndinfo > 0) then
          !if (allocated(dinfo)) deallocate(dinfo)
          !allocate(dinfo(ndinfo))
          call new_bond_info(dinfo,ndinfo)
          call cryst(1)%valence_atom_info(i,dinfo,bvs(i),cn(i))
          !call jav_bvs_dinfo(kmodel,i,ndinfo,dinfo)
      else
          bvs(i) = 0
          cn(i) = 0
      endif
          !write(0,*)'NC_'//cryst(1)%at(i)%glab()//'=',cn(i),ndinfo,bvs(i)
   enddo
   gii = cryst(1)%gii_index(bvs)
   !write(0,*)'GII=',gii
!!!!!test
!corr   call bvtable_tovec(cryst(1)%bvpar,bvec)
!corr     do i=1,size(bvec)
!corr        call bvec(i)%prn(0)
!corr     enddo
!corr   !call bvtable_tovec(cryst(1)%bvpar,bvec)
!!!!!test
!
   end subroutine jav_bond_valence_sum

!----------------------------------------------------------------------

   integer function jav_bvs_ndinfo(kmodel,kat) result(ndinfo)
   USE variables, only: cryst
   USE crystal_phase
   implicit none
   integer, intent(in) :: kmodel,kat
!
   ndinfo = cryst(1)%coord_number(kat)
   !write(0,*)'NB_'//cryst(1)%at(kat)%glab()//'=',ndinfo
!
   end function jav_bvs_ndinfo

!----------------------------------------------------------------------

   subroutine jav_bvs_dinfo(kmodel,kat,ndinfo,dinfo)
   USE variables, only: cryst
   USE crystal_phase
   USE symmgrid
   implicit none
   integer, intent(in)                               :: kmodel,kat,ndinfo
   type(bond_info_t), dimension(ndinfo), intent(out) :: dinfo
   real                                              :: bvs
   integer                                           :: cn
!
   if (ndinfo > 0) then
       call cryst(1)%valence_atom_info(kat,dinfo,bvs,cn)
       !call print_bond_info(cryst(1)%at,cryst(1)%spg,dinfo,0)
   endif
!
   end subroutine jav_bvs_dinfo
