#include <list>
#define NKNOTS 9   //17

#define OTHERS     NUM_COLORS - 2
#define MONOCOL    NUM_COLORS - 1
enum structures {COIL, HELIX, STRAND, TURN, OTHER};
enum labeltypes {LABEL_ATOM, LABEL_SPECIES, LABEL_NUMBER, N_LABELTYPES};
enum hdistances {CH3, CH2, CH, C_ALCHINO, C_Ar, X_CH2, O_H, N_HP, N_HS,
                 S_H, B_H, C_DEF, N_DEF,  NHDistances};

#define LEN_SPEC 8
typedef char len_spec[LEN_SPEC];
#if 0

typedef struct {
                  len_spec Species[MAX_SPECIES];
                  int nk;
               } _SpeciesInfo;

typedef struct {
                 int asym;
                 int oper;
                 int opcode;
               } symm_info_type;
#endif

typedef struct {
                int color;          // numero seq. di specie dell'atomo
                int Z;              // numero atomico
                int ptab;           // numero d'ordine nel file xen
                op_type  op;        // symmetry operator
                //int oper;           // operatore di simmetria
                //int opcode;         // codice operatore
                int seq ;           // numero (sequenziale) del residuo
                int numResiduo;     // numero di residuo (da 0 a 19)
                int ResNum;         // numero (dal file) del residuo
                int catena;         // numero di catena
                int catseq;         // numero sequenziale di catena
                int resseq;         // numero seq. di residuo (nella catena)
                float xyzc[3];      // coordinate frazionarie
                float xyz[3];       // coordinate ortonormalizzate
                float xyzN[3];      // coordinate normalizzate per la grafica
                float vn[3];        // 
                char  lab[MAX_LABLEN]; // label
                int   disegnato;    // flag per grafica
                float radN[4];      // raggio atomico per grafica
                int drawLabel;      // disegna label si'/no
                float ocry;         // occupanza
                float och;          // occupanza
                float B;            // fattore termico
                float Bij[6];       // fattori termici anisotropi
                float mat[4][4];    // 
                float scala[4];     // 
                float semin;        // 
                float seminN;       // 
                float H;            // altezza del picco
                bool  deleted;      // cancellato si/no
                bool  isW;          // acqua si/no
                bool  selected;     // selezionato si/no
                bool  hidden;       // nascosto si/no
                int   heta;         // heteroatom? 1:0
                int   numleg;       // numero di atomi legati
                int   leg[100];     // atomi legati
                float dist[100];    // distanze atomi legati
               } jav_atom_type;

typedef struct {
               int   n1, n2;   // atomi legati
               float dist;     // distanza corrente
               float sigma;    // sigma
               bool  selected; // selezionato si/no
               int   ordleg;   // ordine del legame
               } jav_bond_type;

typedef struct {
               int   n1, n2, n3;  // atomi legati
               float fv;          // angolo tra i legami
               float sigma;       // sigma
               } jav_anglebond_type;

typedef struct {
               int   n1, n2, n3, n4; // atomi legati
               float fv;             // angolo tra i legami
               } jav_torsion_type;

typedef struct {
               int atom;
               int o;
               int o2;
               int c;
               int n;
               int h;
               float Hxyz[3];
               int chain;
               int residue;
               std::string ResLab; 
               bool breakF;
               bool selected;
               bool isAmino;
               float xyz[3];
               } C_Alpha;

typedef struct {
               int  num;
               char *PDBnum;
               char *name;
               char secondary;
               std::vector<int> atoms;
               int  ca, o, c, n, h;
               int  code;
               bool breakF;
               } RESIDUE;

typedef struct {
                char *id;
                int nResidues;
                int nAmino;
                bool isDNA;
                bool isRNA;
                std::vector<RESIDUE> Residues;
                std::vector<RESIDUE> AminoRes;
               } CHAIN;

typedef struct {
        float px, py, pz;      // Spline Control Coordinate
        float tx, ty, tz;      // Spline Direction Vector  
        float hnx, hny, hnz;   // Horizontal Normal Vector
        float vnx, vny, vnz;   // Vertical Normal Vector 
        float dx, dy, dz;      // Ribbon Height Vector  
        float wx, wy, wz;      // Ribbon Width Vector  
        float hinten;          // Horizontal Intensity
        float vinten;          // Vertical Intensity 
        float hsize;           // Horizontal Vector Length 
        float vsize;           // Vertical Vector Length  
        float wide;            // Ribbon Width           
    } Knot;

typedef struct {
                 int  n1,n2,n3;
               } triangle_type;

typedef struct {
                 int nvert;              // numero di vertici del poliedro
                 int xc;                 // centro del poliedro
                 triangle_type face[8];  // facce tringolari del poliedro
                 float d_index;          // distorsion index
               } polyhedron_type;

typedef struct {
                float light_chain[4];
                float labelcolor[4];
                atom_color light_diffuse[NUM_COLORS];
                int LabelS[N_ELEMENTS];
               } statusSav;

namespace GLextmr {
   class Model {
   public:
      Model();
      Model(int status[], float fstatus[], statusSav vec);
      void InitColors();
      size_t truesize();
      void Save(std::ofstream &f);
      void SaveAU(std::ofstream &f, int k);
      void Save(FILE *fp);
      void SaveAU(FILE *fp, int k);
      bool Restore(std::ifstream &f, GLWindow *gl);
      bool Restore(FILE *fp, GLWindow *gl);
      void RestoreAU(std::ifstream &f, int k);
      void RestoreAU(FILE *fp, int k);
      void RipassaModello(int k);
      void setValues(int status[], float fstatus[]);
      void ToggleLabels(int flag);
      void freeModel();
      void freeVectors(bool tofree);
      void freeStructs(bool tofree);
      void PreparaTabelle();
      int  PreparaTabelle(std::string &msg);
      void DeleteResidue(int which);
      void DeleteSideChain(int which);
      void DeleteRing(int which);
      const char *NomeResiduo(int which);
      int  FindChain(const char *curChain);
      int  FindChainId(const char *curChain);
      int  FindResidue(int ch, const char *curResidue, RESIDUE *rr, int atom);
      int  FindConn(int at1, int at2, int nlegs=-1);
      void FillPhList();
      void FillListaAtomi(GtkWidget *dlg, int nodi[]);
      void FillLists(GtkWidget *dlg);
      void UpdateSelectedAtom(int whichAtom, bool to_add=false);
      void UpdateSelectedBond(int whichBond, int whichType);
      void UpdateSelectedResidue(int whichAlpha, bool to_add=false);
      std::vector<char *> ListaAtomi(std::vector<int> atoms, GtkWidget *dlg);
      void UpdateInfoSelections();
      void FillStructureInfo(GtkWidget *dlg);
      void FillStructureInfo(GLWindow *gl, std::stringstream &os, bool im=false);
      void FillListConn(GtkWidget *dlg, bool if_edit=false);
      void FillListAng(GtkWidget *dlg);
      void FillListAtoms(GtkWidget *dlg, bool is_edit=false);
      void FillListTors(GtkWidget *dlg);
      void FillListContacts(GtkWidget *dlg);
      void FillBarChart(GtkWidget *dlg);
      void FillListSphere(GtkWidget *dlg, int nrows, int atomi[],
                          float vdist[], op_type vsym[]);
      void FillComboSpec(GtkWidget *dlg);
      void FillBondValence(GtkWidget *dlg, float *bvs, int *cn);
      void FillBondLengths(GtkWidget *but, int wm, int kat);
      void GetConns();
      void GetEquivs();
      void CalcAngles();
      void CalcTors();
      float Distance(int atom1, int atom2);
      float Ang(int atom1, int atom2, int atom3);
      float Tors(int atom1, int atom2, int atom3, int atom4);
      int  SetColor(int whichAtom, float color1[]);
      void SetRungColor(int whichAtom, float color1[]);
      void DrawModel(GLWindow *gl);
      void DrawSBonds(GLWindow *gl, int nlegs, jav_bond_type *legs, int tipo);
      void DrawAtoms(GLWindow *gl, int i);
      void DrawBonds(GLWindow *gl, int j0);
      void DrawBonds2(GLWindow *gl, int j0, bool ifLigands=false);
      void DrawIsolated(GLWindow *gl);
      void DrawSticks(GLWindow *gl);
      void DrawSticks(GLWindow *gl, int i0, int i1, jav_bond_type *legs,
                      bool iflig=false, float *colore=NULL);
      void Draw2Cylinder (GLWindow *gl, int i0, int i1, jav_bond_type *legs);
      void DrawScaletta (GLWindow *gl);
      void DrawRibbons (GLWindow *gl, bool selected=false, int kmodel=-1);
      void DrawRibbon(int ca,  Knot knot[], int npoints, bool selected=false);
      void DrawTubes(GLWindow *gl, int npoints, Knot knot[], float col[], double factor=1);
      void DrawFlatRibbon(GLWindow *gl, Knot src, Knot dst, float color[]);
      void DrawSolidRibbon(GLWindow *gl, Knot src, Knot dst, float color[]);
      void DrawStrands(GLWindow *gl, int npoints, Knot knot[], float color[]);
      void DrawCircles(GLWindow *gl, int ind);
      void DrawCircles(GLWindow *gl);
      void DrawBalls(GLWindow *gl, int ind, bool ifEll=false);
      void DrawSelected(GLWindow *gl);
      void DrawSelectedBalls(GLWindow *gl);
      void DrawSelectedCylinder(GLWindow *gl, int i0, int i1, jav_bond_type *legs);
      void DrawCyls (GLWindow *gl, bool selected=false, bool ifLigands=false);
      void DrawPeaks (GLWindow *gl);
      void DrawLabels(bool ifSingle=false);
      void DrawLigandsLabels(double model[], double proj[], GLint view[]);
      void DrawAtomLabel(jav_atom_type at, int i, GLdouble x, GLdouble y,
               GLdouble z, PangoFontDescription *fontd, const char *lab=NULL);
      PangoRectangle *GetLabelRect(jav_atom_type at, int i,
                                    PangoFontDescription *fdesc);

      void Normalizza(GLWindow *gl);
      bool IsHydrogen(int which);
      bool IsHeavy(int which);
      bool IsPeak(int which);
      bool IsDna(int atom);
      bool IsHelix(int atom);
      bool IsBetaSheet(int atom);
      bool IsTurn(int atom);
      bool IsLigand(int atom);
      bool IsHetero(int atom);
      bool IsType(int atom, const char *type);
      int  IsTypeBB(int atom, const char *types[], int n);
      bool IsAromatic(int atom);
      bool IsSideChain(int atom);
      bool IsCAlpha(int atom) {return IsType(atom, "CA");}
      bool IsOxygen(int atom) {return IsType(atom, "O");}
      bool IsCarbon(int atom) {return IsType(atom, "C");}
      bool IsNitrogen(int atom) {return IsType(atom, "N");}
      bool IsProtein() {return (nResAmino > 0);}
      int  FindAtom(const char *type, RESIDUE rr, int tipo=0);
      int  GetNumResiduo(int ch, int r);
      std::string GetResName(int res1);
      const char *GetSpecies(int whichNo);
      const char *GetSpeciesName(int whichNo);
      bool CalcolaBaricentro(int i0, int i1, float bc[3]);
      void AllCAlpha();
      float VolumePerAtom(float Volume, int nsymt);
      int FindSpec(std::string &spec);
      atom_type *FromIlmatomToAtom();
      atom_type *FromJavatomToAtom();
      atom_type *FromListToAtom(GtkWidget *list, int *natoms);
      f_bond_type *FromIlmbondToLegm();
      f_bond_type *FromListToLegm(GtkWidget *list);
      jav_anglebond_type *FromListToAngs(GtkWidget *list);
      jav_torsion_type *FromListToTors(GtkWidget *list);
      std::string GetType(int na, ...);
      void FromLegmToList(GtkWidget *list, f_bond_type *legm);
      void FromAngsToList(GtkWidget *list, jav_anglebond_type *angs);
      void FillAngs(int nangles, jav_anglebond_type *angs);
      void FromTorsToList(GtkWidget *list, jav_torsion_type *angs);
      void FillTors(int nangles, jav_torsion_type *angs);
      void CopyAtoms(jav_atom_type *atml, int natoms);
      void DrawPolyhedra(GLWindow *gl);
      void CalcolaNormali();
      void FillPolyhedra(int np, polyhedron_type *p);
      void FillPolyhedraStyles();
      int  FindStyle(int whichPol);
      void NorEll(int na, float el[3][3], float Ell[3][3], float CD[3][3],
                  float semax[3] , int *ifIso);
      void SetEllVal(int na, float Ell[3][3], float CD[3][3], float semax[3]);
      void PreparaEllissoidi(GLWindow *gl);
      void CreaLegsBB();
      void CreaLegsCA();
      void CreaLegsLig();
      void PopulateModel (GtkTreeView *list, int ifClear);
      void CreaMappa(int quale, float pRadius=1.2f, float step=0.3f);
      void FillAtoms(atom_type *atml);
      void FillAtoms(atom_type *atml,  structure_info_type StructInfo, bool au,
                      int *nzo, int *het);
      void FillLegs(int *nlegs, int *nlegs_au, f_bond_type *legm, bool ed=false);
      void exportScene(GLWindow *gl, std::vector<vet3> &xyz);
      int isAtom(int x, int y, int ind);
      void FillPreview(std::stringstream &strm);
      void FillPreview(std::stringstream &strm, float I_coor[][3], int conn[][3]);

      int nModello;                 // numero del modello
      float shininess;
      bool isPDB;                   // se proviene da un file PDB
      float Bmin, Bmax;             // min/max value of thermal factor
      float Bmin_ca, Bmax_ca;       // min/max value of thermal factor(alpha carbons)
      float maxRad[4];              // raggio massimo degli atomi presenti
      float radMax;                                                        
      float baricentro[3];          // coordinate del baricentro
      int  nats;                    // numero di atomi del modello
      int  nats_au;                 // numero di atomi dell'unita' asimmetrica
      int  nlegs_au;                // legami           "    "      "      "
      int  nangbonds;
      int  ntorsions;
      int  nconn;                   // = nconn_prot
      int  nconn_prot;              // numero di legami della proteina
      int  nconn_bb;                // numero di legami del backbone
      int  nconn_ca;                // numero di legami tra i carboni alfa
      int  nconn_lig;               // numero di legami dei ligandi
      int  nconn_mix;               // numero di legami tra ligandi e proteina
      int  nconn_nuc;               // numero di legami dei nucleotidi
      float radii[MAX_SPECIES][2];  // raggio covalente per ogni specie
      int  numSpec;                 // numero di specie presenti
      int  numT;                    // dimensioni della Bond Table;
      int  Species[MAX_SPECIES];    // n. di specie atomica ( = no del fortran)
      int  Content[MAX_SPECIES];    // n. di atomi per specie ( = nw del fortran)
      len_spec LabSpec[MAX_SPECIES];// label specie
      int  ptab[MAX_SPECIES];       // ptab per specie
      bool created;                 // flag per grafica
      bool ifVis;                   //  "   "   "
      bool ifLabels;                //  "   "   "
      int  notLabelH;               //  "   "   "
      int  LabelS[N_ELEMENTS];      //  "   "   "
      bool ifCentro;                //  struttura centrosimmetrica si'/no
      int  plotStyle;               // plot style corrente
      int  drawLigand;              // flag per grafica
      bool ifSymmetry;              // symmetry si'/no
      float klevel[3];              // packing range x simmetria (da 0 a 10)
      int  FitOpt;                  // fit option x simmetria (da 1 a 4)
      bool ifChangedSymmetry;       //  "   "   modificata
      bool ifDrawH;                 // draw hydrogens si'/no
      bool ifDrawW;                 // draw watre molecules si'/no
      bool ifResize;                // resize su tutti gli atomi o solo a.u.
      int  colorMode;               // tipo di colore
      int  labelType;               // tipo di label (0 = nome, 1 = specie)
      bool ifSurface;               // richiesta superficie di VanDerWaals
      bool Surface_c;               // creata superficie di VanDerWaals
      bool ifPoly;                  // richiesti poliedri si'/no
      int   nChains;                // numero di catene
      int   nResTot;                // numero di residui
      int   nResAmino;              // numero di residui veri (amminoacidi)
      bool  hasSecondary;           // struttura secondaria determinata si'/no
      float RF;                     // R-factor
      int nlegsH;
      int nlegsW;
      int   nucleic;                // 0 inorganic or protein; 1 dna; 2 rna
      float light_chain[4];         // luci e colori per grafica
      float labelcolor[4];          // luci e colori per grafica
      atom_color light_diffuse[NUM_COLORS];  // luci e colori per grafica
      int   numbStructs[4];         // numero di coils, eliche, di strands, e di turns
      float distHn[NHDistances];    // distanze X-H normalizzate
      int nPoly;                    // numero di Poliedri
      int nPolyStyles;              // numero di Poliedri distinti
      char *modname;                // nome del modello
      jav_atom_type *atm;           // info sugli atomi 
      jav_atom_type *atm_au;        // info sugli atomi (a.u.)
      int  *iEqui;                  // equivalenti per simmetria
      jav_bond_type *bonds;         // legami 
      jav_bond_type *bonds_bb;      // legami backbone
      jav_bond_type *bonds_ca;      // legami c_alpha and/or phospho...
      jav_bond_type *bonds_lig;     // legami ligandi
      jav_bond_type *bonds_mix;     // legami ligandi/proteina
      jav_bond_type *bonds_nuc;     // legami nucleotidi
      jav_bond_type *legsH;         // legami idrogeno
      jav_bond_type *legsW;         // legami Wandervaals
      char  *Secondary;             // info sulla struttura secondaria (+)
      jav_anglebond_type *anglebonds;// legami + angoli
      jav_torsion_type *torsions;   // angoli di torsione
      polyhedron_type *Polyhedra;   // poliedri
      polyhedron_style *PolyStyle;  //stile poliedri
      int *NConns;                  // per ogni atomo numero degli atomi connessi
      int *NEquivs;                 // per ogni atomo numero degli atomi equivalenti
      std::vector<CHAIN> Chains;    // catene;
      std::vector<C_Alpha> c_alpha; //carboni alfa 
      std::vector<C_Alpha> c_alphab;//carboni alfa 
      std::vector<int> atoms_sel;   //atomi selezionati
      std::vector<int> bonds_sel;   //legami selezionati
      std::vector<int> resid_sel;   //residui selezionati
      std::vector<int> phlist   ;   //
      std::vector<char *> atom_types;//
      std::vector<char *> AtomTypes;//
      std::vector<std::vector<int> > Conns   ;// per ogni atomo lista degli atomi connessi
      std::vector<std::vector<int> > Equivs  ;// per ogni atomo lista degli atomi equivalenti

// (+)   struttura secondaria
//       'E' 'e'   --> beta sheet
//       'H' 'h'   --> elica
//       'G' 'g'   --> elica
//       'P' 'p'   --> elica
   };

void add_model(GtkWidget *window, int i, GtkWidget *father, GtkWidget *parent);
void model_settings(GtkWidget *window, GtkWidget *dlg, GtkWidget *vbox);
void CApplicaSimmetria(int *which);
}

