module ochemdb

implicit none

contains

   subroutine export_for_ochemdb(atom,legm,elem,cell,spg,sname)
!
!  Generate file csv for statistical analysis of distances
!
   USE elements
   USE atom_type_util
   USE unit_cell
   USE spginfom
   USE connect_mod
   USE strutil
   USE fileutil
   USE nrutil, only: swap
   USE arrayutil
   type(atom_type), dimension(:), allocatable, intent(in)    :: atom
   type(bond_type), dimension(:), allocatable                :: legm
   type(element_type), dimension(:), allocatable, intent(in) :: elem
   type(cell_type), intent(in)                               :: cell
   type(spaceg_type), intent(in)                             :: spg
   character(len=*), intent(in)                              :: sname
   integer                                                   :: i,a1,a2,a3
   type(file_handle)                                         :: fdb
   type(angle_type), dimension(:), allocatable               :: ang
   type(container_type), dimension(:), allocatable           :: conn
   character(len=NLEN_LAB), dimension(:), allocatable        :: svet
   integer                                                   :: nmaxbonds,spos,strcode
   type(sg_info_type)                                        :: infospg
   character(len=1), dimension(0:1)                          :: strtype = ['I','O']
   character(len=:), allocatable                             :: crys
   type(atom_type), dimension(:), allocatable                :: atoms
   type(bond_type), dimension(:), allocatable                :: legms
   integer, dimension(:), allocatable                        :: charge
!
   if (numatoms(atom) == 0) return
!
   call init_for_symm(atom,legm,atoms,legms)
   call expand_contacts(atom,cell,spg,atoms,legms,.false.)
   if (numbonds(legms) == 0) return
!
   call legm_to_connect(numatoms(atoms),legms,conn)
!
!  Write file with: ;ID;elem1;elem2;dist;
   call fdb%fopen(trim(sname)//'d.csv','w')
   do i=1,numbonds(legms)
      if (.not.order_is_ok(atoms(legms(i)%n1)%z(),atoms(legms(i)%n2)%z())) then !Sort Z: es. C-N but not N-C
          call swap(legms(i)%n1,legms(i)%n2)
      endif
      a1 = legms(i)%n1
      a2 = legms(i)%n2
      write(fdb%handle(),'(a)')';'//trim(sname)//';'  &
                       //trim(atoms(a1)%spec())//';'//trim(atoms(a2)%spec())//';'//r_to_s(legms(i)%dist)//';'
   enddo
   call fdb%fclose()
!
   nmaxbonds = maxval(conn%nat)
   allocate(svet(nmaxbonds))
!
!  Write file with: ;ID;charge1,charge2,label1,label2;conn1;conn2;
   call fdb%fopen(trim(sname)//'d1.csv','w')
   allocate(charge(numatoms(atom)))
   charge = atoms%charge()
   where(charge == 0) charge = -999
   do i=1,numbonds(legms)
      a1 = legms(i)%n1
      a2 = legms(i)%n2
      write(fdb%handle(),'(a)',advance='no')';'//trim(sname)//';'    &
                       //i_to_s(charge(a1))//';'//i_to_s(charge(a2))//';'  &
                       //atoms(a1)%glab()//';'//atoms(a2)%glab()//';'
!
!     Write connected atoms to a1 and a2 in ascending order
      if (conn(a1)%nat > 1) then
          svet(:conn(a1)%nat-1) = atoms(pack(conn(a1)%pos(:),mask=conn(a1)%pos /= a2))%spec()
          if (conn(a1)%nat > 2) then
              call svec_sort_heap_a(conn(a1)%nat-1,svet)
              write(fdb%handle(),'(a)',advance='no')cat_svet(svet,conn(a1)%nat-1,sep='-')
          else
              write(fdb%handle(),'(a)',advance='no')trim(svet(1))
          endif
      endif
      write(fdb%handle(),'(a)',advance='no')';'
      if (conn(a2)%nat > 1) then
          svet(:conn(a2)%nat-1) = atoms(pack(conn(a2)%pos(:),mask=conn(a2)%pos /= a1))%spec()
          if (conn(a2)%nat > 2) then
              call svec_sort_heap_a(conn(a2)%nat-1,svet)
              write(fdb%handle(),'(a)',advance='no')cat_svet(svet,conn(a2)%nat-1,sep='-')
          else
              write(fdb%handle(),'(a)',advance='no')trim(svet(1))
          endif
      endif
      write(fdb%handle(),'(a)',advance='yes')';'
   enddo
   call fdb%fclose()
!
!  Angles
   call legm_to_ang(legms,atoms,cell%get_g(),ang)
   if (numangles(ang) > 0) then
       call fdb%fopen(trim(sname)//'a.csv','w')
       do i=1,numangles(ang)
          if (.not.order_is_ok(atoms(ang(i)%n1)%z(),atoms(ang(i)%n3)%z())) then !Sort Z: es. C-N-H but not H-N-C
              call swap(ang(i)%n1,ang(i)%n3)
          endif
          a1 = ang(i)%n1
          a2 = ang(i)%n2
          a3 = ang(i)%n3
          write(fdb%handle(),'(a)')';'//trim(sname)//';'//trim(atoms(a1)%spec())//';'//trim(atoms(a2)%spec())  &
                                 //';'//trim(atoms(a3)%spec())//';'//r_to_s(ang(i)%val)//';' 
       enddo
       call fdb%fclose()
!
!      Write file with: ;ID;charge1;charge2;charge3;label1;label2;label3,conn1;conn2;conn3;
       call fdb%fopen(trim(sname)//'a1.csv','w')
        
       do i=1,numangles(ang)
          a1 = ang(i)%n1
          a2 = ang(i)%n2
          a3 = ang(i)%n3
          write(fdb%handle(),'(a)',advance='no')';'//trim(sname)//';'    &
                       //i_to_s(charge(a1))//';'//i_to_s(charge(a2))//';'//i_to_s(charge(a3))//';'  &
                       //atoms(a1)%glab()//';'//atoms(a2)%glab()//';'//atoms(a3)%glab()//';'
!
!         Write connected atoms to a1 in ascending order
          if (conn(a1)%nat > 1) then
             svet(:conn(a1)%nat-1) = atoms(pack(conn(a1)%pos(:),mask=conn(a1)%pos /= a2))%spec()
             if (conn(a1)%nat > 2) then
                 call svec_sort_heap_a(conn(a1)%nat-1,svet)
                 write(fdb%handle(),'(a)',advance='no')cat_svet(svet,conn(a1)%nat-1,sep='-')
             else
                 write(fdb%handle(),'(a)',advance='no')trim(svet(1))
             endif
          endif
          write(fdb%handle(),'(a)',advance='no')';'
!
!         Write connected atoms to a1 in ascending order
          if (conn(a2)%nat > 2) then
             svet(:conn(a2)%nat-2) = atoms(pack(conn(a2)%pos(:),mask=conn(a2)%pos /= a3 .or. conn(a2)%pos /= a1))%spec()
             if (conn(a2)%nat > 3) then
                 call svec_sort_heap_a(conn(a2)%nat-2,svet)
                 write(fdb%handle(),'(a)',advance='no')cat_svet(svet,conn(a1)%nat-2,sep='-')
             else
                 write(fdb%handle(),'(a)',advance='no')trim(svet(1))
             endif
          endif
          write(fdb%handle(),'(a)',advance='no')';'
!
!         Write connected atoms to a3 in ascending order
          if (conn(a3)%nat > 1) then
             svet(:conn(a3)%nat-1) = atoms(pack(conn(a3)%pos(:),mask=conn(a3)%pos /= a2))%spec()
             if (conn(a3)%nat > 2) then
                 call svec_sort_heap_a(conn(a3)%nat-1,svet)
                 write(fdb%handle(),'(a)',advance='no')cat_svet(svet,conn(a3)%nat-1,sep='-')
             else
                 write(fdb%handle(),'(a)',advance='no')trim(svet(1))
             endif
          endif
          write(fdb%handle(),'(a)',advance='yes')';'
       enddo
       call fdb%fclose()
   endif
!
!  Space group
   if (spg%num == 0) return
   call fdb%fopen(trim(sname)//'spg.csv','w')
   infospg = get_spg_info(spg%num,spos)
   if (spos > 0) then
       strcode = is_organic(elem%z,nint(elem%nw))
       if (strcode == 2) strcode = 1  ! metallorganic -> organic
       if (spg%csys_code == CS_Trigonal) then
           if (uc_is_rhombohedral(cell%get_par(),0.05)) then
               crys = trim(cry_sys(spg%csys_code))//';R;'
           elseif (uc_is_hexagonal(cell%get_par(),0.05)) then
               crys = trim(cry_sys(spg%csys_code))//';H;'
           else   ! possible problem with cell
               return
           endif
       else
           crys = trim(cry_sys(spg%csys_code))//';;'
       endif
       write(fdb%handle(),'(a,i0,a)')';'//trim(sname)//';'//trim(spg%symbol_xhm)//';', &
             spg%num,';'//trim(infospg%hm2)//';'//strtype(strcode)//';'//crys
   endif
!
   end subroutine export_for_ochemdb

end module ochemdb
