MODULE pearsonf
   USE progtype, only: rparam_type
   implicit none

   type pearson_function_type
      type(rparam_type) :: u,v,w
      type(rparam_type) :: b0,b1,b2
      type(rparam_type) :: asym
   end type pearson_function_type

   real, parameter, private :: BETAMIN=0.51, BETAMAX=20.0

   real, dimension(:), allocatable, private :: gamrav,cpearv,cpear2v,betav

CONTAINS

   subroutine init_calc(n)
   USE arrayutil
   integer, intent(in)          :: n
!
   if (allocated(gamrav) .and. n == size(gamrav)) return
   call new_array(gamrav,n)
   call new_array(cpearv,n)
   call new_array(cpear2v,n)
   call new_array(betav,n)
!
   end subroutine init_calc

!------------------------------------------------------------------------------------

   function init_pearson_param()  result(param)
   USE progtype, only: rparam_type
   type(pearson_function_type) :: param
   
   param%u = rparam_type(val=0.0,str='U')
   param%v = rparam_type(val=0.0,str='V')
   param%w = rparam_type(val=0.01,str='W')

   param%b0 = rparam_type(val=2.0,str='beta0')
   param%b1 = rparam_type(val=0.0,str='beta1')
   param%b2 = rparam_type(val=0.0,str='beta2')
   param%asym = rparam_type(val=0.0,str='asym')

   end function init_pearson_param

!------------------------------------------------------------------------------------

   subroutine pearson(par,ref,xcount,fs,scal,ycount,nwave,rapporto_int,ier)
   USE reflection_type_util, only:reflection_type   
   USE trig_constants, only:dtor,pi
   USE arrayutil
   USE profile_function
!
   type(pearson_function_type), intent(in)            :: par
   type(reflection_type), dimension(:), intent(inout) :: ref
   real, dimension(:), intent(in)                     :: xcount
   real, dimension(:), intent(in)                     :: fs    
   real, intent(in)                                   :: scal
   real, dimension(:), intent(out)                    :: ycount
   integer, intent(in)                                :: nwave
   real, intent(in)                                   :: rapporto_int
   integer, intent(out)                               :: ier
   real                                               :: b0,b1,b2
   real                                               :: fu,fv,fw
   real                                               :: asym
   integer                                            :: j,kal,nnn
   integer                                            :: iniz,ifin
   real                                               :: tthdeg
   real                                               :: tdeg
   real                                               :: cpear,cpear2
   real                                               :: uvwg,uvw2,uvwqg
   real                                               :: ffx
   real                                               :: deltath,deltath2
   real                                               :: argbeta,ccas
   real                                               :: asterm,pears
   real                                               :: beta,pconst   !!!!!,betad,gambeta,gambetad
   real                                               :: gamra
   real, dimension(2)                                 :: rap
   integer                                            :: negfw1,negfw2
   real                                               :: epsmch = epsilon(1.0)
   real :: tmax
!
   b0 = par%b0%val
   b1 = par%b1%val
   b2 = par%b2%val
   fu = par%u%val
   fv = par%v%val
   fw = par%w%val
   asym = par%asym%val
   negfw1 = 0
   negfw2 = 0
   ier = 0
   ycount(:) = 0.0
   tmax = xcount(size(xcount))
!
   call init_calc(size(ref))
!
   rap = (/1.0,rapporto_int/)
   LOOP_KALPHA : do kal=1,nwave   !!!!!!kalphar
      LOOP_RIFLESSI : do j=1,size(ref)
         tthdeg = ref(j)%tthd
!
         beta = b0 + b1/tthdeg + b2/(tthdeg**2)
         if (beta < betamin .or. beta > betamax) then
             !write(6,*)'WARN BETA=',beta,j
             negfw1 = negfw1 + 1
             !ier = -1
             !return
         endif
         if (beta < betamin) beta = betamin
         if (beta > betamax) beta = betamax
         betav(j) = beta
!
         gamra = gamma(beta) / gamma(beta-0.5)
!
         tdeg = tan (0.5 * tthdeg * dtor)
         uvwqg = fU*tdeg**2 + fV*tdeg + fW
         if (uvwqg < epsmch) then
             !write(6,*)'WARN FWHM=',uvwqg,j
             uvwqg = 0.0001
             negfw2 = negfw2 + 1
             !ier = -1
         endif
         uvwg = sqrt(uvwqg)
         pconst = 2.0**(1.0/beta) - 1.0
         cpear = (2.0 * sqrt(pconst/pi)) / uvwg
         cpear2 = (4.0 * pconst) / uvwqg
!corr         phkl(j,1)=gamra         ! rapporto tra le funzioni gamma
!corr         phkl(j,3)=cpear
!corr         phkl(j,4)=cpear2
         gamrav(j) = gamra
         cpearv(j) = cpear
         cpear2v(j) = cpear2
!
         ref(j)%fwhm = uvwg
         uvw2 =  uvwg * ref(j)%pk * rapIcalc(j)
!TODO: test this!         if (tthdeg-uvw2 > tmax) cycle
!
         iniz = clocate(xcount,tthdeg-uvw2)
         ifin = clocate(xcount,tthdeg+uvw2)
!
         ffx = ref(j)%m*fs(j)**2*ref(j)%po*ref(j)%lp*ref(j)%as*rap(kal)
!
         LOOP_CONTEGGI : do nnn=iniz,ifin
            deltath=xcount(nnn)-tthdeg
            deltath2=deltath**2
            argbeta=(1.0+cpear2*deltath2)
            ccas=1.0/cpear2+deltath2
            pears = gamra * cpear * argbeta**(-beta)
            asterm= 1.0 + asym*deltath**3/(ccas**1.5)
            ycount(nnn)=ycount(nnn) + scal*ffx*pears*asterm
         enddo LOOP_CONTEGGI
      enddo LOOP_RIFLESSI
   enddo LOOP_KALPHA
   if (real(negfw1)/size(ref) > 0.4) ier = 1
   if (real(negfw2)/size(ref) > 0.4) ier = 1
   !if(negfw1>0)write(0,*)'NWGFW=',negfw1,real(negfw1)/size(ref),ier
   !if(negfw2>0)write(0,*)'NWGFW=',negfw2,real(negfw2)/size(ref),ier
!
   end subroutine pearson

!------------------------------------------------------------------------------------

   subroutine pearson_init(ref,xcount,par,maxlim,rapporto_int,ier) 
   USE reflection_type_util, only:reflection_type 
   USE trig_constants, only:dtor,pi
   USE arrayutil
   USE profile_function
!
   type(pearson_function_type), intent(in)            :: par
   type(reflection_type), dimension(:), intent(inout) :: ref
   real, dimension(:), intent(in)                     :: xcount
   real                                               :: b0,b1,b2
   real                                               :: fu,fv,fw
   real                                               :: asym
   integer, intent(out)                               :: maxlim
   real, intent(in)                                   :: rapporto_int
   integer, intent(out)                               :: ier
   integer                                            :: j,nnn
   real                                               :: tthdeg
   real                                               :: tdeg
   real                                               :: uvwg,uvw2,uvwqg
   real                                               :: deltath,deltath2
   real                                               :: argbeta,ccas
   real                                               :: asterm,pears
   real                                               :: beta,pconst  
   real, dimension(2)                                 :: rap
   integer                                            :: negfw
   real                                               :: cpear,cpear2
   real                                               :: gamra
   integer                                            :: nnref
!                
   if (.not.allocated(profi)) allocate(profi(size(ref))) 
   b0 = par%b0%val
   b1 = par%b1%val
   b2 = par%b2%val
   fu = par%u%val
   fv = par%v%val
   fw = par%w%val
   asym = par%asym%val
   negfw = 0
   ier = 0
!corr   betamin = 0.95
!corr   betamax = 20.0
!
!
   rap = (/1.0,rapporto_int/)
   nnref = size(ref)
   LOOP_RIFLESSI : do j=1,nnref
      tthdeg = ref(j)%tthd
!
      beta = b0 + b1/tthdeg + b2/(tthdeg**2)
      if (beta < betamin .or. beta > betamax) then
          !write(6,*)'WARN BETA=',beta
          ier = 1
          !return
      endif
      if (beta < betamin) beta = betamin
      if (beta > betamax) beta = betamax
      betav(j) = beta
!
!!!      gambeta   = gammae(beta)
      !gambeta   = gamma(beta)
      !betad = beta-0.5
!!!      gambetad  = gammae(betad)
      !gambetad  = gamma(betad)
      !gamra     = gambeta/gambetad
      gamra = gamma(beta) / gamma(beta-0.5)

      tdeg = tan (0.5 * tthdeg * dtor)
      uvwqg = fU*tdeg**2 + fV*tdeg + fW
      if (uvwqg.lt.0.0001) then
          uvwqg = 0.0001
          negfw = negfw + 1
          ier = 1
      endif
      uvwg = sqrt(uvwqg)
      pconst = 2.0**(1.0/beta) - 1.0
      cpear = (2.0 * sqrt(pconst/pi)) / uvwg
      cpear2 = (4.0 * pconst) / uvwqg
      ref(j)%fwhm = uvwg
      uvw2 =  uvwg * ref(j)%pk * rapIcalc(j)
!
      profi(j)%iniz = clocate(xcount,tthdeg-uvw2)
      profi(j)%ifin = clocate(xcount,tthdeg+uvw2)
      allocate(profi(j)%func(profi(j)%ifin-profi(j)%iniz+1))
      profi(j)%func = 0.0  ! forse non serve
!
      LOOP_CONTEGGI : do nnn=profi(j)%iniz,profi(j)%ifin
         deltath=xcount(nnn)-tthdeg
         deltath2=deltath**2
         argbeta=(1.0+cpear2*deltath2)
         ccas=1.0/cpear2+deltath2
         pears = gamra * cpear * argbeta**(-beta)
         asterm= 1.0 + asym*deltath**3/(ccas**1.5)
         profi(j)%func(nnn-profi(j)%iniz+1) = pears*asterm
      enddo LOOP_CONTEGGI
      !profi(j)%fconst = ref(j)%m*ref(j)%po*ref(j)%lp
      profi(j)%fconst = ref(j)%m*ref(j)%lp
   enddo LOOP_RIFLESSI
   maxlim = maxval(profi%ifin)
!
   end subroutine pearson_init

!------------------------------------------------------------------------------------

   subroutine pearson_calc(ref,ycount,sumyco,gpo)
   USE reflection_type_util, only:reflection_type  
   USE profile_function
!
   type(reflection_type), dimension(:), intent(inout) :: ref
   real, dimension(:), intent(out) :: ycount
   real, intent(out)               :: sumyco
   type(rparam_type), intent(in) :: gpo
   integer                         :: j,nnn
   real                            :: ffx
   real                            :: addyco
   integer                         :: iniz0
!
   ycount(:) = 0.0
   sumyco = 0.0
   !write(6,*)'size=',size(ycount)
!
   if (gpo%rcod > 0) then
       do j=1,size(ref)
          ffx = ref(j)%fc*profi(j)%fconst*ref(j)%po
          iniz0 = profi(j)%iniz - 1
          do nnn=profi(j)%iniz,profi(j)%ifin
             addyco =  ffx*profi(j)%func(nnn-iniz0)
             sumyco = sumyco + addyco
             ycount(nnn)=ycount(nnn) + addyco
          enddo 
       enddo 
   else
       do j=1,size(ref)
          ffx = ref(j)%fc*profi(j)%fconst
          iniz0 = profi(j)%iniz - 1
          do nnn=profi(j)%iniz,profi(j)%ifin
             addyco =  ffx*profi(j)%func(nnn-iniz0)
             sumyco = sumyco + addyco
             ycount(nnn)=ycount(nnn) + addyco
          enddo 
       enddo 
   endif
!
   end subroutine pearson_calc
 
!------------------------------------------------------------------------------------

   subroutine decomponi_pearson(par,ref,xcount,scal,dycount,nwave,rapporto_int,ier)
   USE reflection_type_util, only:reflection_type
   USE trig_constants, only:dtor,pi
   USE arrayutil
   USE profile_function
!
   type(pearson_function_type), intent(in)            :: par
   type(reflection_type), dimension(:), intent(inout) :: ref
   real, dimension(:), intent(in)                     :: xcount
   real, intent(in)                                   :: scal
   real, dimension(:,:), intent(out)                  :: dycount
   integer, intent(in)                                :: nwave
   real, intent(in)                                   :: rapporto_int
   integer, intent(out)                               :: ier
   real                                               :: b0,b1,b2
   real                                               :: fu,fv,fw
   real                                               :: asym
   integer                                            :: j,kal,nnn
   integer                                            :: iniz,ifin
   real                                               :: tthdeg
   real                                               :: cpear,cpear2,uvw2
   real                                               :: kffx
   real                                               :: deltath,deltath2
   real                                               :: argbeta,ccas
   real                                               :: asterm,pears
   real                                               :: beta   
   real                                               :: gamra
   real, dimension(2)                                 :: rap
   integer                                            :: negfw
!
   b0 = par%b0%val
   b1 = par%b1%val
   b2 = par%b2%val
   fu = par%u%val
   fv = par%v%val
   fw = par%w%val
   asym = par%asym%val
!corr   betamin = 0.95
!corr   betamax = 20.0
   negfw = 0
   ier = 0
   dycount(:,:) = 0.0
   call init_calc(size(ref))
!
   rap = (/1.0,rapporto_int/)
   LOOP_KALPHA : do kal=1,nwave  !!!!!kalphar
      LOOP_RIFLESSI : do j=1,size(ref)
         tthdeg = ref(j)%tthd
!
         beta = b0 + b1/tthdeg + b2/(tthdeg**2)
         if (beta < betamin .or. beta > betamax) then
             !write(6,*)'WARN BETA=',beta
             negfw = negfw + 1
             !ier = -1
             !return
         endif
         if (beta < betamin) beta = betamin
         if (beta > betamax) beta = betamax
         betav(j) = beta
!
!corr         gamra = gamma(beta) / gamma(beta-0.5)
!
!corr         tdeg = tan (0.5 * tthdeg * dtor)
!corr         uvwqg = fU*tdeg**2 + fV*tdeg + fW
!corr         if (uvwqg.lt.epsmch) then
!corr             uvwqg = 0.0001
!corr             negfw = negfw + 1
             !ier = -1
!corr         endif
!corr         uvwg = sqrt(uvwqg)
!corr         pconst = 2.0**(1.0/beta) - 1.0
!corr         cpear = (2.0 * sqrt(pconst/pi)) / uvwg
!corr         cpear2 = (4.0 * pconst) / uvwqg
!corr         phkl(j,1)=gamra         ! rapporto tra le funzioni gamma
!corr         phkl(j,3)=cpear
!corr         phkl(j,4)=cpear2
         gamra = gamrav(j)
         cpear = cpearv(j)
         cpear2 = cpear2v(j)
!
!corr         ref(j)%fwhm = uvwg
!corr         uvw2 =  uvwg * ref(j)%pk * rapIcalc(j)
         uvw2 =  ref(j)%fwhm * ref(j)%pk * rapIcalc(j)
!
         iniz = clocate(xcount,tthdeg-uvw2)
         ifin = clocate(xcount,tthdeg+uvw2)
!
         kffx = ref(j)%m*ref(j)%po*ref(j)%lp*ref(j)%as*rap(kal)
!
         LOOP_CONTEGGI : do nnn=iniz,ifin
            deltath=xcount(nnn)-tthdeg
            deltath2=deltath**2
            argbeta=(1.0+cpear2*deltath2)
            ccas=1.0/cpear2+deltath2
            pears = gamra * cpear * argbeta**(-beta)
            asterm= 1.0 + asym*deltath**3/(ccas**1.5)
            dycount(j,nnn)=scal*kffx*pears*asterm
         enddo LOOP_CONTEGGI
      enddo LOOP_RIFLESSI
   enddo LOOP_KALPHA
   !if(negfw>0)write(6,*)'NWGFW=',negfw,real(negfw)/size(ref)
   !if(negfw>0)write(0,*)'NWGFW=',negfw,real(negfw)/size(ref)
   if (real(negfw)/size(ref) > 0.4) ier = 1
!
   end subroutine decomponi_pearson

!--------------------------------------------------------------------

   subroutine derpears(fstr,par,inizr,ifinr,xcount,ref,cell,derFct,derOP,deryz,dfw,derp,dercc,derG,  &
                       rapporto_int,scalr,rinfo,nwave,gpo,neutro,nsyn,wave,jsys)
   USE progtype, only: refine_info_type 
   USE trig_constants
   USE arrayutil
   USE profile_function, only: rapIcalc
   USE reflection_type_util
   USE type_constants, only: DP
!
   implicit none
   real, dimension(:), intent(in)                  :: fstr
   type(pearson_function_type), intent(in)         :: par
   integer, intent(in)                             :: inizr,ifinr
   real, dimension(:), intent(in)                  :: xcount
   type(reflection_type), dimension(:), intent(in) :: ref
   real, dimension(6), intent(in)                  :: cell
   real, dimension(:), intent(in)                  :: derFct
   real, dimension(:), intent(in)                  :: derOP
   real, dimension(:), intent(out)                 :: deryz
   real, dimension(:,:), intent(out)               :: dfw
   real, dimension(:,:), intent(out)               :: derp
   real, dimension(:,:), intent(out)               :: dercc
   real, dimension(:), intent(out)                 :: derG
   real, intent(in)                                :: rapporto_int
   type(rparam_type), intent(in)                   :: scalr
   type(refine_info_type), intent(in)              :: rinfo
   integer, intent(in)                             :: nwave
   type(rparam_type), intent(in)                   :: gpo
   integer, intent(in)                             :: neutro, nsyn, jsys
   real, intent(in)                                :: wave
   integer                                         :: j,k,kal,nnn
   integer                                         :: iniz,ifin
   logical                                         :: iderc
   real                                            :: asym,scal
   real                                            :: tthdeg    !!!!,thmin0
   real                                            :: beta,beta0,beta1,beta2
   real                                            :: rdeg,tdeg,tdeg2,cdeg
   real                                            :: uvwg,uvwqg,uvw2
   real                                            :: derft,dlorp
   real                                            :: deltath,deltath2,deltath3
   real                                            :: argbeta,ccas,pears,asterm
   real                                            :: ffb1,ffnop,ffx,ffbx
   real                                            :: absoc,pesolp
   real                                            :: deryop,derpth,derasth
   real, dimension(6)                              :: dercells
   real, dimension(3)                              :: gradasym,vetder
   real, dimension(4)                              :: gradprof,vetder1
   real                                            :: fwU,fwV,fwW
   real                                            :: ln2
   real                                            :: pconst,rapgamma
   real                                            :: deruvw,derbeta
   real, dimension(2)                              :: rap
   real                                            :: deraszero,derfunczero
   real                                            :: betad,gambeta,gambetad
   real                                            :: gammae,dergbeta,gamder,dergbetad
   real(DP)                                        :: mmpsi
   integer                                         :: ier
   real                                            :: kpol
!
   if (neutro == 1 .or. nsyn == 1) then
       kpol = 0.0
   else
       kpol = 0.8
   endif
!corr   thmin0 = thmin - 2.0*zero%val
   deryz = 0.0
   dfw   = 0.0
   derp  = 0.0
   dercc = 0.0
   derG(:)  = 0.0
!
   ln2 = alog(2.0)
   vetder(1) = 1.0
   vetder1(1) = 1.0
   rap(1) = 1.0
   rap(2) = rapporto_int
   asym = par%asym%val
   beta0 = par%b0%val
   beta1 = par%b1%val
   beta2 = par%b2%val
   fwU  = par%u%val
   fwV  = par%v%val
   fwW  = par%w%val
   scal = scalr%val
   iderc = rinfo%refine_cell
   LOOP_KALPHA : do kal=1,nwave
      LOOP_RIFLESSI : do j=inizr,ifinr
         tthdeg = ref(j)%tthd
         derft = derFct(j)   
         beta = betav(j)
!
         gambeta   = gammae(beta)
         betad = beta-0.5
         gambetad  = gammae(betad)
!
         dergbeta  = gambeta*mmpsi(dble(beta),ier)
         dergbetad = gambetad*mmpsi(dble(betad),ier)
         gamder    = (dergbeta*gambetad - gambeta*dergbetad)/gambetad**2   ! ! derivata del rapporto tra le funzioni gamma
!corr                  phkl(j,2)=gamder        ! derivata del rapporto tra le funzioni gamma
!
         pconst = 2.0**(1.0/beta)-1
         rdeg = 0.5 * tthdeg * dtor
         tdeg = sin(rdeg) / cos(rdeg)
         tdeg2 = tdeg * tdeg
         cdeg = cos(rdeg)
         uvwg = ref(j)%fwhm
         uvwqg = uvwg**2
         uvw2 = uvwg * ref(j)%pk * rapIcalc(j)                     ! ampiezza di picco
!
         iniz = clocate(xcount,tthdeg-uvw2)
         ifin = clocate(xcount,tthdeg+uvw2)
!
!corr         rapgamma = phkl(j,1)                                           ! rapporto tra le funzioni gamma
         rapgamma = gamrav(j)                                           ! rapporto tra le funzioni gamma
         if (iderc) then
             dlorp = derivataLP(ref(j)%tthd,kpol)
             dercells = scal*derteta_risp_celldir(jsys,cell,ref(j)%hkl,ref(j)%tthd,wave)        ! derivate di theta rispetto alla cella * scala
!
             vetder(2)=(-(2.0*beta1)/(tthdeg**2)-(4.0*beta2)/(tthdeg**3))   ! der. di beta rispetto theta in rad.
             vetder(3) = (1.0 / cos(rdeg)**2)*(fwV + 2.0*fwU*tdeg)          ! der. quadrato di FWHM rispetto theta in rad.
!
             vetder1(2) = vetder(2)
             vetder1(3) = 0.5*vetder(3)/uvwg                                ! der. FWHM rispetto theta
!corr             vetder1(4) = phkl(j,2) * vetder(2)                         ! der. rapporto fun. gamma rispetto theta
             vetder1(4) = gamder * vetder(2)                         ! der. rapporto fun. gamma rispetto theta
         endif
         pesolp = ref(j)%lp * rap(kal)
         absoc = ref(j)%as
!         ffnop = pesolp*ref(j)%m*(ref(j)%fc**2)*absoc
         ffnop = pesolp*ref(j)%m*(fstr(j)**2)*absoc
         ffx = ffnop*ref(j)%po
!         ffbx = pesolp*ref(j)%m*2.0*ref(j)%fc*ref(j)%po*absoc
         ffbx = pesolp*ref(j)%m*2.0*fstr(j)*ref(j)%po*absoc
         ffb1 = pesolp*ref(j)%m*ref(j)%po*absoc
!
         LOOP_CONTEGGI : do nnn=iniz,ifin
            deltath=xcount(nnn)-tthdeg
            deltath2=deltath**2
            deltath3=deltath**3
!corr            argbeta=(1.0+phkl(j,4)*deltath2)
!corr            ccas=1.0/phkl(j,4)+deltath2
!corr            pears = phkl(j,1) * phkl(j,3) * argbeta**(-beta)
            argbeta=(1.0+cpear2v(j)*deltath2)
            ccas=1.0/cpear2v(j)+deltath2
            pears = gamrav(j) * cpearv(j) * argbeta**(-beta)
            asterm= 1.0 + asym*deltath3/(ccas**1.5)
!
!           Compute derivative of 2theta corrected
            !deraszero = 6.0*asym*deltath2*(deltath2-ccas)/ccas**2.5                    ! der. asym ris. zero
            !derfunczero = pears*deltath*4.0*cpear2v(j)*beta/argbeta                 ! der. pearson ris. zero
            deraszero = -3.0*asym*deltath2*(deltath2-ccas)/ccas**2.5                 ! der. asym ris. 2theta corrected
            derfunczero = -pears*deltath*2.0*cpear2v(j)*beta/argbeta                 ! der. pearson ris. 2theta corrected
            deryz(nnn) = deryz(nnn) + scal * ffx * (derfunczero*asterm + deraszero*pears)
!
            gradasym(3) = (-0.375*asym*deltath**3)/((-1 + 2**(1/beta))*ccas**2.5)      ! der. asym ris. quadr. di FWHM
            gradasym(2) = (-1.5*ln2*2**(-2 + 1/beta)*asym*deltath3*uvwqg)/     &
                            ((-1 + 2**(1/beta))**2*beta**2*ccas**2.5)                  ! der. asym ris. beta
!
!           Derivata Pearson rispetto beta
!
            gradprof(2)=(-argbeta**(-1 - beta)*rapgamma*(                               &
                        ln2*2**(1/beta)*(argbeta* uvwg**2 - 8.*beta*deltath**2*pconst)+ &
                        2.*argbeta*beta**2*uvwg**2*pconst*aLog(argbeta) ))/              &
                        (beta**2*uvwg**3*(pconst/pi)**0.5*pi)
!
!           Derivata Pearson rispetto FWHM
!
            gradprof(3) = pears * (8.0*pconst*beta*deltath2/(uvwqg*argbeta) - 1.0) / uvwg
!
!           Derivata Pearson rispetto al rapporto tra funzioni gamma
!
!corr            gradprof(4)=phkl(j,3) * argbeta**(-beta)
            gradprof(4)=cpearv(j) * argbeta**(-beta)
!
            if(rinfo%refine_profile) then
               deruvw = scal*ffx*(gradprof(3)*asterm*0.5/uvwg+gradasym(3)*pears) !gradprof(3)*0.5/uvwg=der.rispetto FWHM**2
               dfw(nnn,1)=dfw(nnn,1)+deruvw*tdeg2                          !der. rispetto U
               dfw(nnn,2)=dfw(nnn,2)+deruvw*tdeg                           !der. rispetto V
               dfw(nnn,3)=dfw(nnn,3)+deruvw                                !der. rispetto W
!
               derp(nnn,1)=derp(nnn,1) + scal*ffx*pears*deltath3/(ccas**1.5) !der. rispetto asym
!
!corr               derbeta = scal * ffx * (gradprof(2)*asterm+gradasym(2)*pears+gradprof(4)*phkl(j,2)*asterm)
               derbeta = scal * ffx * (gradprof(2)*asterm+gradasym(2)*pears+gradprof(4)*gamder*asterm)
               derp(nnn,2)=derp(nnn,2) + derbeta                             !der. rispetto beta0
               derp(nnn,3)=derp(nnn,3) + derbeta/tthdeg                      !der. rispetto beta1
               derp(nnn,4)=derp(nnn,4) + derbeta/tthdeg**2                   !der. rispetto beta2
            endif
            if(iderc) then
               gradasym(1) = rtod*((6.0*asym*deltath**4)/ccas**2.5 - (6.0*asym*deltath**2)/ccas**1.5)
               derasth = DOT_PRODUCT(gradasym,vetder)                                      !der. asym rispetto theta
!
!              Derivata Pearson rispetto theta
!
               gradprof(1)=rtod*(32.*argbeta**(-1-beta)*beta*deltath*pconst*(pconst/pi)**0.5*rapgamma)/uvwg**3 !der. rispetto theta
               derpth = DOT_PRODUCT(gradprof,vetder1)     ! derivata Pearson rispetto theta
               do k=1,6                                   ! der. rispetto parametri di cella
                  dercc(nnn,k)=dercc(nnn,k)+ dercells(k)*(ffx * (derpth*asterm+derasth*pears) &
                          + (ffb1*derft+ffx/pesolp*dlorp) * pears*asterm)
               enddo
            endif
            if (gpo%rcod > 0) then
                deryop = scal * ffnop * pears * asterm
                derG(nnn)=derG(nnn) + deryop*derOP(j)
            endif
         enddo LOOP_CONTEGGI
      enddo LOOP_RIFLESSI
   enddo LOOP_KALPHA
!
   end subroutine derpears

!--------------------------------------------------------------------

   subroutine derpearsa(ref,par,inizr,ifinr,xcount,der,derc,nwave,rapporto_int,scalr)
   USE progtype, only: deratom
   USE reflection_type_util, only: reflection_type
   USE arrayutil
   USE profile_function, only: rapIcalc
!
   type(reflection_type), dimension(:), intent(in) :: ref
   type(pearson_function_type), intent(in)    :: par
   integer, intent(in)                        :: inizr,ifinr
   real, dimension(:), intent(in)             :: xcount
   type(deratom), dimension(:,:), intent(in)  :: der
   type(deratom), dimension(:,:), intent(out) :: derc
   integer, intent(in) :: nwave
   real, intent(in)                           :: rapporto_int
   type(rparam_type), intent(in) :: scalr
   integer             :: j,jj,kal,nnn
   integer             :: iniz,ifin
   real                :: tthdeg
   real                :: rap
   real                :: asym,scal,beta,uvw2
   real                :: deltath,deltath2,deltath3
   real                :: asterm,pears,ccas,cc
   real                :: ffb1,dydI
   integer :: nat
!
!corr   thmin0 = thmin - 2.0*zero%val
   derc = deratom(0.0,0.0,0.0)
   nat = size(derc,2)
!
   asym = par%asym%val
   scal = scalr%val
   LOOP_KALPHA : do kal=1,nwave   !!!!!kalphar
      LOOP_RIFLESSI : do j=inizr,ifinr
         tthdeg = ref(j)%tthd
         if (kal.eq.1) then
             rap = 1.0
           else
             rap = rapporto_int
         endif
!
         beta = betav(j)
         uvw2 = ref(j)%fwhm * ref(j)%pk * rapIcalc(j)
!
         iniz = clocate(xcount,tthdeg-uvw2)
         ifin = clocate(xcount,tthdeg+uvw2)
!
         ffb1 = rap*ref(j)%lp*ref(j)%m*ref(j)%po*ref(j)%as
         LOOP_CONTEGGI : do nnn=iniz,ifin
            deltath=xcount(nnn)-tthdeg
            deltath2=deltath**2
            deltath3=deltath*deltath2
            !cc=(1.0+phkl(j,4)*deltath2)
            !ccas=1.0/phkl(j,4)+deltath2
            !pears = phkl(j,1) * phkl(j,3) * cc**(-beta)
            cc=(1.0+cpear2v(j)*deltath2)
            ccas=1.0/cpear2v(j)+deltath2
            pears = gamrav(j) * cpearv(j) * cc**(-beta)
            asterm= 1.0 + asym*deltath3/(ccas**1.5)
!
!           calcolo derivate param. strutt. per atomo
!
            dydI = scal * ffb1 * pears * asterm
            do jj=1,nat
               derc(nnn,jj)%co = derc(nnn,jj)%co + dydI * der(j,jj)%co
               derc(nnn,jj)%b = derc(nnn,jj)%b + dydI * der(j,jj)%b
               derc(nnn,jj)%occ = derc(nnn,jj)%occ + dydI * der(j,jj)%occ
            enddo
!
         enddo LOOP_CONTEGGI
      enddo LOOP_RIFLESSI
   enddo LOOP_KALPHA
!
   end subroutine derpearsa

  !--------------------------------------------------------------------
  
   subroutine foleb_pears(ref,inizr,ifinr,Icalc,Iobs,pear,scalr,yoss,ytot,ttetaczero,back)
   USE arrayutil
   USE profile_function, only: rapIcalc
   USE reflection_type_util, only: reflection_type
!
   type(reflection_type), dimension(:), intent(in) :: ref
   integer, intent(in)             :: inizr,ifinr
   real, dimension(:), intent(in)  :: Icalc
   real, dimension(:), intent(out) :: Iobs
   type(pearson_function_type), intent(in) :: pear
   type(rparam_type), intent(in) :: scalr
   real, dimension(:), intent(in) :: yoss,ytot,ttetaczero,back
   integer                         :: iniz,ifin
   real                            :: asym,beta,scal
   integer                         :: j,nnn
   real                            :: tthdeg,deltath,deltath2
   real                            :: uvw2
   real                            :: cc,ccas,pears,asterm,yci,diffo,ycalc
   real                            :: sumy
   integer                         :: nneg
   real :: epsmch = epsilon(1.0)
!
   nneg = 0
   asym = pear%asym%val
   scal = scalr%val
   LOOP_RIFLESSI : do j=inizr,ifinr
      sumy = 0.0
      tthdeg = ref(j)%tthd
!
      beta = betav(j)
      uvw2 = ref(j)%fwhm * ref(j)%pk * rapIcalc(j)
!
      iniz = clocate(ttetaczero,tthdeg-uvw2)
!!!!TODO: use dataset%nc2
      ifin = min(clocate(ttetaczero,tthdeg+uvw2),size(ytot))
!
      LOOP_CONTEGGI : do nnn=iniz,ifin
         deltath=ttetaczero(nnn)-tthdeg
         deltath2=deltath**2
         cc=(1.0+cpear2v(j)*deltath2)
         ccas=1.0/cpear2v(j)+deltath2
         pears = gamrav(j) * cpearv(j) * cc**(-beta)
         asterm= 1.0 + asym*deltath**3/(ccas**1.5)
         yci = scal * pears * asterm
         if (ytot(nnn) > epsmch) then
             diffo = yoss(nnn)-back(nnn)
             if (diffo < 0.0) then
                 cycle
             endif
             ycalc = diffo*yci/ytot(nnn)
         else
             cycle
         endif
         sumy = sumy + ycalc
      enddo LOOP_CONTEGGI
!
      Iobs(j) = Icalc(j) * sumy
      if (Iobs(j) < epsmch) then
          Iobs(j) = 0.0
          nneg = nneg + 1
      endif
!
   enddo LOOP_RIFLESSI
   !if (nneg > 0) write(0,*)'IOBS NEGATIVI n=',nneg,sum(Iobs)
!
   end subroutine foleb_pears

!--------------------------------------------------------------------
#if 0
   subroutine defrange_pears(pear,ref,inizr,ifinr,kvolte)
   USE Counts
!corr   USE RefinecomRef
!corr   USE variables, only:ref
   USE arrayutil
   USE profile_function, only: rapIcalc
   USE reflection_type_util
!
   implicit none
   type(pearson_function_type), intent(in) :: pear
   type(reflection_type), dimension(:), intent(inout) :: ref
   integer, intent(in)    :: inizr,ifinr
   integer, intent(in)    :: kvolte
   integer                :: j,nnn,kal
   real , dimension(5000) :: drpf
   integer                :: nrp,nrpe 
   real                   :: beta,asym
   real                   :: tthdeg
   real                   :: uvw2,ungrad
   real                   :: deltath,deltath2,deltath3,cc,ccas
   real                   :: asterm,pears
   real                   :: summ
   integer                :: iniz,ifin
!
   asym = pear%asym%val
   kal=1
   LOOP_RIFLESSI : do j=inizr,ifinr
     tthdeg = ref(j)%tthd
     beta = betav(j)
     uvw2 =  ref(j)%fwhm * ref(j)%pk * rapIcalc(j)
!
         iniz = clocate(ttetaczero,tthdeg-uvw2)
         ifin = clocate(ttetaczero,tthdeg+uvw2)
!
     nrp = 0
     nrpe = 0
     LOOP_CONTEGGI : do nnn=iniz,ifin
        if (nrp.lt.5000) nrp = nrp + 1
        deltath=ttetaczero(nnn)-tthdeg
        deltath2=deltath**2
        deltath3=deltath*deltath2
!corr        cc=(1.0+phkl(j,4)*deltath2)
!corr        ccas=1.0/phkl(j,4)+deltath2
!corr        pears = phkl(j,3) * cc**(-beta)
         cc=(1.0+cpear2v(j)*deltath2)
         ccas=1.0/cpear2v(j)+deltath2
         pears = cpearv(j) * cc**(-beta)    ! gamrav(j) !!!
        asterm= 1.0 + asym*deltath3/(ccas**1.5)
        drpf(nrp) = pears*asterm
     enddo LOOP_CONTEGGI
!
     ungrad = 1.0
       summ = 0.5*SUM((ttetaczero(2:nrp)-ttetaczero(1:nrp-1))*(drpf(2:nrp) + drpf(1:nrp-1)))*100.0
     if (kvolte.le.4) then
         if (summ.lt.99.5) then
             ref(j)%pk = ref(j)%pk + (100. - summ)
             if (ref(j)%pk > 20.0) ref(j)%pk = 20.0
         endif
         if (summ.gt.99.99) then
             ref(j)%pk = ref(j)%pk - 1.0
             if (ref(j)%pk < 3.0) ref(j)%pk = 3.0
         endif
     endif
   enddo LOOP_RIFLESSI
!
   end subroutine defrange_pears
#endif
END MODULE pearsonf
