#include <utils.h>
#include "grafico.h"

#define PowderPatternWindow "PowderPattern"

typedef struct {
    float   wave;  // = 1.54056
    float   ttmin;  // = 5
    float   ttmax;  // = 70
    float   ttstep;  // = 0.02
    float   fwhm;  // = 0.1
    float   scal;  // = 1000.
               } xpd_option_type;

static GtkWidget *Window = NULL;

gboolean ManageExpose (GtkWidget *widget, gpointer obj, gpointer data);
xpd_option_type xpdG = {1.54056f, 5, 70, 0.02f, 0.1f, 1000.0f};


namespace GraphicSpace 
{
static Grafico *graf = NULL;
}               // end namespace

extern "C" void jav_powder_pattern_(int *km, xpd_option_type *xpd);
extern "C" void jav_export_powder_pattern_(char *f1, int *tipo, size_t lung);

typedef struct {
                int h, k, l;
               }_HKL;

static int Xiniz = -1;
static int Yiniz = -1;
static int Wrect, Hrect;
static int Wm = 1;
static float *tthd = NULL;
static _HKL  *hkl = NULL;
static int nrefl = 0;
static std::string Filename(" ");
static GdkEventMask EventMask = (GdkEventMask)
                               (GDK_BUTTON_PRESS_MASK |
                                GDK_BUTTON_RELEASE_MASK |
                                GDK_BUTTON_MOTION_MASK |
                                GDK_POINTER_MOTION_MASK |
                                GDK_SCROLL_MASK );

static gboolean on_ConfigureEvent (GtkWidget *widget, GdkEventConfigure *,
                             gpointer data)
{
using namespace GraphicSpace;
    graf->Configure(widget);
    return TRUE;
}

void on_change_wavelen(GtkWidget *entry, gpointer user_data)
{
using namespace GraphicSpace;
     //xpdG.wave = (float) atof(gtk_entry_get_text(GTK_ENTRY(entry)));
}

void on_change_fwhm(GtkWidget *entry, gpointer user_data)
{
using namespace GraphicSpace;
     //xpdG.fwhm = (float) atof(gtk_entry_get_text(GTK_ENTRY(entry)));
}

void on_change_scal(GtkWidget *entry, gpointer user_data)
{
using namespace GraphicSpace;
     //xpdG.scal = (float) atof(gtk_entry_get_text(GTK_ENTRY(entry)));
}

void on_change_thetamin(GtkWidget *spin, gpointer user_data)
{
using namespace GraphicSpace;
     //xpdG.ttmin = (float) spin_button_get_value(spin);
}

void on_change_thetamax(GtkWidget *spin, gpointer user_data)
{
using namespace GraphicSpace;
     //xpdG.ttmax = (float) spin_button_get_value(spin);
}

void on_change_step(GtkWidget *spin, gpointer user_data)
{
using namespace GraphicSpace;
     //xpdG.ttstep = (float) spin_button_get_value(spin);
}

static void on_settings(GtkWidget *button, gpointer data)
{
using namespace GraphicSpace;
   GtkWidget *father = gtk_widget_get_toplevel(button);
   int response = GPOINTER_TO_INT(data);
   if(response != 0)
   {
      GtkWidget *w = get_widget(father, "wavelen");
      xpdG.wave   = (float) atof(gtk_entry_get_text(GTK_ENTRY(w)));
      w = get_widget(father, "thetamin");
      xpdG.ttmin  = (float) spin_button_get_value(w);
      w = get_widget(father, "thetamax");
      xpdG.ttmax  = (float) spin_button_get_value(w);
      w = get_widget(father, "step");
      xpdG.ttstep = (float) spin_button_get_value(w);
      w = get_widget(father, "fwhm");
      xpdG.fwhm   = (float) atof(gtk_entry_get_text(GTK_ENTRY(w)));
      w = get_widget(father, "scale");
      xpdG.scal   = (float) atof(gtk_entry_get_text(GTK_ENTRY(w)));
      jav_powder_pattern_(&Wm, &xpdG);
   }
   if(response <= 1)
      gtk_widget_destroy(father);
}

void PatternSettings(GtkWidget *but, gpointer data)
{
using namespace GraphicSpace;
   GtkWidget *father = gtk_widget_get_toplevel(but);
   
   GtkWidget *dlg = gtk_dialog_new ();
   gtk_window_set_title (GTK_WINDOW (dlg), "Powder Pattern Settings");
   gtk_container_set_border_width(GTK_CONTAINER(dlg), 8);
   GtkWidget *vbox1 = gtk_dialog_get_content_area(GTK_DIALOG(dlg));
   gtk_window_set_destroy_with_parent(GTK_WINDOW(dlg), TRUE);
   gtk_window_set_transient_for(GTK_WINDOW(dlg), GTK_WINDOW(father));
   gtk_container_set_border_width(GTK_CONTAINER(vbox1), 8);

   GtkWidget *table = table_new(5, 2, FALSE);
   gtk_box_pack_start(GTK_BOX(vbox1), table, FALSE, FALSE, 4);
   table_set_col_spacings(table, 12);
   table_set_row_spacings(table, 12);
   int k = 0;

   GtkWidget *label = LABEL_NEW("Wavelength", NULL);
   set_alignment(label, 0., 0.5);
   table_attach_exp(table, label, 0, 1, k, k+1);
   GtkWidget *spin = gtk_entry_new();
   g_object_set_data(G_OBJECT(dlg), "wavelen", spin);
   table_attach_exp(table, spin, 1, 2, k, k+1);
   set_width_chars(GTK_ENTRY(spin), 10);
   gtk_entry_set_text(GTK_ENTRY(spin), fToStr(xpdG.wave, 5).c_str());
   g_signal_connect(G_OBJECT(spin), "activate",
                   G_CALLBACK(on_change_wavelen), dlg);
   k++;

   label = LABEL_NEW("Two Thetamin", NULL);
   set_alignment(label, 0., 0.5);
   table_attach_exp(table, label, 0, 1, k, k+1);
   spin = SPIN_NEW(0.001f, 180, xpdG.ttmin, 0.05f, 3);
   g_object_set_data(G_OBJECT(dlg), "thetamin", spin);
   table_attach_exp(table, spin, 1, 2, k, k+1);
   set_width_chars(GTK_ENTRY(spin), 6);
   g_signal_connect(G_OBJECT(spin), "value-changed",
                   G_CALLBACK(on_change_thetamin), dlg);
   k++;

   label = LABEL_NEW("Two Thetamax", NULL);
   set_alignment(label, 0., 0.5);
   table_attach_exp(table, label, 0, 1, k, k+1);
   spin = SPIN_NEW(0.001f, 180, xpdG.ttmax, 0.05f, 3);
   g_object_set_data(G_OBJECT(dlg), "thetamax", spin);
   table_attach_exp(table, spin, 1, 2, k, k+1);
   set_width_chars(GTK_ENTRY(spin), 6);
   g_signal_connect(G_OBJECT(spin), "value-changed",
                   G_CALLBACK(on_change_thetamin), dlg);
   k++;

   label = LABEL_NEW("Step ", NULL);
   set_alignment(label, 0., 0.5);
   table_attach_exp(table, label, 0, 1, k, k+1);
   spin = SPIN_NEW(0.001f, 70, xpdG.ttstep, 0.05f, 3);
   g_object_set_data(G_OBJECT(dlg), "step", spin);
   table_attach_exp(table, spin, 1, 2, k, k+1);
   set_width_chars(GTK_ENTRY(spin), 6);
   g_signal_connect(G_OBJECT(spin), "value-changed",
                   G_CALLBACK(on_change_step), dlg);
   k++;

   label = LABEL_NEW("FWHM (2theta)", NULL);
   set_alignment(label, 0., 0.5);
   table_attach_exp(table, label, 0, 1, k, k+1);
   spin = gtk_entry_new();
   g_object_set_data(G_OBJECT(dlg), "fwhm", spin);
   table_attach_exp(table, spin, 1, 2, k, k+1);
   set_width_chars(GTK_ENTRY(spin), 10);
   gtk_entry_set_text(GTK_ENTRY(spin), fToStr(xpdG.fwhm, 5).c_str());
   g_signal_connect(G_OBJECT(spin), "activate",
                   G_CALLBACK(on_change_fwhm), dlg);

   k++;

   label = LABEL_NEW("Scale ", NULL);
   set_alignment(label, 0., 0.5);
   table_attach_exp(table, label, 0, 1, k, k+1);
   spin = gtk_entry_new();
   g_object_set_data(G_OBJECT(dlg), "scale", spin);
   table_attach_exp(table, spin, 1, 2, k, k+1);
   set_width_chars(GTK_ENTRY(spin), 10);
   gtk_entry_set_text(GTK_ENTRY(spin), fToStr(xpdG.scal, 5).c_str());
   g_signal_connect(G_OBJECT(spin), "activate",
                   G_CALLBACK(on_change_scal), dlg);

  GtkWidget *button = OK_button();
  gtk_dialog_add_action_widget(GTK_DIALOG(dlg), button, GTK_RESPONSE_OK);
  g_signal_connect(G_OBJECT(button), "clicked", G_CALLBACK(on_settings), GINT_TO_POINTER(1));
  button = APPLY_button();
  gtk_dialog_add_action_widget(GTK_DIALOG(dlg), button, GTK_RESPONSE_APPLY);
  g_signal_connect(G_OBJECT(button), "clicked", G_CALLBACK(on_settings), GINT_TO_POINTER(2));
  button = CLOSE_button();
  gtk_dialog_add_action_widget(GTK_DIALOG(dlg), button, GTK_RESPONSE_CANCEL);
  g_signal_connect(G_OBJECT(button), "clicked", G_CALLBACK(on_settings), GINT_TO_POINTER(0));
  g_signal_connect_swapped(G_OBJECT(dlg), "delete-event", G_CALLBACK(gtk_button_clicked), button);
  gtk_button_box_set_layout(GTK_BUTTON_BOX(gtk_widget_get_parent(button)),
      GTK_BUTTONBOX_SPREAD);
  gtk_widget_show_all(dlg);

}

void SavePattern(GtkWidget *but, gpointer data)
{
using namespace gtkext;
   GtkWidget *dlg = gtk_widget_get_toplevel(but);
   const char *filter = "ASCII powder pattern(*.xy)@Reflection list(*.hkl)";
   FilselItem local_it;
   local_it = saveas(dlg, filter, Filename.c_str(), false);
   if(local_it.exitCode > 0)
   {
       int  tipo = local_it.nfilt + 1;
       char *f1 = g_build_filename(local_it.cwd, local_it.fileinp, NULL);
       jav_export_powder_pattern_(f1, &tipo, strlen(f1));
       //g_print("file %s, tipo %d\n", f1, tipo);
   }
}

typedef struct {
                  float v;
                  int p;
               } vord;

bool sort_vect ( vord elem1, vord elem2 )
{
   float v1 = elem1.v;
   float v2 = elem2.v;
   return (v1 < v2);
}

int Myminloc(float *vect, float val)
{
    vector<vord> v1;
    
    if(nrefl <= 0)
       return -1;
    for(int i=0; i<nrefl; i++)
    {
        vord elem;
        elem.v = MRabs(vect[i]-val);
        elem.p = i;
        v1.push_back(elem);
    }
    sort(v1.begin(), v1.end(), sort_vect);
    int result = v1.at(0).p;
    v1.clear();
    return result;
}

std::string display_thetc(int x, int y)
{
using namespace GraphicSpace;
  _iPoint ip = {x, y};
  char *stringa;
  float due_theta, inte;

  _Point p = graf->FromScreen(ip);
  due_theta = p.x;
  inte = p.y;

  int k = Myminloc(tthd, due_theta);
  if (k >= 0)
      stringa = g_strdup_printf("2-Theta: %.4f      I: %.4f      h k l :%4d, %4d, %4d",
                            due_theta, inte, hkl[k].h,  hkl[k].k,  hkl[k].l);
  else
      stringa = g_strdup_printf("2-Theta: %.4f      I: %.4f",
                            due_theta, inte);
  return stringa;
}

static void on_reset_zoom(GtkWidget *button, GtkWidget *widget)
{
using namespace GraphicSpace;
     graf->ResetZoom(button, widget);
}

static gboolean mouse_release  (GtkWidget       *widget,
                                GdkEventButton  *event,
                                GtkWidget       *but)
{
using namespace GraphicSpace;
  graf->MouseRelease(widget, but);
  return FALSE; 
}

static gboolean mouse_move (GtkWidget *w, GdkEventMotion *event, gpointer data)
{
using namespace GraphicSpace;
  int x, y, butt, stato;
  x = (int) event->x;
  y = (int) event->y;
  stato = (int) event->state;
  butt = -1;

  if((stato & GDK_BUTTON3_MASK) == GDK_BUTTON3_MASK )
     butt = 3;
  else if ((stato & GDK_BUTTON1_MASK) == GDK_BUTTON1_MASK )
     butt = 1;

  if(butt < 0)
  {
     gtk_statusbar_push(GTK_STATUSBAR(data), 1, toUTF8(display_thetc(x, y).c_str()));
     return TRUE;
  }
  else
     graf->MouseMove(w, x, y);
  return FALSE;
}

static gboolean mouse_press (GtkWidget *widget, GdkEventButton  *ev, gpointer data)
{
using namespace GraphicSpace;
  graf->SetStartMove(true);
  return FALSE; 
}

GtkWidget *create_powder_pattern_dlg(GtkWidget *father)
{
  GtkWidget *dlg = gtk_dialog_new ();
  gtk_window_set_title (GTK_WINDOW (dlg), "Powder Diffraction Pattern");
  gtk_container_set_border_width(GTK_CONTAINER(dlg), 8);
  GtkWidget *vbox1 = gtk_dialog_get_content_area(GTK_DIALOG(dlg));
  gtk_window_set_destroy_with_parent(GTK_WINDOW(dlg), TRUE);
  gtk_window_set_transient_for(GTK_WINDOW(dlg), GTK_WINDOW(father));

  GtkWidget *viewer = gtk_drawing_area_new ();
  g_object_set_data(G_OBJECT(viewer), "offscreen", NULL);
  g_object_set_data(G_OBJECT(dlg), "drawarea", viewer);
  gtk_box_pack_start(GTK_BOX(vbox1), viewer, TRUE, TRUE, 2);

  gtk_widget_set_size_request(viewer, 550, 350);
  gtk_widget_add_events(viewer, EventMask);

  GtkWidget *statusbar = gtk_statusbar_new ();
  g_object_set_data (G_OBJECT (dlg), "statusbar", statusbar);
  gtk_box_pack_start(GTK_BOX(vbox1), statusbar, FALSE, FALSE, 2);

  (void) g_signal_connect (G_OBJECT (viewer), "configure_event",
                      G_CALLBACK (on_ConfigureEvent), NULL);
  (void) g_signal_connect (G_OBJECT (viewer), DRAW_EVENT,
                      G_CALLBACK (ManageExpose), NULL);
  (void) g_signal_connect (G_OBJECT (viewer), "motion_notify_event",
                      G_CALLBACK (mouse_move), statusbar);
  (void) g_signal_connect (G_OBJECT (viewer), "button_press_event",
                      G_CALLBACK (mouse_press), NULL);

  chiudi_dlg_data *st = g_new0(chiudi_dlg_data, 1);
  st->window = father;
  st->name = PowderPatternWindow;

  GtkWidget *button = myimage_button("  _Export", "save.png");
  gtk_dialog_add_action_widget(GTK_DIALOG(dlg), button, GTK_RESPONSE_ACCEPT);
  g_signal_connect(G_OBJECT(button), "clicked", G_CALLBACK(SavePattern), dlg);
  button = myimage_button("_Settings", "preferences.png");
  gtk_dialog_add_action_widget(GTK_DIALOG(dlg), button, 1);
  g_signal_connect(G_OBJECT(button), "clicked", G_CALLBACK(PatternSettings), dlg);
  button = RESET_button();
  gtk_dialog_add_action_widget(GTK_DIALOG(dlg), button, GTK_RESPONSE_APPLY);
  g_signal_connect(G_OBJECT(button), "clicked", G_CALLBACK(on_reset_zoom), viewer);
  gtk_widget_set_sensitive(button, FALSE);
  (void) g_signal_connect (G_OBJECT (viewer), "button_release_event",
                      G_CALLBACK (mouse_release), button);
  button = CLOSE_button();
  gtk_dialog_add_action_widget(GTK_DIALOG(dlg), button, GTK_RESPONSE_CANCEL);
  g_signal_connect(G_OBJECT(button), "clicked", G_CALLBACK(chiudi_dlg), (gpointer) st);
  g_signal_connect_swapped(G_OBJECT(dlg), "delete-event", G_CALLBACK(gtk_button_clicked), button);
  g_object_set_data(G_OBJECT(father), PowderPatternWindow, dlg);
  gtk_button_box_set_layout(GTK_BUTTON_BOX(gtk_widget_get_parent(button)),
      GTK_BUTTONBOX_SPREAD);

  gtk_widget_show_all(dlg);
  return dlg;
}

void OpenPowderPattern(GtkWidget *window)
{
   GtkWidget *dlg;
   if(g_object_get_data(G_OBJECT(window), PowderPatternWindow))
   {
      dlg = get_widget(window, PowderPatternWindow);
      gdk_window_raise(gtk_widget_get_window(dlg));
   }
   else
   {
      dlg = create_powder_pattern_dlg(window);
      g_object_set_data(G_OBJECT(window), PowderPatternWindow, dlg);
   }
}


void PowderPattern(GtkWidget *window, int whichModel, std::string fn)
{
using namespace GraphicSpace;
    Window = window;
    Wm = whichModel;
    Filename = fn;
    jav_powder_pattern_(&whichModel, &xpdG);
}

extern "C" void do_powder_pattern_(float x[], float y[], int *num,
                        reflection_type *refjav, int *num1)
{
using namespace GraphicSpace;
    if(graf != NULL)
       delete graf;
    graf = new Grafico();
    graf->SetLabels("2 theta", "Intensity");
    graf->AddPlot(x, y, *num, xpdG.ttmin, xpdG.ttmax);
    if(*num1)
    {
       if(tthd != NULL)
          g_free(tthd);
       if(hkl != NULL)
       {
          g_free(hkl);
       }
       tthd = g_new0(float, *num1);
       hkl  = g_new0(_HKL, *num1);
       for(int i=0; i<*num1; i++)
       {
           tthd[i] = refjav[i].tthd;
           hkl[i].h = refjav[i].hkl[0];
           hkl[i].k = refjav[i].hkl[1];
           hkl[i].l = refjav[i].hkl[2];
       }
       graf->AddPlot(tthd,NULL, *num1, xpdG.ttmin, xpdG.ttmax);
       nrefl = *num1;
    }
    OpenPowderPattern(Window);
    GtkWidget *dlg = get_widget(Window, PowderPatternWindow);
    GtkAllocation allocation;
    GtkWidget *wi = get_widget(dlg, "drawarea");
    gtk_widget_get_allocation(wi, &allocation);

    int w = (int)allocation.width;
    int h = (int)allocation.height;

    gpointer context = get_object(wi, "context");

    gpointer pixmap = get_object(wi, "offscreen");

    graf->SetArea(w, h);
    graf->SetContext(context, pixmap);
    graf->Aggiorna(wi);
}
