 MODULE progtype

!corr  integer, parameter  :: dblep = kind(1.0d0)
  integer, parameter  :: ALLOCA   = 1
  integer, parameter  :: DEALLOCA = 0

  type ref_limit_type
     real    :: ttmin     ! 2theta minimo usato
     real    :: ttmax     ! 2theta massimo usato
     integer :: nreftot   ! num. totale di riflessi
     integer :: nrefu     ! num. di riflessi usati
     integer :: nri       ! primo riflesso usato
     integer :: nrf       ! ultimo riflesso usato
     integer :: nctot     ! num. totale di conteggi
     integer :: ncu       ! num. di conteggi usati
     integer :: nci       ! primo conteggio usato
     integer :: ncf       ! ultimo conteggio usato
  end type ref_limit_type
  
  type refine_condition_type
     type(ref_limit_type) :: lim            ! info sui limiti di affinamento 
     integer              :: algo           ! tipo di algoritmo di l.sq.
     integer              :: typefun        ! tipo di funzione di profilo
     logical              :: auto_coefb     ! selezione automatica del numero di coeff. di backgr.?
     logical              :: auto_prof      ! affinamento automatico profilo
     logical              :: auto_stru      ! affinamento automatico struttura
     integer              :: ncauto         ! num. di cicli di affinamento automatico
     logical              :: winmess        ! messagistica attiva?
     integer              :: typewei        ! schema di pesaggio
     real                 :: eps            ! criterio di convergenza
     integer              :: maxcy          ! numero massimo di cicli di affinamento
     integer              :: raction = 0    ! 0=Rietveld 1=Le Bail 2=Pawley
     character(len=6)     :: sprint         ! V=verbose,S=small,C=correlation,E=st.dev.,M=lsq matrices,W=window per chi,B=statusbar
     logical              :: riding_model = .true.
     real                 :: adp_factor = 1.2
  end type refine_condition_type
  
  type refine_info_type
     logical :: refine_cell
     logical :: refine_structure
     logical :: refine_profile
     real    :: dw          ! Durbin Watson
     real    :: rp,rwp      ! Rp,Rwp
     integer :: npar
  end type refine_info_type

  type rparam_type
     real                  :: val  = 0  ! valore del parametro
     real                  :: sd = 0    ! sd
     integer               :: rcod = 0  ! posizione nella matrice jac. del parametro se affinato; 0 se non affinato
     character(len=10)     :: str = ' ' ! stringa associata al parametro
  end type rparam_type

  type deratom
     real, dimension(3)    :: co
     real                  :: b
     real                  :: occ
  end type deratom

  type, public                         :: param_type
    real                               :: val        !Value of parameter
    integer, dimension(:), allocatable :: pat        !Atom(s) associated to parameter
    integer                            :: typepar    !Type of parameter
    real                               :: low        !Low-limit value of parameter
    real                               :: high       !High-limit value of parameter
    integer                            :: bound      !=0 fixed boundaries  1= periodic boundaries
    real                               :: stp        !The step length
    character(len=20)                  :: str        !Stringa associata al parametro 
  end type

  type opti_info_type
     integer           :: npar      ! num. totale di parametri
     integer           :: dofi      ! num. di DOF interni
     integer           :: dofe      ! num. di DOF esterni
     integer           :: nprot     ! num. di rotazioni affinate
     integer           :: npb       ! numeri di parametri termici affinati
     integer           :: npshift   ! numero di paramtri di shift affinati  
     integer           :: nreftot   ! num. totale riflessi 
     integer           :: nref      ! num. di riflessi usati
     integer           :: nco       ! numero di conteggi usati
     real              :: ttmin     ! 2theta min
     real              :: ttmax     ! 2theta max
     real              :: res       ! risoluzione
     integer           :: cftype    ! tipo di funzione costo
     logical           :: doc       ! Dynamical occupancy correction   !!!unused variable
     real              :: rwp_leb   ! Rwp(Le Bail) 
     integer           :: nrun  = 0 ! Number of run
     integer           :: nrunc = 0 ! A quale run sono
     character(len=60) :: cost_function_name
     character(len=20) :: cost_function_name_short
     integer           :: seed      ! seme generatore random
     logical           :: autoalgo  ! selezione automatica del tipo di algo
     integer           :: algo      ! tipo di algoritmo di ottimizzazione
     real              :: sogcf     ! valore soglia per la funzione costo
     real              :: fmin      ! valore minimo della funzione costo
     integer           :: rmin      ! run corrispondente al valore minimo
     character(len=10) :: popt      ! Opzioni di output: !P = draw pattern su viewer principale
                                                         !M = draw molecule su viewer principale
                                                         !V = draw on secondary viewer
                                                         !E = export models on cif files
                                                         !S = stampa parametri
                                                         !B = manage status and progress bar
  end type

  integer, parameter :: NMAXSPECIE = 40  ! numero massimo di specie gestibili

  type structure_info_type
    integer                        :: nspec
    integer, dimension(NMAXSPECIE) :: zspec,vspec
    real, dimension(6)             :: cell
    character(len=16)              :: spaceg
    real                           :: cell_volume
    real                           :: volume_per_atom
    real                           :: r_factor
    integer                        :: icent
    integer                        :: nsymmstr
  end type structure_info_type

  type bond_setting_type
       real :: tolmin, tolmax ! legame se dmin-tolmin < d < dmax+tolmax
       real :: angle          ! legame se l'angolo < angle
  end type bond_setting_type
  type(bond_setting_type), parameter :: DEF_BONDSET = bond_setting_type(0.0,0.0,38)

!#define DISTRIB
#if defined(DISTRIB)
 logical, parameter :: DISTRIBUTED_VER = .true.
#else
 logical, parameter :: DISTRIBUTED_VER = .false.
#endif
 END MODULE progtype
