module qe_frm

!
! For input file description see: 
!                   http://www.quantum-espresso.org/wp-content/uploads/Doc/INPUT_PW.html
!

implicit none 

CONTAINS


   subroutine write_qe_file(filename,atom,cell,spg,structname)
   USE fileutil
   USE atom_type_util
   USE unit_cell
   USE spginfom
   USE elements
   character(len=*), intent(in)                           :: filename
   type(atom_type), dimension(:), allocatable, intent(in) :: atom
   type(cell_type), intent(in)                            :: cell
   type(spaceg_type), intent(in)                          :: spg
   character(len=*), intent(in)                           :: structname
   type(file_handle)                                      :: fnw
   integer                                                :: j_in
   type(element_type), dimension(:), allocatable          :: elem
   integer                                                :: ibrav,i
!
   call fnw%fopen(filename,'w')
   if (.not.fnw%good()) return
   j_in = fnw%handle()
!
   write(j_in,'(a)')'&CONTROL'
   write(j_in,'(4x,a)')"calculation='relax',"
   write(j_in,'(4x,a)')"prefix='"//trim(structname)//"',"
   write(j_in,'(4x,a)')"pseudo_dir='"//get_cwd()//get_separ()//"upf_files"//"',"
   write(j_in,'(4x,a)')"outdir='"//get_homepath()//get_separ()//"scratch"//"',"
   write(j_in,'(4x,a)')"nstep=20,"
   write(j_in,'(4x,a)')"etot_conv_thr=5.D-5,"
   write(j_in,'(4x,a)')"forc_conv_thr=5.D-4,"
   write(j_in,'(4x,a)')"verbosity='high'"
   write(j_in,'("/"/)')
!
   write(j_in,'(a)')'&SYSTEM'
   ibrav = 0
   select case(spg%csys_code)

     case (CS_Cubic)
       select case (spg%lattyp)
         case ('P')
           ibrav = 1
         case ('F')
           ibrav = 2
         case ('I')
           ibrav = 3
       end select
       write(j_in,'(4x,a,i0,a)')"ibrav=",ibrav,","
       write(j_in,'(4x,a,f10.4,a)')"a=",cell%get_a(),","

     case (CS_Trigonal,CS_Hexagonal)
       if (uc_is_hexagonal(cell%get_par(),0.01)) then        ! a=b; al=bet=90; gam=120
           ibrav = 4
           write(j_in,'(4x,a,i0,a)')"ibrav=",ibrav,","
           write(j_in,'(4x,a,f10.4,a)')"a=",cell%get_a(),","
           write(j_in,'(4x,a,f10.4,a)')"c=",cell%get_c(),","
       elseif (uc_is_rhombohedral(cell%get_par(),0.01)) then ! a=b=c; al=bet=gam/=90
           if (spg%lattyp == 'P') then
               ibrav = 5
           else
               ibrav = -5
           endif
           write(j_in,'(4x,a,i0,a)')"ibrav=",ibrav,","
           write(j_in,'(4x,a,f10.4,a)')"a=",cell%get_a(),","
           write(j_in,'(4x,a,f12.6,a)')"cosbc=",cos(dtor*cell%get_alpha()),","
       endif

     case (CS_Tetragonal)
       select case (spg%lattyp)
         case ('P')
           ibrav = 6
         case ('I')
           ibrav = 7
       end select
       write(j_in,'(4x,a,i0,a)')"ibrav=",ibrav,","
       write(j_in,'(4x,a,f10.4,a)')"a=",cell%get_a(),","
       write(j_in,'(4x,a,f10.4,a)')"c=",cell%get_c(),","

     case (CS_Orthorhombic)
       select case (spg%lattyp)
         case ('P')
           ibrav = 8
!corr?         case ('A')
!corr?           ibrav = -9
         case ('C')
           ibrav = 9
         case ('F')
           ibrav = 10
         case ('I')
           ibrav = 11
       end select
       write(j_in,'(4x,a,i0,a)')"ibrav=",ibrav,","
       write(j_in,'(4x,a,f10.4,a)')"a=",cell%get_a(),","
       write(j_in,'(4x,a,f10.4,a)')"b=",cell%get_b(),","
       write(j_in,'(4x,a,f10.4,a)')"c=",cell%get_c(),","

     case (CS_Monoclinic)
       if (axis_direction(spg) == 'b') then
           write(j_in,'(4x,a,i0,a)')"ibrav=",-12,","
           write(j_in,'(4x,a)')"uniqueb=.true.,"
           write(j_in,'(4x,a,f10.4,a)')"a=",cell%get_a(),","
           write(j_in,'(4x,a,f10.4,a)')"b=",cell%get_b(),","
           write(j_in,'(4x,a,f10.4,a)')"c=",cell%get_c(),","
           write(j_in,'(4x,a,f12.6,a)')"cosac=",cos(dtor*cell%get_beta()),","
       else 
           if (spg%ncoper == 0) then
               ibrav = 12
           else
               ibrav = 13
           endif
           write(j_in,'(4x,a,i0,a)')"ibrav=",ibrav,","
           write(j_in,'(4x,a)')"uniqueb=.false.,"
           write(j_in,'(4x,a,f10.4,a)')"a=",cell%get_a(),","
           write(j_in,'(4x,a,f10.4,a)')"b=",cell%get_b(),","
           write(j_in,'(4x,a,f10.4,a)')"c=",cell%get_c(),","
           write(j_in,'(4x,a,f12.6,a)')"cosab=",cos(dtor*cell%get_gamma()),","
       endif

     case (CS_Triclinic)
       write(j_in,'(4x,a,i0,a)')"ibrav=",14,","
       write(j_in,'(4x,a,f10.4,a)')"a=",cell%get_a(),","
       write(j_in,'(4x,a,f10.4,a)')"b=",cell%get_b(),","
       write(j_in,'(4x,a,f10.4,a)')"c=",cell%get_c(),","
       write(j_in,'(4x,a,f12.6,a)')"cosbc=",cos(dtor*cell%get_alpha()),","
       write(j_in,'(4x,a,f12.6,a)')"cosac=",cos(dtor*cell%get_beta()),","
       write(j_in,'(4x,a,f12.6,a)')"cosab=",cos(dtor*cell%get_gamma()),","
     
   end select
   call elements_from_atom(atom,elem)
   write(j_in,'(4x,a,i0,a)')"nat=",numatoms(atom),","
   write(j_in,'(4x,a,i0,a)')"ntyp=",numelem(elem),","
   write(j_in,'(4x,a)')     "ecutwfc = 40.0,"
   write(j_in,'(4x,a)')     "occupations = 'fixed',"
   write(j_in,'(4x,a,i0,a)')"space_group=",spg%num,","
   write(j_in,'(4x,a)')     "vdw_corr = 'grimme-d2',"
   write(j_in,'("/"/)')
!
   write(j_in,'(a)')'&ELECTRONS'
   write(j_in,'(4x,a)')"conv_thr=1.D-6,"
   write(j_in,'(4x,a)')"electron_maxstep = 100,"
   write(j_in,'("/"/)')
!
   write(j_in,'(a)')'&IONS'
   write(j_in,'(4x,a)')"ion_dynamics='bfgs',"
   write(j_in,'("/"/)')
!
   write(j_in,'(a)')'ATOMIC_SPECIES'
   do i=1,numelem(elem)
      write(j_in,'(4x,a,f10.4,4x,a)')eleminfo(elem(i)%z)%lab,elem(i)%weight,trim(eleminfo(elem(i)%z)%lab)//'.pbe-mt_fhi.UPF'
   enddo
!
   write(j_in,'(/a)')'ATOMIC_POSITIONS crystal_sg'
   do i=1,numatoms(atom)
      write(j_in,'(a5,3f12.5)')atom(i)%spec(),atom(i)%xc
   enddo
!
   write(j_in,'(/a)')'K_POINTS automatic'
   write(j_in,'(a)')' 4 4 2 1 1 1'
!
   end subroutine write_qe_file

!-----------------------------------------------------------------------------------------------

   subroutine read_qe_file(filename,atom,cell,spg,errc)
   USE unit_cell
   USE atom_type_util
   USE fileutil
   USE elements
   USE spginfom
   USE errormod
   USE strutil
   USE trig_constants
   USE prog_constants
   character(len=*), intent(in)                            :: filename
   type(atom_type), dimension(:), allocatable, intent(out) :: atom
   type(cell_type), intent(out)                            :: cell
   type(spaceg_type), intent(out)                          :: spg
   type(error_type), intent(out)                           :: errc
   type(file_handle)                                       :: fqe
   integer                                                 :: jout,ier
   character(len=:), allocatable                           :: line,line2
   real, dimension(6) :: rnum
   integer :: ibrav
   integer :: nat,iv
   logical :: okatoms
   integer, parameter :: start_size = 100
   real, dimension(3) :: vet
!
   call fqe%fopen(filename,'r')
   if (.not.fqe%good()) return
   jout = fqe%handle()
   okatoms = .false.
   call spg%set_p1()
   call new_atoms(atom,start_size)
!
   do while(get_line(jout,line,trimmed=.true.))
      if (index(line,'bravais-lattice index') > 0) then
          call get_next_number(line,'=',ibrav,ier)

      elseif (index(line,'celldm') > 0) then
          call parse_line_reals(line,[2,4,6],rnum(1:3),ier)
          if (ier /= 0) go to 10
          read(jout,'(a)')line
          call parse_line_reals(line,[2,4,6],rnum(4:6),ier)
          if (ier /= 0) go to 10
          select case (ibrav)
            case (12,-12,13)    ! Monoclinic P, unique axis c,b,base-centered
              rnum(2:3) = rnum(2:3) * rnum(1)
              rnum(1:3) = AU_TO_ANG*rnum(1:3)
              rnum(4:6) = rtod*acos(rnum(4:6))
              
          end select
          call cell%set(rnum)

      elseif (okatoms) then
          if (len(line) > 0) then
              call cutsta(line,line2=line2)
              !write(0,*)'SPEC=',line2,' AT=',line
              nat = nat + 1
              if (numatoms(atom) < nat) call resize_atoms(atom,numatoms(atom)+start_size) 
              atom(nat)%ptab = pxen_from_specie(line2)
              call getnum(line,vet,iv=iv)
              if (iv /= 3) go to 10
              atom(nat)%xc = vet
          else
              okatoms = .false.
          endif

      elseif (index(line,'ATOMIC_POSITIONS') > 0) then
          okatoms = .true.
          nat = 0

      endif
   enddo
   call resize_atoms(atom,nat)
      !call detect_spg(atom,cell)
   return

10 call errc%set('Error reading QE file: '//trim(filename)//char(10)//'Line: '//trim(line))

   end subroutine read_qe_file

!-----------------------------------------------------------------------------------------------

   logical function is_espresso_file(filename)
!
!  Check if the file is a CRYSTAL file
!
   USE fileutil
   character(len=*), intent(in) :: filename
   type(file_handle)            :: fnw
   character(len=200)           :: line
   integer                      :: ier
!
   is_espresso_file = .false.
   call fnw%fopen(filename,'r')
   if (.not.fnw%good()) return
!
   do while (.not.is_espresso_file)
     read(fnw%handle(),'(a)',iostat=ier)line
     if (ier /= 0) exit
     is_espresso_file = index(line,"Quantum ESPRESSO suite") > 0
   enddo 
   call fnw%fclose()
!
   end function is_espresso_file

end module qe_frm
