MODULE exploreset

USE progtype
USE model_util, only: model_type
USE timer

implicit none

type(model_type), dimension(:), allocatable :: modelset
integer, parameter :: MODEL_DONE = 5
integer, parameter :: MODEL_CFOM = 6

!
!  variabili per gestione grafica
integer :: kprorest   ! il tipo di procedura (RBM,Fourier,E-map)
integer :: lastsel    ! l'ultimo modello selezionato dalla lista
logical :: more_trials = .false.
!!!FIXME - aggiornare numset alla fine di all trials perche stampato da fsrbm 

type(timer_type), dimension(4), private :: timerst
integer, parameter, private             :: TIMERST_FOUR=1, TIMERST_RBM=2, TIMERST_TRIAL=3, TIMERST_TOT=4
character(len=*), private, parameter :: STRTAG='@@'

integer, parameter :: RBM_PROC=1, FOUR_PROC=2, EMAP_PROC=3, COVM_PROC=4, EDM_PROC=5, EDME_PROC=6
CONTAINS

   subroutine init_restart()
   USE model_util
!
   call clear_models(modelset)
   kprorest = 1
   lastsel = 0
!
   end subroutine init_restart

!-------------------------------------------------------------------------------------------------

   subroutine explore_trials(kprocedure,ivtrial,ncyclet,rord)
   USE variables, only:atm
   USE Atomi
   USE atom_type_util
   USE variables, only:ref,lconn
   USE General, only:knw,fname,OUT_FILE,cell,spacegr,celem,ERR_WINDOW,lo
   USE Molcom, only:ModelToCompare,jscreen
   USE compare_struct
   USE TestModel, only:vett_test
   USE atom_type_util
   USE molpnew
   USE drawatoms
   USE messagemod
   USE model_util
   USE unit_cell
   USE reflection_type_util
   USE strutil
   USE sinterface, only: update_model
   integer, intent(in)               :: kprocedure
   integer, dimension(:), intent(in) :: ivtrial ! =1 per i trials da explorare
   integer, intent(in)               :: ncyclet ! numero di cicli di fourier
   logical, intent(in)               :: rord
   integer                           :: i
   integer                           :: ntrial
   logical                           :: print_tb
   real                              :: timerun
   character(len=1)                  :: ap = '"'
   character(len=2)                  :: apv = '",'
   real, dimension(5)                :: infomodel
   real                              :: rf
   integer                           :: ipos
   real                              :: mtrov
   integer                           :: kevent
   integer, dimension(size(ivtrial)) :: veterr  
   integer                           :: nerrt
   character(len=:), allocatable     :: sterr
   integer                           :: nExitCode
   real(DP)                          :: timetot
!
   call timerst(TIMERST_FOUR)%set_type('Time for Fourier/RBM')
   call timerst(TIMERST_RBM)%set_type('Time for RBM')
   call timerst(TIMERST_TRIAL)%set_type('Time for trial')
   call timerst(TIMERST_TOT)%set_type('Total time')
   more_trials = .true.
   call timerst(TIMERST_TOT)%tic()
!
#if defined(DISTRIB)
   print_tb = .false.
#else
   print_tb = .true.
#endif
   ntrial = size(ivtrial)
   veterr(:) = 0
!
   if(print_tb)write(6,'(a)')STRTAG//'            NTRIAL      RF           trovati    dist'
   do i=1,ntrial
      if (ivtrial(i) == 1) then
          call timerst(TIMERST_TRIAL)%tic()
          call timerst(TIMERST_FOUR)%tic()
          call explore_single_trial(kprocedure,i,ncyclet,kevent,veterr(i))
          call timerst(TIMERST_FOUR)%toc()
          if (veterr(i) /= 0) cycle
!
          call trial_to_listset(i,ipos)
!
!         Tempo per ogni run
          call timerst(TIMERST_TRIAL)%toc()
          timerun = timerst(TIMERST_TRIAL)%get_interval()
!
!         Stampa informazioni
          rf = modelset(ipos)%info(2)
          modelset(ipos)%info(MODEL_DONE) = 1
          if (knw == 1 .or. ModelToCompare /= 0) then
              write(0,10)i,rf,secs_to_string(timerun),int(vett_test(3)),int(vett_test(1)),vett_test(4)
              write(6,10)i,rf,secs_to_string(timerun),int(vett_test(3)),int(vett_test(1)),vett_test(4)
          else
              write(0,20)i,rf,secs_to_string(timerun)
              write(6,20)i,rf,secs_to_string(timerun)
          endif
10        format('Set:',i0,'  RF=',f0.4,'   Time:',a,'  found:',i3,'/',i0,'  dist:',f0.4)
20        format('Set:',i0,'  RF=',f0.4,'   Time:',a)
!
          !ivtrial(i) = 0  ! azzera ivtrial per i trial gia' sviluppati
          if (kevent == 2) exit   ! evento di stop dalla grafica
!
      endif
!corr      call get_timestep(1,time_r)
   enddo
!
!  Stampa i tempi
   call timerst(TIMERST_TOT)%toc()
   call print_time(timerst,lo)
!
!  Ordina e stampa la lista dei modelli
   if (rord) call model_sort(modelset,keycode=2,keyadd=MODEL_DONE)
   if (print_tb) then
       call model_print_info(modelset,4,iveti=(/1,3/),mark=STRTAG)
   else
       call model_print_info(modelset,4,iveti=(/1,3/))
   endif
!
!  Seleziona il modello migliore 
   call model_get(modelset(1),atm,fom=infomodel)
   call create_connectivity(atm,lconn,set_cell_type(cell),spacegr,check=.true.)
!corr   nats = size(atm)
!corr   call fillatom(atm)
   call update_model(atm,lconn)
!
!  Stampa info sul modello migliore
   vett_test(3) = infomodel(3)
   vett_test(4) = infomodel(4)
   if (print_tb) then
       mtrov = sum(modelset(:)%info(3)) / nummodels(modelset)
       timetot = timerst(TIMERST_TOT)%get_time()
       write(6,"(a,'best ','found:',i0,'/',i0,' dist:',f0.4,' RF:',f0.6,'  time:',a,' media trov:',f0.2)")    &
                        STRTAG,int(vett_test(3)),int(vett_test(1)),vett_test(4), &
                         infomodel(2),secs_to_string(timetot),mtrov
       write(6,'(a,a,a,a,a,i0,a,a,f0.4,a,a,i0,a,i0,a,a,f0.3,a,a,a,a)')'TB2',     &
                       ap,trim(fname(OUT_FILE)(scan(fname(OUT_FILE),'/\',back=.true.)+1:)),apv,  &
                       ap,int(infomodel(1)),apv,                                     & 
                       ap,infomodel(2),apv,                                          & 
                       ap,int(vett_test(3)),'/',int(vett_test(1)),apv,               &
                       ap,vett_test(4),apv,                                          &
                       ap,secs_to_string(timetot),ap
              
   endif
   write(6,'(/a,i0,1x,a)')' The best model was found on trial n.',int(infomodel(1))
   call print_atoms(atm,title='Structure solution',write_occ=.true.)
!
!  Disegna il modello migliore
   if (jscreen > 0) then
       call draw_atoms(atm,lconn)
!
!      Check for error in restart
       nerrt = count(veterr /= 0) 
       if (nerrt > 0) then
           sterr = 'Error at trial'
           if (nerrt > 1) sterr = sterr//'s'
           do i=1,size(veterr)
              if (veterr(i) /= 0) then
                  sterr = sterr//' '//trim(i_to_s(i))
              endif
           enddo
           call MsgWin2(sterr,ERR_WINDOW,nExitCode,5000)
       endif
   endif
!
   if (knw == 1 .or. ModelToCompare /= 0) then
!corr       call sfcalc(ref,atm,spacegr,celem)
       call fcalcang(ref,atm,spacegr,celem)
       call to_coord_sir(0,atm,ref,size(ref),set_cell_type(cell),spacegr,lo)
       call write_message_found(4)
   endif
   more_trials = .false.
!
   end subroutine explore_trials

! -----------------------------------------------------------------

   subroutine explore_single_trial(kprocedure,ntrial,ncyclefou,kevent,ier)
   USE General, only: ERR_WINDOW,lo,celem
   USE comandiexpo
   USE variables
   USE atomi
   USE atom_type_util
   USE sinterface, only: fillatom
   implicit none
   common /graphic_restart/kgraph_restart,ncyc_restart,nset_restart
   integer, intent(in)  :: kprocedure
   integer, intent(in)  :: ntrial,ncyclefou
   integer, intent(out) :: kevent
   integer, intent(out) :: ier
   integer              :: kgraph_restart,ncyc_restart,nset_restart
!corr   integer              :: nExitCode
!
   write(lo,10) ntrial,ncyclefou
10 format(///,' ',78('+'),//,                         &
              ' Fourier for trial no.',i5,            &
              ' with ',i5,' cycles',//)
!
   ier=0
   kgraph_restart = 1
   ncyc_restart = ncyclefou
   nset_restart = ntrial
   icomat = icomat + 1
   call fourie(ier,kprocedure,kevent)
   if (kprocedure == EMAP_PROC) then
!
!      rimuovi ghost solo per kprocedure=3 (E-MAP)
       call xyz_to_atom(atm,xyzs(:3,:nats),bfac,occr=xyzs(5,:),occh=occpch,nz=nz,lab=aLab,inte=xyzs(4,:),elem=celem)
       !call remove_atoms_from_list(atm,atm%get_nz(),0)  ! rimuovi eventuali atomi con nz=0 (ghost) da e-map
       call delete_ghosts(atm) 
       nats = size(atm)
       call fillatom(atm)
       call frm15(nats)
   endif
   kgraph_restart = 0
   if (ier == 0)  then
       call lastplot()  
       icomat = icomat - 1
   else
      ! if (jscreen > 0) then
      !     call MsgWin2('Error in restart',ERR_WINDOW,nExitCode,5000)
      ! endif
       write(lo,'(1x,62("*"),/," *",22x,"Error in restart",22x,"*",/,1x,62("*"))')
   endif
!
   end subroutine explore_single_trial

!-------------------------------------------------------------------------------------------------

   subroutine trial_to_listset(nums,ipos)
   USE Atomi
   USE stat
   USE atom_type_util
   USE variables
   USE TestModel, only:vett_test
   USE compare_struct
   USE General, only:knw,spacegr,celem,cell,lo
   USE Dialogo, only:ModelToCompare
   USE model_util
   USE reflection_type_util
   USE unit_cell
   integer, intent(in)            :: nums   ! numero del set da aggiungere
   integer, optional, intent(out) :: ipos   ! in che posizione e' stato aggiunto
   real, dimension(5)             :: info
   integer                        :: pos
!
!  Calcola Fc e to_coord
   call xyz_to_atom(atm,xyzs(:3,:nats),bfac,occr=xyzs(5,:),occh=occpch,nz=nz,lab=aLab,inte=xyzs(4,:),elem=celem)
!corr   call sfcalc(ref,atm,spacegr,celem)
   call fcalcang(ref,atm,spacegr,celem)
   if (knw == 1 .or. ModelToCompare /= 0) then
       call to_coord_sir(0,atm,ref,size(ref),set_cell_type(cell),spacegr,lo)
   endif
!
!  Salva il modello nella lista
   info(1) = nums                   ! numero del set
   info(2) = Rvalue(ref%fo,ref%fc)  ! RF
   info(3) = vett_test(3)           ! trovati
   info(4) = vett_test(4)           ! distanza
   pos = trial_to_model(nums)
   if (pos == 0) pos = nums
   call model_set(modelset(pos),atm,fom=info(1:4))
   if (present(ipos)) ipos = pos
!
   end subroutine trial_to_listset

!-------------------------------------------------------------------------------------------------

   subroutine explore_all_trials()
   USE PhaseModule, only:nset,imove
   USE reflection_type_util
   USE strutil
   USE commandsmod
   USE exportfiles
   integer, dimension(:), allocatable :: ivtrial
   integer                            :: ncy,i
   integer                            :: ier
   integer                            :: posc
   integer                            :: kproc
!
   call restart_prelim(ier)
   if (ier == 0) then
       posc = command_position(commands,'alltrials')
       ncy =20 
       kproc = 0
       do i=1,commands(posc)%ndir             ! sostituito con loop per testare piu' di una direttiva
           select case (upper(commands(posc)%strdir(i)(1:3)))
              case ('EMA','DMA')
                !kproc = 3
                kproc = EMAP_PROC
                kprorest = 1
                ncy = 0
              case ('FOU')
                !kproc = 2
                kproc = FOUR_PROC
              case ('RBM')
                !kproc = 1
                kproc = RBM_PROC
              case ('SHI')
                imove = 1
           end select
       enddo
       if (is_directive(commands(posc),'edmo') > 0) then
           if (kproc == EMAP_PROC) then
               kproc = EDME_PROC  ! EDM+EMAP
           else
               kproc = EDM_PROC   ! EDM
           endif
       endif

       !endif
       allocate(ivtrial(nset))
       ivtrial(:) = 1
       call explore_trials(kproc,ivtrial,ncy,rord=.true.)
       call export_files()             ! aggiorna file esterno
   endif
!
   end subroutine explore_all_trials

! -----------------------------------------------------------------

   integer function trial_to_model(itrial) result(kmodel)
   USE model_util
   integer, intent(in) :: itrial
   !integer             :: i
!
   !kmodel = 0
   !do i=1,nummodels(modelset)
   !   if (modelset(i)%info(1) == itrial) then
   !       kmodel = i; exit
   !   endif
   !enddo
   kmodel = find_model(modelset,nummodels(modelset),1,itrial)
!
   end function trial_to_model

END MODULE exploreset

! ----------------------------------------------------------------------
   subroutine restart()
   USE PhaseModule
   USE exploreset
!
   implicit none
!
   integer                            :: ier
   integer                            :: kcycx
   integer                            :: selrow
   integer                            :: kproc
!
   call restart_prelim(ier)
   if (ier == 0) then
       kcycx=20                             ! numero di cicli di fourier
       selrow = 1                          ! il set migliore
       kproc = kprorest
       call open_restart(kproc, nset, int(modelset%info(1)), modelset%info(MODEL_CFOM),   &
            int(modelset%info(MODEL_DONE)), modelset%info(2), selrow)
   endif
!
   end subroutine restart

! -----------------------------------------------------------------

   subroutine restart_prelim(ier)
   USE Molcom
   USE General
   USE PhaseModule
   USE exploreset
   USE variables, only:ref
   USE proginterface
   USE Atomi, only:nats
   USE model_util
   implicit none
   integer, intent(out) :: ier
   real, dimension(302)               :: cfom
   integer, dimension(302)            :: ipos
   real, dimension(11)                :: fom
   real, dimension(2000)              :: ep
   integer, dimension(nrefl)          :: iphb
   integer                            :: i,j
   integer                            :: lsn,lfw,llw,lpb,ll16
   integer                            :: nlastset
!
   call srlin (16,lsn,lfw,llw,lpb,ll16)
   if (lfw.gt.0) then
       ier = 0
!
       call fillref(ref)
!
       call ysfl16(cfom,ipos,nset)  ! leggi il numero di set
!
!      Individua quali set sono stati eseguiti
       if (.not.allocated(modelset)) then
!
!          se la lista e vuota riempi infoset dalla lista 16
           call resize_models(modelset,nset)
           do j=1,nset
              i = ipos(j)
              call snr16(i,fom,ep,iphb)
              modelset(j)%info(MODEL_DONE) = 0
              modelset(j)%info(1) = j
              modelset(j)%info(MODEL_CFOM) = fom(5)
              modelset(j)%info(2) = 0
           enddo
!           
!          aggiungi l'ultimo modello se esiste, cioe' se ho fatto la direttiva fourier
           if (nats > 0) then
               call trial_to_listset(numset)
               modelset(numset)%info(MODEL_DONE) = 1
           endif
       else
          if (lastsel > 0) then
!              
!             aggiorna l'ultimo modello selezionato perche' potrebbe essere stato modificato
              nlastset = int(modelset(lastsel)%info(1))   ! prendi il set dell'ultima selezione
              call trial_to_listset(nlastset)                   ! aggiorna il set
          endif
       endif
!
   else
       call OpenWarning('Cannot restart. No trials stored')
       ier = -1
   endif
!
   end subroutine restart_prelim

! -----------------------------------------------------------------

   subroutine TrialSelected(selrow)
   USE exploreset
   USE Atomi
   USE molpnew
   USE variables
   USE atom_type_util
   USE drawatoms
   USE messagemod
   USE general, only:knw, cell, spacegr
   USE TestModel, only:vett_test
   USE strutil
   USE Molcom, only:ModelToCompare
   USE model_util
   USE exportfiles
   USE connect_mod
   USE Phasemodule, only: numset
   USE unit_cell
   USE sinterface, only: update_model
   implicit none
   integer            :: selrow
   real, dimension(5) :: infomodel
!
   if (modelset(selrow)%info(MODEL_DONE) == 1) then
       call model_get(modelset(selrow),atm,fom=infomodel)
!
!      Crea connettivita' e disegna
       call create_connectivity(atm,lconn,set_cell_type(cell),spacegr,check=.true.,kpr=0)
       call update_model(atm,lconn)
       call draw_atoms(atm,lconn)
       call bond_to_iconn(lconn,iconn) ! aggiorna iconn
       call export_files()             ! aggiorna file esterno
!
       call clear_messages(ipos=2)
       if (selrow == 1) then
           call write_message('The best structure (trial n.'//trim(i_to_s(int(infomodel(1))))//') was selected',pos=2)
       else
           call write_message('The structure of trial n.'//trim(i_to_s(int(infomodel(1))))//' was selected',pos=2)
       endif
       call write_message('RF = ',rnum=infomodel(2),pos=3)
       lastsel = selrow       ! memorizza l'ultimo modello selezionato
       numset = nint(infomodel(1))  ! aggiorna il puntatore al set corrente
       if (knw > 0 .or. ModelToCompare /= 0) then
           vett_test(3) = infomodel(3)           ! trovati
           vett_test(4) = infomodel(4)           ! distanza
           call write_message_found(4)
       endif
   endif
!
   end subroutine TrialSelected

! -----------------------------------------------------------------

   subroutine finishrestart(ivtrial, kproc, ivset, cfomvalue, ivdone, rfvalue, selset)
   USE PhaseModule, only:nset
   USE exploreset
   USE commandsmod
   implicit none
   integer, intent(in)                    :: kproc            
   integer,  dimension(nset), intent(out) :: ivtrial          ! righe con i trial da sviluppare
   integer,  dimension(nset), intent(out) :: ivset
   real, dimension(nset), intent(out)     :: cfomvalue
   integer, dimension(nset), intent(out)  :: ivdone           ! quali trial gia' sviluppati
   real, dimension(nset)                  :: rfvalue          ! RF per trial già sviluppati
   integer                                :: ncyclC       
   integer                                :: iset
   integer                                :: i
   integer                                :: selset
   integer                                :: ndevset
!
   select case (kproc)
       !case (1,2,4)    ! RBM,Fourier,RBM-CONN
       case (RBM_PROC,FOUR_PROC,COVM_PROC)    ! RBM,Fourier,RBM-CONN
          ncyclC = 20
       case (EMAP_PROC)        ! E-map
          ncyclC = 0
   end select 
!
   ivtrial(int(modelset(:)%info(1))) = ivtrial(:)     ! perche' ivtrial punta alle righe della gtklist
!
   if (is_directive_cmd(commands,'dmap')) then   ! non ordinare i modelli se attiva la direttiva dmap
       call explore_trials(kproc,ivtrial,ncyclC,rord=.false.)
   else
       call explore_trials(kproc,ivtrial,ncyclC,rord=.true.)
   endif
!
   do i=1,nset
      ivset(i) = nint(modelset(i)%info(1))
      ivdone(i) = nint(modelset(i)%info(MODEL_DONE))
      rfvalue(i) = modelset(i)%info(2)
      cfomvalue(i) = modelset(i)%info(MODEL_CFOM)
   enddo
!
!  Quale set selezionare sulla gtklist
   ndevset = count(ivtrial(:) == 1)      ! num di set. sviluppati
   if (ndevset == 1) then  ! un solo set sviluppato
       iset = 0
       do i=1,nset        ! cerca il set sviluppato
          if (ivtrial(i) == 1) then
              iset = i
              exit
          endif
       enddo
       do i=1,nset        ! seleziona il set sviluppato
          if (ivset(i) == iset) then
              selset = i
          endif
       enddo
       !write(0,*)'iset=',iset
   else                   ! piu' set sviluppati
       selset = 1         ! selezione sul set migliore
   endif
   ivtrial(:) = 0
!
   end subroutine finishrestart
