   MODULE RefinecomRef
   USE progtype 
   USE pearsonf
   USE psvoigtf
   USE tchzf
   USE background
!
   type(refine_condition_type) :: rfcond   
   type(refine_info_type)      :: rinfo
!
   type(pearson_function_type) :: pear
   type(pvoigt_function_type)  :: pvoi
   type(tch_function_type)     :: tch
!
!  zero-shift
   type(rparam_type)               :: zero
!
!  sample displacement and transparency
   type(rparam_type)               :: sdisp = rparam_type(str='sample di.')
   type(rparam_type)               :: stran = rparam_type(str='sample tr.')
!
!  scala
   type(rparam_type)               :: scalr
!
!  P.O.
   integer               :: kopcorr   ! 0=applico, 1=non applico la correzione
   integer, dimension(3) :: hklpo     ! h,k,l del piano
   type(rparam_type)     :: gpo       ! G
!
!  background
!corr   type(background_type) :: backrtv
   type(back_condition_type) :: backriet
!corr   integer                                 :: npointback         ! numero punti di background
!corr   type(point_type), dimension(:), allocatable  :: pointb        ! posizione punti di background
   type(rparam_type), dimension(NMAXCOEFB) :: coefbkg            ! coefficienti
!corr   real                                    :: thminbkg           ! zero del polinomio
!
!corr type(constraint_type), dimension(:), allocatable :: constrv  ! constraint del rietveld
!
!   real               :: rapporto_int = 1.0
   type(rparam_type), dimension(:), allocatable :: fopar
!
    integer :: nparfun
    integer :: ippar
   integer, parameter :: int4    = selected_int_kind(9)
!corr   real, parameter               :: epsmch = epsilon(epsmch)
!corr   real(dblep)      :: epsmchd
   real    :: lambdar
!
   real :: lebtol=1.0
   integer :: modleb = 1
   integer :: ncurrent_cycle         ! ciclo corrente di affinamento su piu' cicli
   integer :: kstager                ! step di affinamento corrente
   integer :: kstagemax              ! numero massimo di step di affinamento
   integer :: iFirstRietveld         ! =0 (non sono mai entrato in rietveld); =1 (sono gia' entrato in Rietveld)
   integer :: kabsor                 ! richiesta correzione di assorbimento
!
!  Tempo
   real :: start_time, end_time
!
!  Fattori di accordo
   real                :: errep1,errepw1     ! Rp e Rwp con background sottratto
   real                :: errexp             ! Re convenzionale
   real                :: errexp1            ! Re con background sottratto
   real                :: chi2               ! GOF convenzionale
!
   integer :: kalphar
!
   real, allocatable     :: derfopG (:)     ! derfopG(i,plane): der. funz. di Doll. rispetto a G
!
!  Correzione di assorbimento
   real, dimension(7) :: abscoef   ! coefficienti della formula per calcolare la correzione
!
   character(80), dimension(30) :: pgstring
!
!corr   real,    allocatable :: phkl(:,:)
!
!  Variabili sui riflessi
!corr   real, allocatable    :: betariet (:) ! betariet (i): beta
!corr   real, allocatable    :: rapIcalc (:)   ! rapIcalc (i,nph)    : rapporto di intensita'
   integer, allocatable :: noverl(:)        ! codice che indica la sovrapposizione
   real, allocatable    :: delta(:)
   END MODULE RefinecomRef

! -------------------------------------------------------------------------

   MODULE assorbimento
!
!  Coefficienti per interpolazione della correzione di assorbimento
!
!  cabs = costanti della formula di Dwiggins
!  cinv = elementi della matrice inversa * 3
!
!                                                                                         Coefficienti per
   real, parameter, dimension(7,7) :: cabs = reshape(                               &
   (/1.697652726,-0.057832,-0.014737, 0.0052399,-0.0040958, 13.178e-4,-14.500e-5,   & !theta=0  gradi: cabs(1:7,1)
     1.697652726,-0.081900,-0.019551, 0.0012934,-0.0028349, 12.731e-4,-14.846e-5,   & !theta=15 gradi: cabs(1:7,2)
     1.697652726,-0.15651 ,-0.022883,-0.012301 , 0.0096249,-19.881e-4, 14.414e-5,   & !theta=30 gradi: cabs(1:7,3)
     1.697652726,-0.27048 ,-0.027345, 0.006844 , 0.007503 ,-30.211e-4, 34.222e-5,   & !theta=45 gradi: cabs(1:7,4)
     1.697652726,-0.40317 ,-0.008807, 0.040689 ,-0.011295 , 9.4468e-4, 3.1492e-5,   & !theta=60 gradi: cabs(1:7,5)
     1.697652726,-0.51497 , 0.026637, 0.061371 ,-0.029397 , 60.356e-4,-49.206e-5,   & !theta=75 gradi: cabs(1:7,6)
     1.697652726,-0.55837 , 0.041420, 0.068963 ,-0.036556 , 80.965e-4,-70.573e-5/), & !theta=90 gradi: cabs(1:7,7)
                                          (/7,7/), order=(/1,2/))
!
   real, parameter, dimension(7,7) :: cinv = reshape(                               &
   (/3.,-73.        , 518.       ,-1600.      , 2432.      ,-1792.      , 512.,     & !theta=0  gradi: cinv(1:7,1)
     0., 89.56921938,-842.4101615, 2889.948452,-4628.512517, 3515.405007,-1024.,    & !theta=15 gradi: cinv(1:7,2)
     0.,-24.        , 488.       ,-2192.      , 4032.      ,-3328.      , 1024.,    & !theta=30 gradi: cinv(1:7,3)
     0., 12.        ,-268.       , 1536.      ,-3328.      , 3072.      ,-1024.,    & !theta=45 gradi: cinv(1:7,4)
     0.,-8.         , 184.       ,-1136.      , 2752.      ,-2816.      , 1024.,    & !theta=60 gradi: cinv(1:7,5)
     0., 6.430780618,-149.5898385, 950.0515478,-2411.487483, 2628.594993,-1024.,    & !theta=75 gradi: cinv(1:7,6)
     0.,-3.         , 70.        ,-448.       , 1152.      ,-1280.      , 512. /),  & !theta=90 gradi: cinv(1:7,7)
                                          (/7,7/), order=(/1,2/))
   END MODULE assorbimento
! ----------------------------------------------------------------------
   MODULE CommonExpo
      common /searchfour/ isearchfo
      common/anna1/clp(3000),inizc(3000),ifinc(3000),sumobs,sumobsnobac
      common /step/ thet1(10),thet2(10),ncoun1(10),ncoun2(10)   &
      ,nref1(10),nref2(10),ymx(10),nnt                          &
      ,thet11(10),thet22(10),ncoun11(10),ncoun22(10)            &
      ,nref11(10),nref22(10),sscl(10),scala(10)
      common /picbkg/ backp(10000,2),ntyp(10000),bkk(11,10),tmm(10)  &
      ,nback, ifcamb(10),noback
   END MODULE CommonExpo

