MODULE sinterface

implicit none

CONTAINS 

   subroutine fillatom(atom)
   USE Atomi
   USE atom_basic
   type(atom_type), dimension(:), intent(in) :: atom
   integer                                   :: i,j
!
   do i=1,nats
      do j=1,3
         xyzs(j,i) = atom(i)%xc(j)
      enddo
      nz(i) = atom(i)%get_nz()
      xyzs(4,i) = atom(i)%inte
      xyzs(5,i) = atom(i)%ocry
      occpch(i) = atom(i)%och
      bfac(i) = atom(i)%biso
      aLab(i) = atom(i)%lab
   enddo
!
   end subroutine fillatom

!--------------------------------------------------------------------------------------

   subroutine atom_to_xyzs(atom,legm)
!
!  Copia atom e legm in variabili xyzs e iconn
!
   USE Atomi
   USE connect_mod
   type(atom_type), dimension(:), intent(in)                        :: atom
   type(bond_type), dimension(:), intent(in), allocatable, optional :: legm
   integer                                                          :: i,j
!
   nats = size(atom)
   if (nats > MaxAtoms) then
!corr       write(0,*)'Too many atoms!'
       return
   endif
   do i=1,nats
      do j=1,3
         xyzs(j,i) = atom(i)%xc(j)
      enddo
      nz(i) = atom(i)%get_nz()
      xyzs(4,i) = atom(i)%inte
      xyzs(5,i) = atom(i)%ocry
      occpch(i) = atom(i)%och
      bfac(i) = atom(i)%biso                                                                    
      aLab(i) = atom(i)%lab
   enddo
!
   if (present(legm)) call bond_to_iconn(legm,iconn)
!
   end subroutine atom_to_xyzs

!--------------------------------------------------------------------------------------
   
   subroutine xyzs_to_atom(atom)
   USE Atomi
   USE atom_type_util
   USE General, only: celem
   type(atom_type), intent(inout), dimension(:), allocatable :: atom
   call xyz_to_atom(atom,xyzs(:3,:nats),bfac,occr=xyzs(5,:),occh=occpch,nz=nz,lab=aLab,inte=xyzs(4,:),elem=celem)
   end subroutine xyzs_to_atom

!--------------------------------------------------------------------------------------

   subroutine update_model(atom,legm) 
   USE variables
   USE atom_type_util
   USE connect_mod
   type(atom_type), dimension(:), allocatable, intent(in) :: atom
   type(bond_type), dimension(:), allocatable, intent(in) :: legm
!
!  aggiorna xyzs e iconn
   if (numatoms(atom) > 0) then
       call atom_to_xyzs(atom,legm)
!
!      aggiorna atm
       call copy_atoms(atm,atom)
!
!      aggiorna lconn
       call copy_bonds(lconn,legm)
!
!      Update crystal phase 1
       call cryst(1)%set_atoms(atom)
       call cryst(1)%set_bonds(legm)
   endif
!
   end subroutine update_model

!--------------------------------------------------------------------------------------

   subroutine crystal_to_expo(cryst)
   USE general, only: spacegr,cell,celem
   USE crystal_phase
   USE variables, only: ref,atm,lconn
   USE proginterface, only: ref_to_expo
   type(crystal_phase_t), intent(in) :: cryst
!
   if (cryst%is_spg()) then
       spacegr = cryst%spg
       call set_spaceg_new(spacegr)
   endif
   cell(:6) = cryst%get_cellpar()
   call cryst%get_scattering(celem)
!
   call cryst%get_reflections(ref)
   call ref_to_expo(ref)
!
   call cryst%get_atoms(atm)
   call cryst%get_bonds(lconn)
!
   end subroutine crystal_to_expo

!--------------------------------------------------------------------------------------

   subroutine update_atm()
   USE Atomi
   USE variables
   USE atom_type_util
   USE connect_mod
   USE General, only: cell,celem
   USE unit_cell
   type(cell_type) :: cellt
!
   if (nats > 0) then
!
!      aggiorna atm
       call xyz_to_atom(atm,xyzs(:3,:nats),bfac,occr=xyzs(5,:),occh=occpch,nz=nz,lab=aLab,inte=xyzs(4,:),elem=celem)
!
!      aggiorna lconn
       cellt = set_cell_type(cell)
       call iconn_to_legm(iconn,lconn,atm,cellt%get_g())
!
!      Update crystal phase 1
       call cryst(1)%set_atoms(atm)
       call cryst(1)%set_bonds(lconn)
   endif
!
   end subroutine update_atm

!--------------------------------------------------------------------------------------

   function spaceg_string() result(space)
   USE Symmetry
   USE strutil
   character(len=16) :: space
   write(space,'(16a1)')nSpaceG
   space(1:1) = upper(space(1:1))
   end function spaceg_string

!--------------------------------------------------------------------------------------

   logical function is_cell_defined_list()
   USE General, only:cell, InfoOK
!
!  InfoOK(1) == 1 significa che e' stata creata la lista 1
   is_cell_defined_list = all(cell(1:6) > 0.0) .and. InfoOK(1) == 1
!
   end function is_cell_defined_list

!--------------------------------------------------------------------------------------

   logical function is_space_defined_list()
   USE strutil
   USE General, only:InfoOK
   character(len=16) :: spg
   spg = spaceg_string()
   call s_filter(spg)
!
!  InfoOK(1) == 1 significa che e' stata creata la lista 2
   is_space_defined_list = len_trim(spg) /= 0 .and. InfoOK(2) == 1
   end function is_space_defined_list

!--------------------------------------------------------------------------------------

   subroutine set_cell_as_defined()
   USE General, only:cell, InfoOK
   if (all(cell(1:6) > 0.0)) InfoOK(1) = 1
   end subroutine set_cell_as_defined

!--------------------------------------------------------------------------------------

   subroutine set_space_as_defined()
   USE General, only:InfoOK
   if (len_trim(spaceg_string()) /= 0) InfoOK(2) = 1
   end subroutine set_space_as_defined

!--------------------------------------------------------------------------------------

   logical function is_cell_defined()
   USE General, only:cell
!
   is_cell_defined = all(cell(1:6) > 0.0)
!
   end function is_cell_defined

!--------------------------------------------------------------------------------------

   logical function is_space_defined()
   USE strutil
   character(len=16) :: spg
   spg = spaceg_string()
   call s_filter(spg)
   is_space_defined = len_trim(spg) /= 0 
   end function is_space_defined

!--------------------------------------------------------------------------------------

   subroutine create_cell()
   USE General, only: cell
   real, dimension(6) :: cella
   cella(1:3) = 10.0
   cella(4:6) = 90
   cell(:6) = cella
!corr   call set_cell(cella)
   end subroutine create_cell

!--------------------------------------------------------------------------------------

   subroutine set_spaceg(space,ierspace)
   USE Symmetry
   USE General, only: spacegr
   USE spginfom
   USE variables, only: cryst
   character(len=*), intent(in)   :: space
   integer, intent(out), optional :: ierspace
   integer                        :: ishif
   integer, dimension(3)          :: kshif
   character(len=16)              :: spaceg
   integer                        :: ier
!
   ier = 0
   if (len_trim(space) > 0) then
       spaceg(:) = space
       call StandardSpg(spaceg,ier)
       if (ier == 0) then
           ishif = 0
           kshif(:) = 0
           call insym(ishif,kshif,ier)
           spacegr = init_spaceg_type(space) ! set space group in structure spacegr
           call cryst(1)%set_spg(spacegr)
           call xspinb()  
       endif
   endif
       !write(0,*)'NSYM=',nsym,nsymtot,latt,jsys
   if (present(ierspace)) ierspace = ier
!
   end subroutine set_spaceg

!--------------------------------------------------------------------------------------

   subroutine set_spaceg_new(spg)
   USE spginfom
   USE Symmetry
   USE General, only:cell, spacegr
   USE reflection_type_util
   USE variables, only: cryst
   type(spaceg_type), intent(in) :: spg
   common /ori/ ntra,taboTC(8,3)
   integer :: ntra
   real :: taboTC
!corr   integer                       :: i,j
!
   spacegr = spg   ! save spacegroup in a global variable
   call cryst(1)%set_spg(spacegr)
!
!  Symbol
   if (len_trim(spg%symbol_xhm) > 0) then
       read(spg%symbol_xhm,'(16a1)') nSpaceG
   elseif (len_trim(spg%symbol_alt(1)) > 0) then
       read(spg%symbol_alt(1),'(16a1)') nSpaceG
   endif
!
!  Numero di operatori e centratura
!corr   nsymTot = spg%nsymop
!corr   totop = spg%nsymop
   pts = spg%ncoper
!corr   ncentr = spg%ncoper
!
   call get_symm_from_spg(spg,cell,kmat,tmat,jsys,latt,nsym,icent)
!
!corr   nSymmStrings = spg%nsymop
!   if (icent == 1) call CheckCentro(nsym)    ! serve????? contiene la xspinb che serve
   if (nsym <= 24) then  ! se nsym > 24 (es. F -4 2 3) supero la dim di ismat/tsmat
       call musym()    ! ismat, tsmat 
       call Preoper()  ! serve
   else  ! dare un warning
   endif
!
!  Direzione polare
!corr   kpol = 0
!corr   vpol(:) = 0
!corr   if (spg%polarx) vpol(1) = 1
!corr   if (spg%polary) vpol(2) = 1
!corr   if (spg%polarz) vpol(3) = 1
!corr   kpol=100*vpol(1)+10*vpol(2)+vpol(3)
!!!
!FIXME: set seminvariants: jsvet, isvet
!!!
!
!  Origini permesse
   call get_origini_spg(spg)  !!!FIXME jsvet
   call CheckRhombAx(nSpaceG, cell, ifRhombAx)
   jspini = -1
   !corrcall xspinb()
   call get_origini_spg(spacegr)
   taboTC(:,:) = tabo  ! copy tabo in taboTC per vecchia to_coord
      !write(0,*)'lattice=',trim(spg%lattyp),latt
!!!???   if3den = ?????
!!!???   call calmod(ngc,nbr,ncs,sem,nse)
!
   end subroutine set_spaceg_new

!--------------------------------------------------------------------------------------

   integer function get_kpol(spg)
   USE spginfom
   type(spaceg_type), intent(in) :: spg
   get_kpol = 0
   if (spg%polarx) get_kpol = 100
   if (spg%polary) get_kpol = get_kpol + 10
   if (spg%polarz) get_kpol = get_kpol + 1
   end function get_kpol

!--------------------------------------------------------------------------------------

   subroutine get_origini_spg(spg)
   USE spginfom
   USE Symmetry
   implicit none
   type(spaceg_type), intent(in) :: spg
   integer                       :: i,j
   integer                       :: kpol
!
!  Origini permesse
!corr   OkOrig = 1
   nOrigRoxi = spg%norig_ind
          !write(0,*)'norig=',nOrigRoxi,spg%norig
   do i=1,nOrigRoxi
      tabo(i,:) = spg%orig_ind(:,i)
      if(spg%polarx .and. spg%polary .and. spg%polarz) then
          tabo(i,:3) = 128
      else if( spg%polarx .and. spg%polary)    then
          tabo(i,1) = 128
          tabo(i,2) = 128
      else if( spg%polarx .and. spg%polarz)    then
          tabo(i,1) = 128
          tabo(i,3) = 128
      else if( spg%polary .and. spg%polarz)    then
          tabo(i,2) = 128
          tabo(i,3) = 128
      else if( spg%polarx)    then
          tabo(i,1) = 256
      else if( spg%polary)    then
          tabo(i,2) = 256
      else if( spg%polarz)    then
          if (jsys == 5 .and. latt == 1) then
              tabo(i,3) = 128
          else
              tabo(i,3) = 256 ! Primitive (latt=1) .and. Hexagonal (jsys=5)
          endif
      end if
   enddo
!
!  per xyzmax
   xyzmax(:) = -9999
   do i=1,nOrigRoxi
      do j=1,3
         if (tabo(i,j) > xyzmax(j)) then
             xyzmax(j) = tabo(i,j)
         endif
      end do
   end do
   do j=1,3
      if (xyzmax(j) < 0.6) then
          xyzmax(j) = 2.0
      elseif (xyzmax(j) < 0.7) then
          xyzmax(j) = 32.0   !  caso 1/3, 2/3 
      elseif (xyzmax(j) < 0.9) then
          xyzmax(j) = 32.0   !  caso 1/6, 5/6 
      elseif (xyzmax(j) < 1.0) then
          xyzmax(j) = 4.0
      endif
   end do
!
!  if3den (code from OriginiRoxi)
   if3den = 0
   kpol = get_kpol(spg)
   if (icent == 1) then
       if (latt == 7 .and. jsys == 5) then  ! Rhombohedrical (latt=7) and Hexagonal (jsys=5)
           if3den = 1
       endif
   else
       if (latt == 1 .and. jsys == 5) then  ! Primitive (latt=1) .and. Hexagonal (jsys=5)
           if (latt == 1) then
               if (kpol == 1) then        ! Floating origin along z
                   if3den = 1 
               else
                   if (jsvet(1) == 128) then
                       if3den = 1 
                   endif
               endif
           endif
       endif
   endif
   end subroutine get_origini_spg

!--------------------------------------------------------------------------------------
 
   subroutine set_cell_and_spg(crystal,modif)
   USE crystal_phase 
   USE General, only:alambda
   USE PatternRef, only:thmin,thmax
   USE strutil
   USE spginfom
   USE atom_type_util
   USE view
   USE unit_cell
   USE errormod
   type(crystal_phase_t), intent(inout) :: crystal
   logical, intent(out) :: modif
   character(18)      :: spacecharnew, spacechar
   real, dimension(6) :: cellnew
   integer            :: kreturn   !!!!, ier
   type(cell_type) :: cellt
   type(error_type) :: err
!
!
!  Create cell if doesn't exist (possible in case of File > Import Pattern > SA)
   if (crystal%is_cell()) then
       cellnew(:) = crystal%get_cellpar()
       cellt = set_cell_type(cellnew) ! save old cell in cellt
   else
       cellnew(:) = cellt%get_par()
   endif
!
!  Create s.g. if doesn't exist  
   if (crystal%is_spg()) then
       spacechar = crystal%spg%symbol_xhm
   else
       spacechar = 'P 1'
   endif
!
   spacecharnew = spacechar
   call cell_and_spg_dlg(cellnew, spacecharnew, kreturn)                             
   if (kreturn == 1) then   ! OK pressed
       modif = .false.
       if (any(abs(cellt%get_par() - cellnew(:)) > epsilon(1.0))) then
           modif = .true.
           call crystal%set_cell(cellnew)
           if (crystal%natoms() > 0) then
               call coord_in_newcell(crystal%at,cellt,crystal%cell)
           endif
           write(0,*) 'cell modificata: new',crystal%get_cellpar(),crystal%natoms()
           !write(0,*) 'cell modificata: old',cellt%get_par(),crystal%natoms()
       endif
!
       if (.not.s_eqidb(spacechar,spacecharnew)) then
           modif = .true.
           call crystal%set_spg(spacecharnew,err)
           if (err%signal) call err%print()
           !write(0,*) 'g.s. modificato'
       endif
       if (modif) then
           call crystal%make_reflections(thmin,thmax,alambda)
           call crystal_to_expo(crystal)
       endif
   else
       if (.not.crystal%is_spg()) then ! don't leave cell undefined
           call crystal%set_spg(spacecharnew,err)
           if (err%signal) call err%print()
       endif
   endif
!
   end subroutine set_cell_and_spg

END MODULE sinterface
