MODULE ssmoothing  

implicit none  

CONTAINS

   function savgol_smooth(xdata,nleft,nright,oder,opol)  result(xs)
   USE nr
!
   real, dimension(:)              :: xdata  ! dati su cui applicare lo smoothing
   integer                         :: nleft  ! numero di punti a sinistra
   integer                         :: nright ! numero di punti a destra
   integer                         :: oder   ! ordine della derivata (=0 per smoothing della funzione)
   integer                         :: opol   ! ordine della polinomiale
   real, dimension(size(xdata))    :: xs     ! smoothing
   real, dimension(nleft+nright+1) :: coef   ! coefficienti di Savinsky-Golay
   real, dimension(-nleft:nright)  :: cout
   integer                         :: npunti   
   integer                         :: inip,finp
   integer                         :: i,j
!
!  calcola i coefficienti
   coef = savgol(nleft,nright,oder,opol)
!
!  riordina i coefficienti
   cout(-nleft:0) = coef(nleft+1:1:-1)
   cout(1:nright) = coef(nleft+nright+1:nleft+2:-1)
!
   npunti = size(xdata)   ! num. di punti
!
!  Punti iniziali. Andamento costante prima del punto n.1
   do i=1,nleft
      xs(i) = xdata(1)*sum(cout(-nleft:-i)) + sum(cout(-i+1:nright)*xdata(1:i+nright))
   enddo
!
!  Punti centrali
   inip = nleft + 1
   finp = npunti - nright
   do i=inip,finp
      xs(i) = sum(cout(-nleft:nright)*xdata(i-nleft:i+nright))
   enddo
!
!  Punti finali. Andamento costante dopo l'ultimo punto.
   do i=finp+1,npunti
      j = npunti - i + 1
      xs(i) = xdata(npunti)*sum(cout(j:nright)) + sum(cout(-nleft:j-1)*xdata(i-nleft:npunti))
   enddo
!
   end function savgol_smooth
   
  !-----------------------------------------------------------------------------------

   function averagesmooth(datavet,npoints)
!
!  Esegue uno smoothing su datavet usando la tecnica della media
!
   implicit none
   real, dimension(:), intent(in) :: datavet ! vettore in ingresso
   integer, intent(in)            :: npoints ! 2*npoints + 1 e' il numero di punti mediati
   real, dimension(size(datavet)) :: averagesmooth
   integer                        :: punti
   integer                        :: i
   real                           :: const
!
   punti = size(datavet)
   const = 1./(2.*npoints+1)
!
!  Punti all' estrema sinistra; si aggiungono npoints punti uguali all' estremo inferiore
!
   do i=1,npoints
      averagesmooth(i) = const*((npoints-i+1)*datavet(1)+sum(datavet(1:i+npoints)))
   enddo
   !averagesmooth(1) = datavet(1)
   !do i=2,npoints
   !   !averagesmooth(i) = const*((npoints-i+1)*datavet(1)+sum(datavet(1:i+npoints)))
   !   averagesmooth(i) = sum(datavet(1:2*i-1))/(2*i-1)
   !enddo
   !    write(0,*)'punto n.1',averagesmooth(1:npoints)
!
!  Punti centrali
!
   do i=npoints+1,punti-npoints
      averagesmooth(i) = const*(sum(datavet(i-npoints:i+npoints)))
   enddo
!
!  Punti all'estrema destra; si aggiungono npoints punti uguali all' estremo superiore
!
   do i=punti-npoints+1,punti
      averagesmooth(i) = const*((npoints+i-punti)*datavet(punti)+sum(datavet(i-npoints:punti)))
   enddo
!
   end function averagesmooth

END MODULE SSMOOTHING
