 MODULE tchzf
   USE trig_constants
   USE progtype, only: rparam_type
   implicit none
  
   type tch_function_type
      type(rparam_type) :: u,v,w,z
      type(rparam_type) :: x,y
      type(rparam_type) :: asym
   end type tch_function_type

   real, parameter, private :: CONSTG1 = 4.0*alog(2.0)
   real, parameter, private :: CONSTG2 = sqrt(CONSTG1/pi)
   real, parameter, private :: CONSTL  = 2.0/pi

   real, dimension(:), allocatable, private :: etav

   private :: init_calc

 CONTAINS 
   
   subroutine init_calc(n)
   USE arrayutil
   integer, intent(in)          :: n
!
   if (allocated(etav) .and. n == size(etav)) return
   call new_array(etav,n)
!
   end subroutine init_calc

!----------------------------------------------------------------------------------------------------

   function init_tchz_param()  result(param)
   USE progtype, only: rparam_type
   type(tch_function_type) :: param

   param%u = rparam_type(val=0.0,str='U')
   param%v = rparam_type(val=0.0,str='V')
   param%w = rparam_type(val=0.01,str='W')

   param%x = rparam_type(val=0.01,str='X')
   param%y = rparam_type(val=0.0,str='Y')
   param%z = rparam_type(val=0.0,str='Z')
   param%asym = rparam_type(val=0.0,str='asym')

   end function init_tchz_param

!------------------------------------------------------------------------------------

   subroutine psvoigt_tchz(par,ref,xcount,fs,scal,ycount,nwave,rapporto_int,ier)
   USE reflection_type_util, only:reflection_type
   USE arrayutil
   USE profile_function
!
   implicit none
   type(tch_function_type), intent(in)                :: par
   type(reflection_type), dimension(:), intent(inout) :: ref
   real, dimension(:), intent(in)                     :: xcount
   real, dimension(:), intent(in)                     :: fs    
   real, intent(in)                                   :: scal
   real, dimension(:), intent(out)                    :: ycount
   integer, intent(in)                                :: nwave
   real, intent(in)                                   :: rapporto_int
   integer, intent(out)                               :: ier
   integer                                            :: iniz,ifin
   integer                                            :: j,kal,nnn
   real                                               :: eta
   real                                               :: fU,fV,fW,fZ,fX,fY
   real                                               :: thrad,tanth,costh
   real                                               :: uvw2
   real                                               :: hg,hl,fwhm,qh,argfwhm
   real                                               :: asym
   real                                               :: tthdeg
   real                                               :: cgaus2,cgaus1,cloren1,cloren2
!corr   real                                               :: constg2,constg1
!corr   real                                               :: constl
   real                                               :: ffx
   real                                               :: deltath,deltath2
   real                                               :: pvoigt,asterm
   real, dimension(2)                                 :: rap
   real                                               :: arghg
   integer                                            :: neg1,neg2,negfw
!
   neg1=0
   neg2=0
   ier = 0
   ycount(:) = 0.0
!
   call init_calc(size(ref))
!
   rap(1) = 1.0
   rap(2) = rapporto_int
   fU = par%u%val
   fV = par%v%val
   fW = par%w%val
   fZ = par%z%val
   fX = par%x%val
   fY = par%y%val
   asym = par%asym%val
!corr   constg1 = 4.0*alog(2.0)
!corr   constg2 = sqrt(constg1/pi)
!corr   constl  = 2.0/pi
   LOOP_KALPHA : do kal=1,nwave
      LOOP_RIFLESSI : do j=1,size(ref)
         tthdeg = ref(j)%tthd  
         thrad = 0.5 * tthdeg * dtor
         costh = cos(thrad)
         tanth = sin(thrad) / costh
!
         arghg = fU*tanth**2 + fV*tanth + fW + fZ/costh**2
         if (arghg < epsilon(0.1)) then
           ! write(6,*)'NEGATIV'
             arghg = 0.0000001
             neg1 = neg1+1
           ! ierprof = 1
           ! return
         endif
         hg = sqrt (arghg)                                        !componente gaussiana della FWHM
         hl = fX*tanth + fY/costh                                 !componente lorentziana della FWHM
         argfwhm = hg**5 + 2.69269*hg**4*hl + 2.42843*hg**3*hl**2 + 4.47163*hg**2*hl**3 + 0.07842*hg*hl**4 + hl**5
         if (argfwhm < epsilon(0.1)) then
         !!!!    write(0,*)'WARNING argfwhm = ',argfwhm,j
             argfwhm = 0.0000001
             neg2 = neg2+1
         !   write(0,*)'WARNING argfwhm'
         !   write(6,*)'WARNING argfwhm'
         !   ierprof = 1
         !   return
         endif
         fwhm = argfwhm**0.2
         qh = hl / fwhm
         eta = 1.36603*qh - 0.47719*qh**2 + 0.11116*qh**3         !PV mixing parameter
!
         cgaus1 =   CONSTG1/fwhm**2
         cgaus2 =   CONSTG2/fwhm
         cloren1 = CONSTL / fwhm
         cloren2 = 4.0 / fwhm**2
!
         ref(j)%fwhm = fwhm
         etav(j) = eta
         uvw2 =  fwhm * ref(j)%pk * rapIcalc(j)                  !ampiezza del picco
!
         iniz = clocate(xcount,tthdeg-uvw2)
         ifin = clocate(xcount,tthdeg+uvw2)
!
         ffx = ref(j)%m*fs(j)**2*ref(j)%po*ref(j)%lp*ref(j)%as*rap(kal)
!!!!         IcalcK(j,kal) = ffx
!
         LOOP_CONTEGGI : do nnn=iniz,ifin
            deltath=xcount(nnn)-tthdeg
            deltath2=deltath**2
            pvoigt = eta*cloren1/(1.0+cloren2*deltath2) + (1.0-eta)*cgaus2*exp(-cgaus1*deltath2)
            asterm=1.0-(asym*sign(1.0,deltath)*deltath2)/tanth
            ycount(nnn)=ycount(nnn)+scal*ffx*pvoigt*asterm
         enddo LOOP_CONTEGGI
      enddo LOOP_RIFLESSI
   enddo LOOP_KALPHA
!
   !negfw = neg1+neg2
   negfw = neg1
   if (negfw>0)then
!!!       write(0,*)'VALORI FUNZIONE NEGATIVI ',neg1,neg2
       !if (real(negfw)/size(ref) > 0.4) ierprof = 1
   endif
!
   end subroutine psvoigt_tchz
!--------------------------------------------------------------------

   subroutine derpsvoi_tchz(par,inizr,ifinr,xcount,ref,cell,derFct,derOP,deryz,dfw,derp,dercc,derG,    &
                            rapporto_int,scalr,rinfo,nwave,gpo,neutro,nsyn,wave,jsys)
   USE progtype, only: refine_info_type
   USE arrayutil
   USE profile_function
   USE reflection_type_util
!
   implicit none
   type(tch_function_type), intent(in)             :: par
   integer, intent(in)                             :: inizr,ifinr
   real, dimension(:), intent(in)                  :: xcount
   type(reflection_type), dimension(:), intent(in) :: ref
   real, dimension(6), intent(in)                  :: cell
   real, dimension(:), intent(in)                  :: derFct
   real, dimension(:), intent(in)                  :: derOP
   real, dimension(:), intent(out)                 :: deryz
   real, dimension(:,:), intent(out)               :: dfw
   real, dimension(:,:), intent(out)               :: derp
   real, dimension(:,:), intent(out)               :: dercc
   real, dimension(:), intent(out)                 :: derG
   real, intent(in)                                :: rapporto_int
   type(rparam_type), intent(in)                   :: scalr
   type(refine_info_type), intent(in)              :: rinfo
   integer, intent(in)                             :: nwave
   type(rparam_type), intent(in)                   :: gpo
   integer, intent(in)                             :: neutro, nsyn, jsys
   real, intent(in)                                :: wave
   integer                                         :: j,k,kal,nnn
   integer                                         :: iniz,ifin
   logical                                         :: iderc
   real                                            :: asym,scal
   real                                            :: cgaus2,cgaus1,cloren1,cloren2
!corr   real                                            :: constl,constg2,constg1
   real                                            :: qh,eta,detaqh,dqhfwhm
   real                                            :: detah
   real                                            :: tthdeg
   real                                            :: uvw2
   real                                            :: pesolp,absoc
   real                                            :: derft,dlorp
   real                                            :: ffnop,ffx,ffb1
   real                                            :: deltath,deltath2,cc
   real                                            :: asterm,pvoigt,pgau,plor
   real                                            :: deruvw,derasth
   real                                            :: derpth,deryop
   real, dimension(6)                              :: dercells
   real                                            :: derfunczero,deraszero
   real, dimension(2)                              :: rap
   real, dimension(3)                              :: vetder,gradprof
   real                                            :: fU,fV,fW,fZ,fX,fY
   real                                            :: hg,hl,fwhm,argfwhm,pfw
   real                                            :: derfg,derfl
   real                                            :: dfwhmg,dfwhml
   real                                            :: derhgt,derhlt
   real                                            :: dconst
   real                                            :: thrad,tanth,tanth2,senth,costh
   real                                            :: secth2
   real, dimension(6)                              :: ct
   real, dimension(6)                              :: hgv,hlv
   integer, dimension(6)                           :: ci
   real                                            :: arghg
   real                                            :: kpol
!
   if (neutro == 1 .or. nsyn == 1) then
       kpol = 0.0
   else
       kpol = 0.8
   endif
   deryz = 0.0
   dfw   = 0.0
   derp  = 0.0
   dercc = 0.0
   derG(:)  = 0.0
!
   ct = (/1.0,2.69269,2.42843,4.47163,0.07842,1.0/)
   ci = (/0,1,2,3,4,5/)
   vetder(1) = 1.0
   rap(1) = 1.0
   rap(2) = rapporto_int
   asym = par%asym%val
   fU = par%u%val
   fV = par%v%val
   fW = par%w%val
   fZ = par%z%val
   fX = par%x%val
   fY = par%y%val
   scal = scalr%val
   iderc = rinfo%refine_cell
!corr   constg1 = 4.0*alog(2.0)
!corr   constg2 = sqrt(CONSTG1/pi)
!corr   constl  = 2.0/pi
   LOOP_KALPHA : do kal=1,nwave
      LOOP_RIFLESSI : do j=inizr,ifinr
         tthdeg = ref(j)%tthd  
         thrad = 0.5 * tthdeg * dtor
         costh = cos(thrad)
         senth = sin(thrad)
         tanth = senth / costh
         tanth2 = tanth**2
!
         arghg = fU*tanth2 + fV*tanth + fW + fZ/costh**2
         if (arghg < epsilon(0.1)) then
         !   write(0,*)'NEGATIV'
             arghg = 0.0000001
         endif
         hg = sqrt ( arghg )    !componente gaussiana della FWHM
         hl = fX*tanth + fY/costh                                 !componente lorentziana della FWHM
         hgv = hg**ci(6:1:-1); hlv = hl**ci                       !potenze di hg e hv
         argfwhm = sum(ct*hgv*hlv)
         if (argfwhm < 0.0) then
             argfwhm = 0.0000001
             write(6,*)'WARN DERIVATE'
         endif
         fwhm = argfwhm**0.2                                      !FWHM
         pfw = 0.2*argfwhm**(-0.8)
         dfwhmg = pfw*sum(ct(:5)*ci(6:2:-1)*hgv(2:)*hlv(:5))      !der. di FWHM risp. hg
         dfwhml = pfw*sum(ct(2:6)*ci(2:6)*hgv(2:6)*hlv(:5))       !der. di FWHM risp. hl
!
         cgaus1 =  CONSTG1 / fwhm**2
         cgaus2 =  CONSTG2 / fwhm
         cloren1 = CONSTL / fwhm
         cloren2 = 4.0 / fwhm**2
!
         qh = hl / fwhm
         eta = 1.36603*qh - 0.47719*qh**2 + 0.11116*qh**3         !PV mixing parameter
         detaqh = 1.36603 - 2.0*0.47719*qh + 3.0*0.11116*qh**2    !der. eta risp. qh
         dqhfwhm = -hl/fwhm**2                                    !der. qh risp. fwhm
         detah  = detaqh*dqhfwhm                                  !der. eta risp. h
!
         derft = derFct(j) 
         secth2 = 1.0 / costh**2
         !!!!!!!ref(j)%pk = 10.
         uvw2 = fwhm * ref(j)%pk * rapIcalc(j)
!
         iniz = clocate(xcount,tthdeg-uvw2)
         ifin = clocate(xcount,tthdeg+uvw2)
!
         if (iderc) then
            dlorp = derivataLP(ref(j)%tthd,kpol)
            derhgt = secth2*(fV + 2.0*(fU+fZ)*tanth)*0.5/hg         !der. hg risp. theta
            derhlt = secth2*(fX + fY*senth)                         !der. hl risp. theta
            vetder(3) = dfwhmg*derhgt + dfwhml*derhlt               !der. FWHM risp. theta
            vetder(2) = detaqh*(derhlt/fwhm+dqhfwhm*vetder(3))      !der. eta risp. theta
            dercells = scal*derteta_risp_celldir(jsys,cell,ref(j)%hkl,ref(j)%tthd,wave) !(der. di theta risp. alla cella) * scala
         endif
         pesolp = ref(j)%lp
         absoc = ref(j)%as
         ffnop = pesolp*ref(j)%m*(ref(j)%fc**2)*absoc
         !!!!op = oprcorr(j,:3)
         !!!!opT = sum(frac*op)
         !!!!opT = op(1)
         ffx = ffnop*ref(j)%po
         ffb1 = rap(kal)*pesolp*ref(j)%m*ref(j)%po*absoc
         LOOP_CONTEGGI : do nnn=iniz,ifin
            deltath=xcount(nnn)-tthdeg
            deltath2=deltath**2
            cc=(1.0+cloren2*deltath2)
            plor= cloren1/cc
            pgau= cgaus2*exp(-cgaus1*deltath2)
            pvoigt = eta*plor + (1.0-eta)*pgau
            asterm=1.0-(asym*sign(1.0,deltath)*deltath2)/tanth
!
!           Derivative of 2theta corrected
            !deraszero = 4.0*asym*sign(1.0,deltath)*deltath/tanth                       ! der. asym. rispetto zero
            !derfunczero = 4.0*deltath*(cloren2*plor*eta/cc + cgaus1*(1.0-eta)*pgau)    ! der. pvoigt rispetto zero
            deraszero = -2.0*asym*sign(1.0,deltath)*deltath/tanth                       ! der. asym. rispetto 2theta corrected
            derfunczero = -2.0*deltath*(cloren2*plor*eta/cc + cgaus1*(1.0-eta)*pgau)    ! der. pvoigt rispetto 2theta corrected
            deryz(nnn) = deryz(nnn) + scal * ffx * (derfunczero*asterm + deraszero*pvoigt)
!
!           Derivata Pseudo-Voigt rispetto eta
!
            gradprof(2) = plor - pgau
!
!           Derivata Pseudo-Voigt rispetto FWHM
!
            gradprof(3) = ( eta*plor*(8.0*deltath2-cc*fwhm**2)/cc + (eta-1)*pgau*(fwhm**2-2.0*CONSTG1*deltath2) ) / fwhm**3
!
            if (rinfo%refine_profile) then
              dconst = scal*ffx*asterm
              deruvw = (gradprof(2)*detah+gradprof(3))                                       !der. rispetto FWHM
!
              derfg  = dconst*deruvw*dfwhmg/(2.0*hg)
              dfw(nnn,1)=dfw(nnn,1) + derfg * tanth2                         !der. rispetto U
              dfw(nnn,2)=dfw(nnn,2) + derfg * tanth                          !der. rispetto V
              dfw(nnn,3)=dfw(nnn,3) + derfg                                  !der. rispetto W
              dfw(nnn,4)=dfw(nnn,4) + derfg / costh**2                       !der. rispetto Z
!
              derfl = dconst*(deruvw*dfwhml+gradprof(2)*detaqh/fwhm)
              dfw(nnn,5)=dfw(nnn,5) + derfl * tanth                          !der. rispetto X
              dfw(nnn,6)=dfw(nnn,6) + derfl / costh                          !der. rispetto Y
        !!!      if (j==1) then
        !!!  write(6,'(a6,x,i3,8f15.5)')'TCHDER=',nnn,deltath,derfwhm(nnn,5,nph)/(scal * ffx * asterm),  &
        !!!  deruvw/(scal * ffx * asterm),derfl/(scal * ffx * asterm),thrad,tanth,dfwhml,detaqh/fwhm
        !!!      endif
!
              derp(nnn,1)=derp(nnn,1) - scal*ffx*pvoigt*sign(1.0,deltath)*deltath2/tanth !der. rispetto asym
            endif
            if (iderc) then
               derasth=asym*sign(1.0,deltath)*deltath*(rtod*4.0/tanth + deltath/(sin(thrad)**2)) !der.asym risp. thrad
!
!              Derivata Psvoig rispetto theta in radianti del riflesso
!
               gradprof(1) = rtod*4.0*deltath*(eta*cloren2*plor/cc + (1-eta)*cgaus1*pgau)
!
               derpth = DOT_PRODUCT(gradprof,vetder)       !derivata rispetto theta del riflesso
            !  if (j==1) then
            ! !!!!     write(6,*)'hl,hg,h,eta',hl,hg,fwhm,eta
            !      write(6,'(a6,x,i3,9f15.5)')'TCHCELL',nnn,ttetaczero(nnn),derpth,gradprof,vetder,thrad
            !  endif
               do k=1,6
                   dercc(nnn,k)=dercc(nnn,k)+dercells(k)*            &
                        (ffx*(derpth*asterm+derasth*pvoigt)+(ffb1*derft+ffx/pesolp*dlorp)*pvoigt*asterm)
               enddo
            endif
!
            if (gpo%rcod > 0) then
                deryop = scal * ffnop * pvoigt * asterm
                derG(nnn)=derG(nnn) + deryop*derOP(j)
            endif
         enddo LOOP_CONTEGGI
      enddo LOOP_RIFLESSI
   enddo LOOP_KALPHA
!
   end subroutine derpsvoi_tchz

!--------------------------------------------------------------------

   subroutine dertchza(ref,par,inizr,ifinr,xcount,der,derc,nwave,rapporto_int,scalr)
   USE progtype, only: deratom
   USE reflection_type_util, only: reflection_type
   USE arrayutil
   USE profile_function
!
   implicit none
   type(reflection_type), dimension(:), intent(in) :: ref
   type(tch_function_type), intent(in)             :: par
   integer, intent(in)                             :: inizr,ifinr
   real, dimension(:), intent(in)                  :: xcount
   type(deratom), dimension(:,:), intent(in)       :: der
   type(deratom), dimension(:,:), intent(out)      :: derc
   integer, intent(in)                             :: nwave
   real, intent(in)                                :: rapporto_int
   type(rparam_type), intent(in)                   :: scalr
   integer                                         :: j,jj,kal,nnn
   integer                                         :: iniz,ifin
   real                                            :: tthdeg
   real                                            :: rdeg,tdeg
   real, dimension(2)                              :: rap
   real                                            :: asym,scal,eta,uvw2
   real                                            :: deltath,deltath2
   real                                            :: asterm,pvoigt
   real                                            :: ffb1,dydI
!corr   real                                            :: constg1,constg2,constl
   real                                            :: cgaus1,cgaus2,cloren1,cloren2
   real                                            :: fwhm
   integer                                         :: nat
!
   derc = deratom(0.0,0.0,0.0)
   nat = size(derc,2)
!
   rap(1) = 1.0
   rap(2) = rapporto_int
   asym = par%asym%val
   scal = scalr%val
   !!!!frac = parcor(4:6)
!corr   constg1 = 4.0*alog(2.0)
!corr   constg2 = sqrt(constg1/pi)
!corr   constl  = 2.0/pi
   LOOP_KALPHA : do kal=1,nwave
      LOOP_RIFLESSI : do j=inizr,ifinr
         tthdeg = ref(j)%tthd
         rdeg = 0.5 * tthdeg * dtor
         tdeg = sin(rdeg) / cos(rdeg)
!
         fwhm = ref(j)%fwhm
         cgaus1 =   CONSTG1/fwhm**2
         cgaus2 =   constg2/fwhm
         cloren1 = CONSTL / fwhm
         cloren2 = 4.0 / fwhm**2
!
         eta = etav(j)
         uvw2 = fwhm * ref(j)%pk * rapIcalc(j)
!
         iniz = clocate(xcount,tthdeg-uvw2)
         ifin = clocate(xcount,tthdeg+uvw2)
!
         !!!!opT = sum(frac*oprcorr(j,:))
         !!!!opT = oprcorr(j,1)
         ffb1 = rap(kal)*ref(j)%lp*ref(j)%m*ref(j)%po*ref(j)%as
         LOOP_CONTEGGI : do nnn=iniz,ifin
            deltath=xcount(nnn)-tthdeg
            deltath2=deltath**2
            pvoigt = eta*cloren1/(1.0+cloren2*deltath2) + (1.0-eta)*cgaus2*exp(-cgaus1*deltath2)
            asterm=1.0-(asym*sign(1.0,deltath)*deltath2)/tdeg
!
!           calcolo derivate param. strutt. per atomo
!
            dydI = scal * ffb1 * pvoigt * asterm
            do jj=1,nat
               derc(nnn,jj)%co = derc(nnn,jj)%co + dydI * der(j,jj)%co
               derc(nnn,jj)%b = derc(nnn,jj)%b + dydI * der(j,jj)%b
               derc(nnn,jj)%occ = derc(nnn,jj)%occ + dydI * der(j,jj)%occ
            enddo
!
         enddo LOOP_CONTEGGI
      enddo LOOP_RIFLESSI
   enddo LOOP_KALPHA
!
   end subroutine dertchza

  !------------------------------------------------------------------------------------------

   subroutine foleb_tchz(ref,inizr,ifinr,Icalc,Iobs,tch,scalr,yoss,ytot,ttetaczero,back)
   USE arrayutil
   USE profile_function, only: rapIcalc
   USE reflection_type_util, only: reflection_type
!
   type(reflection_type), dimension(:), intent(in) :: ref
   integer, intent(in)                             :: inizr,ifinr
   real, dimension(:), intent(in)                  :: Icalc
   real, dimension(:), intent(out)                 :: Iobs
   type(tch_function_type), intent(in)             :: tch
   type(rparam_type), intent(in)                   :: scalr
   real, dimension(:), intent(in)                  :: yoss,ytot,ttetaczero,back
!corr   type(reflection_type), dimension(:), intent(in) :: ref
!corr   integer, intent(in)                             :: inizr,ifinr
!corr   real, dimension(:), intent(in)                  :: Icalc
!corr   real, dimension(:), intent(out)                 :: Iobs
   integer                                         :: iniz,ifin
   real                                            :: asym,eta,scal
   integer                                         :: j,nnn
   real                                            :: tthdeg,deltath,deltath2
   real                                            :: uvw2
   real                                            :: asterm,yci,diffo,ycalc
   real                                            :: pvoigt
   real                                            :: sumy
!corr   real                                            :: constg1,constg2,constl
   real                                            :: cgaus1,cgaus2
   real                                            :: cloren1,cloren2
   real                                            :: fwhm
   real                                            :: thrad,tanth
   real                                            :: epsmch = epsilon(1.0)
!
   asym = tch%asym%val
   scal = scalr%val
!
!corr   constg1 = 4.0*alog(2.0)
!corr   constg2 = sqrt(constg1/pi)
!corr   constl  = 2.0/pi
!
   LOOP_RIFLESSI : do j=inizr,ifinr
      sumy = 0.0
      tthdeg = ref(j)%tthd
      thrad = 0.5 * tthdeg * dtor
      tanth = sin(thrad) / cos(thrad)
!
      eta = etav(j)
      uvw2 = ref(j)%fwhm * ref(j)%pk * rapIcalc(j)
!
      fwhm = ref(j)%fwhm
      cgaus1 =   CONSTG1/fwhm**2
      cgaus2 =   CONSTG2/fwhm
      cloren1 = CONSTL / fwhm
      cloren2 = 4.0 / fwhm**2
!
      iniz = clocate(ttetaczero,tthdeg-uvw2)
      ifin = clocate(ttetaczero,tthdeg+uvw2)
!
      LOOP_CONTEGGI : do nnn=iniz,ifin
         deltath=ttetaczero(nnn)-tthdeg
         deltath2=deltath**2
         pvoigt = eta*cloren1/(1.0+cloren2*deltath2) + (1.0-eta)*cgaus2*exp(-cgaus1*deltath2)
         asterm=1.0-(asym*sign(1.0,deltath)*deltath2)/tanth
         yci = scal * pvoigt * asterm
         if (ytot(nnn) > epsmch) then
            diffo = yoss(nnn)-back(nnn)
            if (diffo.lt.0.0) then
                !diffo = 0.0
                cycle
            endif
            ycalc = diffo*yci/ytot(nnn)
         else
            !ycalc = 0.0
            cycle
         endif
         sumy = sumy + ycalc
      enddo LOOP_CONTEGGI
!
      Iobs(j) = Icalc(j) * sumy
!!!!      if (Iobs(j) < 0.0) Iobs(j) = 0.0
      if (Iobs(j) < epsmch) Iobs(j) = 0.0
!
   enddo LOOP_RIFLESSI
!
   end subroutine foleb_tchz

 !------------------------------------------------------------------------------------------

 END MODULE tchzf
